/**
 * 
 */
package archeopterix.algorithm;

import java.util.List;

import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.solution.Architecture;
import archeopterix.solution.ConstraintHandling;

/**
 * @author JMontgomery
 *
 */
public interface PopulationGenerator {
	
	public static final Class<?> DEFAULT_SOLUTION_REPRESENTATION = archeopterix.solution.Architecture.class;
	
	/**
	 * Generates a new collection {@code size} solutions using the current
	 * configuration's {@linkplain
	 * au.edu.swin.archeopterix.conf.Conf#getSolutionRepresentation() solution
	 * representation} and without any constraint violation handling.
	 */
	public List<Architecture> generatePopulation(int size, EmbeddedSystemProblem problem);

	/**
	 * Generates a new collection {@code size} solutions using the current
	 * configuration's {@linkplain
	 * au.edu.swin.archeopterix.conf.Conf#getSolutionRepresentation() solution
	 * representation} with the specified constraint violation handling
	 * approach.
	 */
	public List<Architecture> generatePopulation(int size, EmbeddedSystemProblem problem,
			ConstraintHandling constraintHandling);
	
	/**
	 * Generates a new collection of {@code size} solutions. 
	 */
	public List<Architecture> generatePopulation(int size, EmbeddedSystemProblem problem,
			ConstraintHandling constraintHandling, Class<?> archRep);

	/**
	 * Generates {@code target.size() - toSize} solutions and adds them to
	 * {@code target}; their representation is {@linkplain
	 * au.edu.swin.archeopterix.conf.Conf#getSolutionRepresentation() that set}
	 * in the current configuration.
	 */
	public List<Architecture> generatePopulation(List<Architecture> target, int toSize, EmbeddedSystemProblem problem,
			ConstraintHandling constraintHandling);

	/**
	 * Generates {@code target.size() - toSize} solutions and adds them to
	 * {@code target}; returns a reference to {@code target}. 
	 */
	public List<Architecture> generatePopulation(List<Architecture> target, int toSize, EmbeddedSystemProblem problem,
			ConstraintHandling constraintHandling, Class<?> archRep);
	public List<Architecture> resizePopulation(List<Architecture> target, int toSize, EmbeddedSystemProblem problem, ConstraintHandling constraintHandling);
}
