package archeopterix.algorithm;
public class Watch {
	  /**
	  * Start the Watch.
	  *
	  * @throws IllegalStateException if the Watch is already running.
	  */
	  public void start(){
	    if ( fIsRunning ) {
	      throw new IllegalStateException("Must stop before calling start again.");
	    }
	    //reset both start and stop
	    fStart = System.currentTimeMillis();
	    fStop = 0;
	    fIsRunning = true;
	    fHasBeenUsedOnce = true;
	  }

	  /**
	  * Stop the Watch.
	  *
	  * @throws IllegalStateException if the Watch is not already running.
	  */
	  public String GetElapsed(){
		    StringBuilder result = new StringBuilder();
		    result.append(System.currentTimeMillis() - fStart);
		    result.append(";ms");
		    return result.toString();
	  }
	  public void Reset(){
		  	fStart = System.currentTimeMillis();
	  }
	  public void stop() {
	    if ( !fIsRunning ) {
	      throw new IllegalStateException("Cannot stop if not currently running.");
	    }
	    fStop = System.currentTimeMillis();
	    fIsRunning = false;
	  }

	  /**
	  * Express the "reading" on the Watch.
	  *
	  * @throws IllegalStateException if the Stopwatch has never been used,
	  * or if the Watch is still running.
	  */
	  public String toString() {
	    validateIsReadable();
	    StringBuilder result = new StringBuilder();
	    result.append(fStop - fStart);
	    result.append(" ms");
	    return result.toString();
	  }

	  /**
	  * Express the "reading" on the Watch as a numeric type.
	  *
	  * @throws IllegalStateException if the Stopwatch has never been used,
	  * or if the Watch is still running.
	  */
	  public long toValue() {
	    validateIsReadable();
	    return fStop - fStart;
	  }

	  // PRIVATE ////
	  private long fStart;
	  private long fStop;

	  private boolean fIsRunning;
	  private boolean fHasBeenUsedOnce;

	  /**
	  * Throws IllegalStateException if the watch has never been started,
	  * or if the watch is still running.
	  */
	  private void validateIsReadable() {
	    if ( fIsRunning ) {
	      String message = "Cannot read a Watch which is still running.";
	      throw new IllegalStateException(message);
	    }
	    if ( !fHasBeenUsedOnce ) {
	      String message = "Cannot read a Watch which has never been started.";
	      throw new IllegalStateException(message);
	    }
	  }

}
