/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package archeopterix.algorithm.aco;

import archeopterix.problem.componentselection.algorithm.aco.CSACOOperators;
import java.util.HashMap;
import java.util.Map;
import java.util.NoSuchElementException;

/** OperatorFactory for Simulated Annealing. The concrete clases that implements
 * the abstract ACOOperators should be added to the ACOOperators array.
 */
public class ACOOperatorsFactory {
	
	private Map<Class<?>, ACOOperators> repToOperatorsMap = new HashMap<Class<?>, ACOOperators>();

	/**
	 * Please add new OperatorClass object from each concrete implementatin of the
	 * the Simulated Annealing Operators.
	 */
	private static ACOOperators[] availableOps = {
		
        new CSACOOperators()
	};
	
	private static ACOOperatorsFactory instance;
	
	private ACOOperatorsFactory() { }
	
	public synchronized static ACOOperatorsFactory instance() {

		if (instance == null)
			instance = new ACOOperatorsFactory();
		return instance;
	}

	/**
	 *
	 * @param solRepClass
	 * @return the aproporiate Simulated annealing operators object corresponds
	 * to the given solReprentation.
	 */
	public ACOOperators getSuitableOperators(Class<?> solRepClass) {
		ACOOperators ops = repToOperatorsMap.get(solRepClass);
		if (ops == null) {
			for (int i = 0; i < availableOps.length && ops == null; i++) {
				if (availableOps[i].canOperateOn(solRepClass))
					ops = availableOps[i];
			}
			if (ops == null)
				throw new NoSuchElementException("No appropriate SA operators found for " + solRepClass.getCanonicalName());
			repToOperatorsMap.put(solRepClass, ops);
		}
		return ops;
	}

}
