package archeopterix.algorithm.ea;
import archeopterix.algorithm.PopulationGenerator;
import archeopterix.algorithm.learning.LearningAlgorithm;
import archeopterix.algorithm.learning.utils.Parameter;
import archeopterix.algorithm.learning.utils.ValueParameter;
import archeopterix.solution.comparator.GenericComparator;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.solution.Architecture;
import archeopterix.solution.ConstraintHandling;
import archeopterix.solution.OutOfOptionsException;
import archeopterix.solution.Repairable;
import archeopterix.util.PropertyMap;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

/**
 *
 * @author Alda
 */
public final class AdaptiveGA<T extends Architecture> extends NSGA  {   
    private LearningAlgorithm learning;
    
    public AdaptiveGA() {
	super();
        try{
            learning = (LearningAlgorithm) Class.forName("archeopterix.algorithm.learning.benchmarks.PMLearner").newInstance();
        }catch(Exception ex){
            System.out.println(ex.toString());
        }
        GenericComparator cmp = null;
        try{
        cmp = (GenericComparator) Class.forName("archeopterix.solution.comparator.MSOPSComparator").newInstance();
        }catch(Exception ex){
            System.out.println(ex.toString());
        }
        List<Parameter> parameters = new LinkedList<Parameter>();
        Parameter mRate = new ValueParameter("mutationRate",2,0.01,0.5);
        mRate.initialiseBins();
        Parameter cRate = new ValueParameter("crossoverRate", 2, 0.7, 1.0);
        cRate.initialiseBins();
        parameters.add(mRate);
        parameters.add(cRate);
        learning.initialise(cmp,parameters);
    }
    
    @Override
    public void oneIteration(){
        super.oneIteration();
        try{
            learning.onIterationComplete(population, currIteration);
        }
        catch(Exception ex){
            System.out.println(ex.toString());
        }
    }
    @Override
    public void evolve(){
        int iMatingPoolSize = matingPool.size();
        Architecture[] children ={null, null};
        int childrenCounter = 0;
        while (childrenCounter < childrenPerIteration){
            children[0] = matingPool.get(rnd.nextInt(iMatingPoolSize));
            children[1]= matingPool.get(rnd.nextInt(iMatingPoolSize));
            crossoverProbability = ((Double)children[0].getParameterMap().get("crossoverRate").getCurrentBin().getValue()+(Double)children[0].getParameterMap().get("crossoverRate").getCurrentBin().getValue())/(double)2;
            if(rnd.nextDouble()<crossoverProbability){
                try{
                    crossoverCounter++;
                    children = operators.singlePointCrossover(children[0], children[1], problem);
                    for (int i = 0; i < children.length; i++){
                        children[i].setFeasible(problem.validate(children[i]));
                        if (!children[i].isFeasible()){
                            if (constraintHandling == ConstraintHandling.REPAIR){
                                ((Repairable) children[i]).attemptRepair(problem);
                                 children[i].setFeasible(true);
                            } else{
                                children[i].setConstraintViolation(problem);
                            }
                        }
                    }
                    processChild(children[0]);
                    childrenCounter++;
                    processChild(children[1]);
                    childrenCounter++; 

                } catch (OutOfOptionsException ex){
                    crossoverFailureCounter++;
                    continue;
                }
            }
            mutationProbability = (Double)children[0].getParameterMap().get("mutationRate").getCurrentBin().getValue();
            if (rnd.nextDouble() < mutationProbability){
                try{
                    mutationCounter++;
                    if (mutationOperator == MutationOperator.SINGLE_POINT_MUTATION){
                        children[0] = operators.singlePointMutation(children[0], problem);
                        children[1] = operators.singlePointMutation(children[1], problem);
                    } else if (mutationOperator == MutationOperator.SWAP_MUTATATION){
                        children[0] = operators.swapMutation(children[0], problem);
                        children[1] = operators.swapMutation(children[1], problem);
                    }
                    else if (mutationOperator==MutationOperator.UNIFORM_MUTATION){
                        children[0] = operators.uniformMutation(children[0], problem, mutationProbability);
                        children[1] = operators.uniformMutation(children[1], problem, mutationProbability);
                    }
                    children[0].setFeasible(problem.validate(children[0]));
                    children[1].setFeasible(problem.validate(children[1]));
                    if (!children[0].isFeasible()){
                        if (constraintHandling == ConstraintHandling.REPAIR){
                            ((Repairable) children[0]).attemptRepair(problem);
                            children[0].setFeasible(true);
                        } else{
                            children[0].setConstraintViolation(problem);
                        }
                    }
                    if (!children[1].isFeasible()){
                        if (constraintHandling == ConstraintHandling.REPAIR){
                            ((Repairable) children[1]).attemptRepair(problem);
                            children[1].setFeasible(true);
                        } else{
                            children[1].setConstraintViolation(problem);
                        }
                    }
                    processChild(children[0]);
                    childrenCounter++;
                    processChild(children[1]);
                    childrenCounter++; 
                } catch (OutOfOptionsException ex){
                    System.err.println(ex.getMessage());
                    mutationFailureCounter++;
                }   
            }   
        }
         
    }
    
    
    @Override
    public void initialise(EmbeddedSystemProblem problem, PropertyMap externalParams, Collection<Architecture> partialInitPop, PopulationGenerator popGenerator)
    {
        super.initialise(problem, externalParams, partialInitPop, popGenerator);
        learning.assignRandomParametersToSolutions(population, currIteration);
        
    }
}
