/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package archeopterix.algorithm.ea;

import archeopterix.problem.componentselection.algorithm.nsga.CSEAOperators;
import java.util.HashMap;
import java.util.Map;
import java.util.NoSuchElementException;

import archeopterix.problem.deployment.algorithm.nsga.ECUBasedDepEAOperators;
import archeopterix.problem.deployment.algorithm.nsga.OriginalDepEAOperators;
import archeopterix.problem.redundancy.algorithm.nsga.RAEAOperators;

/**
 *
 * @author imeedeniya
 */
public class EAOperatorsFactory {
	
	private Map<Class<?>, EAOperators> repToOperatorsMap = new HashMap<Class<?>, EAOperators>();

	private static EAOperators[] availableOps = {
		new OriginalDepEAOperators(),
		new ECUBasedDepEAOperators(),
        new RAEAOperators(),
        new CSEAOperators()
	};
	
	private static EAOperatorsFactory instance;
	
	private EAOperatorsFactory() { }
	
	public synchronized static EAOperatorsFactory instance() {
		if (instance == null)
			instance = new EAOperatorsFactory();
		return instance;
	}
	
	public EAOperators getSuitableOperators(Class<?> solRepClass) {
		EAOperators ops = repToOperatorsMap.get(solRepClass);
		if (ops == null) {
			for (int i = 0; i < availableOps.length && ops == null; i++) {
				if (availableOps[i].canOperateOn(solRepClass))
					ops = availableOps[i];
			}
			if (ops == null)
				throw new NoSuchElementException("No appropriate EA operators found for " + solRepClass.getCanonicalName());
			repToOperatorsMap.put(solRepClass, ops);
		}
		return ops;
	}

}
