package archeopterix.algorithm.ea;

import archeopterix.algorithm.PopulationGenerator;
import archeopterix.algorithm.Stats;
import archeopterix.conf.Conf;
import archeopterix.evaluation.Attribute;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.solution.Architecture;
import archeopterix.solution.OutOfOptionsException;
import archeopterix.util.PropertyMap;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.logging.Level;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * FIXME Won't currently work as i_CtoMRatio parameter disabled.
 *
 */
public class MOGA extends EA {
	private static final Log log = LogFactory.getLog(MOGA.class);

	//FIXME Change the way singlePointCrossover and mutation are done so that their respective probabilities are used, rather than this
	private int 	i_CtoMRatio; // Crossover to Mutation Ratio

        @Override
	public void initialise(EmbeddedSystemProblem problem, PropertyMap params, Collection<Architecture> partialInitPop, PopulationGenerator popGenerator) {
		super.initialise(problem, params, partialInitPop, popGenerator);

		//FIXME This will have to go
//		i_CtoMRatio		= iCtoMRatio;

		if (log.isDebugEnabled()) {
			//log.debug( "[" + oStopWatch.GetElapsed() + "] :Starting" );
			//log.debug( "[" + oStopWatch.GetElapsed() + "] :After Generating Popluation" );
			log.debug(m_Watch.GetElapsed()+
					" H" +i_ECUs + "_C"+ i_Comps +
					" PopSize=" + populationSize + " MaxEvals="+ maxEvaluations);
			//log.debug("After Sort :"+ lst_Population.size()+ "\n"+ lst_Population.toString());
			//log.debug("Initial Population"+ lst_Population.toString());
		}
		UpdateSolutions(population);
		printCurrentNondominatedSolutions("INITIAL");
		if (log.isDebugEnabled())
			log.debug( m_Watch.GetElapsed()+ ": INITIAL POPULATION: Avg.F= " + (d_SumOfFitness/ populationSize) );
		Collections.sort(population, comp);
	}

        @Override
	public void oneIteration(){
		PerformReproduction();
		//log.debug("New Index =" + iIndex + oDep3 );
		PerformExpiring();
		//Deployment oMedianDep = lst_Population.get( (populationSize / 2) +1  );
		//Deployment oMedianDep = lst_Population.get( 0 );
		if (log.isDebugEnabled()) {
			//log.debug("[" + oStopWatch.GetElapsed() + "] :Current Median : F= " + oMedianDep.d_Fitness + " Attributes:" + oMedianDep.GetAttributeList().toString());
			log.debug(m_Watch.GetElapsed()+ ": Itr: " + (currIteration+1) + " Evals=" +
					Stats.instance().getEvaluationCounter() +" Avg.F= " + (d_SumOfFitness/ populationSize) );
		}
		//if (((i+1)%10)== 0){
		UpdateSolutions(population);
		//PrintCurrentSolutions("After Iteration " + i);
		//}
		
		incrementIteration();
	}

        @Override
	public void finalise(Log log) {
		//Logger.Logln("\n[" + oStopWatch.GetElapsed() + "] :Algorithm Output : Size ="+ lst_Population.size()+ lst_Population.toString());
		//PrintCurrentPopulation("Before Pareto Set");
		GetParetoSet();
		//System.out.println(" --- Population :" + sText + " ---");
		log.debug("Pareto SET");
		for( int i=0; i< population.size(); i++){
			Architecture oDep = population.get(i); 
			log.debug("POP" + i + "," + oDep.toString());
		}

		//Logger.Logln(m_Watch.GetElapsed() + " :Pareto Set Size:"+ lst_Solutions.size() + " Avg.F= " + (d_SumOfFitness/ lst_Solutions.size()) + lst_Solutions.toString());
		printCurrentNondominatedSolutions("Final");
		//int iRnd1 = rnd.nextInt(populationSize);
		//int iRnd2 = rnd.nextInt(populationSize);
		//List lstAttributes = m_Analyzer.evaluateAttributes(oDep);
	}
	
	//FIXME Needed? Should extractNondominatedFront (or similar) in Algorithm do this?
	private void GetParetoSet(){
		for( int i=0; i < population.size(); i++){
			for ( int j=0; j < population.size(); j++ ){
				if (i==j)
					continue;
				if ( population.get(j).isDominated(population.get(i))){
					d_SumOfFitness -= population.get(j).d_Fitness;
					population.remove(j);
					i = (i>j)?(i-1):i;
					j = (j>0)?(j-1):0;
				}
			}
		}
	}
	
	
	private void PerformReproduction(){
		Random random = Conf.instance().random;
// 		TODO::Mating Pool Selection
		for (int i = 0; i < childrenPerIteration; i++ ) {
                    int iIndex1 = random.nextInt(populationSize);
                    int iIndex2 = random.nextInt(populationSize);
                    Architecture oDep1 = population.get(iIndex1);
                    Architecture oDep2 = population.get(iIndex2);
                    Architecture oChild =null;
                    Architecture[] oChildren ={null, null};
                    Architecture oMutator = null;

                    //Logger.Logln("Checking " + iIndex1 + " AND " + iIndex2 );
                    if ((i%i_CtoMRatio)== 0) {
                            //DO mutation
                            if ( oDep1.d_Fitness > oDep2.d_Fitness ){
                                    oMutator = oDep2;
                                    //Logger.Logln("Mutating " + iIndex2 );
                            }
                            else {
                                    oMutator = oDep1;
                                    //Logger.Logln("Mutating " + iIndex1 );
                            }
                try {
                    oChild = operators.swapMutation(oMutator, problem);
                } catch ( OutOfOptionsException ex) {
                    java.util.logging.Logger.getLogger(MOGA.class.getName()).log(Level.SEVERE, null, ex);
                }
			}
			else {
				
				boolean bSuccess =false;
				
				for (int iTry=0; iTry<i_Comps ; iTry++){
                    try {
                        oChildren = operators.singlePointCrossover(oDep1, oDep2, problem);
                    } catch (OutOfOptionsException ex) {
                        java.util.logging.Logger.getLogger(MOGA.class.getName()).log(Level.SEVERE, null, ex);
                    }
                    problem.validate(oChild);
	 
				}
				if ( !bSuccess){
					if (log.isDebugEnabled())
						log.debug("Crossover FAILED :" + oDep1 + oDep2 );
					continue;
				}
				//TODO:: Do BackTaracking here
			}
			problem.evaluateAttributes(oChild);
			//FIXME Should maxima be updated now? I.e., is child definitely going to be added?

			
			d_SumOfFitness += calculateFitness(oChild);
			// TODO:: Update lst_MaxAttribValues here and do a fitness calculation again (if necessary)
			int iNewIndex = Collections.binarySearch(population, oChild, comp);
			if (iNewIndex < 0) {
				population.add(-iNewIndex-1, oChild);
			} else {
				population.add(iNewIndex, oChild);
			}
			populationSize = population.size();
		}
	}
	

	
	private void PerformExpiring(){
		for ( int i = 0; i < childrenPerIteration; i++ ) {
			d_SumOfFitness -= population.get(0).d_Fitness;
			population.remove(0);
		}
		populationSize = population.size();
	}
	
	private void GenerateLocalPopulation(LinkedList<Architecture> lstPopulation, int iPopSize){
		population.clear();
		int iPopIndex = 0;
		for ( int i= 0; i < populationSize; i++) {
			if (iPopIndex >= lstPopulation.size())
				iPopIndex = 0;
			Architecture oDep = lstPopulation.get(iPopIndex).clone();
			problem.evaluateAttributes(oDep);
			List<Attribute> lstAttributes = oDep.getAttributeList();
			for ( int j=0; j<lstAttributes.size(); j++ ){
				Attribute oAtt = lstAttributes.get(j);
				updateMaxAttributeValues(oAtt, j);
			}
			population.add(oDep);
			iPopIndex++;
		}
		d_SumOfFitness = 0;
		for (int i = 0; i < populationSize; i++) {
			d_SumOfFitness += calculateFitness(population.get(i));
		}
	}


	
	
}
