package archeopterix.algorithm.exact;


import archeopterix.algorithm.Algorithm;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import archeopterix.algorithm.PopulationGenerator;
import archeopterix.problem.Component;
import archeopterix.problem.Host;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.deployment.solution.OriginalDeploymentSolution;
import archeopterix.solution.Architecture;
import archeopterix.util.PropertyMap;

//FIXME This algorithm appears to only work with Deployments
public class BruteForce extends Algorithm {
	private static final Log log = LogFactory.getLog(BruteForce.class);

	private int[][] ai_Loc;
	private int[]   ai_HM; //Host Memories
	private int[]   ai_Dep; // Current Deployment as array format
	private int[]   ai_CM; //Component Memories

	private int i_Feasible;

	
	@Override
	public void initialise(EmbeddedSystemProblem problem, PropertyMap params, Collection<Architecture> partialInitPop, PopulationGenerator popGenerator) {
		super.initialise(problem, params, partialInitPop, popGenerator);
		ai_Loc = new int[i_ECUs][i_Comps];
		ai_HM = new int[i_ECUs];
		ai_Dep = new int[i_Comps];
		ai_CM = new int[i_Comps];

		List<Component> comps = problem.getComponents();
		for (int iC = 0; iC < i_Comps; iC++) {
			Component oComp = comps.get(iC);
			ai_CM[iC] = oComp.getIntProperty(Component.MEMORY);
			ai_Dep[iC] = -1;
		}
		
		List<Host> ecus = problem.getECUs();
		for (int iH = 0; iH < i_ECUs; iH++) {
			Host oECU = ecus.get(iH);
			ai_HM[iH] = oECU.getIntProperty(Host.MEMORY);
			for (int iC=0; iC<i_Comps; iC++){
				ai_Loc[iH][iC] = oECU.getIntFromListProperty(Host.LOC_LIST, iC);
			}
			System.out.println("Loc H"+ iH + "=" + Arrays.toString(ai_Loc[iH]));
		}
		System.out.println("Component Memory : "+ Arrays.toString(ai_CM));
		System.out.println("Host Memory : "+ Arrays.toString(ai_HM));
	}

	public void oneIteration() {
		i_Feasible = 0;
		if (log.isDebugEnabled())
			log.debug(m_Watch.GetElapsed()+":Starting  H" +i_ECUs + "_C"+ i_Comps);
		/*for (int iC = 0; iC < i_Comps; iC++) {
                //for (int iC = (i_Comps-1); iC >= 0; iC--) {
			TryAllCompFrom(iC);
			printCurrentSolutions("Feasible Count="+i_Feasible+" at Component :"+ iC);
		}*/
		enumerateSolutions(0);
		if (log.isDebugEnabled())
			log.debug("Search Complete.Search Space="+ Math.pow(i_Comps, i_ECUs) + " Feasible="+ i_Feasible);
		printCurrentNondominatedSolutions("Final Pareto Set");
		
		incrementIteration();
	}
	
	private void enumerateSolutions(int iCompIndex) {
		if (iCompIndex == i_Comps) { //Go here only get VALI+FULL deployment
			i_Feasible++;
			//Logger.Logln(Logger.LOGFILE,"FEASIBLE:" +);
			OriginalDeploymentSolution oDep = getCurrentDeployment();
			problem.evaluateAttributes(oDep);
			checkDominance(oDep);

		} else {
			for (int iH = 0; iH < i_ECUs; iH++) {
				if (ai_Loc[iH][iCompIndex] == 1){
					if (ai_HM[iH] >= ai_CM[iCompIndex]){
						int iOldHost = ai_Dep[iCompIndex];
						if (iOldHost != -1) {
							ai_HM[iOldHost] +=  ai_CM[iCompIndex];
						}
						ai_Dep[iCompIndex] = iH;
						ai_HM[iH] -=  ai_CM[iCompIndex];
						enumerateSolutions(iCompIndex+1);
					}
				}
				//m_Dep.set(iCompIndex, iH);
				//enumerateSolutions(iCompIndex+1);
			}
			//System.out.println("Could not allocate for C"+ iCompIndex);
		}
	}

	private OriginalDeploymentSolution getCurrentDeployment(){
		OriginalDeploymentSolution oDep = new OriginalDeploymentSolution(i_ECUs, i_Comps);
		for (int iC = 0; iC < i_Comps; iC++) {
			oDep.set(iC, ai_Dep[iC]);
		}
		return oDep;
	}

	
}