package archeopterix.algorithm.fitnessLandscapeCharacterisation;

import archeopterix.algorithm.PopulationGenerator;
import archeopterix.algorithm.localSearch.LocalSearch;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.deployment.solution.OriginalDeploymentSolution;
import archeopterix.solution.Architecture;
import archeopterix.util.PropertyMap;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import org.apache.commons.logging.Log;

/**
 *
 * @author Alda
 */
public class FitnessPredictor extends LocalSearch{
    private List<Predictor> fitnessPredictors;
    //private double firstStepLength = 0.1;
    private  int stoppingCondition=0;
    private List<Double> errors;
    
    public FitnessPredictor(){
        errors=new LinkedList<Double>();
        fitnessPredictors = new LinkedList<Predictor>();
    }
    
    @Override
    public void oneIteration() {
        Architecture newSolution = new OriginalDeploymentSolution(problem);
        //List<Double> fitnessSer = new LinkedList<Double>();
        problem.evaluateAttributes(newSolution);   
        double initialFitness = newSolution.getAttributeValue(0);
        boolean changed = neighbourhoodSearch(0, newSolution);
        double firstStepFitness = newSolution.getAttributeValue(0);
        newSolution.setFeasible(problem.validate(newSolution));
        boolean feasible=newSolution.isFeasible();
        Predictor bestPred = selectPredictor(firstStepFitness-initialFitness);
        if (changed&&feasible){
            changed=true;
            for(int ic=1;ic<i_Comps&&changed&&feasible;ic++){
                changed=neighbourhoodSearch(ic,newSolution);
                newSolution.setFeasible(problem.validate(newSolution));
                feasible=newSolution.isFeasible();
            }
            
        }
        double secondStepFitness = newSolution.getAttributeValue(0);
        if(secondStepFitness>solution.getAttributeValue(0)){
            solution=newSolution.clone();
            problem.evaluateAttributes(solution);
        }
        if(bestPred!=null){
            double error = ((secondStepFitness-firstStepFitness)-bestPred.getSecondStepFitnessDiff())*((secondStepFitness-firstStepFitness)-bestPred.getSecondStepFitnessDiff());
            errors.add(error);
        }
        incrementIteration();
    }
    
    public final Predictor selectPredictor(double firstStepFitnessDiff){
        double diff =100;
        Predictor bestBredictor=null;
        if(fitnessPredictors!=null&&fitnessPredictors.size()>0)
        bestBredictor = fitnessPredictors.get(0);
        for(Predictor pred: fitnessPredictors){
            if(pred.getFirstStepFitnessDiff()-firstStepFitnessDiff<diff){
                bestBredictor = pred;
                diff=pred.getFirstStepFitnessDiff()-firstStepFitnessDiff;
            }     
        }
        return bestBredictor;
    }
    
    @Override
    public void initialise(EmbeddedSystemProblem oproblem, PropertyMap newparams, Collection<Architecture> partialInitPop, 
        PopulationGenerator opopGenerator){        
        super.initialise(oproblem,newparams,partialInitPop,opopGenerator);
        solution = new OriginalDeploymentSolution(problem);
        problem.evaluateAttributes(solution);
        initialisePredictors();
        System.out.println("Finished initialising predictors!");
    }
    
    public void initialisePredictors(){
        while(stoppingCondition<10000){
            boolean feasible=false;
            stoppingCondition++;
            Architecture newSolution=null;
            int count=0;
            while(!feasible && count<10){
                newSolution = new OriginalDeploymentSolution(problem); 
                newSolution.setFeasible(problem.validate(newSolution));
                count++;
                if (newSolution.isFeasible()){
                    feasible=true;
                    count=0;
                }
            }
            if(feasible==true){
                problem.evaluateAttributes(newSolution);   
                double initialFitness = newSolution.getAttributeValue(0);
                boolean changed = neighbourhoodSearch(0, newSolution);            
                if (changed){
                    changed=true;
                    double firstStepFitness = newSolution.getAttributeValue(0);
                    Architecture lastSol = null;
                    for(int ic=1;ic<i_Comps&&feasible&&changed;ic++){  
                        lastSol = newSolution.clone();
                        problem.evaluateAttributes(lastSol);
                        changed=neighbourhoodSearch(ic, (OriginalDeploymentSolution) newSolution);
                        newSolution.setFeasible(problem.validate(newSolution));
                        feasible=newSolution.isFeasible();
                    }
                    double secondStepFitness = lastSol.getAttributeValue(0);
                    if(secondStepFitness>solution.getAttributeValue(0)){
                        solution=lastSol.clone();
                        problem.evaluateAttributes(solution);
                    }
                    addPredictor(initialFitness,firstStepFitness,secondStepFitness, null); 
                }
            }
        }
    }
    public void addPredictor(double initialFitness, double firstStepFitness, double secondStepFitness, List<Double> fitnessSeries){
        if(firstStepFitness==initialFitness && secondStepFitness==firstStepFitness){
            return;
        }
        if(!enoughCanberraDistance(firstStepFitness-initialFitness, secondStepFitness-firstStepFitness)){
            return;
        }
        Predictor pred = new Predictor(firstStepFitness-initialFitness, secondStepFitness-firstStepFitness);
        pred.setfitnessSeries(fitnessSeries);
        fitnessPredictors.add(pred);   
    }
            
    public boolean enoughCanberraDistance(double firstStep, double secondStep){
        boolean add=true;
        for(Predictor pred: fitnessPredictors){
            if(firstStep-pred.getFirstStepFitnessDiff()<1.0E-6&&secondStep-pred.getSecondStepFitnessDiff()<1.0E-6)
                add=false;
        }
        return add;
    }       
    
    public double getCanberraDistance(){
        double canberraDistance = 0.0;
        for(Predictor predA: fitnessPredictors){
            for(Predictor predB: fitnessPredictors){
                if(predA.getFirstStepFitnessDiff()!=0.0||predB.getFirstStepFitnessDiff()!=0.0)
                    canberraDistance+=(predA.getFirstStepFitnessDiff()-predB.getFirstStepFitnessDiff())/(predA.getFirstStepFitnessDiff() + predB.getFirstStepFitnessDiff());
                if(predA.getSecondStepFitnessDiff()!=0.0||predB.getSecondStepFitnessDiff()!=0.0){
                    canberraDistance+=(predA.getSecondStepFitnessDiff()-predB.getSecondStepFitnessDiff())/(predA.getSecondStepFitnessDiff() + predB.getSecondStepFitnessDiff());
                }
            }
        }
        return canberraDistance;    
    }
    
    @Override
    public void finalise(Log log) {
        double overallErr=0.0;
        for(Double error: errors){
            overallErr+=error;
        }
        log.debug("CanberraDistance:"+getCanberraDistance()+":Overall Error:"+overallErr+":Number of Predictors:"+ fitnessPredictors.size()+":Best solution:"+solution);
    }
    
}
