package archeopterix.algorithm.fitnessLandscapeCharacterisation;

import java.util.LinkedList;
import java.util.List;
import org.apache.commons.math3.stat.correlation.SpearmansCorrelation;
import org.apache.commons.math3.stat.regression.SimpleRegression;

/**
 *
 * @author Alda
 */
public class Predictor {
    private double firstStepConstraintsDiff;
    private double secondStepConstraintsDiff;
    private double firstStepFitnessDiff;
    private double secondStepFitnessDiff;
    private List<Double> constraintSeries;
    private List<Double> fitnessSeries;
    private double fitnessConstraintCorr;
    private double constraintStepCorrelation;
    private double fitnessStepCorrelation;
    private double constraintSlope;
    private double fitnessSlope;
    
    public Predictor(){
        constraintSeries = new LinkedList<Double>();
        fitnessSeries = new LinkedList<Double>();
    }
    
    public double getFirstStepConstraintDiff(){
        return firstStepConstraintsDiff;
    }
    
    public double getSecondStepConstraintDiff(){
        return secondStepConstraintsDiff;
    }
    
    public double getFirstStepFitnessDiff(){
        return firstStepConstraintsDiff;
    }
    
    public double getSecondStepFitnessDiff(){
        return secondStepConstraintsDiff;
    }
    
    public double getCorrelation(){
        SpearmansCorrelation cor = new SpearmansCorrelation();
        double[] constraintArray = new double[constraintSeries.size()]; 
        double[] fitnessArray = new double[fitnessSeries.size()];
        for(int i=0; i<constraintSeries.size();i++){
            constraintArray[i]=constraintSeries.get(i);
            fitnessArray[i] = fitnessSeries.get(i);
        }
        fitnessConstraintCorr = cor.correlation(constraintArray, fitnessArray);
        return fitnessConstraintCorr;
    }
    
    public double getConstraintStepCorrelation(){
        double[] constraintArray = new double[constraintSeries.size()];
        double[] steps = new double[constraintSeries.size()];
        for(int i=0; i<constraintSeries.size();i++){
            constraintArray[i]=constraintSeries.get(i);
            steps[i]=i;
        }
        SpearmansCorrelation cor = new SpearmansCorrelation();
        constraintStepCorrelation = cor.correlation(constraintArray,steps);
        return constraintStepCorrelation;
    }
     public double getFitnessStepCorrelation(){
        double[] constraintArray = new double[fitnessSeries.size()];
        double[] steps = new double[fitnessSeries.size()];
        for(int i=0; i<fitnessSeries.size();i++){
            constraintArray[i]=fitnessSeries.get(i);
            steps[i]=i;
        }
        SpearmansCorrelation cor = new SpearmansCorrelation();
        fitnessStepCorrelation = cor.correlation(constraintArray,steps);
        return fitnessStepCorrelation;
    }
    public double getConstraintSlope(){
        double[][] constraintArray = new double[constraintSeries.size()][2]; 
        for(int i=0; i<constraintSeries.size();i++){
            constraintArray[i][1]=constraintSeries.get(i);
            constraintArray[i][0]=i;
        }
        SimpleRegression rg = new SimpleRegression();
        rg.addData(constraintArray);
        constraintSlope = rg.getSlope();
        return constraintSlope;
    }
    public double getFitnessSlope(){
        double[][] fitnessArray = new double[fitnessSeries.size()][2]; 
        for(int i=0; i<fitnessSeries.size();i++){
            fitnessArray[i][1]=fitnessSeries.get(i);
            fitnessArray[i][0]=i;
        }
        SimpleRegression rg = new SimpleRegression();
        rg.addData(fitnessArray);
        fitnessSlope = rg.getSlope();
        return fitnessSlope;
    }
    
    public void setConstraintSeries(List<Double> constraintSer){
        constraintSeries = constraintSer;
    }
    
    public void setfitnessSeries(List<Double> fitnessSer){
        fitnessSeries = fitnessSer;
    }
    
    public Predictor(double fStepConstraints, double sStepConstraints, double fStepFitness, double sStepFitness){
        firstStepConstraintsDiff = fStepConstraints;
        secondStepConstraintsDiff = sStepConstraints;
        firstStepFitnessDiff = fStepFitness;
        secondStepFitnessDiff = sStepFitness;
    }
    
    public Predictor(double fStepFitness, double sStepFitness){
        firstStepFitnessDiff = fStepFitness;
        secondStepFitnessDiff = sStepFitness;
    }
    
    public void setPredictor(double fStepConstraints, double sStepConstraints, double fStepFitness, double sStepFitness){
        firstStepConstraintsDiff = fStepConstraints;
        secondStepConstraintsDiff = sStepConstraints;
        firstStepFitnessDiff = fStepFitness;
        secondStepFitnessDiff = sStepFitness;
    }
    
    @Override
    public String toString(){
        String st= "FirstFitness:"+firstStepFitnessDiff+":Secondfitness:"+secondStepFitnessDiff;
        st+=":FirstConstraints:"+firstStepConstraintsDiff+":SecondConstraints:"+secondStepConstraintsDiff;
        return st;
    }
    
    
}
