package archeopterix.algorithm.heuristic;

import java.io.FileNotFoundException;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


import archeopterix.algorithm.Algorithm;
import archeopterix.algorithm.PopulationGenerator;
import archeopterix.algorithm.Stats;
import static archeopterix.algorithm.ea.EA.PARAM_POP_SIZE;
import archeopterix.conf.Conf;
import archeopterix.problem.EmbeddedSystemProblem;

import archeopterix.problem.deployment.solution.OriginalDeploymentSolution;
import archeopterix.reporting.HypervolumeDevelopment;
import archeopterix.solution.Architecture;
import archeopterix.util.PropertyMap;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.LinkedList;

public class BayesianHeuristic extends Algorithm {

	private static final Log log = LogFactory.getLog(BayesianHeuristic.class);
	public static final String PARAM_N_SOLUTIONS = "nSolutions";
	private LinkedList<LinkedList<Integer>> u_ij;
	private LinkedList<LinkedList<Integer>> u_ij_a;
	private ProbabilityTable p_a_ij;
	private ProbabilityTable p_ij_a;
	private ProbabilityTable p_ij;
	protected ProbabilityTable[] arr_PTable;
	protected int i_PTableCount;
	protected Comparator<Architecture>[] arr_Comparator;
	protected List<Solution> solutions;
        protected int populationSize;
	PrintStream fileStream;
	FileOutputStream out;

        protected BayesianHeuristic() {
            super();
            params.put(PARAM_POP_SIZE, 100);
        }
	@Override
	public void initialise(EmbeddedSystemProblem problem, PropertyMap params, Collection<Architecture> partialInitPop, PopulationGenerator popGenerator) {
		super.initialise(problem, params, partialInitPop, popGenerator);
                populationSize = params.getInt(PARAM_POP_SIZE);
		i_PTableCount = Conf.instance().getNEvaluators();
		arr_PTable = new ProbabilityTable[i_PTableCount];
		p_a_ij = new ProbabilityTable(i_Comps, i_ECUs);
		p_ij_a = new ProbabilityTable(i_Comps, i_ECUs);
		p_ij = new ProbabilityTable(i_Comps, i_ECUs);
		arr_Comparator = (Comparator<Architecture>[]) new Comparator<?>[i_PTableCount];
		solutions=new LinkedList<Solution>();
		try {
			ReinitLocalObjects();
		} catch (FileNotFoundException ex) {
			Logger.getLogger(BayesianHeuristic.class.getName()).log(Level.SEVERE, null, ex);
		}
	}


	protected void ReinitLocalObjects() throws FileNotFoundException {
		p_ij.reset();
		p_a_ij.reset();
		p_ij_a.reset();
		out = new FileOutputStream("./utils/Results/BayesianHeuristic.txt");
		fileStream = new PrintStream(out);
	}

	public ProbabilityTable getPTable(int iAttribute) {
		return p_ij_a;
	}

	public double getProbabilityValue(int iRow, int iCol) {
		return p_ij_a.getProbability(iRow, iCol);
	}

	public void setProbabilityValue(int iRow, int iCol, double dP) {
		p_ij_a.updateProbability(iCol, iCol, dP);
	}

	public void calculateProbability() {

		int k_a = 0;
		double sum = 0.0;
		double average = 0.0;
		int sSize = solutions.size();
		for (int i = 0; i < sSize; i++) {
			sum = sum + solutions.get(i).getArchitecture().getAttributeList().get(0).getNormalisedOptimiserValue();
		}
		average = sum / solutions.size();
		
		u_ij = new LinkedList<LinkedList<Integer>>();
		u_ij_a = new LinkedList<LinkedList<Integer>>();
		
		for (int i=0; i<i_Comps; i++){
			LinkedList<Integer> lstRow1 = new LinkedList<Integer>();
			LinkedList<Integer> lstRow2 = new LinkedList<Integer>();
			for (int j=0; j<i_ECUs; j++){
				lstRow1.add(0);
				lstRow2.add(0);
			}
			 u_ij.add(lstRow1);
			 u_ij_a.add(lstRow1);
		}

		int k = 0;
		for (int iS = 0; iS < populationSize; iS++) {
			OriginalDeploymentSolution oDep = solutions.get(iS).getArchitecture();
			for (int iComp = 0; iComp < i_Comps; iComp++) {
				k++;
				int indexHost = oDep.getHostIndex(iComp);
				u_ij.get(iComp).set(indexHost, (u_ij.get(iComp).get(indexHost) + 1));
				if (oDep.getAttributeList().get(0).getNormalisedOptimiserValue() > average) {
					k_a++;
					u_ij_a.get(iComp).set(indexHost, (u_ij.get(iComp).get(indexHost) + 1));

				}
			}
			double p_Ka = (double) k_a / (double) k;
			for (int iComp = 0; iComp < i_Comps; iComp++) {
				for (int iECU = 0; iECU < i_ECUs; iECU++) {
					double p_V_ij = (double) (u_ij.get(iComp).get(iECU)) / (double) k;
					if(p_V_ij>0)
					p_ij.updateProbability(iComp,iECU, p_V_ij);

					////************** ProbBest - EVIDENCE ******************////

					double paij = (double) (u_ij_a.get(iComp).get(iECU)) / (double) (k_a);
					double PaVij = paij / p_V_ij;
					if(PaVij>0)
					p_a_ij.updateProbability(iComp,iECU, PaVij);

					////************** Posterior Probability ******************////

					double PVija = PaVij * p_V_ij / p_Ka;
					if(PVija>0&&!Double.isNaN(PVija))
					p_ij_a.updateProbability(iComp,iECU,PVija);
				}
			}
		}
	}

	@Override
	public void oneIteration() {
		if (log.isInfoEnabled()) {
			log.info(m_Watch.GetElapsed() + " H" + i_ECUs + "_C" + i_Comps + " Solutions=" + populationSize);
		}
		Solution oSol;
		for (int iSolution = 0; iSolution < populationSize; iSolution++) {
	
				oSol = new Solution(this, problem);
				//oSol.ConstructSolution();

			problem.evaluateAttributes(oSol.getArchitecture());
			solutions.add(oSol);
		}
		calculateProbability();

		printCurrentNondominatedSolutions("Final Pareto Set");
		fileStream.println(nondominatedSolutions.get(0));
		fileStream.println(nondominatedSolutions.toString());
		fileStream.println(nondominatedSolutions.get(0).getSerializedString());
		//double hv =  HypervolumeDevelopment.getInstance().getHypervolume(nondominatedSolutions);
		//fileStream.println(hv);
	Stats.instance().printStatistics();
	}

	//FIXME Needs to be converted into oneIteration(), with initial and final code put elsewhere
	public void Run() {
		if (log.isInfoEnabled()) {
			log.info(m_Watch.GetElapsed() + ":Starting " 
					+ " H" + i_ECUs + "_C" + i_Comps
					+ " Solutions=" + populationSize);
		}

		for (int iSolutions = 0; iSolutions < populationSize; iSolutions++) {

			Solution oSol;
			do {
				oSol = new Solution(this, problem);

			} while (!problem.validate(oSol.getArchitecture()));

			problem.evaluateAttributes(oSol.getArchitecture());
		}

		printCurrentNondominatedSolutions("Final Pareto Set");
		//Stats.instance().printStatistics();

	}


}
