
package archeopterix.algorithm.learning;

import java.util.List;
import java.util.Map;
import archeopterix.algorithm.learning.utils.Bin;
import archeopterix.algorithm.learning.utils.CommonFunctions;
import archeopterix.algorithm.learning.utils.Parameter;
import archeopterix.solution.Architecture;
import java.util.HashMap;



public class ForecastBasedLearner<T> extends LearningAlgorithm<T>{

    /**
     * Populate New parameters for Executor Instances.
     * Create ARIMA model for each parameter and use to forecast next parameter
     * usages
     * @param List<Solution>
     * @param iCompletedIteration LastCompletedIteration
     */
    @Override
    public void prepareInstancesForNextIteration(List<T> population,int iCompletedIteration){

         for(T sol:population){
            prepareInstancesForNextIteration(sol, iCompletedIteration);
        }
    }

    @Override
     public void learn(int iCompletedIteration, List<T> pop){
        if  ( ( iCompletedIteration ) < 3){
            for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
                Parameter param = (Parameter)paramEntry.getValue();
                for( Bin bin : param.getListBins()){
                    bin.getBinPerformance().addPredictedPerformance(iCompletedIteration,bin.getBinPerformance().getAveragePerformance(iCompletedIteration));
                }
            }
        }
    }
     
    public void doFinalTasks(List<T> population) {
        for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter) paramEntry.getValue();
            for( Bin bin : param.getListBins()){
                for (int i=0; i < bin.getBinPerformance().getSuccessRateHistory().size(); i++){
                    System.out.println("Parameter:" + param.getName()+":Bin:"+bin.getIndex()+":Itr:"+i+":Performance:"+bin.getBinPerformance().getSuccessRateHistory().get(i)+":PredictedPerformance:"+bin.getBinPerformance().getPredictedSuccessRateHistory().get(i));
                }
            }
        }

    }

    @Override
    public void prepareInstancesForNextIteration(T solution, int iCompletedIteration) {
            HashMap<Object, Parameter> solParameterMap = new HashMap<Object, Parameter>();
            for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
                Parameter param = (Parameter) paramEntry.getValue();
                Parameter newParameter = param.clone();
                List<Bin> lstBins = param.getListBins();
                double[] adProbValues = new double[lstBins.size()];
                for( int i=0; i<lstBins.size(); i++)
                    adProbValues[i] = (Double)(lstBins.get(i)).getBinPerformance().getPredictedPerformance(iCompletedIteration);//(lstBins.get(i)).getSuccessProbHistory(iCompletedIteration-1);
                int iBinIndex = CommonFunctions.doRoulette(adProbValues);
                Bin newCurrentBin = newParameter.getBin(iBinIndex);
                newParameter.setCurrentBin(newCurrentBin);
                solParameterMap.put(newParameter.getName(), newParameter);
                System.out.println("Iteration:"+iCompletedIteration+":Parameter:"+newParameter.getName()+":Value:"+newParameter.getBin(iBinIndex).getValue());

            }
            ((Architecture)solution).setParameterMap(solParameterMap);
    }

}

