package archeopterix.algorithm.learning;

import archeopterix.algorithm.learning.utils.Membership;
import archeopterix.algorithm.learning.utils.Vector;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Alda
 */

public abstract class GenericClustering {
    
    protected Vector[] patterns;
    protected Membership mm;
    protected int nClusters;
    protected int nClasses;
    protected final String clusteringType = "";

    public GenericClustering(){}
    public GenericClustering(int iClasses, int iClusters, Vector... patts){
        nClasses = iClasses;
        nClusters = iClusters;
        patterns = patts;
        mm = new Membership(patterns.length, nClusters);
    }

    public void initialise(int iClasses, int iClusters, List<Double> values, List<Double> names){
        nClusters = iClusters;
        nClasses = iClasses;
        initialise(values, names);
    }
    
    public void initialise(List<Double> values, List<Double> names){
        List<Vector> observationList = new ArrayList<Vector>();
        for(int i=0;i<values.size();i++)
            observationList.add(new Vector(names.get(i).toString(),values.get(i)));
        
        patterns = new Vector[observationList.size()];
        for (int i = 0; i < observationList.size(); i++) 
            patterns[i] = observationList.get(i);
        mm = new Membership(patterns.length);
    }

    public void initialise(double[] values, double[] names){
        List<Vector> observationList = new ArrayList<Vector>();
        for(int i=0;i<values.length;i++)
            observationList.add(new Vector(String.valueOf(names[i]),values[i]));

        patterns = new Vector[observationList.size()];
        for (int i = 0; i < observationList.size(); i++)
            patterns[i] = observationList.get(i);
        mm = new Membership(patterns.length);

    }
    public abstract void cluster();
    public abstract void classify();
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder("");
        sb.append("pattern \t\t cluster" + "\n");
        int[] clusters = mm.getClusters();
        for (int i = 0; i < patterns.length; i++) {
            sb.append(patterns[i]).append("\t").append(clusters[i]).append("\n");
        }
        return sb.toString();
    }

    public Membership getMembershipMatrix(){
        return mm;
    }
    protected Vector[] getPatternsWithIndexes(List<Integer> indexes) {
        Vector[] newPatterns = new Vector[indexes.size()];
        for (int i = 0; i < newPatterns.length; i++) {
            newPatterns[i] = patterns[indexes.get(i)];
        }
        return newPatterns;
    }


    public double classEntropy(int a, int b){
        double entropy=0.0;
        int[] ces = new int[nClasses];
        int clLength = b-a+1;
        for(int i=a; i<b; i++)
            ces[mm.getClusters()[i]]++;

        for(int i=0; i<nClasses; i++){
            if(ces[i]>0)
                entropy+=ces[i]/(double)clLength*Math.log((double)ces[i]/(double)clLength);
        }

        return -entropy;
    }

}
