package archeopterix.algorithm.learning;

import java.util.List;
import org.apache.commons.math3.filter.DefaultMeasurementModel;
import org.apache.commons.math3.filter.DefaultProcessModel;
import org.apache.commons.math3.filter.KalmanFilter;
import org.apache.commons.math3.filter.MeasurementModel;
import org.apache.commons.math3.filter.ProcessModel;
import org.apache.commons.math3.linear.Array2DRowRealMatrix;
import org.apache.commons.math3.linear.ArrayRealVector;
import org.apache.commons.math3.linear.RealMatrix;
import org.apache.commons.math3.linear.RealVector;
import org.apache.commons.math3.random.JDKRandomGenerator;
import org.apache.commons.math3.random.RandomGenerator;
import archeopterix.algorithm.learning.utils.Bin;
import archeopterix.algorithm.learning.utils.BinPerformance;
import archeopterix.solution.comparator.GenericComparator;
import archeopterix.algorithm.learning.utils.Parameter;


/**
 *
 * @author Alda
 */
public class KalmanFilterLearner<T> extends ForecastBasedLearner<T> {
    private KalmanFilter filter;
    private RealVector pNoise;
    private RealVector mNoise;
    private RealMatrix A;
    private RealVector x;
    private RealMatrix H;
    private double processNoise = 1e-5d;
    
    @Override
     public void initialise(GenericComparator comp, List<Parameter> parameters) {
        super.initialise(comp, parameters);
        // A = [ 1 ]
        A = new Array2DRowRealMatrix(new double[] { 1d });
        // no control input
        RealMatrix B = null;
        x = new ArrayRealVector(new double[] {1d });
        // H = [ 1 ]
        H = new Array2DRowRealMatrix(new double[] { 1d });
        // Q = [ 0 ]
        RealMatrix Q = new Array2DRowRealMatrix(new double[] { processNoise });
        // R = [ 0 ]
        RealMatrix R = new Array2DRowRealMatrix(new double[] { 0 });
        RealMatrix P0 = new Array2DRowRealMatrix(new double[] { 1d });
        
        ProcessModel pm = new DefaultProcessModel(A, B, Q, x, P0);
        MeasurementModel mm = new DefaultMeasurementModel(H, R);
        filter = new KalmanFilter(pm, mm);
        pNoise = new ArrayRealVector(1);
        mNoise = new ArrayRealVector(1);
                        
    }
   @Override
    public void learn(int iCompletedIteration, List<T> pop){
        for( Parameter param : lstParametersToControl.values()){
            List<Bin> lstBins = param.getListBins();
            RandomGenerator rand = new JDKRandomGenerator();
            for( int i=0; i<lstBins.size();i++){
                BinPerformance binPerf = lstBins.get(i).getBinPerformance();
                double performance = (Double)binPerf.getAveragePerformance(iCompletedIteration);            
                filter.predict();
                 // simulate the process
                pNoise.setEntry(0, processNoise * rand.nextGaussian());

                // x = A * x + p_noise
                x = A.operate(x).add(pNoise);
                mNoise.setEntry(0,performance);
                RealVector z = H.operate(x).add(mNoise);
                filter.correct(z);
                double next = filter.getStateEstimation()[0];
                binPerf.addPredictedPerformance(iCompletedIteration, next);
            }         
        }
    }
}
