package archeopterix.algorithm.learning;
import archeopterix.algorithm.learning.utils.Bin;
import archeopterix.algorithm.learning.utils.CommonFunctions;
import archeopterix.solution.comparator.GenericComparator;
import archeopterix.algorithm.learning.utils.Parameter;
import archeopterix.conf.Conf;
import archeopterix.solution.Architecture;
import java.util.*;
import java.lang.Object.*;

public abstract class LearningAlgorithm<T> {
    protected Map<String, Parameter> lstParametersToControl;
    protected GenericComparator performanceMetric;
    protected Random random = Conf.instance().random;
    public void initialise(GenericComparator comp, List<Parameter> parameters) {
        performanceMetric = comp;
        lstParametersToControl = new HashMap<String, Parameter>();    
        for(Parameter newParameter: parameters){         
            lstParametersToControl.put(newParameter.getName().toString(),newParameter);
        }
    }

    /**
     * This will be called on completion of each iteration.
     * @param population
     * @param iCurrentIteration
     */
    public void onIterationComplete(List<T> population, int iCurrentIteration){
        for(T sol : population){
            if(((Architecture)sol).getParameterMap()==null)
               assignRandomParametersToSolutions(sol, iCurrentIteration);
            for (Parameter prm: ((Architecture) sol).getParameterMap().values()){
                Bin curSolutionBin = prm.getCurrentBin();
                Parameter controlledParam=lstParametersToControl.get(prm.getName().toString());
                Bin controlledBin= controlledParam.getBin(curSolutionBin.getBinIndex());
                controlledBin.getBinPerformance().addUsage(iCurrentIteration, curSolutionBin.getValue());
                controlledBin.getBinPerformance().addPerformance(iCurrentIteration, ((Architecture)sol).getRanking().getAverageRanking());
            }
        }
        for(Parameter param: lstParametersToControl.values()){
            for(Bin bin:param.getListBins()){
                List<Double> perf= bin.getBinPerformance().getPerformanceHistory().get(iCurrentIteration);
                double[] arrPer = new double[perf.size()];
                for(int i=0;i<perf.size();i++){
                    arrPer[i]=perf.get(i);
                }
                double avgPerf= CommonFunctions.getAverage(arrPer);
                bin.getBinPerformance().addAveragePerformance(iCurrentIteration, avgPerf);
                System.out.println("Iteration:"+iCurrentIteration+":Parameter:"+param.getName()+":Index:"+bin.getIndex()+":Performance"+avgPerf);
            }
        }
        learn(iCurrentIteration,population);
        prepareInstancesForNextIteration(population, iCurrentIteration);
          
    }

    public void onIterationComplete(T sol, int iCurrentIteration){
        if(((Architecture)sol).getParameterMap()==null)
           assignRandomParametersToSolutions(sol, iCurrentIteration);
        for (Parameter prm: ((Architecture) sol).getParameterMap().values()){
            Bin curSolutionBin = prm.getCurrentBin();
            Parameter controlledParam=lstParametersToControl.get(prm.getName().toString());
            Bin controlledBin= controlledParam.getBin(curSolutionBin.getBinIndex());
            controlledBin.getBinPerformance().addUsage(iCurrentIteration, curSolutionBin.getValue());
            controlledBin.getBinPerformance().addPerformance(iCurrentIteration, ((Architecture)sol).getAttributeValue(0));
        }
        for(Parameter param: lstParametersToControl.values()){
            for(Bin bin:param.getListBins()){
                List<LinkedList<Double>> performance= bin.getBinPerformance().getPerformanceHistory();
                if(performance.size()>iCurrentIteration){
                    List<Double> perf= performance.get(iCurrentIteration);
                    double[] arrPer = new double[perf.size()];
                    for(int i=0;i<perf.size();i++){
                        arrPer[i]=perf.get(i);
                    }
                    double avgPerf= CommonFunctions.getAverage(arrPer);
                    bin.getBinPerformance().addAveragePerformance(iCurrentIteration, avgPerf);
                }
            }
        }
        List<T> pop = new LinkedList<T>();
        pop.add(sol);
        learn(iCurrentIteration,pop);
        prepareInstancesForNextIteration(sol, iCurrentIteration);       
    }
    protected abstract void learn(int iCompletedIteration, List<T> pop);
    protected abstract void prepareInstancesForNextIteration(List<T> population, int iCompletedIteration);
    public void prepareInstancesForNextIteration(T sol,int iCompletedIteration){   
        HashMap<Object, Parameter> solParameterMap = new HashMap<Object, Parameter>();
        for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter) paramEntry.getValue();
            Parameter newParameter = param.clone();
            List<Bin> lstBins = param.getListBins();
            double[] adProbValues = new double[lstBins.size()];
            for( int i=0; i<lstBins.size(); i++)
                adProbValues[i] = (Double)lstBins.get(i).getBinPerformance().getAveragePerformance(iCompletedIteration);//(lstBins.get(i)).getSuccessProbHistory(iCompletedIteration-1);
            int iBinIndex = CommonFunctions.doRoulette(adProbValues);

            Bin newCurrentBin = newParameter.getBin(iBinIndex);
            newParameter.setCurrentBin(newCurrentBin);
            solParameterMap.put(newParameter.getName(), newParameter);
        }
        ((Architecture)sol).setParameterMap(solParameterMap);
    }
        
    public void assignRandomParametersToSolutions(List<T> population, int iCompletedIteration){
        for (T sol : population) {
            assignRandomParametersToSolutions(sol, iCompletedIteration);
        }
    }
	
     public void assignRandomParametersToSolutions(T sol, int iCompletedIteration){
            HashMap<Object, Parameter> solutionParametersMap = new HashMap<Object, Parameter>();
            for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
                Parameter prm = (Parameter)paramEntry.getValue();
                Parameter newParam = prm.clone();
                newParam.initialiseRandomly(iCompletedIteration);
                solutionParametersMap.put(newParam.getName(), newParam);
            }
            ((Architecture)sol).setParameterMap(solutionParametersMap);
    }
	
}
