
package archeopterix.algorithm.learning;

import archeopterix.algorithm.learning.utils.Bin;
import archeopterix.algorithm.learning.utils.ValueBin;
import archeopterix.algorithm.learning.utils.Parameter;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public class ParameterRangeLearner<T> extends SuccessBasedLearner<T>{

    @Override
    public void learn(int iCompletedIteration, List<T> pop){
        super.learn(iCompletedIteration,pop);
        adaptParameterBins(iCompletedIteration);
    }

    public void adaptParameterBins(int iCompletedIteration){
        for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter) paramEntry.getValue();
            List<Bin> lstBins = param.getListBins();
            double bestBinPerformance = (Double)(lstBins.get(1)).getBinPerformance().getSuccessRateHistory().get(iCompletedIteration);
            double worstBinPerformance = bestBinPerformance;
            for( Bin bin : lstBins ){
                double binPerformance = (Double)(bin).getBinPerformance().getSuccessRateHistory().get(iCompletedIteration);
                if (binPerformance>bestBinPerformance){
                    bestBinPerformance = binPerformance;
                }
                if (binPerformance<worstBinPerformance){
                    worstBinPerformance = binPerformance;
                }
                if(binPerformance==0.0){
                    bin.getBinPerformance().addSuccessRate(iCompletedIteration, 0.01);
                }
            }
            List<Bin> newLstBins = new LinkedList<Bin>();
            for( Bin xbin : lstBins ){
                ValueBin bin = (ValueBin)xbin;
                double binPerformance = (Double)bin.getBinPerformance().getSuccessRateHistory().get(iCompletedIteration);
                double oldMax = bin.getMax();
                double oldMin = bin.getMin();
                if(binPerformance>=bestBinPerformance){
                    double intervalLength = ((Double)oldMax-(Double)oldMin)/2;
                    double newMax = (Double)oldMin+intervalLength / 2;
                    double newMin = (Double)oldMin+intervalLength/2+0.0001;

                    Bin newBin1 = bin.clone();
                    ((ValueBin)newBin1).setMax(newMax);
                    Bin newBin2 = bin.clone();
                    ((ValueBin)newBin2).setMin(newMin);
                    newLstBins.add(newBin1);
                    newLstBins.add(newBin2);
                }
                else if(binPerformance<=worstBinPerformance){
                    if(newLstBins.size()>0||bin.getMax()==((ValueBin)param.getListBins().get(lstBins.size()-1)).getMax()){
                        ((ValueBin)newLstBins.get(newLstBins.size()-1)).setMax(oldMax);
                    }
                    else{
                        Bin newBin = ((ValueBin)bin).clone();
                        newLstBins.add(newBin);
                    }
                }
                else{
                    Bin newBin = bin.clone();
                    newLstBins.add(newBin);
                }
            }
            param.setListBins(newLstBins);
        }
    }

}
