package archeopterix.algorithm.learning;
import archeopterix.algorithm.learning.utils.Bin;
import archeopterix.algorithm.learning.utils.CommonFunctions;
import archeopterix.algorithm.learning.utils.Parameter;
import archeopterix.solution.Architecture;
import java.util.List;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author Alda
 * @param <T>
 */
public class SuccessBasedLearner<T> extends LearningAlgorithm<T>{
    private double threshold=0.1;

    @Override
    public void learn(int iCompletedIteration, List<T> population){
        for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter) paramEntry.getValue();
            Architecture sol = (Architecture)getPercentile(population, threshold);
            for(Bin bin : param.getListBins()){
                int inAboveAvg = bin.getBinPerformance().getOutperformerCount(performanceMetric, sol.getAttributeValue(0), iCompletedIteration, true);
                int inTotalUsage = bin.getBinPerformance().getUsage(iCompletedIteration);
                double dSuccessProbability = (inTotalUsage > 0)? ((double)inAboveAvg/(double)inTotalUsage) :0.01;
                (bin).getBinPerformance().addSuccessRate(iCompletedIteration,dSuccessProbability);
                
            }
        }
    }

    public T getPercentile(List<T> population, double dPercentile){
            Collections.sort(population, performanceMetric);
            int iIndex = (int) ( (1- dPercentile) * population.size());
            return population.get(iIndex);
    }


     /**
     * Printing of results
     */
    
    public void doFinalTasks(List<T> population) {
        for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter) paramEntry.getValue();
            for(Bin bin : param.getListBins()){
                List<Double> performanceHistory = bin.getBinPerformance().getSuccessRateHistory();
                for (int i=0; i < performanceHistory.size(); i++){
                    String printRes = "Parameter:" + param.getName()+":Bin:"+bin.getIndex()+":Itr:"+i+":SucessRate:"+performanceHistory.get(i);
                }
            }
        }
    }

    /**
     * Populate New parameters for Executor Instances.
     * Use Previously calculated Success rates and select the bin from a discrete distribution
     * weighted from the success rates.
     * @param List<Solution> population 
     * @param iCompletedIteration LastCompletedIteration
     */

    @Override
    public void prepareInstancesForNextIteration(List<T> population,int iCompletedIteration){
        for(T sol:population){
            HashMap<Object, Parameter> solParameterMap = new HashMap<Object, Parameter>();
            for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
                Parameter param = (Parameter) paramEntry.getValue();
                Parameter newParameter = param.clone();
                List<Bin> lstBins = param.getListBins();
                double[] adProbValues = new double[lstBins.size()];
                for( int i=0; i<lstBins.size(); i++)
                    adProbValues[i] = (Double)(lstBins.get(i)).getBinPerformance().getSuccessRateHistory().get(iCompletedIteration);//(lstBins.get(i)).getSuccessProbHistory(iCompletedIteration-1);
                int iBinIndex = CommonFunctions.doRoulette(adProbValues);

                Bin newCurrentBin = newParameter.getBin(iBinIndex);
                newParameter.setCurrentBin(newCurrentBin);
                solParameterMap.put(newParameter.getName(), newParameter);
            }
            ((Architecture)sol).setParameterMap(solParameterMap);
        }
    }
}
