
package archeopterix.algorithm.learning.benchmarks;


import archeopterix.algorithm.learning.SuccessBasedLearner;
import archeopterix.algorithm.learning.utils.Bin;
import archeopterix.algorithm.learning.utils.Parameter;
import archeopterix.solution.Architecture;
import archeopterix.util.CommonFunctions;
import java.util.List;


public class AverageCreditAssignment<T extends Architecture> extends SuccessBasedLearner<T>{

	/**
	 * Populate New parameters for Executor Instances.
	 * Use Previously calculated Success rates and select the bin from a discrete distribution
	 * weighted from the success rates.
	 * @param mapExInstance
	 * @param iCompletedIteration LastCompletedIteration
	 */
	@Override
	public void prepareInstancesForNextIteration(List<T> population,int iCompletedIteration){

            int window = 5;
            for(T sol: population){
                for( Parameter param : sol.getParameterMap().values()){
                    List<Bin> lstBins = param.getListBins();
                    int inBins = lstBins.size();
                    double[] adProbValues = new double[inBins];
                    for( int i=0; i<inBins; i++)
                        adProbValues[i] = (lstBins.get(i)).getBinPerformance().getAverageSuccessInWindow(window, iCompletedIteration);
                    int iBinIndex = CommonFunctions.doRoulette(adProbValues);
                    param.setCurrentBin(lstBins.get(iBinIndex));
                   
                }
            }
	}
}
