/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package archeopterix.algorithm.learning.benchmarks;

import java.util.LinkedList;

public final class CreditAssignment {



	LinkedList<Double> wRewards = new LinkedList<Double>();
	private int W;//size of the sliding window
	int type; //extreme (0), average (1) and rank-based
	double bestReward;
	double sumReward;
        protected int times;
	boolean normalized;
	double maxReward; //keeps the maximum value of all the ops, for normalization
	boolean delta_fitness; //used to select between delta_fitness and fitness (few hours to deadline, so global variable for sure! :-)

	//CreditAssignment(int _W, bool _delta_fitness=0, int _type = 0, int _norm = 0);
	//CreditAssignment(int _W, int _type = 0, int _norm = 0);
	public CreditAssignment(int _W, boolean _delta_fitness, int _type, boolean _norm){
		W=_W;
                times=0;
		type = _type;
		normalized = _norm;
		delta_fitness = _delta_fitness;
		clearCredit();
	}

	public boolean isNorm(){
		//cout << "isNorm" << endl;
		return normalized;
	}

	public int getType(){
		return type;
	}

	public int getTimesOp(int op){
		return wRewards.size();
	}

//to use with mutation
public void addFitnessImprov(double fit_old, double fit_new){
	//cout << "addFitnessImprov1" << endl;
	double diff = fit_new - fit_old;
	if(delta_fitness)
		updateQueue(diff);
	else{
		if (diff <= 0) fit_new = 0;
		updateQueue(fit_new);
	}

}

//### I could have twice the information, by using both generated offspring to reward the operators
//### In the current way, there is always twice the information for the mutation operators
//### What might explain the difference in the best values found for W (to be checked).
//to use with crossover
public void addFitnessImprov(double fit_old1, double fit_old2, double fit_new1, double fit_new2){
	//cout << "addFitnessImprov2" << endl;
	double best_old, best_new;

	if(fit_old1 > fit_old2) best_old = fit_old1;
	else best_old = fit_old2;

	if(fit_new1 > fit_new2) best_new = fit_new1;
	else best_new = fit_new2;

	double diff = best_new - best_old;
	if(delta_fitness)
		updateQueue(diff);
	else{
		if(diff <= 0) best_new = 0;
		updateQueue(best_new);
	}
}

public void addReward(double reward){
    times++;
    updateQueue(reward);
}

public int getTimes(){
    return times;
}

public void setMaxReward(double _max){
	//cout << "setMaxReward" << endl;
	if(_max < 0) maxReward = 0;
	else maxReward = _max;
}

public double getMaxReward(){
	//cout << "getMaxReward" << endl;
	return maxReward;
}

public double getReward(boolean nonorm){
	if(type==0) return getBestReward(nonorm); //Extreme
	else if(type==1) return getAvgReward(nonorm); //Average
	else if(type==2) return getLastReward(nonorm); //Instantaneous
	else return 0;
}

public double getLastReward(boolean nonorm){
	//cout << "getLastReward" << endl;
	double instRew, divide;

	if(!normalized || nonorm || maxReward==0) divide=1.0;
	else divide = maxReward;

	if(wRewards.size()>0){
		instRew = wRewards.getFirst();
		double reward = instRew/divide;
		if(reward > 0) return reward;
	}

	return 0;
}

public double getBestReward(boolean nonorm){
	//cout << "getBestReward" << endl;
	double divide;

	if(!normalized || nonorm || maxReward==0) divide=1.0;
	else divide=maxReward;

	double reward=bestReward/divide;
	if(reward > 0)	return reward;
	return 0;
}

public double getAvgReward(boolean nonorm){
	//cout << "getAvgReward" << endl;
	double divide;
	double avg = (double)sumReward/(double)wRewards.size();

	if(!normalized || nonorm || maxReward==0) divide=1.0;
	else divide=maxReward;

	double reward = avg/divide;
	if(reward>0) return reward;
	return 0;
}

public void updateQueue(double reward){
	//cout << "updateQueue \t" << op << "\t" << reward << "\t" << wRewards.size() << "\t" << W << endl;
	sumReward += reward; //update the sum, used to extract the average, counts also the negative rewards.
	wRewards.add(reward); //insert the latest reward in the queue

	if(reward > bestReward)	//if it is better than the actual best,
            bestReward = reward;//update the best

	if(wRewards.size() > W){ 		//if the queue becomes bigger than W
            double back = wRewards.getLast(); //keeps the last one's value
            sumReward -= back;			//used to extract the average
            wRewards.remove(back); 		//removes the last one from the queue
            if(back == bestReward){		//if last was the best
                bestReward = 0;
                //deque<double>::reverse_iterator myIterator;
                for(Double d : wRewards){
                    if(d>bestReward)
                        bestReward = d;
                }
            }
	}
}

public boolean checkFullQueue(){
	if(wRewards.size()<W) return false;
	else	return true;
}

public void printQueue(){
//cout << "printQueue" << endl;
	
   for(double d :wRewards){
	   System.out.println(d);
   }
}

public void clearCredit(){
	if(wRewards!=null)
	wRewards.clear();
	bestReward = sumReward = maxReward = 0;
}

}
