package archeopterix.algorithm.learning.benchmarks;

import archeopterix.algorithm.learning.LearningAlgorithm;
import archeopterix.algorithm.learning.utils.Bin;
import archeopterix.algorithm.learning.utils.BinPerformance;
import archeopterix.algorithm.learning.utils.Parameter;
import archeopterix.solution.Architecture;
import archeopterix.solution.comparator.GenericComparator;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;


public class PMLearner<T extends Architecture> extends LearningAlgorithm<T> {

    protected double p_min = 0.15, alpha = 0.8;
    protected	int W = 50;
    protected int typeCAS = 0;
    protected boolean normalized = false;
    private HashMap<Object,ProbMatching> opSelector;
    protected int timeTotal = 4;
   @Override
     public void initialise(GenericComparator comp, List<Parameter> parameters) {
        super.initialise(comp, parameters);
            CreditAssignment crA;
            opSelector = new HashMap<Object,ProbMatching>();
            for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
                Parameter param = (Parameter) paramEntry.getValue();
                ProbMatching opS= new ProbMatching(param.getNumberBins(),  p_min, alpha);
                opS.credit = new LinkedList<CreditAssignment>();
                List<Bin> lstBins = param.getListBins();
                int inBins = lstBins.size();
                for(int i=0; i<inBins;i++){
                        crA = new CreditAssignment(W, true, typeCAS, normalized);
                        opS.credit.add(crA);
                }
                opSelector.put(param.getName(),opS);
            }
    }


    @Override
    protected void learn(int iCompletedIteration, List<T> pop){
	int iMinimumIterations =2;
        if  ( ( iCompletedIteration ) < iMinimumIterations){
            return;
        }
        for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
            Parameter param = (Parameter)paramEntry.getValue();
            List<Bin> lstBins = param.getListBins();
            for(Bin bin:lstBins){
                BinPerformance bPer = bin.getBinPerformance();
                double reward = bPer.getAveragePerformance(iCompletedIteration)- bPer.getAveragePerformance(iCompletedIteration-1); 
                opSelector.get(param.getName()).credit.get(bin.getBinIndex()).addReward(reward);
                opSelector.get(param.getName()).applyReward(bin.getBinIndex());
            }
        }
    }
    
    @Override
    public void prepareInstancesForNextIteration(List<T> population,int iCompletedIteration){
        for(T sol: population){
            prepareInstancesForNextIteration(sol, iCompletedIteration);
        }

    }
       @Override
    public void prepareInstancesForNextIteration(T sol, int iCurrentIteration){
        for( Parameter param : sol.getParameterMap().values()){
                List<Bin> lstBins = param.getListBins();
                int iBinIndex = opSelector.get(param.getName()).selectOperator(timeTotal);
                param.setCurrentBin(lstBins.get(iBinIndex));
            }
    }

}