
package archeopterix.algorithm.learning.forecast;

import archeopterix.algorithm.learning.ForecastBasedLearner;
import archeopterix.algorithm.learning.utils.Bin;
import archeopterix.algorithm.learning.utils.Parameter;
import archeopterix.util.CommonFunctions;
import archeopterix.util.statistics.timeseries.ARIMA.AutoARIMA;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;

public class ARIMAForecast<T> extends ForecastBasedLearner<T>{
    public static String AUTOREGRESSION_DEPTH = "AutoregressionDepth";
    public static String MOVING_AVERAGE_DEPTH = "MovingAverageDepth";
    public static String MUTIPLICITY = "Multiplicity";

    /**
     * Populate New parameters for Executor Instances.
     * Create ARIMA model for each parameter and use to forecast next parameter
     * usages
     * @param mapExInstance
     * @param iCompletedIteration LastCompletedIteration
     */
    @Override
    public void learn(int iCompletedIteration, List<T> pop){
        if  ( ( iCompletedIteration ) < 3){
            for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
                Parameter param = (Parameter)paramEntry.getValue();
                for( Bin bin : param.getListBins()){
                    bin.getBinPerformance().addPredictedPerformance(iCompletedIteration,bin.getBinPerformance().getAveragePerformance(iCompletedIteration));
                }
            }
        }
        else{
            for( Parameter param : lstParametersToControl.values()){
                List<Bin> lstBins = param.getListBins();
                Random rnd = new Random();
                int inBins = lstBins.size();
                double[] adProbValues = new double[inBins];
                String sRul = "";
                for( int i=0; i<inBins; i++){
                    //Obtain the history of the bin's performance over the iterations
                    List<Double> adHistory = lstBins.get(i).getBinPerformance().getSuccessRateHistory();
                    List<Double> diffSeries = new LinkedList<Double>();
                    diffSeries.add(0,0.0);
                    for(int j=1; j<adHistory.size(); j++){
                        diffSeries.add(j, adHistory.get(j)-adHistory.get(j-1));
                    }
                    int[] adIntIndexSet = CommonFunctions.getIntIndexSet(diffSeries);
                    double[] dHis=CommonFunctions.arrayFromList(diffSeries);
                    AutoARIMA autoArima = new AutoARIMA(adIntIndexSet,dHis);
                    try{
                        autoArima.compute(5);
                    }catch(Exception ex){
                        System.out.println(ex.toString());
                        return;
                    }
                    autoArima.forecast(1);
                    double[] outlierForecast = autoArima.getForecast();
                    double average = CommonFunctions.getAverage(adHistory);
                    adProbValues[i]=outlierForecast[0]+average;
                    lstBins.get(i).getBinPerformance().addPredictedPerformance(i, adProbValues[i]);
                }
            }        
        }
    }

}

