package archeopterix.algorithm.learning.forecast;

import archeopterix.algorithm.learning.ForecastBasedLearner;
import archeopterix.algorithm.learning.utils.Bin;
import archeopterix.algorithm.learning.utils.Parameter;
import archeopterix.util.CommonFunctions;
import archeopterix.util.statistics.EWMA;
import java.util.List;
import java.util.Map;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class EWMAForecast<T> extends ForecastBasedLearner<T>{

    private final static Log log = LogFactory.getLog(EWMAForecast.class);
    private double d_Lambda = 0.3; /** Degree of Weighting Decrease **/
    private double d_ExplorationRatio = 0.1; /** Exploration Ratio in Roulette Wheel selection **/

    @Override
    public void learn(int iCompletedIteration, List<T> pop){
        if  ( ( iCompletedIteration ) < 3){
            for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
                Parameter param = (Parameter)paramEntry.getValue();
                for( Bin bin : param.getListBins()){
                    bin.getBinPerformance().addPredictedPerformance(iCompletedIteration,bin.getBinPerformance().getAveragePerformance(iCompletedIteration));
                }
            }
        }
        else{
            for( Parameter param : lstParametersToControl.values()){
                List<Bin> lstBins = param.getListBins();
                int inBins = lstBins.size();
                double[] adProbValues = new double[inBins];
                String sRul = "";
                for( int i=0; i<inBins; i++){
                    //Obtain the history of the bin's performance over the iteration
                    List<Double> adHistory = lstBins.get(i).getBinPerformance().getSuccessRateHistory();
                    double[] history = CommonFunctions.arrayFromList(adHistory);
                    EWMA ewmaModel = new EWMA(history, d_Lambda);
                    adProbValues[i]= ewmaModel.getNextPrediction();
                    lstBins.get(i).getBinPerformance().addPredictedPerformance(iCompletedIteration,adProbValues[i]);            
                }
            }
        }
    }
}
