package archeopterix.algorithm.learning.forecast;
import archeopterix.algorithm.learning.ForecastBasedLearner;
import archeopterix.algorithm.learning.utils.Bin;
import archeopterix.algorithm.learning.utils.Parameter;
import archeopterix.util.CommonFunctions;
import java.util.List;
import java.util.Map;
import jsc.datastructures.PairedData;
import jsc.regression.PearsonCorrelation;

public class PearsonForecast<T> extends ForecastBasedLearner<T>{

    @Override
    public void learn(int iCompletedIteration, List<T> pop){
        if  ( ( iCompletedIteration ) < 3){
            for(Map.Entry paramEntry : lstParametersToControl.entrySet()){
                Parameter param = (Parameter)paramEntry.getValue();
                for( Bin bin : param.getListBins()){
                    bin.getBinPerformance().addPredictedPerformance(iCompletedIteration,bin.getBinPerformance().getAveragePerformance(iCompletedIteration));
                }
            }
        }
        else{
            for( Parameter param : lstParametersToControl.values()){
                List<Bin> lstBins = param.getListBins();
                for( Bin bin:lstBins){
                    List<Double> adHistory = bin.getBinPerformance().getAveragePerformance();
                    double[] history = CommonFunctions.arrayFromList(adHistory);
                    double[] adIndexSet = CommonFunctions.getIndexSet(history);
                    if(history.length>0){
                        PairedData o_PairedData = new PairedData(adIndexSet, history);
                        PearsonCorrelation o_PearsonCorrelation = null;
                        try {
                            o_PearsonCorrelation = new PearsonCorrelation(o_PairedData);
                        } catch (Exception ex) {
                            System.out.println(ex.toString());
                            return;
                        }
                        double A = o_PearsonCorrelation.getA();
                        double B = o_PearsonCorrelation.getB();
                        //double r = o_PearsonCorrelation.getR();
                        double next = A + B * (iCompletedIteration+1);
                        bin.getBinPerformance().addPredictedPerformance(iCompletedIteration, next);
                    }
                }
            }
        }
    }

}
