package archeopterix.algorithm.learning.utils;

import java.util.LinkedList;
import java.util.List;
/**
 *
 * @author Alda
 */
public class BinPerformance {
    private List<LinkedList<Double>> performanceHistory;
    private List<LinkedList<Object>> usageHistory;
    private List<Double> predictedPerformanceHistory;
    private List<Double> successRateHistory;
    private List<Double> predictedSuccessRateHistory;
    private List<LinkedList<Integer>> classifierHistory;
    private List<Double> averagePerformance;
    
    public BinPerformance(){     
        predictedPerformanceHistory= new LinkedList<Double>();
        performanceHistory = new LinkedList<LinkedList<Double>>();
        successRateHistory = new LinkedList<Double>();
        predictedSuccessRateHistory= new LinkedList<Double>();
        usageHistory = new LinkedList<LinkedList<Object>>();
        averagePerformance = new LinkedList<Double>();
        classifierHistory = new LinkedList<LinkedList<Integer>>();
    }
    
    @Override
    public BinPerformance clone() {
        BinPerformance newBinPerformace = new BinPerformance();
        newBinPerformace.addPredictedPerformanceHistory(predictedPerformanceHistory);
        newBinPerformace.addPredictedSuccessRateHistory(predictedSuccessRateHistory);
        newBinPerformace.addSuccessRateHistory(successRateHistory);
        newBinPerformace.addUsageHistory(usageHistory);
        newBinPerformace.addPerformanceHistory(performanceHistory);
        newBinPerformace.addClassifierInformationHistory(classifierHistory);
        return newBinPerformace;
    }
    
    
     public void addClassifierInformationHistory(List<LinkedList<Integer>> newHistory){
        if (newHistory != null){
            if(classifierHistory==null)
                classifierHistory = new LinkedList<LinkedList<Integer>>();
            for(int i=0; i<newHistory.size()&& newHistory.get(i)!=null;i++){
                if(classifierHistory.isEmpty()||classifierHistory.size()<=i)
                    classifierHistory.add(i,new LinkedList<Integer>());
                for(int j=0;j<newHistory.get(i).size();j++)
                    classifierHistory.get(i).add(newHistory.get(i).get(j));
            }
        }
    }
    public void addClassifierInformation(int iHistoryIndex, Integer value){
        if(classifierHistory==null)
            classifierHistory = new LinkedList<LinkedList<Integer>>();
        if(classifierHistory.isEmpty()||classifierHistory.size()<=iHistoryIndex)
            classifierHistory.add(iHistoryIndex, new LinkedList<Integer>());
        classifierHistory.get(iHistoryIndex).add(value);
    }

   public List<LinkedList<Integer>> getClassifierHistory(){
       return classifierHistory;
   }
   
   public void addAveragePerformance(int iHistoryIndex, Double value){
        if(averagePerformance==null)
            averagePerformance = new LinkedList<Double>();
        while(averagePerformance.isEmpty()||averagePerformance.size()<(iHistoryIndex+1))
            averagePerformance.add(value);
   }
   public void addAveragePerformance(List<Double> newPerformance){
        if(newPerformance!=null){
            averagePerformance = new LinkedList<Double>();
            for(int i=0; i<newPerformance.size();i++){
                for(Double ob : newPerformance)
                    averagePerformance.add(ob);
            }
        }
    }
   public List<Double> getAveragePerformance(){
       return averagePerformance;
   }
    public void addUsage(int iHistoryIndex, Object value){
        if(usageHistory==null)
            usageHistory = new LinkedList<LinkedList<Object>>();
        while(usageHistory.isEmpty()||usageHistory.size()<=(iHistoryIndex+1))
            usageHistory.add(new LinkedList<Object>());
        
        usageHistory.get(iHistoryIndex).add(value);
    }

    
    public List<LinkedList<Object>> getUsageHistory(){
        return usageHistory;
    }

    public void addUsageHistory(List<LinkedList<Object>> newUsage){
        if(newUsage!=null){
            usageHistory = new LinkedList<LinkedList<Object>>();
            for(int i=0; i<newUsage.size()&&newUsage.get(i)!=null;i++){
                if(usageHistory.isEmpty()||usageHistory.size()<=i)
                    usageHistory.add(i,new LinkedList<Object>());
                for(Object ob : newUsage.get(i))
                    usageHistory.get(i).add(ob);
            }
        }
    }
   
   
   public void addPerformanceHistory(List<LinkedList<Double>>newHistory){
        if(performanceHistory==null)
            performanceHistory = new LinkedList<LinkedList<Double>>();
        for(int i=0; i<newHistory.size();i++){
            if(performanceHistory.isEmpty()||performanceHistory.size()<=i)
                performanceHistory.add(i, new LinkedList<Double>());
            for(int j=0;j<newHistory.get(i).size();j++)
                performanceHistory.get(i).add(j,newHistory.get(i).get(j));
        }
    }


    public double getPredictedPerformance(int iter){
        if(predictedPerformanceHistory.size()>iter)
            return predictedPerformanceHistory.get(iter);
        return 0.01;
    }

    public void addPredictedPerformance(int iHistoryIndex, Double dValue){
        if(predictedPerformanceHistory==null){
            predictedPerformanceHistory=new LinkedList<Double>();
        }
        while(predictedPerformanceHistory.size()<=iHistoryIndex){
            predictedPerformanceHistory.add(0.01);
        }
        predictedPerformanceHistory.set(iHistoryIndex, dValue);
    }
    public void addPredictedPerformanceHistory(List<Double> predictedHistory){
        predictedPerformanceHistory=new LinkedList<Double>();
        for(Double val:predictedHistory)
            predictedPerformanceHistory.add(val);
    }
    public void clearHistory(){
        if(performanceHistory!=null) performanceHistory.clear();
    }

    public List<LinkedList<Double>> getPerformanceHistory(){
        if(performanceHistory==null)
            performanceHistory=new LinkedList<LinkedList<Double>>();
        return performanceHistory;
    }

  
    public void addSuccessRate(int iHistoryIndex, Double dValue){
        if(successRateHistory!=null)
            successRateHistory = new LinkedList<Double>();
        while(successRateHistory.size()<=iHistoryIndex)
            successRateHistory.add(dValue);
        successRateHistory.add(iHistoryIndex,dValue);
    }

    private void addSuccessRateHistory(List<Double> history){
        successRateHistory=new LinkedList<Double>();
        for(Double dVal: history)
            successRateHistory.add(dVal);
    }

    public List<Double> getSuccessRateHistory(){return successRateHistory;}
    
    public List<Double> getPredictedSuccessRateHistory(){
        return predictedSuccessRateHistory;
    }
    
    public void addPredictedSuccessRate(int iHistoryIndex, Double dValue){
        if(predictedSuccessRateHistory==null){
            predictedSuccessRateHistory=new LinkedList<Double>();
        }
        while(predictedSuccessRateHistory.size()<=iHistoryIndex){
            predictedSuccessRateHistory.add(0.01);
        }
        predictedSuccessRateHistory.set(iHistoryIndex, dValue);
    }
    private void addPredictedSuccessRateHistory(List<Double>predictedHistory){
        predictedSuccessRateHistory=new LinkedList<Double>();
        for(Double val:predictedHistory)
            predictedSuccessRateHistory.add(val);
    }


    public void addPerformance(int iHistoryIndex, Double dVal){
        if (performanceHistory == null){
            performanceHistory = new LinkedList<LinkedList<Double>>();
        }
        while(performanceHistory.isEmpty()||performanceHistory.size()<=(iHistoryIndex+1)){
            performanceHistory.add(new LinkedList<Double>());
        }
        performanceHistory.get(iHistoryIndex).add(dVal);
    }
    public double getAverageSuccessInWindow(int window, int iHistoryIndex){
        double avg = 0.0;
        for (int i=1;i<window;i++){
            avg=avg+(Double)successRateHistory.get(iHistoryIndex-i);
        }
        avg = avg/(double)window;
        return avg;
    }
    public double getExtremeSuccessInWindow(int window, int iHistoryIndex){
        double max = 0.0;
        for (int i=1;i<window;i++){
            if(max>(Double)successRateHistory.get(iHistoryIndex-i))
            max=(Double)successRateHistory.get(iHistoryIndex-i);
        }
        return max;
    }
    public double getAveragePerformance(int iteration){
        if(performanceHistory==null || performanceHistory.size()<1||performanceHistory.size()<(iteration+1)){
            return 0;
        }
        int iCnt = performanceHistory.get(iteration).size();
        if ( iCnt== 0 )
            return 0;
        double dSum = 0.0;
        for(Object d : performanceHistory.get(iteration))
            dSum += (Double)d;
        return dSum/ (double)iCnt;
    }
    public int getOutperformerCount(Object pMetric, double dTh, int itr, boolean maximise){
        int iCnt = 0;
        if(performanceHistory!=null&&performanceHistory.size()>0&&performanceHistory.size()>itr){
            LinkedList<Double> perf= performanceHistory.get(itr);
            if(maximise){
                for(Double d : perf){
                    if (d >= dTh)
                        iCnt++;
                }
            }
            else{
                for(Double d : perf){
                    if (d <= dTh)
                        iCnt++;
                }
            }
        }
        return iCnt;
    }
    public int getUsage(int iCompletedIteration){
        if(usageHistory.size()>iCompletedIteration)
            return usageHistory.get(iCompletedIteration).size();
        return 0;

    }
}
