package archeopterix.algorithm.learning.utils;
import java.util.Arrays;

/**
 *
 * @author Alda
 */
public class Vector {

    private double EPSILON = 0.000000001;
    private double[] values;
    private String name;

    public Vector(double... newValues) {
        values = newValues;
    }
    public String getName(){
        return name;
    }

    @Override
    public Vector clone(){
        Vector newVector = new Vector(Arrays.copyOf(values, values.length));
        newVector.name = name;
        newVector.EPSILON = EPSILON;
        return newVector;
    }
    public double[] getValues(){
        return values;
    }
    public Vector(String newName, double... newValues) {
        name = newName;
        values = newValues;
    }

    public double euclideanNorm(Vector otherVector) {
        if (values.length != otherVector.getValues().length)
            throw new IllegalArgumentException();
        double sumOfSquares = 0;
        for (int i = 0; i < values.length; i++) {
            sumOfSquares += (values[i] - otherVector.getValues()[i]) * (values[i] - otherVector.getValues()[i]);
        }
        return Math.sqrt(sumOfSquares);
    }

    @Override
    public boolean equals(Object obj) {
        if(!(obj instanceof Vector)) return false;
        Vector other = (Vector) obj;
        for (int i = 0; i < values.length; i++) {
            if (Math.abs((values[i] - other.values[i])) > EPSILON)
                return false;
        }
        return true;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(name + " = [");
        for (int i = 0; i < values.length; i++) {
            sb.append(values[i]);
            if (i < values.length - 1)
                sb.append(", ");
        }
        return sb.append("]").toString();
    }

    public int getNearestPointIndex(Vector[] points) {
        if (points == null || points.length<1)
            throw new IllegalArgumentException();
        int nearestVectorIndex = 0;
        double minDistance = euclideanNorm(points[0]);
        for (int i = 1; i < points.length; i++) {
            double distance = euclideanNorm(points[i]);
            if (distance < minDistance) {
                nearestVectorIndex = i;
                minDistance = distance;
            }
        }
        return nearestVectorIndex;
    }
}