package archeopterix.algorithm.localSearch;
import archeopterix.algorithm.PopulationGenerator;
import archeopterix.algorithm.learning.LearningAlgorithm;
import archeopterix.algorithm.learning.utils.Parameter;
import archeopterix.algorithm.learning.utils.ObjectParameter;
import archeopterix.problem.Component;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.deployment.solution.OriginalDeploymentSolution;
import archeopterix.solution.Architecture;
import archeopterix.util.PropertyMap;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 *
 * @author Alda
 */
public final class AdaptiveVariableNeighbourhood<T extends Architecture> extends LocalSearch  {   
    private LearningAlgorithm learning;
    private final static Log log = LogFactory.getLog(AdaptiveVariableNeighbourhood.class);
    protected String PARAM_SEARCH_STRATEGY = "searchStrategy";
    protected String PARAM_NEIGHBOURHOOD = "neighbourhood";
    protected String PARAM_FIRSTVSBEST = "firstVSbest";
    public AdaptiveVariableNeighbourhood() {
	super();
        System.out.println("=================AdaptiveMultiOperator====================");
        try{
            learning = (LearningAlgorithm) Class.forName("archeopterix.algorithm.learning.benchmarks.PMLearner").newInstance();
        }catch(Exception ex){
            System.out.println(ex.toString());
        }
        List<Parameter> parameters = new LinkedList<Parameter>();
        String[] sStrategyParams={"variableNeigbourhood","perturbate"};
        Parameter searchStrategy = new ObjectParameter(PARAM_SEARCH_STRATEGY,2, sStrategyParams);
        searchStrategy.initialiseBins();
        parameters.add(searchStrategy);  
        
        String[] sNeighParameters = {"kopt","oneFlip"};
        Parameter neighbourhoodParam = new ObjectParameter(PARAM_NEIGHBOURHOOD,2,sNeighParameters);
        neighbourhoodParam.initialiseBins();
        parameters.add(neighbourhoodParam);
        
        String[] sFirstvsBest = {"firstImprovement","bestImprovement"}; 
        Parameter firstVSbestParam = new ObjectParameter(PARAM_FIRSTVSBEST,2,sFirstvsBest);
        firstVSbestParam.initialiseBins();
        parameters.add(firstVSbestParam);
        
        learning.initialise(comp,parameters);
    }
    
    @Override
    public void initialise(EmbeddedSystemProblem problem, PropertyMap externalParams, Collection<Architecture> partialInitPop, PopulationGenerator popGenerator){
        super.initialise(problem, externalParams, partialInitPop, popGenerator);
        solution = new OriginalDeploymentSolution(problem);     
        problem.evaluateAttributes(solution);
        learning.assignRandomParametersToSolutions(solution, currIteration);
        neighbourhoodSearch(0,(OriginalDeploymentSolution)solution);
        problem.evaluateAttributes(solution);
        
    }
    
    @Override
    public void oneIteration(){ 
        String searchStrategy = (String)solution.getParameterMap().get(PARAM_SEARCH_STRATEGY).getCurrentBin().getValue();
        OriginalDeploymentSolution tempSol = (OriginalDeploymentSolution)solution.clone();
        problem.evaluateAttributes(tempSol);
        if(searchStrategy.equalsIgnoreCase("variableNeigbourhood")){
            neighbourhoodSearch(0,(OriginalDeploymentSolution)solution);
        }
        else{
            perturbate(tempSol);
            oneFlip(0,tempSol);
        }
        if(comp.compare(tempSol, solution)==1){
                solution=tempSol.clone();
                problem.evaluateAttributes(solution);
        }
        try{
            learning.onIterationComplete(solution, currIteration);
        }
        catch(Exception ex){
            System.out.println(ex.toString());
        }
        //log.debug("Evals:"+getNumberEvaluations()+":Fitness:"+solution.getAttributeValue(0));
        incrementIteration();
    }
    
    public void perturbate(OriginalDeploymentSolution tempSol) {
            boolean coLocViolated;
            boolean swapImpossible;

            // if we don't find a possible swap after twice the number of components (list length)
            // we assume something is wrong and moves are impossible
            Random rnd = new Random();
            List<Component> comps = problem.getComponents();
            for (int i = 0; i < i_Comps * 2; i++) {
                swapImpossible = false;
                coLocViolated = false;
                int iMutationID1 = rnd.nextInt(i_Comps);
                int iMutationID2;
                // if (comps.get(iMutationID1).mustColocate())
                //     continue;
                int ecuIndex1 = tempSol.getHostIndex(iMutationID1);
                int ecuIndex2 = 0;
                int timeout = 0;
                do {
                    iMutationID2 = rnd.nextInt(i_Comps);
                    ecuIndex2 = tempSol.getHostIndex(iMutationID2);
                    if (timeout > comps.get(iMutationID1).getCanUseHostListLength()) {
                        swapImpossible = true;
                        break;
                    }
                    timeout++;
                } while (ecuIndex1 == ecuIndex2);

                if (swapImpossible) {
                    continue;
                }
                if (!comps.get(iMutationID1).isCompatibleHost(problem.getECUs().get(ecuIndex2))) {
                    continue;
                }

                if (!comps.get(iMutationID2).isCompatibleHost(problem.getECUs().get(ecuIndex1))) {
                    continue;
                }

                //check all other allocations to same Host - are there incompatibilities?

                for (int j = 0; j < i_Comps; j++) {
                    //assumption is that components are not prohibited from colocating with themselves
                    if (j != iMutationID1 && tempSol.getHostIndex(j) == ecuIndex1) {
                        if (!comps.get(iMutationID2).canColocateWith(comps.get(j))) {
                            coLocViolated = true;
                            break;
                        }
                    } else if (j != iMutationID2 && tempSol.getHostIndex(j) == ecuIndex2) {
                        if (!comps.get(iMutationID1).canColocateWith(comps.get(j))) {
                            coLocViolated = true;
                            break;
                        }
                    }
                }
                if (coLocViolated) {
                        continue;
                }
                //assuming here that colocation restrictions and poss Host assignments are common to the group members.
                if (comps.get(iMutationID1).mustColocate()) {
                    List<Component> sourceGroupMembers = comps.get(iMutationID1).getMustColocateWith();
                    for (Component comp : sourceGroupMembers) {
                        tempSol.set(comp.index, ecuIndex2);
                    }
                }

                if (comps.get(iMutationID2).mustColocate()) {
                    List<Component> targetGroupMembers = comps.get(iMutationID2).getMustColocateWith();
                    for (Component comp : targetGroupMembers) {
                        tempSol.set(comp.index, ecuIndex1);
                    }

                }
                tempSol.set(iMutationID1, ecuIndex2);
                tempSol.set(iMutationID2, ecuIndex1);
            }
    }
    
    @Override
    public boolean neighbourhoodSearch(int iComp, Architecture sol){
        String neighbourhood = (String)solution.getParameterMap().get(PARAM_NEIGHBOURHOOD).getCurrentBin().getValue();
       // System.out.println(neighbourhood);
        if(neighbourhood.equalsIgnoreCase("kopt"))
            return kOpt(0,2, sol);
        else
            return oneFlip(iComp, sol);
    }
    
    @Override
    public void finalise(Log log) {
        m_Watch.stop();
        log.debug("BestFitness:"+solution.getAttributeValue(0));
    }
}
