
package archeopterix.algorithm.localSearch;

import archeopterix.algorithm.Algorithm;
import archeopterix.algorithm.PopulationGenerator;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.deployment.solution.OriginalDeploymentSolution;
import archeopterix.solution.Architecture;
import archeopterix.solution.comparator.SolutionComparator;
import archeopterix.util.PropertyMap;
import java.util.Collection;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 *
 * @author Alda
 */
public class LocalSearch extends Algorithm{

    protected Architecture solution;
    private final static Log log = LogFactory.getLog(AdaptiveVariableNeighbourhood.class);
    
    public LocalSearch(){
        super();
        comp.setObjectiveForComparison(0);
    }
        @Override
    public double getQuality(){
        return solution.getAttributeValue(0);
        
    }
    @Override
    public void initialise(EmbeddedSystemProblem problem, PropertyMap externalParams, Collection<Architecture> partialInitPop, PopulationGenerator popGenerator){
        
        super.initialise(problem, externalParams, partialInitPop, popGenerator);
        try{
            comp = (SolutionComparator) Class.forName("archeopterix.solution.comparator.SolutionComparator").newInstance();
            comp.setObjectiveForComparison(0);
        }catch(Exception ex){
            System.out.println(ex.toString());
        }
        //log.info(problem);
        solution = new OriginalDeploymentSolution(problem);

        problem.evaluateAttributes(solution); 
        
    }
    @Override
    public void oneIteration() {
        neighbourhoodSearch(0, solution);
        incrementIteration();
    }
    @Override
    public void finalise(Log log) {
        m_Watch.stop();
        log.debug("Best Fitness: "+solution.getAttributeValue(0));
    }
    
    public boolean oneFlip(int iComp, Architecture sol){        
        Architecture bestSol = ((OriginalDeploymentSolution)sol).clone();
        problem.evaluateAttributes(bestSol);
        int bestHostId = ((OriginalDeploymentSolution)solution).getHostIndex(iComp);
        boolean changed =false;
        for(int iHost=0; iHost<i_ECUs; iHost++){
            ((OriginalDeploymentSolution)sol).set(iComp, iHost);
            problem.evaluateAttributes(sol);
            if(comp.compare(sol, bestSol)==1){
                bestSol = ((OriginalDeploymentSolution)sol).clone();
                problem.evaluateAttributes(bestSol);
                bestHostId = iHost;
                changed=true;
            }
        }
        ((OriginalDeploymentSolution)sol).set(iComp, bestHostId);
        problem.evaluateAttributes(sol);
        return changed;       
    }
    
    public boolean neighbourhoodSearch(int start, Architecture sol){
            //return kOpt(sol, iComp,1);
             return oneFlip(start, sol);
    }
    
    public boolean kOpt(int start, int k, Architecture sol){
        OriginalDeploymentSolution tempSol = (OriginalDeploymentSolution)solution.clone();
        problem.evaluateAttributes(tempSol);
        boolean changed=false;
        double bestFitness = sol.getAttributeValue(start);
        int bestMove = start;
        while(start<i_Comps-k){
            int host1= tempSol.getHostIndex(start);
            int host2=tempSol.getHostIndex(start+k);
            tempSol.set(start, host2);
            tempSol.set(start+k, host1);        
            problem.evaluateAttributes(tempSol);
            double newFitness = tempSol.getAttributeValue(0);
            if(newFitness>bestFitness){
                changed =true;
                bestMove = start;
            }
            start++;
        }
        if(changed){
            int host1= ((OriginalDeploymentSolution)sol).getHostIndex(bestMove);
            int host2= ((OriginalDeploymentSolution)sol).getHostIndex(bestMove+k);
            ((OriginalDeploymentSolution)sol).set(bestMove, host2);
            ((OriginalDeploymentSolution)sol).set(bestMove+k, host1);
            problem.evaluateAttributes(sol);
        }         
        return changed;
    }
    
}
