package archeopterix.algorithm.paco;

import java.util.List;
import java.util.LinkedList;
import java.util.Random;

import archeopterix.conf.Conf;
import archeopterix.problem.Component;
import archeopterix.problem.Host;
import archeopterix.problem.deployment.solution.ECUBasedDeployment;
import archeopterix.solution.Architecture;
import archeopterix.problem.deployment.solution.OriginalDeploymentSolution;


public class PheromoneMatrix {
	
	private double[][] pheroMatrix;
	private int compCount, ecuCount;
	private double pheroAddition, initPheromone;
	private Random rnd;

	
	public PheromoneMatrix(int compCount, int ecuCount){
		this.compCount = compCount;
		this.ecuCount = ecuCount;
		pheroMatrix = new double[compCount][ecuCount];
		
		initPheromone = 0.01;
		pheroAddition = 0.9;

		rnd = Conf.instance().random;
	}
	
	
	public PheromoneMatrix(int compCount, int ecuCount,	double pheroAddition)
	{
		this(compCount, ecuCount);
		this.pheroAddition = pheroAddition;	
	}

	
	public void init()
	{
		for(int i = 0; i < compCount; i++)
                    for(int j = 0; j < ecuCount; j++)
			pheroMatrix[i][j]=initPheromone;					
	}
	
	
	public double getPheromone(int iComp, int iEcu)
	{
		return pheroMatrix[iComp][iEcu];
	}
	
	
	public void setPheromone(int iComp, int iEcu, double pheromone)
	{
		pheroMatrix[iComp][iEcu] = pheromone;
	}
	

	/* 
	 * For each of the k chosen solutions add 0.9/k of pheromone
	 * to each matrix where comp -> ecu assignment exists
	 */
	protected void updatePheromones(List<Architecture> solutions)
	{
            ECUBasedDeployment sol;
            int solCount = solutions.size();
            List<Component> ecuSpecificCompList;
            for(int j = 0; j < solCount; j++)
            {
                sol = (ECUBasedDeployment) solutions.get(j);
                for(int i = 0; i < ecuCount; i++)
                {
                    ecuSpecificCompList = sol.getComponentListForHost(i, null);
                    for (Component component : ecuSpecificCompList)
                    {
                         pheroMatrix[component.index][i] += pheroAddition/solCount;
                    }
                }
            }
	}
	

	public int maxSelect(int iComp, List<Host> ecus)
	{
		double value = 0;
		List<Integer> indexes = new LinkedList<Integer>();
				
		for(Host e: ecus)
			if(pheroMatrix[iComp][e.index] > value) value = pheroMatrix[iComp][e.index];
		
		for(Host e:ecus)
			if(pheroMatrix[iComp][e.index] == value) indexes.add(e.index);
		
		return indexes.get(rnd.nextInt(indexes.size())); 
	}
	
	
	public int rouletteSelect(int iComp, List<Host> ecus)
	{
		return rouletteSelect(iComp, ecus, rnd.nextDouble());
	}
	
	
	public int rouletteSelect(int iComp, List<Host> ecus, double rnd)
	{	
		double sum = 0;
		double[] tempArray = new double[ecus.size()];
		int ecuIndex = 0;

		for(int i=0; i< tempArray.length;i++)
		{
			sum+=pheroMatrix[iComp][ecus.get(i).index];
			tempArray[i]=sum;
		}
		
		while (rnd > tempArray[ecuIndex]/sum) ecuIndex++;
		return ecus.get(ecuIndex).index;
	}
}
