/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package archeopterix.algorithm.sa;

import archeopterix.problem.deployment.algorithm.sa.ECUBasedDepSAOperators;
import java.util.HashMap;
import java.util.Map;
import java.util.NoSuchElementException;

/** OperatorFactory for Simulated Annealing. The concrete clases that implements
 * the abstract SAOperators should be added to the SAOperators array.
 *
 * @author imeedeniya
 */
public class SAOperatorsFactory {
	
	private Map<Class<?>, SAOperators> repToOperatorsMap = new HashMap<Class<?>, SAOperators>();

	/**
	 * Please add new OperatorClass object from each concrete implementatin of the
	 * the Simulated Annealing Operators.
	 */
	private static SAOperators[] availableOps = {
		new ECUBasedDepSAOperators()
	};
	
	private static SAOperatorsFactory instance;
	
	private SAOperatorsFactory() { }
	
	public synchronized static SAOperatorsFactory instance() {

		if (instance == null)
			instance = new SAOperatorsFactory();
		return instance;
	}

	/**
	 *
	 * @param solRepClass
	 * @return the aproporiate Simulated annealing operators object corresponds
	 * to the given solReprentation.
	 */
	public SAOperators getSuitableOperators(Class<?> solRepClass) {
		SAOperators ops = repToOperatorsMap.get(solRepClass);
		if (ops == null) {
			for (int i = 0; i < availableOps.length && ops == null; i++) {
				if (availableOps[i].canOperateOn(solRepClass))
					ops = availableOps[i];
			}
			if (ops == null)
				throw new NoSuchElementException("No appropriate SA operators found for " + solRepClass.getCanonicalName());
			repToOperatorsMap.put(solRepClass, ops);
		}
		return ops;
	}

}
