package archeopterix.conf;


import java.io.File;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import org.apache.log4j.FileAppender;
import org.apache.log4j.Logger;

import archeopterix.algorithm.PopulationGenerator;
import archeopterix.algorithm.learning.utils.Parameter;
import archeopterix.evaluation.ArchitectureEvaluator;
import archeopterix.problem.SchedulingStratergy;
import archeopterix.util.PropertyMap;
import archeopterix.util.io.ConfXMLParser;
import java.io.Serializable;
import java.util.Vector;


/**
 * TODO Describe the purpose of this class.
 */
public class Conf implements Serializable{
	
    public static final String DEFAULT_LOG_FILE_NAME_PREFIX = "Log_";
    public static final String DEFAULT_RESULTS_FILE_NAME_PREFIX = "results_";

    /**
     * Log4J name for main log.
     */
    public static final String MAIN_LOG = "logfile";

    /**
     * Log4J name for results log (which applies no formatting to logged
     * messages, leaving this up to application).
     */
    public static final String RESULTS_LOG = "Results";

    private int i_MaxRedundancy = 3;
	private int nObj=3;
    private SchedulingStratergy schedulingStratergy = SchedulingStratergy.FIXED;
        
	//FIXME Add PopulationGenerator class to configuration
	//FIXME Add constraint handling approach(es) to configuration
	
	/**
	 * Parameter settings for each class of algorithm (whether or not that
	 * algorithm is currently being used).
	 */
	private HashMap<Class<?>, PropertyMap> parametersByClass;
	
	/**
	 * The type of algorithm that is to be used.
	 */
	private Class<?> algClass;

	/**
	 * The type of problem that is to be solved.
	 */
	private Class<?> problemClass;
	
	/**
	 * Source of the problem; possibly not needed here since Problem instance
	 * could come from another source.
	 */
	private File problemFile;
        private boolean problemVaries;
	
	/**
	 * Active attribute/objective evaluators.
	 */
	private List<ArchitectureEvaluator> evaluators;
	
	/**
	 * An unmodifiable view of {@code evaluators} suitable for returning to
	 * those that require access to them.
	 */
	private List<ArchitectureEvaluator> unmodifiableEvaluators;
        
        private List<Parameter> lstParametersToControl;
	
	private Class<?> solRepClass = PopulationGenerator.DEFAULT_SOLUTION_REPRESENTATION;
	
	/**
	 * Single, global Random instance; use {@link #setRandomSeed(int)} to
	 * reset the seed.
	 */
	public final Random random = new Random();
	
	/**
	 * Number of times to run the main algorithm.
	 */
	private int nRepeats = 1;
	
	/**
	 * Indicates if a separate results file should be generated for each of
	 * the {@linkplain #getRunRepeats() n} algorithm runs.
	 */
	private boolean runResultsSeparate;

	/**
	 * The source of random seeds for successive runs of the main algorithm.
	 */
	private RandomSeedSource randomSeedSource = RandomSeedSource.CLOCK;

	/**
	 * The first random seed to use when {@link #resetRandomSeed()} is called
	 * if {@code randomSeedSource == RandomSeedSource.SEQUENTIAL}.
	 */
	private long firstRandomSeed = 1;
	
	/**
	 * Stored copy of current random seed, since {@code Random} doesn't report
	 * this.
	 */
	private long currRandomSeed = 1;
	
	private String logFileNamePrefix = DEFAULT_LOG_FILE_NAME_PREFIX;
	
	private String resultsFileNamePrefix = DEFAULT_RESULTS_FILE_NAME_PREFIX;
	private String populationFileName = null;
	private static Conf instance;
	
	
	private Conf() {
		parametersByClass = new HashMap<Class<?>, PropertyMap>();
		evaluators = new Vector<ArchitectureEvaluator>();
		unmodifiableEvaluators = Collections.unmodifiableList(evaluators);
		setRandomSeed(firstRandomSeed); //as a default starting point
		runResultsSeparate = true; //default behaviour is to generate one file per run
	}
	
	public synchronized static Conf instance() {
            if (instance == null)
                    instance = new Conf();
            return instance;
	}
	
	public synchronized void loadFromXML(String confXMLFile) {
            clearAll();
            ConfXMLParser.loadConfig(confXMLFile, this);
	}
	
	public void initLogging() {
            //Problem problem = ProblemXMLParser.loadProblem( problemFile , getProblemClass());
            FileAppender fileAppender = (FileAppender) Logger.getRootLogger().getAppender(MAIN_LOG);
            String fileName = getProblemFileRootName() + ".log";
            System.out.println("attempt to set file : " + logFileNamePrefix + fileName + " FA =" + fileAppender);
            fileAppender.setFile(logFileNamePrefix + fileName);
            fileAppender.activateOptions();

            if (nRepeats == 1 || !runResultsSeparate)
                initResultsLog(0);
	}
	
	public void initResultsLog(int n) {
            FileAppender resultsAppender = (FileAppender) Logger.getLogger(RESULTS_LOG).getAppender(RESULTS_LOG); //log category and appender have same name
            resultsAppender.setFile(resultsFileNamePrefix + getProblemFileRootName() + (n > 0 ? "_" + n : "") + ".log");
            resultsAppender.activateOptions();
            //Note: Have removed writing to the results file this.toString() as the results file is for algorithms' use;
            //it is not for this class to dictate its contents. Add an appropriate logging message to each algorithm
            //that needs to write the configuration to the results file.
	}
	
	public void clearAll() {
            clearParameters();
            clearEvaluators();
            algClass = null;
            problemClass = null;
            problemFile = null;
            resetRandomSeed();
	}
	
	public void clearParameters() { parametersByClass.clear(); }
	
	public void addParameters(Class<?> algClass, PropertyMap params) {
            parametersByClass.put(algClass, params);
	}
	
	public PropertyMap getApplicableParameters(Class<?> algClass){
            //FIXME Need to make this set values in class hierarchy order, so that the same parameters defined for more specific classes are set last, thereby overriding the superclass's settings
            //Tedious approach is required if we allow parameters to be specified at the superclass level as well as the concrete implementation level
            PropertyMap paramsUnion = new PropertyMap();
            for (Class<?> clazz : parametersByClass.keySet()){
                if (clazz.isAssignableFrom(algClass)){
                    paramsUnion.addAll(parametersByClass.get(clazz));
                }
            }
            return paramsUnion;
	}

	public int getEvaluatorsCount(){ return evaluators.size(); }
	
	public Class<?> getAlgorithmClass() { return algClass; }
	
	public void setAlgorithmClass(Class<?> algClass) { this.algClass = algClass; }
	
	public Class<?> getProblemClass() { return problemClass; }
	
	public void setProblemClass(Class<?> problemClass) { this.problemClass = problemClass; }
	
	public File getProblemFile() { return problemFile; }
	public int getNObj(){
		return nObj;
	}
	public void setProblemFile(File problemFile) { this.problemFile = problemFile; }

        public String getProblemFileRootName()
        {
            return problemFile.getName().replaceAll("\\.xml$", "");
        }
	
	public void clearEvaluators() {
		evaluators.clear();
	}
	
    public ArchitectureEvaluator addEvaluator(Class<?> evaluatorClass, Integer id) {
        ArchitectureEvaluator evaluator = null;
        try {
            if (id == null) {
                evaluator = (ArchitectureEvaluator) evaluatorClass.newInstance();
            } else {
                evaluator = (ArchitectureEvaluator) evaluatorClass.getConstructor(Integer.class).newInstance(id);
            }
            evaluators.add(evaluator);
        } catch (Exception e) {
            throw new RuntimeException("Unabled to instantiate active attribute evaluator "
                    + evaluatorClass.getCanonicalName(), e);
        }
        return evaluator;
    }

	/**
	 * Returns an unmodifiable list of the active evaluators.
	 */
	public List<ArchitectureEvaluator> getEvaluators() { return unmodifiableEvaluators; }
	
	public int getNEvaluators() { return evaluators.size(); }
	
	public Class<?> getSolutionRepresentation() { return solRepClass; }
	
	public void setSolutionRepresentation(Class<?> solRepClass) { this.solRepClass = solRepClass; }
	
	public void setRandomSeedUsingCurrentTime() { setRandomSeed( System.currentTimeMillis() ); }
	
	public void setRandomSeed(long seed) { random.setSeed(currRandomSeed = seed); }
	
	public int getRunRepeats() { return nRepeats; }
	
	public void setRunRepeats(int nRepeats) { this.nRepeats = nRepeats;	}
	
	public boolean isRunResultsSeparate() { return runResultsSeparate; }

	public boolean setRunResultsSeparate(boolean separate) { return this.runResultsSeparate = separate; }

	public RandomSeedSource getRandomSeedSource() { return randomSeedSource; }

	public void setRandomSeedSource(RandomSeedSource randomSeedSource) { this.randomSeedSource = randomSeedSource; }

	public long getInitialRandomSeed() { return firstRandomSeed; }
	
	public void setInitialRandomSeed(long firstRandomSeed) { this.firstRandomSeed = firstRandomSeed; }

        public void setParametersToControl(List<Parameter> params){
            lstParametersToControl = params;
        }
        public List<Parameter> getParametersToControl(){
            
            return lstParametersToControl;
        }
    public void setSchedulingStratergy(SchedulingStratergy stratergy){ schedulingStratergy = stratergy;}

    public int getMaxRedundancyLevel(){return i_MaxRedundancy;}
    public SchedulingStratergy getSchedulingStratergy(){return schedulingStratergy;}

	/**
	 * Resets the random seed using the source and initial value stored in this
	 * configuration.
	 */
	public void resetRandomSeed() {
		if (randomSeedSource == RandomSeedSource.SEQUENTIAL)
			setRandomSeed(firstRandomSeed);
		else //randomSeedSource == RandomSeedSource.CLOCK
			setRandomSeedUsingCurrentTime();
	}

	/**
	 * If the source of random seeds is a sequential series of numbers, resets
	 * the random seed to the next one in the sequence; otherwise uses the
	 * current system time. 
	 */
	public void useNextRandomSeed() {
		if (randomSeedSource == RandomSeedSource.SEQUENTIAL)
			setRandomSeed(currRandomSeed + 1);
		else //randomSeedSource == RandomSeedSource.CLOCK
			setRandomSeedUsingCurrentTime();
	}

	public String getLogFileNamePrefix() { return logFileNamePrefix; }
	
	public void setLogFileNamePrefix(String logFileNamePrefix) { this.logFileNamePrefix = logFileNamePrefix; }
	
	public String getResultsFileNamePrefix() { return resultsFileNamePrefix; }
	
	public void setResultsFileNamePrefix(String resultsFileNamePrefix) { this.resultsFileNamePrefix = resultsFileNamePrefix; }
	public void setPopulationFileName(String popFileName) { this.populationFileName = popFileName; }
	public String getPopulationFileName() { return this.populationFileName; }

    @Override
	public String toString() {
		StringWriter result = new StringWriter();
		PrintWriter out = new PrintWriter( result );
 
		out.println("----------- Experiment Configurations ---------");
		out.println("Run details:");
		out.println("Algorithm: " + algClass.getCanonicalName());
		out.print("Repeats: " + nRepeats + " with random seeds taken ");
		if (randomSeedSource == RandomSeedSource.SEQUENTIAL)
			out.println("sequentially from " + firstRandomSeed + " through " + (firstRandomSeed + nRepeats - 1) );
		else //randomSeedSource == RandomSeedSource.CLOCK)
			out.println("from the system time" );
		
		out.println("Various algorithm parameters:");
		for (Map.Entry<Class<?>,PropertyMap> algParams : parametersByClass.entrySet()) {
			out.println("For " + algParams.getKey().getCanonicalName());
			if (algParams.getKey().isAssignableFrom(algClass))
				out.println(" (used by main algorithm)");
			out.println( algParams.getValue().toFormattedString("  ") );
		}

		out.println("Problem: " + problemClass.getCanonicalName() + " defined in " + problemFile);
		out.println("Objectives:");
		for (ArchitectureEvaluator evaluator : evaluators)
			out.println(evaluator);
		out.println("----------- END Configurations --------- ");
		return result.toString();
	}

    public void setProblemVaries(boolean varies)
    {
        problemVaries = varies;
    }

    public boolean getProblemVaries()
    {
        return problemVaries;
    }
}
