package archeopterix.evaluation;

import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.solution.Architecture;
import archeopterix.util.PropertyMap;
import archeopterix.util.probability.MyStatistics;
import archeopterix.util.probability.UncertaintyHandler;
import java.io.Serializable;


public abstract class ArchitectureEvaluator implements Serializable{
    private String name;
    private double maxValue;
    private double minValue;
    private boolean shouldMinimise;
    protected UncertaintyHandler uncertaintyHandler;
    protected PropertyMap mapParameters;
    protected boolean subjectUncertainty = false;

    protected ArchitectureEvaluator(String name, boolean shouldMinimise) {
        this.name = name;
        this.shouldMinimise = shouldMinimise;
    }

    public void setParameters(PropertyMap params){
        mapParameters = params;
        if (mapParameters.containsKey("maxvalue")){
            maxValue = mapParameters.getDouble("maxvalue");
        }
        if (mapParameters.containsKey("minvalue")){
            minValue = mapParameters.getDouble("minvalue");
        }
        if (mapParameters.containsKey("subjectuncertainty")){
            subjectUncertainty = mapParameters.getBoolean("subjectuncertainty");
            if (subjectUncertainty){
                uncertaintyHandler = new UncertaintyHandler();
            }
        }

    }
    public UncertaintyHandler getUncertaintyHandler(){
        return uncertaintyHandler;
    }
    public String getName() {
        return name;
    }

    public boolean isMinimisationObjective() {
        return shouldMinimise;
    }

    public void setMaxValue(double dValue) {
        maxValue = dValue;
    }

	public void setMinValue(double value) {
        minValue = value;
    }

    public double getMaxValue() {
        return maxValue;
    }

    @Override
    public String toString() {
        return "Evaluator: " + name;
    }

    public abstract double evaluate(EmbeddedSystemProblem oContext, Architecture oDep);

    public double evaluateNormalised(EmbeddedSystemProblem oContext, Architecture oDep) {
        return evaluate(oContext, oDep) / maxValue;
    }

	public double getMinValue()
	{
		return minValue;
	}

      protected MyStatistics getStats(Architecture oArch) {
        if (! subjectUncertainty ){
            System.err.println("getStat Rejected. Evaluator :" + this.getName() + " is not under Uncertainty!");
            return null;
        }
        Object oVal = oArch.getProperty(this.getName());
        if (oVal == null) {
            MyStatistics oStat = new MyStatistics();
            oArch.setProperty(this.getName(), oStat);
            return oStat;
        }
        return (MyStatistics) oVal;
    }
}
