package archeopterix.evaluation;

import java.util.List;

import archeopterix.problem.Host;
import archeopterix.problem.NetworkProperties;
import archeopterix.problem.InteractionProperties;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.deployment.solution.DeploymentSolution;
import archeopterix.problem.deployment.solution.ECUBasedDeployment;
import archeopterix.solution.Architecture;
import archeopterix.util.Pair;
import java.io.Serializable;

public class ComOHEvaluator extends ArchitectureEvaluator implements Serializable{

    //TODO: find a reasonable way of actually calculating this
    public ComOHEvaluator() {
        super("ComOH", true);
    }

    /*	public double evaluate(Context oContext, Architecture oArch){
    Deployment oDep = (Deployment)oArch;
    float fSumFND = 0;
    float fSumFE = 0;
    float fSumBR = 0;

    for ( int iCompi =0; iCompi < oContext.i_Comps; iCompi++)
    {
    //Comp oCompi = (Comp)lst_Comp.get(iCompi);
    for ( int iCompj = 0; iCompj < oContext.i_Comps; iCompj++)
    {
    //Comp oCompj = (Comp)lst_Comp.get(iCompj);
    //Host oHosti = oDep.GetHost(oCompi);
    //Host oHostj = oDep.GetHost(oCompj);
    int iHosti	= oDep.getHostIndex(iCompi);
    int iHostj	= oDep.getHostIndex(iCompj);
    if ( oContext.IsInteracting (iCompi, iCompj) )
    {
    fSumFND += oContext.GetInteractionParameter(iCompi, iCompj, Defs.FREQUENCY) * oContext.GetNetworkParameter(iHosti, iHostj, Defs.NW_DELAY); //TODO
    fSumFE += oContext.GetInteractionParameter(iCompi, iCompj, Defs.FREQUENCY) *  oContext.GetInteractionParameter(iCompi, iCompj, Defs.EVENT_SIZE);
    fSumBR += oContext.GetNetworkParameter(iHosti, iHostj, Defs.RELIABILITY) * oContext.GetNetworkParameter(iHosti, iHostj, Defs.BANDWIDTH);
    }
    }
    }
    float fRet = (fSumFND + (fSumFE/fSumBR));

    this.Evaluate2(oContext, oArch);
    return fRet; // TODO:: return fRet;

    }*/
    public double evaluate(EmbeddedSystemProblem oContext, Architecture oArch) {
        if (oArch instanceof ECUBasedDeployment) {
            ECUBasedDeployment oDep = (ECUBasedDeployment) oArch;
            float fSumFND = 0;
            float fSumFE = 0;
            float fSumBR = 0;
            //James M: Have changed int[] to Pair<Integer> to match new result of getNextAssignedPair();
            //  if desired the components of these objects could be taken out and assigned to primitives to
            //  speed up their use below.
            //  FIXME However, it is really unclear what the result of getNextAssignedPair() represents; it also appears inefficient to calculate it for each component since the result is always the next component in the lists of component assigned to ECUs and it must walk each list in turn until that point is reached
            Pair<Integer> pair1, pair2;

            List<Host> hosts = oContext.getECUs();

            //FIXME Should this loop at least skip i = j and perhaps only examine j > i?
            for (int iCompi = 0, c = oContext.getComponentCount(); iCompi < c; iCompi++) {
                //pairs are structured components first (pair[0] is component)
                pair1 = oDep.getNextAssignedPair(iCompi);
                //  System.out.print("Compare comp "+c1);
                //Comp oCompi = (Comp)lst_Comp.get(iCompi);
                int c1 = pair1.getA();
                int host1 = pair1.getB();
                for (int iCompj = 0; iCompj < c; iCompj++) {
                    pair2 = oDep.getNextAssignedPair(iCompj);
                    int c2 = pair2.getA();
                    int host2 = pair2.getB();
                    //  System.out.println(" with comp "+c2);

                    if (oContext.areInteracting(c1, c2)) {
                        double dFreq, dNWdelay, dEvtSize, dNWReliability, dNWBW;
                        dFreq = oContext.getInteractionProperties(c1, c2).getDouble(InteractionProperties.FREQUENCY);
                        dEvtSize = oContext.getInteractionProperties(c1, c2).getDouble(InteractionProperties.EVENT_SIZE);
                        fSumFE += dFreq * dEvtSize;
                        if (host1 == host2) { // In the Same Host
                            fSumBR += (1 - oContext.getECU(pair1.getB()).getDoubleProperty(Host.FAILURE_RATE));
                        } else {
                            dNWdelay = 0.0; //Double.MAX_VALUE
                            dNWReliability = 0.0;
                            dNWBW = 0.0;
                            if (oContext.areConnected(pair1.getB(), pair2.getB())) {
                                dNWdelay = oContext.getNetworkProperties(pair1.getB(), pair2.getB()).getDouble(NetworkProperties.NW_DELAY);
                                dNWReliability = oContext.getNetworkProperties(pair1.getB(), pair2.getB()).getDouble(NetworkProperties.RELIABILITY);
                                dNWBW = oContext.getNetworkProperties(pair1.getB(), pair2.getB()).getDouble(NetworkProperties.BANDWIDTH);
                            }
                            fSumFND += dFreq * dNWdelay;
                            fSumBR += dNWReliability * dNWBW;

                        }
                    }
                }
            }
            float fRet = (fSumFND + (fSumFE / fSumBR));
            System.out.flush();
            return fRet; // TODO:: return fRet;


            } else if (oArch instanceof DeploymentSolution) {
            DeploymentSolution oDep = (DeploymentSolution) oArch;
            float fSumFND = 0;
            float fSumFE = 0;
            float fSumBR = 0;

            for (int iCompi = 0; iCompi < oContext.getComponentCount(); iCompi++) {
                //Comp oCompi = (Comp)lst_Comp.get(iCompi);
                for (int iCompj = 0; iCompj < oContext.getComponentCount(); iCompj++) {
                    int iHosti = oDep.getHostIndex(iCompi);
                    int iHostj = oDep.getHostIndex(iCompj);
                    if (oContext.areInteracting(iCompi, iCompj)) {
                        double dFreq, dNWdelay, dEvtSize, dNWReliability, dNWBW;
                        dFreq = oContext.getInteractionProperties(iCompi, iCompj).getDouble(InteractionProperties.FREQUENCY);
                        dEvtSize = oContext.getInteractionProperties(iCompi, iCompj).getDouble(InteractionProperties.EVENT_SIZE);
                        fSumFE += dFreq * dEvtSize;
                        if (iHosti == iHostj) { // In the Same Host
                            fSumBR += (1 - oContext.getECU(iHosti).getDoubleProperty(Host.FAILURE_RATE));
                        } else {
                            dNWdelay = 0.0; //Double.MAX_VALUE;
                            dNWReliability = 0.0;
                            dNWBW = 0.0;
                            if (oContext.areConnected(iHosti, iHostj)) {
                                dNWdelay = oContext.getNetworkProperties(iHosti, iHostj).getDouble(NetworkProperties.NW_DELAY);
                                dNWReliability = oContext.getNetworkProperties(iHosti, iHostj).getDouble(NetworkProperties.RELIABILITY);
                                dNWBW = oContext.getNetworkProperties(iHosti, iHostj).getDouble(NetworkProperties.BANDWIDTH);
                            }
                            fSumFND += dFreq * dNWdelay;
                            fSumBR += dNWReliability * dNWBW;
                        }
                    }
                }
            }
            float fRet = (fSumFND + (fSumFE / fSumBR));
            return fRet; // TODO:: return fRet;
            
            } else {
            System.err.println("ComOHEvaluator:Un supported architecture to evaluate.");
            return 0.0;
        }


    }
}
