package archeopterix.evaluation;

import java.util.List;

import archeopterix.problem.Component;
import archeopterix.problem.Host;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.deployment.solution.OriginalDeploymentSolution;
import archeopterix.problem.redundancy.solution.RedundancyAlloc;
import archeopterix.solution.Architecture;


public class CostEvaluator extends ArchitectureEvaluator {

	public CostEvaluator() {
		super("Cost", true);
	}

	@Override
	public double evaluate(EmbeddedSystemProblem oContext, Architecture oArch) {
		double dCost = 0.0;
		//FIXME Should implement subclasses of CostEvaluator for each type of problem
		if (oArch instanceof OriginalDeploymentSolution) {
			List<Host> hosts = oContext.getECUs();
			OriginalDeploymentSolution oDep = (OriginalDeploymentSolution)oArch;
			for (int iH = 0, c = oContext.getECUCount(); iH < c; iH++) {
				if ( oDep.getComponentCount(iH) > 0 )
					dCost += hosts.get(iH).getDoubleProperty(Host.COST);
			}
		}
		else if(oArch instanceof RedundancyAlloc) {
			List<Component> comps = oContext.getComponents();
			RedundancyAlloc oRA = (RedundancyAlloc)oArch;
			for (int iC = 0, c = oContext.getComponentCount(); iC < c; iC++){
				int iRA = oRA.getRALevel(iC);
				dCost += comps.get(iC).getDoubleProperty(Component.COST) * (1+iRA);
			}
		}

		return dCost;
	}

}
