package archeopterix.evaluation;

import java.util.List;

import archeopterix.problem.Host;
import archeopterix.problem.InteractionProperties;
import archeopterix.problem.NetworkProperties;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.deployment.solution.DeploymentSolution;
import archeopterix.problem.deployment.solution.ECUBasedDeployment;
import archeopterix.solution.Architecture;
import archeopterix.util.Pair;
import java.io.Serializable;

public class DataComRelEvaluator extends ArchitectureEvaluator implements Serializable {

    public DataComRelEvaluator()
	{
        super("DataComRel", false);
    }

    @Override
    public double evaluate(EmbeddedSystemProblem oProblem, Architecture oArch) {
        if (oArch instanceof ECUBasedDeployment) {
            ECUBasedDeployment oDep = (ECUBasedDeployment) oArch;
            float fSumFR = 0;
            float fSumF = 0;
            //James M: Have changed int[] to Pair<Integer> to match new result of getNextAssignedPair();
            //  if desired the components of these objects could be taken out and assigned to primitives to
            //  speed up their use below.
            //  FIXME However, it is really unclear what the result of getNextAssignedPair() represents; it also appears inefficient to calculate it for each component since the result is always the next component in the lists of component assigned to ECUs and it must walk each list in turn until that point is reached
            Pair<Integer> pair1, pair2;

            List<Host> ecus = oProblem.getECUs();

            //FIXME Should this only consider j > i (i < c - 1), or at least i != j?
            for (int i = 0, c = oProblem.getComponentCount(); i < c; i++) {
                pair1 = oDep.getNextAssignedPair(i);
                int c1 = pair1.getA();
                int host1 = pair1.getB();
                //Comp oCompi = (Comp)lst_Comp.get(iCompi);
                for (int j = 0; j < c; j++) {
                    pair2 = oDep.getNextAssignedPair(j);
                    int c2 = pair2.getA();
                    int host2 = pair2.getB();
                    //pair1[0], pair2[0]) is the components
                    if (oProblem.areInteracting(c1, c2)) {
                        //s_Result += "\n IP Read C[" + iCompi +"], C["+ iCompj + "] NP READ H[" + iHosti + "], H[" + iHostj + "]";
                        //Host oHosti = oDep.GetHost(oCompi);
                        //Host oHostj = oDep.GetHost(oCompj);


                        double dNWRel = 1.0;
                        if (host1 == host2) {
                            dNWRel = (1 - ecus.get(host1).getDoubleProperty(Host.FAILURE_RATE));
                        } else {
							try{
                            dNWRel = oProblem.getNetworkProperties(host1, host2).getDouble(NetworkProperties.RELIABILITY);
							}catch(Exception ex){
								System.out.println("Data com Rel Error:" + ex.toString());
							}
						}
                        fSumFR += oProblem.getInteractionProperties(c1, c2).getDouble(InteractionProperties.FREQUENCY) * dNWRel; //TODO
                        fSumF += oProblem.getInteractionProperties(c1, c2).getDouble(InteractionProperties.FREQUENCY);
                    }
                }
            }
            return (fSumFR / fSumF);
        } else if (oArch instanceof DeploymentSolution) {
            DeploymentSolution oDep = (DeploymentSolution) oArch;
            float fSumFR = 0;
            float fSumF = 0;

            for (int iCompi = 0; iCompi < oProblem.getComponentCount(); iCompi++) {
                //Comp oCompi = (Comp)lst_Comp.get(iCompi);
                for (int iCompj = 0; iCompj < oProblem.getComponentCount(); iCompj++) {
                    int iHosti = oDep.getHostIndex(iCompi);
                    int iHostj = oDep.getHostIndex(iCompj);
                    if (oProblem.areInteracting(iCompi, iCompj)) {
                        double dFreq, dNWReliability;
                        dFreq = oProblem.getInteractionProperties(iCompi, iCompj).getDouble(InteractionProperties.FREQUENCY);
                        fSumF += dFreq;
                        if (iHosti == iHostj) { // In the Same Host
                            dNWReliability = (1 - oProblem.getECU(iHosti).getDoubleProperty(Host.FAILURE_RATE));
                        } else {
                            dNWReliability = 0.0;
                            if (oProblem.areConnected(iHosti, iHostj)) {
                                dNWReliability = oProblem.getNetworkProperties(iHosti, iHostj).getDouble(NetworkProperties.RELIABILITY);
                            }
                        }
                        fSumFR += (dFreq * dNWReliability);
                    }
                }
            }
            return (fSumFR / fSumF);
        } else {
            System.err.println("DataComRelEvaluator:Un supported architecture to evaluate.");
            return 0.0;
        }
    }
}
