/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package archeopterix.evaluation;

import archeopterix.conf.Conf;
import archeopterix.problem.Component;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.Host;
import archeopterix.problem.InteractionProperties;
import archeopterix.problem.NetworkProperties;
import archeopterix.problem.SchedulingStratergy;
import archeopterix.problem.deployment.solution.DeploymentSolution;
import archeopterix.solution.Architecture;
import archeopterix.util.PropertyMap;
import archeopterix.util.matrix.Matrix;
import archeopterix.util.probability.MonteCarlo;
import archeopterix.util.probability.MyStatistics;
import java.io.BufferedReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.Serializable;
import java.util.Iterator;
import java.util.List;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class PCTLEvaluator extends ArchitectureEvaluator implements Serializable {
    public static final String PARAM_PCTL_FORMULA_FILE = "pctlFormulaFile";
    public static final String PARAM_PRISM_MODELS_DIRECTORY = "prismModelsDirectory";
    public static final String PARAM_PRISM_COMMAND = "prismCommand";

    transient protected EmbeddedSystemProblem m_Problem = null;
    protected boolean b_SeviceReliability = false; //Service Reliability
    protected int i_ServiceID = 0;
    Matrix mat_P = null;
    Matrix mat_Fail = null;
    Matrix mat_Success = null;
    double[] arr_StateEnergyRewards = null;
    double[] arr_StatePerfRewards = null;
    
    private int i_FileIndex = 0;
    private String s_PCTLFormulaFile = "Cheung80Rel.pctl";
    private String s_PrismCommand = "/home/indika/Documents/Programming/PRISM/prism-4.0.1-linux32/bin/prism";
    private String s_PrismModelsFolder = "./";
    private static final Log log = LogFactory.getLog(PCTLEvaluator.class);

    public PCTLEvaluator() {
        super("PCTL", false);
    }

    public PCTLEvaluator(Integer iServiceID) {
        super("PCTL" + iServiceID, false);
        b_SeviceReliability = true;
        i_ServiceID = iServiceID;
    }

    @Override
    public void setParameters(PropertyMap params){
        super.setParameters(params);
        s_PCTLFormulaFile = params.getString(PARAM_PCTL_FORMULA_FILE);
        s_PrismCommand = params.getString(PARAM_PRISM_COMMAND);
        s_PrismModelsFolder = params.getString(PARAM_PRISM_MODELS_DIRECTORY);
    }
    
    @Override
    public double evaluate(EmbeddedSystemProblem oProblem, Architecture oArch) {
        if (m_Problem == null) {
            m_Problem = oProblem;
        }
        if (oArch instanceof DeploymentSolution) {
             DeploymentSolution oDep = (DeploymentSolution) oArch;
            if (subjectUncertainty) {
                double dVal = 0.0;
                MyStatistics oStat = getStats(oArch);
                uncertaintyHandler.ResetRuns();
                while (uncertaintyHandler.hasMoreRuns(oStat)) {
                    dVal = evaluatePCTLProperty(oDep);
                    oStat.update(dVal, true);
                }
                return uncertaintyHandler.getEstimate(oStat);
            } else
                return evaluatePCTLProperty(oDep);
        }
        
        return 0.0;
    }

    private double evaluatePCTLProperty(DeploymentSolution oDep){
            createTransistionMatrix(oDep);
            double dRet = 0;
            if ( createPRISMModel("DTMC" + i_FileIndex + ".pm") ){
                dRet = executePrism( "DTMC" + i_FileIndex + ".pm", s_PCTLFormulaFile );
            }
            //i_FileIndex++;
            /*int iComps = m_Problem.getComponentCount();
            Matrix mT = ((Matrix.identity(iComps + 2, iComps + 2)).minus(mat_P));
            if (mT.det() == 0.0) {
                log.error("Det = 0.0");
            }
            return mT.inverse().get(0, iComps + 1);
             *
             */
            return dRet;
    }
    /**
     * Transistion matrix for Chenug80 Reliability Evaluation.
     * + State reward structures for energy consumpation
     * and response time ignoering (only using state rewards)
     *
     * @param oDep
     */
    protected void createTransistionMatrix(DeploymentSolution oDep) {
        int iComps = m_Problem.getComponentCount();
        mat_P = new Matrix(iComps + 2, iComps + 2, 0); // +1 is for super initial state (state 0), +1 for super final state
        mat_Fail = new Matrix(iComps + 2, 1, 0); // Column of Failure probabilities
        mat_Success = new Matrix(iComps + 2, 1, 0); // Column of SuccessProbabilities
        arr_StateEnergyRewards = new double[iComps];
        arr_StatePerfRewards = new double[iComps];
        for (int iCompi = 0; iCompi < m_Problem.getComponentCount(); iCompi++) {
            double dQ0 = 0;
            //if (subjectUncertainty) {
            //    dQ0 = b_SeviceReliability ? MonteCarlo.instance().Sample(m_Problem.getComponent(iCompi).getPDFFromListProperty(Component.INITILIZE_PROB_DT, i_ServiceID)) : MonteCarlo.instance().Sample(m_Problem.getComponent(iCompi).getPDFProperty(Component.INITILIZE_PROB_DT));
            //} else {
                dQ0 = b_SeviceReliability ? m_Problem.getComponent(iCompi).getDoubleFromListProperty(Component.INITILIZE_PROB, i_ServiceID) : m_Problem.getComponent(iCompi).getDoubleProperty(Component.INITILIZE_PROB);
            //}
            mat_P.set(0, iCompi + 1, dQ0); //Note. it always sets the fist row
            double dRowSum = 0;
            int iHi = oDep.getHostIndex(iCompi);
            double dLi = getECUFailureRate(iHi); //L for Lamda
            double dExecutionTime = getSojournTime(iCompi, iHi, oDep);
            double dRelCompi = Math.exp(-1 * dLi * dExecutionTime);
            double dPowerDissipation = getECUPowerDissipation(iHi);
            arr_StateEnergyRewards[iCompi] = dExecutionTime* dPowerDissipation ;
            arr_StatePerfRewards[iCompi]= dExecutionTime;
            
            double dFailureProb =  (1 - dRelCompi); //Transistion to Fail state
            
            for (int iCompj = 0; iCompj < m_Problem.getComponentCount(); iCompj++) {
                if (m_Problem.areInteracting(iCompi, iCompj)) {

                    double dPij = 0;
                    if (subjectUncertainty){
                        dPij = b_SeviceReliability ? MonteCarlo.instance().Sample(m_Problem.getInteractionProperties(iCompi, iCompj).getPDFFromList(InteractionProperties.TRANSITION_PROB_DT, i_ServiceID)) : MonteCarlo.instance().Sample(m_Problem.getInteractionProperties(iCompi, iCompj).getPDF(InteractionProperties.TRANSITION_PROB_DT));
                    } else {
                        dPij = b_SeviceReliability ? m_Problem.getInteractionProperties(iCompi, iCompj).getDoubleFromList(InteractionProperties.TRANSITION_PROB, i_ServiceID) : m_Problem.getInteractionProperties(iCompi, iCompj).getDouble(InteractionProperties.TRANSITION_PROB);
                    }

                    //Including Communication Failures
                    double dRelij = 1;
                    int iHj = oDep.getHostIndex(iCompj);
                    double dCLij = b_SeviceReliability ? m_Problem.getInteractionProperties(iCompi, iCompj).getDoubleFromList(InteractionProperties.COMM_LOAD_KB, i_ServiceID) : m_Problem.getInteractionProperties(iCompi, iCompj).getDouble(InteractionProperties.COMM_LOAD_KB);
                    PropertyMap oNW = m_Problem.getNetworkProperties(iHi, iHj);
                    if (oNW != null) {//Network exists
                        int iDRij = oNW.getInt(NetworkProperties.DATA_RATE_KBPS);
                        double dLij = getNetworkFailureRate(oNW);
                        dRelij = Math.exp(-1 * dLij * (dCLij / iDRij));
                        dFailureProb += (dPij * (1- dRelij));
                    }
                    
                    dPij = dPij * dRelCompi * dRelij;
                    dRowSum += dPij;
                    mat_P.set(iCompi + 1, iCompj + 1, dPij);
                }
            }
            if (dRowSum < Double.MIN_VALUE) { //Absoarbing state
                mat_P.set(iCompi + 1, iComps + 1, dRelCompi);
                dRowSum += dRelCompi;
            }
            //Normalising
            dRowSum += dFailureProb;
            for (int j = 0; j < iComps + 1; j++) {
                mat_P.set(iCompi + 1, j, mat_P.get(iCompi + 1, j) / dRowSum);
            }

            mat_Fail.set(iCompi + 1, 0, dFailureProb / dRowSum); //Set failure probabilties
        }
        //Normalise first raw
        /*double dTemp = 0;
        for (int j = 0; j < iComps + 2; j++) {
            dTemp += mat_P.get(0, j);
        }
        for (int j = 0; j < iComps + 2; j++) {
            mat_P.set(0, j, mat_P.get(0, j)/dTemp);
        }*/

        mat_Success.set(iComps + 1, 0, 1); //From superfinal state, a transistion to success state
        // System.out.println(mat_Fail + " SUCC " + mat_Success);
    }

    /**
     * Creates a Prism Model file for Cheung80 Reliability evaluation
     */
    protected boolean createPRISMModel(String sModelFile) {
        PrintWriter prismFile = null; //For Data File
        try {
            int iRows = mat_P.getRowDimension();
            prismFile = new PrintWriter(new FileWriter(s_PrismModelsFolder + sModelFile));
            prismFile.println("dtmc\n");
            prismFile.println("module Cheung80");
            prismFile.println("s : [0.." + (iRows - 1) + "] init 0; //s[0] intial state.");
            prismFile.println(" // s[0] super intial state.");
            prismFile.println(" // s[" + (iRows - 1) + "] super final state.");
            prismFile.println("x : [0..1] init 1; // New states for reliability evaluation. x[0] = fail, x[1] = success");
            prismFile.println("completed :  bool init false; //Execution complete state");
            //prismFile.println("label \"success\" =  x=1;");
            //prismFile.println("label \"fail\" =  x=0;");

            for (int i = 0; i < iRows; i++) {
                String sRow = "";
                for (int j = 0; j < iRows; j++) {
                    double dPij = mat_P.get(i, j);
                    if (dPij > Double.MIN_VALUE) {
                        if (sRow.equals("")) //First Entry for row
                        {
                            sRow = "[] s=" + i + "-> " + dPij + " : (s'=" + j + ")";
                        } else {
                            sRow += (" + " + dPij + " : (s'=" + j + ")");
                        }
                    }
                }
                double dFi = mat_Fail.get(i, 0); //Failure probability of state
                if (dFi > Double.MIN_VALUE) {
                    if (sRow.equals("")) //First Entry for row
                    {
                        sRow = "[] s=" + i + "-> " + dFi + " : (x'=0)";
                    } else {
                        sRow += (" + " + dFi + " : (x'=0)");
                    }
                }
                if (!sRow.equals("")) {
                    prismFile.println(sRow + ";");
                } else { //Super final state
                    prismFile.println("[] s=" + i + "-> 1 : (x' = 1 ) & (completed' = true);");
                }
            }
            prismFile.println("endmodule\n\n");
            
            prismFile.println("rewards \"energyrewards\"");
            for (int i = 0; i < arr_StateEnergyRewards.length; i++){
                prismFile.println("s=" + (i+1) + " : " + arr_StateEnergyRewards[i] + ";");
            }
            prismFile.println("endrewards\n\n");

            prismFile.println("rewards \"perfrewards\"");
            for (int i = 0; i < arr_StatePerfRewards.length ; i++){
                prismFile.println("s=" + (i+1) + " : " + arr_StatePerfRewards[i] + ";");
            }
            prismFile.println("endrewards\n\n");

        } catch (IOException e) {
            log.error("Error writing the PRISM model :" + s_PrismModelsFolder + sModelFile);
            return false;
        } finally {
            if (prismFile != null) {
                log.trace("Prism Model File Created :" + s_PrismModelsFolder + sModelFile);
                prismFile.close();
            }
            return true;
        }
    }

    /**
     * Execute prism and capture the result
     * @return
     */
    protected double executePrism(String sModelFile, String sFormulaFile) {
        String sCommand = s_PrismCommand + " "
                    + s_PrismModelsFolder + sModelFile + " "
                    + sFormulaFile;
        try {
            log.trace(sCommand);
            Process p = Runtime.getRuntime().exec(sCommand);
            p.waitFor();
            BufferedReader reader = new BufferedReader(new InputStreamReader(p.getInputStream()));
            String line = reader.readLine();
            while (line != null) {
                if (line.startsWith("Result:")){
                    String[] asParts = line.split(" ");
                    log.trace("RESULT;"+ s_PCTLFormulaFile + ";" + asParts[1]);
                    return Double.parseDouble( asParts[1] );
                }
                //else
                //    System.out.println("NOMATCH -" +line);
                line = reader.readLine();
            }
            System.err.println("Could not extract prism results :" + (i_FileIndex++));
        } catch (IOException e1) {
            log.error("IO Error in Command: " + sCommand + " ERROR:" + e1.toString());
        } catch (InterruptedException e2) {
            log.error("Interrupted in  Command: " + sCommand + " ERROR:"+ e2.toString());
        }
        return 0;
    }

    protected double getECUFailureRate(int iECUIndex) {
        if (subjectUncertainty) {
            return MonteCarlo.instance().Sample(m_Problem.getECU(iECUIndex).getPDFProperty(Host.FAILURE_RATE_DT));
        }
        return m_Problem.getECU(iECUIndex).getDoubleProperty(Host.FAILURE_RATE);
    }

    protected double getECUPowerDissipation(int iECUIndex) {
        if (subjectUncertainty) {
            return MonteCarlo.instance().Sample(m_Problem.getECU(iECUIndex).getPDFProperty(Host.POWER_DISSIPATION_DT));
        }
        return m_Problem.getECU(iECUIndex).getDoubleProperty(Host.POWER_DISSIPATION);
    }


    protected double getNetworkFailureRate(PropertyMap oNW) {
        if (subjectUncertainty) {
            return MonteCarlo.instance().Sample(oNW.getPDF(NetworkProperties.FAILURE_RATE_DT));
        }
        return oNW.getDouble(NetworkProperties.FAILURE_RATE);
    }
    protected double getSojournTime(int iCompID, int iHostID, DeploymentSolution oDep) {

        int iMIPS = m_Problem.getECU(iHostID).getIntProperty(Host.SPEED_MIPS);
        if (Conf.instance().getSchedulingStratergy() == SchedulingStratergy.FIXED) {
            double dMI = 0.0;
            List<Component> lstOtherComps = oDep.getComponentListForHost(iHostID, m_Problem);
            Iterator<Component> ite = lstOtherComps.iterator();
            while (ite.hasNext()) {
                Component oComp = ite.next();
                if (b_SeviceReliability) {
                    //Summing up for all services
                    List<Double> lstMIs = oComp.getDoubleListProperty(Component.COMPUTATIONS_MI);
                    Iterator<Double> ite2 = lstMIs.iterator();
                    while (ite2.hasNext()) {
                        dMI += ite2.next();
                    }
                } else {
                    dMI += oComp.getDoubleProperty(Component.COMPUTATIONS_MI);
                }
            }
            return (dMI / (double) iMIPS);
        } else {
            double dMI = b_SeviceReliability ? m_Problem.getComponent(iCompID).getDoubleFromListProperty(Component.COMPUTATIONS_MI, i_ServiceID) : m_Problem.getComponent(iCompID).getDoubleProperty(Component.COMPUTATIONS_MI);
            return (dMI / (double) iMIPS);
        }

    }
}
