package archeopterix.evaluation;

import archeopterix.conf.Conf;
import archeopterix.problem.Component;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.Host;
import archeopterix.problem.InteractionProperties;
import archeopterix.problem.NetworkProperties;
import archeopterix.problem.SchedulingStratergy;
import archeopterix.problem.componentselection.solution.ComponentSelectionSolution;
import archeopterix.problem.deployment.solution.DeploymentSolution;
import archeopterix.problem.redundancy.solution.RedundancyAlloc;
import archeopterix.solution.Architecture;
import archeopterix.util.PropertyMap;
import archeopterix.util.matrix.Matrix;
import archeopterix.util.probability.MonteCarlo;
import archeopterix.util.probability.MyStatistics;
import java.io.Serializable;
import java.util.Iterator;
import java.util.List;

public class ReliabilityEvaluator extends ArchitectureEvaluator implements Serializable {

    transient protected Matrix mat_Q0;
    transient protected Matrix mat_Pij;
    transient protected Matrix mat_Vi;//Expected No of visits for Comps
    transient private Matrix mat_Q0_BK; //For roubst values
    transient private Matrix mat_Pij_BK; // For roubst values
    transient protected EmbeddedSystemProblem m_Problem = null;
    protected boolean b_SeviceReliability = false; //Service Reliability
    protected int i_ServiceID = 0;

    public ReliabilityEvaluator() {
        super("Rel", false);

    }

    public ReliabilityEvaluator(String sName) {
        super(sName, false);

    }

    public ReliabilityEvaluator(Integer iServiceID) {
        super("RelSrv" + iServiceID, false);
        b_SeviceReliability = true;
        i_ServiceID = iServiceID;
    }

    public ReliabilityEvaluator(String sName, Integer iServiceID) {
        super(sName + "Srv" + iServiceID, false);
        b_SeviceReliability = true;
        i_ServiceID = iServiceID;
    }

    // Littlewood Model .. Refer Goseva-Popstojanova07T
	/*public double evaluate(Context oContext, Deployment oDep){

    double dLamdaS =0.0;
    double dDenominator = 0.0; //( Sum of PIi * Sum(Pij*mij) )
    double dSumAiLi = 0.0;
    double dSumBijVij = 0.0;
    for ( int iCompi =0; iCompi < oContext.i_Comps; iCompi++){
    int iHi 		= oDep.GetHostIndex(iCompi);
    double dPIi  = oContext.GetCompParameter(iCompi, Defs.STEDY_STATE_PROB );
    double dLi 	= oContext.GetHostParameter(iHi, Defs.FAILURE_INTENSITY ); //L for Lamda
    double dSumPijMij	= 0.0;
    //Comp oCompi = (Comp)lst_Comp.get(iCompi);
    for ( int iCompj = 0; iCompj < oContext.i_Comps; iCompj++){
    if ( oContext.IsInteracting (iCompi, iCompj)) {
    double dPij = oContext.GetInteractionParameter(iCompi, iCompj, Defs.INTERACTION_PROBABILITY );
    double dMij = oContext.GetInteractionParameter(iCompi, iCompj, Defs.MEAN_EXEC_TIME );
    double dBij = dPIi * dPij; // Only the numerator
    dSumPijMij += dPij * dMij;

    int iHj = oDep.GetHostIndex(iCompj);
    double dVij = oContext.GetNetworkParameter(iHi, iHj, Defs.FAILURE_INTENSITY);
    dSumBijVij += (dBij * dVij);
    }
    }
    double dAi = dPIi * dSumPijMij; // Only the numerator
    dSumAiLi += (dAi*dLi);
    dDenominator += dAi;
    }
    dLamdaS = ((dSumAiLi + dSumBijVij)/ dDenominator );
    return (1-dLamdaS);
    }*/
    //Assayad04G Model
    @Override
    public double evaluate(EmbeddedSystemProblem oProblem, Architecture oArch) {
        if (m_Problem == null) {
            m_Problem = oProblem;
            ComputeExpectedVisitsMatrix();
        }
        //applyUncertainty();
        if (oArch instanceof DeploymentSolution) {
            //ReComputeExpectedVisitsMatrix();
            DeploymentSolution oDep = (DeploymentSolution) oArch;
            if (subjectUncertainty) {
                double dVal = 0.0;
                MyStatistics oStat = getStats(oArch);
                uncertaintyHandler.ResetRuns();
                while (uncertaintyHandler.hasMoreRuns(oStat)) {
                    dVal = EvaluateReliabilityKubat89(oProblem, oDep);
                    oStat.update(dVal, true);
                }
                return uncertaintyHandler.getEstimate(oStat);
            } else {
                return EvaluateReliabilityKubat89(oProblem, oDep);
            }
        } else if (oArch instanceof RedundancyAlloc) {
            //ReComputeExpectedVisitsMatrix();
            RedundancyAlloc oRA = (RedundancyAlloc) oArch;
            if (subjectUncertainty) {
                double dVal = 0.0;
                MyStatistics oStat = getStats(oArch);
                uncertaintyHandler.ResetRuns();
                while (uncertaintyHandler.hasMoreRuns(oStat)) {
                    dVal = EvaluateReliabilityKubat89(oProblem, oRA);
                    oStat.update(dVal, true);
                }
                return uncertaintyHandler.getEstimate(oStat);
            } else {
                return EvaluateReliabilityKubat89(oProblem, oRA);
            }
            //return (1-dTemp);

        } else if (oArch instanceof ComponentSelectionSolution) {
            //ReComputeExpectedVisitsMatrix();
            ComponentSelectionSolution oCS = (ComponentSelectionSolution) oArch;
            if (subjectUncertainty) {
                double dVal = 0.0;
                MyStatistics oStat = getStats(oArch);
                uncertaintyHandler.ResetRuns();
                while (uncertaintyHandler.hasMoreRuns(oStat)) {
                    dVal = EvaluateReliabilityKubat89(oProblem, oCS);
                    oStat.update(dVal, true);
                }
                return uncertaintyHandler.getEstimate(oStat);
            } else {
                return EvaluateReliabilityKubat89(oProblem, oCS);
            }
            //return (1-dTemp);
        }
        return 0.0;
        //return Math.exp(dRet);
        //return Math.pow(dRet, oContext.d_ArrivalRate);
        //return dRet * oProblem.getService(0).getArrivalRate();
    }

    /**
     * Hierachical Reliability Evaluation Model (Kubat89)
     * Component Selection Problems
     * @param oProblem
     * @param oCS
     * @return
     */
    protected double EvaluateReliabilityKubat89(EmbeddedSystemProblem oProblem, ComponentSelectionSolution oCS) {
        double dRet = 1.0;
        for (int iCompi = 0; iCompi < oProblem.getComponentCount(); iCompi++) {
            int iOption = oCS.getCurrentSelection(iCompi);
            Component oComp = oProblem.getComponent(iCompi);
            double dCompRel;
            if (subjectUncertainty) {
                dCompRel = MonteCarlo.instance().Sample(oComp.getPDFFromListProperty(Component.OPTION_RELIABILITY_DT, iOption));
            } else {
                dCompRel = oComp.getDoubleFromListProperty(Component.OPTION_RELIABILITY, iOption);
            }

            double dVi = mat_Vi.get(iCompi, 0);
            double dRel = Math.pow(dCompRel, dVi);
            ;
            dRet *= dRel;
            for (int iCompj = 0; iCompj < oProblem.getComponentCount(); iCompj++) {
                if (oProblem.areInteracting(iCompi, iCompj)) {
                    double dLi = oProblem.getInteractionProperties(iCompi, iCompj).getDouble(InteractionProperties.FAILURE_RATE);
                    double dTime = oProblem.getInteractionProperties(iCompi, iCompj).getDouble(InteractionProperties.TX_TIME) / (3.6E5);
                    dRel = Math.exp(-1 * dLi * dTime);
                    double dPij = b_SeviceReliability ? oProblem.getInteractionProperties(iCompi, iCompj).getDoubleFromList(InteractionProperties.TRANSITION_PROB, i_ServiceID) : oProblem.getInteractionProperties(iCompi, iCompj).getDouble(InteractionProperties.TRANSITION_PROB);
                    dRel = Math.pow(dRel, (dVi * dPij));
                    dRet *= dRel;
                }
            }
        }
        return dRet;
    }

    /**
     * Hierachical Reliability Evaluation Model (Kubat89)
     * Redundancy Allocation Problem
     * @param oProblem
     * @param oCS
     * @return
     */
    protected double EvaluateReliabilityKubat89(EmbeddedSystemProblem oProblem, RedundancyAlloc oRA) {
        double dRet = 1.0;
        for (int iCompi = 0; iCompi < oProblem.getComponentCount(); iCompi++) {
            double dLi = oProblem.getComponent(iCompi).getDoubleProperty(Component.FAILURE_RATE); //L for Lamda
            double dTime = b_SeviceReliability ? oProblem.getComponent(iCompi).getDoubleFromListProperty(Component.SOJOURN_TIME, i_ServiceID) / (3.6E5) : oProblem.getComponent(iCompi).getDoubleProperty(Component.SOJOURN_TIME) / (3.6E5);

            int iRA = oRA.getRALevel(iCompi);
            double dRel = Math.exp(-1 * dLi * dTime);
            dRel = 1 - Math.pow((1 - dRel), (double) (iRA + 1));
            //double dVi 		= oContext.GetCompParameterDouble(iCompi, Defs.EXPECTED_VISITS, i_ServiceIndex );
            double dVi = mat_Vi.get(iCompi, 0);
            //dRel = Math.pow(dRel, dVi);
            //dRet += (Math.log(dRel) * dVi);
            dRet *= dRel;
            for (int iCompj = 0; iCompj < oProblem.getComponentCount(); iCompj++) {
                if (oProblem.areInteracting(iCompi, iCompj)) {
                    dLi = oProblem.getInteractionProperties(iCompi, iCompj).getDouble(InteractionProperties.FAILURE_RATE);
                    dTime = oProblem.getInteractionProperties(iCompi, iCompj).getDouble(InteractionProperties.TX_TIME) / (3.6E5);
                    dRel = Math.exp(-1 * dLi * dTime);
                    dRel = 1 - Math.pow((1 - dRel), (double) (iRA + 1));
                    double dPij = b_SeviceReliability ? oProblem.getInteractionProperties(iCompi, iCompj).getDoubleFromList(InteractionProperties.TRANSITION_PROB, i_ServiceID) : oProblem.getInteractionProperties(iCompi, iCompj).getDouble(InteractionProperties.TRANSITION_PROB);
                    dRel = Math.pow(dRel, (dVi * dPij));
                    //dRet += (Math.log(dRel) * (dVi * dPij));
                    dRet *= dRel;
                }
            }
            //if (dRel < 0.999)//To Avoid dTemp being 0
            //dTemp *= (1-dRel);
        }
        return dRet;
    }

    /**
     * Hierachical Reliability Evaluation Model (Kubat89)
     * Deployment Candidates
     * @param oProblem
     * @param oCS
     * @return
     */
    protected double EvaluateReliabilityKubat89(EmbeddedSystemProblem oProblem, DeploymentSolution oDep) {
        double dRet = 1.0;
        for (int iCompi = 0; iCompi < oProblem.getComponentCount(); iCompi++) {
            int iHi = oDep.getHostIndex(iCompi);
            double dLi = getECUFailureRate(iHi); //L for Lamda
            double dRel = Math.exp(-1 * dLi * getSojournTime(iCompi, iHi, oDep));
            double dVi = mat_Vi.get(iCompi, 0);
            dRel = Math.pow(dRel, dVi);
            dRet *= dRel;
            for (int iCompj = 0; iCompj < oProblem.getComponentCount(); iCompj++) {
                if (oProblem.areInteracting(iCompi, iCompj)) {
                    int iHj = oDep.getHostIndex(iCompj);
                    double dCLij = b_SeviceReliability ? oProblem.getInteractionProperties(iCompi, iCompj).getDoubleFromList(InteractionProperties.COMM_LOAD_KB, i_ServiceID) : oProblem.getInteractionProperties(iCompi, iCompj).getDouble(InteractionProperties.COMM_LOAD_KB);
                    PropertyMap oNW = oProblem.getNetworkProperties(iHi, iHj);
                    if (oNW != null) {//Network exists
                        int iDRij = oNW.getInt(NetworkProperties.DATA_RATE_KBPS);
                        double dLij = getNetworkFailureRate(oNW);
                        dRel = Math.exp(-1 * dLij * (dCLij / iDRij));
                        double dVij = mat_Vi.get(iCompi, 0) * mat_Pij.get(iCompi, iCompj);
                        dRel = Math.pow(dRel, dVij);
                        dRet *= dRel;
                    }
                }
            }
        }
        return dRet;
    }

    protected double getSojournTime(int iCompID, int iHostID, DeploymentSolution oDep) {

        int iMIPS = m_Problem.getECU(iHostID).getIntProperty(Host.SPEED_MIPS);
        if (Conf.instance().getSchedulingStratergy() == SchedulingStratergy.FIXED) {
            double dMI = 0.0;
            List<Component> lstOtherComps = oDep.getComponentListForHost(iHostID, m_Problem);
            Iterator<Component> ite = lstOtherComps.iterator();
            while (ite.hasNext()) {
                Component oComp = ite.next();
                if (b_SeviceReliability) {
                    //Summing up for all services
                    List<Double> lstMIs = oComp.getDoubleListProperty(Component.COMPUTATIONS_MI);
                    Iterator<Double> ite2 = lstMIs.iterator();
                    while (ite2.hasNext()) {
                        dMI += ite2.next();
                    }
                } else {
                    dMI += oComp.getDoubleProperty(Component.COMPUTATIONS_MI);
                }
            }
            return (dMI / (double) iMIPS);
        } else {
            double dMI = b_SeviceReliability ? m_Problem.getComponent(iCompID).getDoubleFromListProperty(Component.COMPUTATIONS_MI, i_ServiceID) : m_Problem.getComponent(iCompID).getDoubleProperty(Component.COMPUTATIONS_MI);
            return (dMI / (double) iMIPS);
        }

    }

    protected double getECUFailureRate(int iECUIndex) {
        if (subjectUncertainty) {
            return MonteCarlo.instance().Sample(m_Problem.getECU(iECUIndex).getPDFProperty(Host.FAILURE_RATE_DT));
        }
        return m_Problem.getECU(iECUIndex).getDoubleProperty(Host.FAILURE_RATE);
    }

    protected double getNetworkFailureRate(PropertyMap oNW) {
        if (subjectUncertainty) {
            return MonteCarlo.instance().Sample(oNW.getPDF(NetworkProperties.FAILURE_RATE_DT));
        }
        return oNW.getDouble(NetworkProperties.FAILURE_RATE);
    }

    /**
     * Apply some uncertainty to the Transistion probability and initialisation probability vectors
     */
    protected void applyUncertainty() {
        //Applying uncertainty.
        double[][] adPij = mat_Pij_BK.getArray();
        for (int iComp = 0; iComp < m_Problem.getComponentCount(); iComp++) {
            adPij[iComp] = MonteCarlo.instance().SampleDirchelet(adPij[iComp]);
        }
        double[][] adQ0T = mat_Q0_BK.transpose().getArray();
        adQ0T[0] = MonteCarlo.instance().SampleDirchelet(adQ0T[0]);
        for (int iComp = 0; iComp < m_Problem.getComponentCount(); iComp++) {
            mat_Q0.set(iComp, 0, adQ0T[0][iComp]);
        }
    }

    protected void ComputeExpectedVisitsMatrix() {
        mat_Q0 = new Matrix(m_Problem.getComponentCount(), 1);
        mat_Pij = new Matrix(m_Problem.getComponentCount(), m_Problem.getComponentCount());
        for (int iCompi = 0; iCompi < m_Problem.getComponentCount(); iCompi++) {
            double dQ0 = b_SeviceReliability ? m_Problem.getComponent(iCompi).getDoubleFromListProperty(Component.INITILIZE_PROB, i_ServiceID) : m_Problem.getComponent(iCompi).getDoubleProperty(Component.INITILIZE_PROB);
            mat_Q0.set(iCompi, 0, dQ0);
            for (int iCompj = 0; iCompj < m_Problem.getComponentCount(); iCompj++) {
                if (m_Problem.areInteracting(iCompi, iCompj)) {
                    double dPij = b_SeviceReliability ? m_Problem.getInteractionProperties(iCompi, iCompj).getDoubleFromList(InteractionProperties.TRANSITION_PROB, i_ServiceID) : m_Problem.getInteractionProperties(iCompi, iCompj).getDouble(InteractionProperties.TRANSITION_PROB);
                    mat_Pij.set(iCompi, iCompj, dPij);
                }
            }
        }
        //Expected Number of Visits for Components
        Matrix T = (Matrix.identity(mat_Pij.getColumnDimension(), mat_Pij.getRowDimension())).minus(mat_Pij.transpose());
        mat_Vi = T.inverse().times(mat_Q0);
    }

    protected void ReComputeExpectedVisitsMatrix() {
        if (mat_Q0 == null) {
            mat_Q0 = new Matrix(m_Problem.getComponentCount(), 1);
            mat_Pij = new Matrix(m_Problem.getComponentCount(), m_Problem.getComponentCount());
            ComputeExpectedVisitsMatrix();
            mat_Q0_BK = new Matrix(mat_Q0.getArrayCopy());
            mat_Pij_BK = new Matrix(mat_Pij.getArrayCopy());
        }

        double dSumQ0 = 0.0;//For Normalization
        for (int iCompi = 0; iCompi < m_Problem.getComponentCount(); iCompi++) {
            Component oComp = m_Problem.getComponent(iCompi);
            double dQ0 = b_SeviceReliability ? MonteCarlo.instance().Sample(oComp.getPDFFromListProperty(Component.INITILIZE_PROB_DT, i_ServiceID)) : MonteCarlo.instance().Sample(oComp.getPDFProperty(Component.INITILIZE_PROB_DT));
            mat_Q0.set(iCompi, 0, dQ0);
            dSumQ0 += dQ0;
            double dSumPij = 0.0; //For Normalization
            for (int iCompj = 0; iCompj < m_Problem.getComponentCount(); iCompj++) {
                if (m_Problem.areInteracting(iCompi, iCompj)) {
                    PropertyMap oInteraction = m_Problem.getInteractionProperties(iCompi, iCompj);
                    double dPij = b_SeviceReliability ? MonteCarlo.instance().Sample(oInteraction.getPDFFromList(InteractionProperties.TRANSITION_PROB_DT, i_ServiceID)) : MonteCarlo.instance().Sample(oInteraction.getPDF(InteractionProperties.TRANSITION_PROB_DT));
                    mat_Pij.set(iCompi, iCompj, dPij);
                    dSumPij += dPij;
                }
            }
            if (dSumPij > 0) {
                for (int iCompj = 0; iCompj < m_Problem.getComponentCount(); iCompj++) {
                    mat_Pij.set(iCompi, iCompj, mat_Pij.get(iCompi, iCompj) / dSumPij);
                }
            }
        }
        if (dSumQ0 > 0) {
            for (int iCompi = 0; iCompi < m_Problem.getComponentCount(); iCompi++) {
                mat_Q0.set(iCompi, 0, mat_Q0.get(iCompi, 0) / dSumQ0);
            }
        }

        Matrix T = (Matrix.identity(mat_Pij.getColumnDimension(), mat_Pij.getRowDimension())).minus(mat_Pij.transpose());
        mat_Vi = T.inverse().times(mat_Q0);
    }
}
