/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package archeopterix.evaluation;

import archeopterix.problem.Component;
import archeopterix.problem.Host;
import archeopterix.problem.InteractionProperties;
import archeopterix.problem.NetworkProperties;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.deployment.solution.DeploymentSolution;
import archeopterix.problem.redundancy.solution.RedundancyAlloc;
import archeopterix.solution.Architecture;
import archeopterix.util.PropertyMap;
import archeopterix.util.matrix.Matrix;
import archeopterix.util.probability.MonteCarlo;
import archeopterix.util.probability.MyStatistics;
import java.io.Serializable;

/**
 *
 * @author imeedeniya
 */
public class RobustReliabilityEvaluator extends ReliabilityEvaluator implements Serializable {

    transient private Matrix mat_Q0_BK;
    transient private Matrix mat_Pij_BK;

    public RobustReliabilityEvaluator() {
        super("RobustRel");
    }

    public RobustReliabilityEvaluator(Integer iServiceID) {
        super("RobustRel", iServiceID.intValue());
    }

    @Override
    public double evaluate(EmbeddedSystemProblem oProblem, Architecture oArch) {
        if (m_Problem == null) {
            m_Problem = oProblem;
            ComputeExpectedVisitsMatrix();
        }

        //applyUncertainty();

        //return EvaluateReliabilityKubat89(oProblem, oDep);
//            return EvaluateReliabilityCheung80(oProblem, oDep);

        //double[] dData = new double[i_Runs];
        double dVal = 0.0;
        MyStatistics oStat = getStats(oArch);
        uncertaintyHandler.ResetRuns();
        int iSample = 0;
        //ChartViewer oChart = new ChartViewer("Sample Values",ChartViewer.CHART_TYPE_XY_LINE,false, false);
        while (uncertaintyHandler.hasMoreRuns(oStat)) {
            //ReComputeExpectedVisitsMatrix();
            if (oArch instanceof DeploymentSolution) {
                dVal = EvaluateReliabilityKubat89(oProblem, (DeploymentSolution) oArch);
            } else if (oArch instanceof RedundancyAlloc) {
                dVal = EvaluateReliabilityKubat89(oProblem, (RedundancyAlloc) oArch);
            }
            oStat.update(dVal,true);
            //ChartViewer.Ins().UpdateXY("Sample", null, iSample++, dVal);
            //oChart.UpdateXY("Reliability", null, iSample++, dVal);
        }
        //oChart.View();
        //oChart.saveChart("./", "Samples");

        //LogFile.LogDoubleArray(dData, Conf.Ins().s_RootFolder+"/Sol_"+ oDep.getBufferString()+".dat");
        return uncertaintyHandler.getEstimate(oStat);
        /*
        double dMean = oDep.am_Stat[0].getMean();
        double dVar = oDep.am_Stat[0].getVariance();

         */
        /*HashMap<Double,Double> mapFD = Utils.getDistribution(dData, 100);
        ChartViewer oChart = new ChartViewer("Frequency Distribution",ChartViewer.CHART_TYPE_XY_LINE,false, false);
        oChart.SetData(s_Name , mapFD);
        oChart.AddText(0.2,0.85, "Avg ="+ dMean);
        oChart.AddText(0.2,0.75, "Var ="+ dVar);
        //oChart.SetOtherDistributions(mapFD, dMean, dVar);

        oChart.View();
        oChart.SaveChartAsJPG(Conf.Ins().s_RootFolder+"/Sol_"+ oDep.getDepString()+".jpg");
        LogFile.LogDoubleArray(dData, Conf.Ins().s_RootFolder+"/Sol_"+ oDep.getDepString()+".dat");
        try{
        System.in.read();
        }
        catch(Exception e){}
         */

        /*
        ChartViewer oChart2 = new ChartViewer("Probability Distribution",ChartViewer.CHART_TYPE_XY_LINE,false, false);
        oChart2.SetOtherDistributions(mapFD, dMean, dVar);
        //oChart.SaveChartAsJPG(Conf.Ins().s_RootFolder+"/Sol_"+ oDep.getDepString()+"_"+i_ServiceIndex+".jpg");
        oChart2.AddText(0.2,0.85, "Avg ="+ dMean);
        oChart2.AddText(0.2,0.75, "Var ="+ dVar);
        oChart2.View();

        //System.out.println ("n=" + oDep.am_Stat[i_ServiceIndex].getCount());
        //ChartViewer.Ins().UpdateXY(s_Name,null, dMean, dVar);
        //ChartViewer.Ins().View();
        //return Utils.GetBetaEstimate(dMean, dVar, d_Confidance);
        //return Utils.GetNormalEstimate(dMean, dVar, d_Confidance);
         */
        //return Utils.getTriangularEstimate(dMean, Math.abs(dVar), d_Confidance);
        //return Utils.getNormalEstimate(dMean, dVar, d_Confidance);

    }

    @Override
    protected double getECUFailureRate(int iECUIndex) {
        double dSample = MonteCarlo.instance().Sample(m_Problem.getECU(iECUIndex).getPDFProperty(Host.FAILURE_RATE_DT));
        //double dPE = m_Problem.GetHostParameter(iECUIndex, Defs.FAILURE_RATE );
        //System.out.println("ECU PE="+ dPE + " SAM="+ dSample + " DIFF = "+ (dSample - dPE));
        return dSample;
    }

    @Override
    protected double getNetworkFailureRate(PropertyMap oNW) {
        double dSample = MonteCarlo.instance().Sample(oNW.getPDF(NetworkProperties.FAILURE_RATE_DT));
        //double dPE = m_Problem.GetNetworkParameter(iECUi, iECUj, Defs.FAILURE_RATE);
        //System.out.println("NW PE="+ dPE + " SAM="+ dSample + " DIFF = "+ (dSample - dPE));
        return dSample;

    }

    protected void applyUncertainty() {
        //Applying uncertainty.
        double[][] adPij = mat_Pij_BK.getArray();
        for (int iComp = 0; iComp < m_Problem.getComponentCount(); iComp++) {
            adPij[iComp] = MonteCarlo.instance().SampleDirchelet(adPij[iComp]);
        }

        //System.out.println ( "Q Before Uncertainty" + mat_Q0.toString() );
        double[][] adQ0T = mat_Q0_BK.transpose().getArray();
        adQ0T[0] = MonteCarlo.instance().SampleDirchelet(adQ0T[0]);
        for (int iComp = 0; iComp < m_Problem.getComponentCount(); iComp++) {
            mat_Q0.set(iComp, 0, adQ0T[0][iComp]);
        }
    }

    @Override
    protected void ReComputeExpectedVisitsMatrix() {
        if (mat_Q0 == null) {
            mat_Q0 = new Matrix(m_Problem.getComponentCount(), 1);
            mat_Pij = new Matrix(m_Problem.getComponentCount(), m_Problem.getComponentCount());
            ComputeExpectedVisitsMatrix();
            mat_Q0_BK = new Matrix(mat_Q0.getArrayCopy());
            mat_Pij_BK = new Matrix(mat_Pij.getArrayCopy());
        }

        double dSumQ0 = 0.0;//For Normalization
        for (int iCompi = 0; iCompi < m_Problem.getComponentCount(); iCompi++) {
            Component oComp = m_Problem.getComponent(iCompi);
            double dQ0 = b_SeviceReliability ? MonteCarlo.instance().Sample(oComp.getPDFFromListProperty(Component.INITILIZE_PROB_DT, i_ServiceID)) : MonteCarlo.instance().Sample(oComp.getPDFProperty(Component.INITILIZE_PROB_DT));
            mat_Q0.set(iCompi, 0, dQ0);
            dSumQ0 += dQ0;
            double dSumPij = 0.0; //For Normalization
            for (int iCompj = 0; iCompj < m_Problem.getComponentCount(); iCompj++) {
                if (m_Problem.areInteracting(iCompi, iCompj)) {
                    PropertyMap oInteraction = m_Problem.getInteractionProperties(iCompi, iCompj);
                    double dPij = b_SeviceReliability ? MonteCarlo.instance().Sample(oInteraction.getPDFFromList(InteractionProperties.TRANSITION_PROB_DT, i_ServiceID)) : MonteCarlo.instance().Sample(oInteraction.getPDF(InteractionProperties.TRANSITION_PROB_DT));
                    mat_Pij.set(iCompi, iCompj, dPij);
                    dSumPij += dPij;
                }
            }
            if (dSumPij > 0) {
                for (int iCompj = 0; iCompj < m_Problem.getComponentCount(); iCompj++) {
                    mat_Pij.set(iCompi, iCompj, mat_Pij.get(iCompi, iCompj) / dSumPij);
                }
            }
        }
        if (dSumQ0 > 0) {
            for (int iCompi = 0; iCompi < m_Problem.getComponentCount(); iCompi++) {
                mat_Q0.set(iCompi, 0, mat_Q0.get(iCompi, 0) / dSumQ0);
            }
        }


        //System.out.println ( "Pij for Service "+ i_ServiceID + " =" +  mat_Pij.toString() );
        //System.out.println ( "Q0 for Service "+ i_ServiceID + " =" + mat_Q0.toString() );

        //System.out.println ( "S=" + i_ServiceIndex + "EX[i]" + PrintMatrix(E) );
        //Expected Number of Visits for Components
        Matrix T = (Matrix.identity(mat_Pij.getColumnDimension(), mat_Pij.getRowDimension())).minus(mat_Pij.transpose());
        mat_Vi = T.inverse().times(mat_Q0);
        //System.out.println ( "S=" + i_ServiceIndex + "EX[i]" + PrintMatrix(E) );
        //System.out.println ( "Expected Visits in Service"+ i_ServiceID+" =" + mat_Vi.toString() );
    }

    //Returns the statistics object for the attribute

}
