package archeopterix.evaluation;

import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.royalroad.solution.RoyalRoadSolution;
import archeopterix.solution.Architecture;
import java.util.LinkedList;

public class RoyalRoadEvaluator extends ArchitectureEvaluator {
	public RoyalRoadEvaluator() {
		super("RoyalRoadEvaluator", false);
	}

    public RoyalRoadEvaluator(String sName) {
        super(sName, false);

    }

	public double evaluate(EmbeddedSystemProblem oProblem, Architecture oArch) {

		RoyalRoadSolution royalSol = (RoyalRoadSolution)oArch;
		int blockCount = royalSol.blockCount;
		int blockSize = royalSol.blockSize;
		double partValue = royalSol.partValue;
		double partFitness = royalSol.partFitness;
		int levels = royalSol.levels;
		int regionSize = royalSol.regionSize;
		double bonusStartValue = royalSol.bonusStartValue;
		double bonusValue = royalSol.bonusValue;

		int[] blockComplete = new int[blockCount];

		// PART calculation
		double part = 0;
		for(int i = 0; i < blockCount;i++)
		{
			int currBlockCount = getBitCount(i*regionSize, i*regionSize+blockSize, royalSol.getLstAlloc());
			
			if(currBlockCount==blockSize) blockComplete[i]=1;
			else if(currBlockCount<=partFitness) part = part + partValue*currBlockCount;
			else part = part - partValue*(currBlockCount-partFitness);
		}

		// BONUS calculation
		double bonus = 0;
		for(int i=0;i<=levels;i++)
		{
			int levelCompleted = 0;
			int levelSize = (int) Math.pow(2,i);
			for(int j=0;j<blockCount;j += levelSize)
			{
				boolean completed = true;
				for(int k=j;k<j+levelSize;k++)
					if(blockComplete[k]==0)
						{
							completed = false;
							break;
						}
				if(completed) levelCompleted++;
			}
			if(levelCompleted>0) bonus += bonusStartValue + (levelCompleted-1) * bonusValue;
		}

		return (part + bonus);
	}
	public int getBitCount(int start, int end, LinkedList<Integer> lstAlloc)
	{
		int count = 0;
		for(int i=start ; i < end; i++) if(lstAlloc.get(i)==1) count++;
		return count;
	}
}
