package archeopterix.evaluation;

import java.util.List;

import archeopterix.problem.Component;
import archeopterix.problem.Host;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.deployment.solution.DeploymentSolution;
import archeopterix.solution.Architecture;


public class SchedulingLengthEvaluator extends ArchitectureEvaluator {
	public SchedulingLengthEvaluator() {
		super("SchedulingLength", true);
	}

        public SchedulingLengthEvaluator(String sName) {
        super(sName, true);

    }


	@Override
	public double evaluate(EmbeddedSystemProblem oProblem, Architecture oArch) {
		double dAvgSL = 0.0;
		if (oArch instanceof DeploymentSolution) {
			DeploymentSolution oDep = (DeploymentSolution)oArch;
                        List<Host> lstECUs = oProblem.getECUs();
                        double dSumSL = 0.0;
                        for (Host oECU : lstECUs){
                            int iMIPS = oECU.getIntProperty(Host.SPEED_MIPS);
                            List<Component> lstOtherComps = oDep.getComponentListForHost(oECU.index, oProblem);
                            double dMI = 0.0;
                            for(Component oComp : lstOtherComps){
                                Object oComMI = oComp.getPropertyMap().get(Component.COMPUTATIONS_MI);
                                if (oComMI instanceof List)
                                {
                                    List<Double> lstMIs = (List<Double>)oComMI;//oComp.getDoubleListProperty(Component.COMPUTATIONS_MI);
                                    for(Double dSrvMI : lstMIs){
                                        dMI += dSrvMI;
                                    }
                                }
                                else
                                    dMI += (Double) (oComMI == null ? 0.0 : oComMI);// oComp.getDoubleProperty(Component.COMPUTATIONS_MI);
                            }
                            dSumSL += (dMI / (double) iMIPS);
                        }
                        dAvgSL = dSumSL/(double) oProblem.getECUCount();
		}
		else{
                    dAvgSL = 0;
		}
		return dAvgSL;
	}

}