/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package archeopterix.gui;

import java.awt.Font;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.xy.IntervalXYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.ApplicationFrame;
import org.jfree.chart.renderer.xy.*;
import org.jfree.chart.ChartUtilities;

import java.util.*;
import java.io.File;
import java.io.IOException;
import org.jfree.data.UnknownKeyException;
import org.jfree.chart.annotations.XYTextAnnotation;
import archeopterix.util.probability.Distribution;
import archeopterix.util.probability.TriangularDistribution;
import archeopterix.util.statistics.NormalDistribution;
import java.io.FileWriter;
import java.io.PrintWriter;
import org.jfree.data.xy.XYDataItem;

public class ChartViewer{

    private static final long serialVersionUID = 1L;
    public static final int CHART_TYPE_XY_LINE = 0;
    public static final int CHART_TYPE_XY_AREA = 1;
    public static final int CHART_TYPE_XY_SCATTER = 2;

    private static ChartViewer m_Ins =null;

    private boolean b_ShowDataPoints = false;
    private boolean b_ShowDataLabels = false;
    private int i_Width = 750;
    private int i_Height = 500;
    private int i_ChartType = CHART_TYPE_XY_LINE;
    private String s_ChartType = "linespoints"; //For Gnuplot
    private String s_Title ="";
    private XYSeriesCollection m_XYSeriesCol = null;
    private JFreeChart m_Chart = null;
    private ApplicationFrame m_Frame = null;
    


    public static ChartViewer Ins(){
        if (m_Ins == null)
            m_Ins = new ChartViewer("", CHART_TYPE_XY_LINE, false, false);
        return m_Ins;
    }

    public ChartViewer(String sTitle, int iChartType, boolean bShowDataPoints, boolean bShowDataLabels) {
        s_Title = sTitle;
        i_ChartType = iChartType;
        b_ShowDataPoints = bShowDataPoints;
        b_ShowDataLabels = bShowDataLabels;
        m_XYSeriesCol = new XYSeriesCollection();
        m_Chart = createChart(m_XYSeriesCol);
		m_Chart.setTitle(sTitle);
    }

    public void View(){
        if (m_Frame == null){
            m_Frame = new ApplicationFrame(s_Title);
            final ChartPanel chartPanel = new ChartPanel(m_Chart);
            chartPanel.setPreferredSize(new java.awt.Dimension(i_Width, i_Height));
            m_Frame.setContentPane(chartPanel);
        }
        //IntervalXYDataset oDataset = m_XYSeriesCol;
        //JFreeChart oChart = createChart(oDataset);
        m_Frame.pack();
        m_Frame.setVisible(true);
    }

    public void SetData(String sSeriesID, Map<Double,Double> mapData){
        //Reset();
        Iterator it = mapData.entrySet().iterator();
        int i=0;
        while (it.hasNext()) {
            Map.Entry pairs = (Map.Entry)it.next();
            //System.out.println(pairs.getKey() + " = " + pairs.getValue());
            UpdateXY(sSeriesID, String.valueOf(i), ((Double)pairs.getKey()).doubleValue(), ((Double)pairs.getValue()).doubleValue());
            i++;
        }
    }

    //Temporary function
    public void SetOtherDistributions(Map<Double,Double> mapData, double dMean, double dVar){
        NormalDistribution oNorm = new NormalDistribution(dMean, dVar);

        double dAlpha = dMean * ( dMean - dMean*dMean - dVar) / dVar;
        double dBeta = (1- dMean)* ( dMean - dMean*dMean - dVar) / dVar;
        //System.out.println("Alpha=" + dAlpha + " Beta =" + dBeta);
        //BetaDistribution oBeta = Utils.getBestfitBetaDist(dMean, dVar);
        TriangularDistribution oTri = Distribution.getBestfitTriangleDist(dMean, dVar);
        Distribution.plotPDF(oTri, "Triangular", oTri.getMin(), oTri.getMax(), 100, null);
        Iterator it = mapData.entrySet().iterator();
        int i=0;
        while (it.hasNext()) {
            Map.Entry pairs = (Map.Entry)it.next();
            double dKey = ((Double)pairs.getKey()).doubleValue();
            double dVal = ((Double)pairs.getValue()).doubleValue();
            //System.out.println(pairs.getKey() + " = " + pairs.getValue());
            UpdateXY("Normal", String.valueOf(i),dKey, oNorm.probability(dKey) );
            UpdateXY("Tri", String.valueOf(i),dKey, oTri.probability(dKey) );
            i++;
        }
    }

    public void UpdateXY(String sSeriesID, String sItemID,  double dX, double dY){
        XYSeries oSeries = null;
        try{
            oSeries = m_XYSeriesCol.getSeries(sSeriesID);
        }
        catch (UnknownKeyException e){
            oSeries = new XYSeries(sSeriesID);
            m_XYSeriesCol.addSeries(oSeries);
        }
        oSeries.add(dX, dY);
        if (b_ShowDataLabels){
            final XYTextAnnotation annotation = new XYTextAnnotation(sItemID, dX,dY);
            annotation.setFont(new Font("SansSerif", Font.PLAIN, 10));
            ((XYPlot)m_Chart.getPlot()).addAnnotation(annotation);
        }

    }

    public void AddText(double dXratio, double dYratio, String sText){
        double dX = m_XYSeriesCol.getDomainLowerBound(false) + dXratio * ( m_XYSeriesCol.getDomainUpperBound(false) -m_XYSeriesCol.getDomainLowerBound(false));
        double dY = m_XYSeriesCol.getRangeLowerBound(false) + dYratio * ( m_XYSeriesCol.getRangeUpperBound(false) -m_XYSeriesCol.getRangeLowerBound(false));
        final XYTextAnnotation annotation = new XYTextAnnotation(sText, dX,dY);
        annotation.setFont(new Font("SansSerif", Font.PLAIN, 12));
        ((XYPlot)m_Chart.getPlot()).addAnnotation(annotation);
    }

    public void Reset(){
        m_XYSeriesCol.removeAllSeries();
        
    }

    /**
     * Save as JPEG and EPS Formats
     * @param sFileName
     */
    public void saveChart(String sPath, String sPlotName){
        try {
            ChartUtilities.saveChartAsJPEG(new File(sPath +"/"+ sPlotName + ".jpg"), m_Chart, i_Width, i_Height);
        } catch (IOException e) {
            System.err.println("Problem occurred in saving chart:"+ sPlotName + ".jpg");
        }
        createGNUPlotFiles(sPath, sPlotName);
    }

    /**
     * Save the chart series data into a file and creates gnuplot script.
     * Can use following shell script to plot them all
     *  for iFile in *.plt;
     *    do
     *      gnuplot $iFile;
     *    done
     * @param sPlotName
     */
    public boolean createGNUPlotFiles(String sPath, String sPlotName) {
        boolean success = true;
        PrintWriter outData = null; //For Data File
        PrintWriter outCmd = null; // For gnuplot Commands
        double dXMin= Double.MAX_VALUE;
        double dYMin = Double.MAX_VALUE;
        double dXMax = Double.MIN_VALUE;
        double dYMax = Double.MIN_VALUE;
        try {
            outData = new PrintWriter(new FileWriter(sPath + "/" + sPlotName + ".dat"));
            outCmd = new PrintWriter(new FileWriter(sPath + "/" + sPlotName + ".plt"));
            
            int iDataSets = m_XYSeriesCol.getSeriesCount();
            String sPS = ""; //Plot Specifier
            for (int iDS = 0; iDS < iDataSets; iDS++) {
                XYSeries oSeries = m_XYSeriesCol.getSeries(iDS);
                if (iDS > 0)
                    sPS += (",\\\n\t");

                int iItems = oSeries.getItemCount();
                for (int iItem = 0; iItem < iItems; iItem++) {
                    XYDataItem oItem = oSeries.getDataItem(iItem);
                    double dX = oItem.getXValue();
                    double dY = oItem.getYValue();
                    dXMax = Math.max(dXMax, dX);
                    dXMin = Math.min(dXMin, dX);
                    dYMax = Math.max(dYMax, dY);
                    dYMin = Math.min(dYMin, dY);
                    outData.println( dX+ "\t" + dY);
                }
                sPS += "'" + sPlotName + ".dat' index " + iDS + " with " + s_ChartType + " ls " +
                        (iDS+1) + " title '"+  m_XYSeriesCol.getSeriesKey(iDS) + "'";
                outData.println("\n\n");
            }
            double dXGutter = 0.1 * (dXMax-dXMin);
            double dYGutter = 0.1 * (dYMax - dYMin);
            outCmd.println("# Title" + s_Title);
            outCmd.println("set terminal postscript eps enhanced");
            outCmd.println("set size 1,1");
            outCmd.println("set output '" + sPlotName + ".eps'");
            outCmd.println("set xrange [" + (dXMin- dXGutter)  + ":" + (dXMax+ dXGutter)+ "]");
            outCmd.println("set yrange [" + (dYMin- dYGutter)  + ":" + (dYMax+ dYGutter)+ "]");
            outCmd.println("set style line 1 lt 1 pt 7 lc rgb \"black\" lw 1"); //2
            outCmd.println("set style line 2 lt 3 pt 3 lc rgb \"black\" lw 1"); //2
            outCmd.println("set style line 3 lt 0 pt 7 lc rgb \"black\" lw 1"); //4
            outCmd.println("set style line 4 lt 1 pt 4 lc rgb \"black\" lw 1"); //4
            outCmd.println("plot " + sPS) ;
            System.out.println("Gnuplot data,plot files created ! :" + sPlotName);
            /*try{
                Runtime.getRuntime().exec("gnuplot ./" + sPlotName + ".plt");
            }catch (RuntimeException re){
                System.err.println("Could not Execute gnuplot from java for :" + sPlotName);
            }*/
        } catch (IOException e) {
            System.err.println("Error writing gnuplot data/plot files :" + sPlotName);
            success = false;
        } finally {
            if (outData != null) outData.close();
            if (outCmd != null) outCmd.close();
        }
        return success;
    }

    private JFreeChart createChart(IntervalXYDataset dataset) {
        JFreeChart chart = null;
        switch(i_ChartType){
            case CHART_TYPE_XY_AREA:
                chart = ChartFactory.createXYAreaChart(
                        null, //Title
                        null, // X title
                        null, // Ytitle
                        dataset,
                        PlotOrientation.VERTICAL,
                        true, // Legand
                        true,
                        false
                        );
                b_ShowDataPoints = false;// Does Not Suppport.
                s_ChartType = "linespoints"; //TODO FIX LATER
                break;
            case CHART_TYPE_XY_LINE:
                chart = ChartFactory.createXYLineChart(
                        null, //Title
                        null, // X title
                        null, // Ytitle
                        dataset,
                        PlotOrientation.VERTICAL,
                        true, // Legand
                        true,
                        false
                        );
                s_ChartType = "linespoints";
                break;
            case CHART_TYPE_XY_SCATTER:
                chart = ChartFactory.createScatterPlot(
                    null, //Title
                    null, // X title
                    null, // Ytitle
                    dataset,
                    PlotOrientation.VERTICAL,
                    true, // Legand
                    true,
                    false
                    );
                s_ChartType = "points";
                break;
        }
        //final JFreeChart chart = ChartFactory.createScatterPlot(
         
        //final JFreeChart chart = ChartFactory.createXYAreaChart(
            

        if (b_ShowDataPoints){
            XYPlot oplot = (XYPlot) chart.getPlot();
            XYLineAndShapeRenderer orr = (XYLineAndShapeRenderer) oplot.getRenderer() ;
            orr.setShapesVisible(true);
        }
        return chart;
    }

}
