package archeopterix.gui;

import archeopterix.evaluation.Attribute;
import archeopterix.solution.Architecture;

import java.awt.Color;  
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.util.List;
import javax.swing.JFrame;
import javax.swing.JTabbedPane;
import org.jfree.chart.ChartPanel; 
import org.jfree.chart.JFreeChart; 
import org.jfree.chart.axis.NumberAxis; 
import org.jfree.chart.plot.XYPlot; 
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer; 
import org.jfree.data.xy.*; 

/**  * A demo scatter plot.  */ 

public class ChartWindow implements ChartUpdateListener{

        /**
         * A demonstration application showing a scatter plot.
         *
         * @param title  the frame title.
         */
           
        XYSeriesCollection hypervolumeDataset;
	DefaultXYDataset objectiveSpaceDataset;
        XYSeries hypervolSeries;

	public ChartWindow()
	{
            
		hypervolumeDataset = new XYSeriesCollection();

                hypervolSeries = new XYSeries("Hypervolume", false, true);
                hypervolumeDataset.addSeries(hypervolSeries);
		objectiveSpaceDataset = new DefaultXYDataset();
		/*		dataset.addSeries("Red circles", new double[][]
				{
				{ 1, 2, 3 },
				{ 2, 4, 1 } });
		*/NumberAxis xAxis = new NumberAxis("iterations");
		NumberAxis yAxis = new NumberAxis("hypervolume");
		NumberAxis objectiveXAxis = new NumberAxis("x");
		NumberAxis objectiveYAxis = new NumberAxis("y");
                objectiveYAxis.setAutoRangeIncludesZero(false);
	        objectiveYAxis.setTickMarkInsideLength(2.0f);
	        objectiveYAxis.setTickMarkOutsideLength(0.0f);

                objectiveXAxis.setAutoRangeIncludesZero(false);
                objectiveXAxis.setTickMarkInsideLength(2.0f);
	        objectiveXAxis.setTickMarkOutsideLength(0.0f);
		XYItemRenderer renderer = new XYLineAndShapeRenderer(false, true);
		XYItemRenderer objectiveRenderer = new XYLineAndShapeRenderer(false,
				true);
		renderer.setSeriesPaint(0, Color.BLUE);
		objectiveRenderer.setSeriesPaint(0, Color.BLUE);
                objectiveRenderer.setSeriesShape(0, new Rectangle2D.Double(-2.5, -2.5,
				5, 5));
                objectiveRenderer.setSeriesPaint(1, Color.GREEN);
                objectiveRenderer.setSeriesShape(1, new Rectangle2D.Double(-2.5, -2.5,
				5, 5));
                objectiveRenderer.setSeriesPaint(2, Color.BLACK);
                objectiveRenderer.setSeriesShape(2, new Rectangle2D.Double(-2.5, -2.5,
				5, 5));
                objectiveRenderer.setSeriesPaint(3, Color.RED);
                objectiveRenderer.setSeriesShape(3, new Rectangle2D.Double(-2.5, -2.5,
				5, 5));
                objectiveRenderer.setSeriesPaint(4, Color.YELLOW);
                objectiveRenderer.setSeriesShape(4, new Rectangle2D.Double(-2.5, -2.5,
				5, 5));
		// renderer.setSeriesPaint(2, Color.GREEN);

		renderer.setSeriesShape(0, new Ellipse2D.Double(-2.5, -2.5, 5, 5));
                renderer.setSeriesOutlinePaint(0, Color.black);
                renderer.setSeriesOutlinePaint(1, Color.black);
		
                objectiveRenderer.setSeriesOutlinePaint(0, Color.black);
                objectiveRenderer.setSeriesOutlinePaint(1, Color.black);
		// renderer.setSeriesShape(2, new Rectangle2D.Double(-7, -3, 14, 6));
		XYPlot objectivePlot = new XYPlot(objectiveSpaceDataset,
				objectiveXAxis, objectiveYAxis, objectiveRenderer);
		XYPlot plot = new XYPlot(hypervolumeDataset, xAxis, yAxis, renderer);
                plot.setNoDataMessage("NO DATA YET");
	        plot.setDomainZeroBaselineVisible(true);
	        plot.setRangeZeroBaselineVisible(true);

                //NumberAxis domainAxis = (NumberAxis) plot.getDomainAxis();


	        //NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();

		JFreeChart objectiveChart = new JFreeChart(
				"Objective Space Distribution", new Font("Tahoma", 2, 18),
				objectivePlot, true);

		JFreeChart chart = new JFreeChart("Hypervolume Development", new Font(
				"Tahoma", 2, 18), plot, true);

		JFrame frame = new JFrame("Deployment Plotting");
		// frame.setContentPane(new ChartPanel(chart));

		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.addTab("Objective Space", null, new ChartPanel(
				objectiveChart),
				"Shows the distribution in the objective space.");
		tabbedPane.addTab("Hypervolume Development", null, new ChartPanel(chart),
				"How the hypervolume develops over time.");
		frame.setContentPane(tabbedPane);

		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		int windowWidth = Toolkit.getDefaultToolkit().getScreenSize().width - 20;
		int windowHeight = Toolkit.getDefaultToolkit().getScreenSize().height - 50;

		frame.setPreferredSize(new Dimension(windowWidth, windowHeight));

		frame.pack();
		frame.setVisible(true);
	}

	    
            public void addNewSeries(String sText,  List<Architecture> lst_sol)
            {
                double[][] data = new double[2][lst_sol.size()];
                for (int i = 0; i < lst_sol.size(); i++)
                {
                    List<Attribute> att = lst_sol.get(i).getAttributeList();
                    data[0][i] = att.get(att.size()-2).getValue();
                    data[1][i] = att.get(att.size()-1).getValue();
                }
               
               objectiveSpaceDataset.addSeries(sText, data);
               
            }

            public void addPointToHypervolume(int evaluation, double hypervolume)
            {
                hypervolSeries.add(evaluation, hypervolume);
            }
	    
	}
