package archeopterix.problem;


import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import archeopterix.util.PropertyMap;

public class Component extends ObjectWithProperties {
	public static final String LABEL = "Component";
	//Properties of Components; if will be the same for all problems should use an enum
	public static final String COLOC_LIST = "CoLocalisationList";
	public static final String COMPUTATIONS_MI = "ComputationsInMI";
	public static final String COST = "Cost";
	public static final String ENERGY_PER_VISIT = "EnergyPerVisit";
	public static final String EXPECTED_VISITS = "Vi";
	public static final String FAILURE_RATE = "FailureRate";
	public static final String IDLE_ENERGY = "IdleEnergy";
	public static final String INITILIZE_PROB = "InitializeProbability";
        public static String INITILIZE_PROB_DT	= "InitializeProbabilityDT";
        public static final String MEMORY = "Mem";
	public static final String GROUP_MEMORY = "MemIncludingColocated";
	public static final String SOJOURN_TIME = "SojournTime";
	public static final String OPTION_SOJOURN_TIME = "OptionSojournTime";
	public static final String OPTION_SOJOURN_TIME_DT = "OptionSojournTimeDT";
        public static final String OPTION_RELIABILITY = "OptionReliability";
        public static final String OPTION_RELIABILITY_DT = "OptionReliabilityDT";
	public final int index;
	private ArrayList<Component> mustCoLocate;
	//TODO Also store these as a BitSet to make checking for conflicts faster
	private ArrayList<Component> mustNotCoLocate;
	private ArrayList<Host> canUseHost;

	public Component(String name, int index, PropertyMap propertyMap) {
		super(name, propertyMap);
		mustCoLocate = new ArrayList<Component>();
		mustNotCoLocate = new ArrayList<Component>();
		canUseHost = new ArrayList<Host>();
		this.index = index;
	}

    @Override
	public String toString() {
		return "(" + name + ", properties " + propertyMap + ")";
	}
        public int getNumberOptions(){return canUseHost.size();}
	public void addMustCoLocate(Component other) { mustCoLocate.add(other); }

	public void addMustNotCoLocate(Component other) { mustNotCoLocate.add(other); }

	public List<Component> getMustColocateWith() { return mustCoLocate; }

	public List<Component> getMustNotColocateWith() { return mustNotCoLocate; }
	
	public List<Host> getCanUseHosts(){return canUseHost;}

	public void addCanUseHost(Host ecu) { canUseHost.add(ecu); }

	public int getCanUseHostListLength() { return canUseHost.size(); }

	public Host getHostOption(int o) { return canUseHost.get(o); }

	public boolean isCompatibleHost(Host host) { return canUseHost.contains(host); }

	public boolean mustColocate() { return mustCoLocate.size() != 0; }

	public boolean canColocateWith(Component other) { return !mustNotCoLocate.contains(other); }

	public boolean conflicts(List<Component> comps){
		boolean okToAssign = true;
		//check if components in the same group have colocate restrictions with other comp.
		Iterator<Component> mustNotColocateWith = this.getMustNotColocateWith().iterator();
		while (mustNotColocateWith.hasNext() && okToAssign)
			okToAssign = ! comps.contains(mustNotColocateWith.next());

		return !okToAssign;
	}

}