package archeopterix.problem;

import java.io.Serializable;
import java.util.Comparator;

/**
 * Compares {@link Component}s based first on the number of alternative ECUs to
 * which they may be assigned, with fewer options appearing before more
 * options; if equal then {@code Component}s are further compared based on
 * their size (including that of any other components with which they must be
 * colocated), with larger size appearing before smaller size.
 * 
 * @author jmontgomery
 */
public class ComponentOptionCountAndSizeComparator implements Comparator<Component>, Serializable
{
	private static final long serialVersionUID = 1L;

    public int compare(Component c1, Component c2) {
		//Ascending order for ECU option count
		if ( c1.getCanUseHostListLength() < c2.getCanUseHostListLength() )
			return -1;
		else if (c1.getCanUseHostListLength() > c2.getCanUseHostListLength())
			return 1;
		else { //Descending order for memory requirement
			int c1Size = c1.getIntProperty( Component.GROUP_MEMORY );
			int c2Size = c2.getIntProperty( Component.GROUP_MEMORY );
			if (c1Size > c2Size)
				return -1;
			else if (c1Size < c2Size)
				return 1;
			else return 0;
		}
	}

}

