package archeopterix.problem;

import archeopterix.algorithm.Stats;
import archeopterix.conf.Conf;
import archeopterix.constraints.ConstraintHandler;
import archeopterix.evaluation.ArchitectureEvaluator;
import archeopterix.evaluation.Attribute;
import archeopterix.solution.Architecture;
import archeopterix.util.PropertyMap;
import java.io.Serializable;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

    public class EmbeddedSystemProblem implements Serializable{

	private static final Log log = LogFactory.getLog(EmbeddedSystemProblem.class);
	protected int i_Comps;
	protected int i_ECUs;
	protected List<Component> lst_Comps;
	protected List<Component> unmodifiableComps;
	protected List<Host> lst_ECUs;
	protected List<Host> unmodifiableECUs;
	protected List<Service> lst_Services;       //FIXME : Initialization and populate values
	protected List<List<PropertyMap>> m_NetworkMatrix;
	protected List<List<PropertyMap>> m_InteractionsMatrix;

	private double maxMemConstraint;
	private ComponentOptionCountComparator componentOptionComparator = new ComponentOptionCountComparator();
	private ComponentOptionCountAndSizeComparator componentOptionAndSizeComparator = new ComponentOptionCountAndSizeComparator();
	private List<Component> lst_compConstrainedAssignments;
	private List<Component> lst_compRefinedConstrainedAssignments;
	private List<ConstraintHandler> lst_Constraints = new ArrayList<ConstraintHandler>();
	
	//FIXME Temporarily caching conf instance
	private Conf conf;
        
        public List<ConstraintHandler> getListConstraints(){
            return lst_Constraints;
        }

	//***************************************************************************************
	private void initCommonObjects() {
		//FIXME Should maximum values be taken as parameters or are they known by each evaluator
		//Note Evaluators are are instantiated by Conf
//		m_RelEvaluator = new ReliabilityEvaluator();
//		m_CostEvaluator = new CostEvaluator();
//		m_ComOHEvaluator = new ComOHEvaluator();
//		m_DataComRelEvaluator = new DataComRelEvaluator();
//		m_ResponseTimeEvaluator = new ResponseTimeEvaluator();
//		arr_SRelEvaluators	= new ServiceReliabilityEvaluator[Conf.instance().i_Services];
		//For Service Reliability evaluation
		/*int services = 0;
		for (ArchitectureEvaluator evaluator : Conf.instance().getEvaluators())
		if (evaluator instanceof ServiceReliabilityEvaluator)
		services++;
		if ( services > 0 ) {
		amat_Q0 = new Matrix[services];
		amat_Vi = new Matrix[services];
		amat_Pij = new Matrix[services];
		for (int i = 0; i < services; i++){
		//				arr_SRelEvaluators[i] = new ServiceReliabilityEvaluator( "Srv"+i , i, 1);
		amat_Q0[i] = new Matrix (i_Comps, 1);
		amat_Pij[i] = new Matrix (i_Comps, i_Comps);
		for (int iCompi= 0; iCompi < i_Comps; iCompi++){
		amat_Q0[i].set(iCompi, 0, lst_Comps.get(iCompi).getDoubleFromListProperty( Component.INITILIZE_PROB, i ) );
		for ( int iCompj = 0; iCompj < i_Comps; iCompj++) {
		amat_Pij[i].set(iCompi, iCompj, getInteractionProperties(iCompi, iCompj).getDoubleFromList( InteractionProperties.TRANSITION_PROB, i ) );
		}
		}
		//Expected Number of Visits for Components
		Matrix T =  ( Matrix.identity( amat_Pij[i].getColumnDimension(), amat_Pij[i].getRowDimension() )).minus( amat_Pij[i].transpose() );
		amat_Vi[i] = T.inverse().times(amat_Q0[i]);
		//System.out.println ( "S=" + i_ServiceIndex + "EX[i]" + PrintMatrix(E) );
		System.out.println ( "Vi[" + i + "] =" + amat_Vi[i].toString() );
		}
		}
		else{
		//This is a tempry hack.. :) //TODO
		amat_Q0 = new Matrix[1];
		amat_Vi = new Matrix[1];
		amat_Pij = new Matrix[1];
		//arr_SRelEvaluators[i] = new ServiceReliabilityEvaluator( "Srv"+i , i);
		amat_Q0[0] = new Matrix (i_Comps, 1);
		amat_Pij[0] = new Matrix (i_Comps, i_Comps);
		for (int iCompi= 0; iCompi < i_Comps; iCompi++){
		amat_Q0[0].set(iCompi, 0, lst_Comps.get(iCompi).getDoubleProperty( Component.INITILIZE_PROB ) );
		for ( int iCompj = 0; iCompj < i_Comps; iCompj++){
		if ( getInteractionProperties(iCompi, iCompj) != null )
		amat_Pij[0].set(iCompi, iCompj, getInteractionProperties(iCompi, iCompj).getDouble( InteractionProperties.TRANSITION_PROB ) );
		else
		amat_Pij[0].set(iCompi, iCompj, 0);
		}
		}
		//Expected Number of Visits for Components
		Matrix T =  ( Matrix.identity( amat_Pij[0].getColumnDimension(), amat_Pij[0].getRowDimension() )).minus( amat_Pij[0].transpose() );
		amat_Vi[0] = T.inverse().times(amat_Q0[0]);
		//System.out.println ( "S=" + i_ServiceIndex + "EX[i]" + PrintMatrix(E) );
		System.out.println ( "Vi[0] =" + amat_Vi[0].toString() );
		}*/
	}

	/**
	 * This method enables the other classes (like algorithms) to enable/disable constraint checks.
	 * @param constraintClass Class of the constrainthandler
	 * @param bEnabled Enable/disable the constraint check
	 */
	public void setConstraintCheck(Class<?> constraintClass, boolean bEnabled) {
            for (int i = 0; i < lst_Constraints.size(); i++) {
                ConstraintHandler oConst = lst_Constraints.get(i);
                if (oConst.getClass() == constraintClass) {
                    oConst.setEnabled(bEnabled);
                }
            }
	}

	/**
	 * Add new constraint
	 * @param oConst
	 */
	public void addConstraint(ConstraintHandler oConst) {
            lst_Constraints.add(oConst);
	}
	public EmbeddedSystemProblem(List<Host> oHostList, List<Component> oCompList, List<List<PropertyMap>> matNetwork, List<List<PropertyMap>> matInteractions) {
            lst_ECUs = oHostList;
            lst_Comps = oCompList;
            unmodifiableComps = Collections.unmodifiableList(lst_Comps);
            unmodifiableECUs = Collections.unmodifiableList(lst_ECUs);

            m_NetworkMatrix = matNetwork;
            m_InteractionsMatrix = matInteractions;

            //Populating i_index variables
            i_ECUs = lst_ECUs.size();
            i_Comps = lst_Comps.size();
            //s_Result	= new String();
            //FIX ME TEMP
            lst_Services = new Vector<Service>();
            lst_Services.add(new Service(0, 3.6E3)); //per hour (30 per second)
            initCommonObjects();
            //preprocess();
            preprocessComponents();
            calculateMaxMemoryConstraint();
            realityCheck();
            conf = Conf.instance();
	}
	public List<Component> getComponents() {
            return unmodifiableComps;
	}

	public int getComponentCount() {
            return lst_Comps.size();
	}

	public List<Host> getECUs() {
            return unmodifiableECUs;
	}

	public int getECUCount() {
            return lst_ECUs.size();
	}

	public Component getComponent(int iCompIndex) {
            return lst_Comps.get(iCompIndex);
	}

	public Host getECU(int iECUInex) {
            return lst_ECUs.get(iECUInex);
	}
	//FIXME Probably want to provide an unmodifiable view of this list

	public List<List<PropertyMap>> getNetworkMatrix() {
            return m_NetworkMatrix;
	}

	//FIXME Probably want to provide an unmodifiable view of this list
	public List<List<PropertyMap>> getInteractionsMatrix() {
            return m_InteractionsMatrix;
	}
	public void calculateMaxMemoryConstraint() {
            maxMemConstraint = 0;
            for (int i = 0; i < lst_Comps.size(); i++) {
                maxMemConstraint += lst_Comps.get(i).getIntProperty(Component.MEMORY);
            }
	}

	public double getMaxMemoryConstraint() {
            return maxMemConstraint;
	}

	//******************************************************************************
	//FIXME Creation of attributes should probably not be done here; they could be created with solutions
	public double evaluateAttributes(Architecture dep) {
            long lInTime = System.currentTimeMillis();
            Stats.instance().incrementEvaluationCounter();

            if (dep.getAttributeList() == null) {
                List<ArchitectureEvaluator> evaluators = conf.getEvaluators();
                List<Attribute> lstAttributes = new Vector<Attribute>(evaluators.size());
                for (ArchitectureEvaluator evaluator : evaluators) {
                        lstAttributes.add(new Attribute(evaluator));
                }
                dep.setAttributeList(lstAttributes);
            }
            double dFitness = 0.0;
            List<Attribute> lstAttributes = dep.getAttributeList();
            for (int j = 0; j < lstAttributes.size(); j++) {
                Attribute oAtt = lstAttributes.get(j);
                dFitness += oAtt.evaluate(this, dep);
                //FIXME This is where the EmbeddedSystemProblem had asked the Algorithm to update its maximum vales; this functionality will need to go in the relevant Algorithm
        }
            dep.d_Fitness = dFitness;
            Stats.instance().l_EvalTime += (System.currentTimeMillis() - lInTime);
            return dFitness;
	}

	public void evaluateAttributes(List<Architecture> population) {
            for (Architecture oArch : population) {
                evaluateAttributes(oArch);
            }
	}
	//FIXME These numerous convenience methods may not be required; any callers can already retrieve the relevant component, Host or pairwise property map and call the appropriate methods on those.

	public PropertyMap getNetworkProperties(int ci, int cj) {
		return m_NetworkMatrix.get(ci).get(cj);
	}

	public boolean areInteracting( int ci, int cj) {
            if(m_InteractionsMatrix.get(ci).get(cj) != null)
		return true;
            else{
                return false;
            }
	}

	public boolean areConnected(int Hi, int Hj) {
		return m_NetworkMatrix.get(Hi).get(Hj) != null;
	}

	public PropertyMap getInteractionProperties(int ci, int cj) {
		return m_InteractionsMatrix.get(ci).get(cj);
	}
	public boolean validate(Architecture arch) {
            for (ConstraintHandler constraint : lst_Constraints) {
                if (!constraint.checkConstraint(arch)) {
                    return false;
                }
            }
            return true;
	}
        
	/**
	 * Allows a specific constraint to be tested. Currently potentially
	 * inefficient if many constraints being enforced as must iterate over
	 * entire list of constraints until appropriate one found. Returns
	 * {@code true} if the solution respects the constraint or if the
	 * specified constraint isn't enabled (isn't present), {@code false}
	 * otherwise.
	 * <p>
	 * Added by EJM 2010-09-07 to support NSGA's use of specific constraints.
	 * @return {@code true} if solution respects the given constraint or the
	 * constraint is not being enforced, {@code false} otherwise. 
	 */
	public boolean validate(Architecture arch, Class<?> constraintClass) {
            for (ConstraintHandler constraint : lst_Constraints) {
                if (constraint.getClass() == constraintClass) //comparison is safe as only one instance of each Class object created
                {
                    return constraint.checkConstraint(arch);
                }
            }
            return true;
	}

	private void preprocessComponents() {
            lst_compConstrainedAssignments = new ArrayList<Component>();
            lst_compRefinedConstrainedAssignments = new ArrayList<Component>();

            //Those components that are part of any group
            Set<Component> groupMembers = new HashSet<Component>();
            int[] totalMemRequired = new int[i_Comps];

            for (int iCompi = 0; iCompi < i_Comps; iCompi++) {
                Component oCompi = lst_Comps.get(iCompi);
                totalMemRequired[iCompi] += oCompi.getIntProperty( Component.MEMORY );
                //add to separate list for sorting
                lst_compConstrainedAssignments.add(oCompi);

                List<Integer> lstColoc = (List<Integer>) oCompi.getIntListProperty(Component.COLOC_LIST);
                for (int iCompj = 0; iCompj < i_Comps; iCompj++) { //starting Changed to iCompi
                    int iCoLoc = lstColoc.get(iCompj);
                    if (iCoLoc == -1) {
                        oCompi.addMustNotCoLocate(lst_Comps.get(iCompj));
                    } else if (iCoLoc == 1) {
                        oCompi.addMustCoLocate(lst_Comps.get(iCompj));
                        groupMembers.add( lst_Comps.get(iCompj) );
                        //Duplication of calculation but saves time doing separate iteration over the list of components
                        totalMemRequired[iCompi] += lst_Comps.get(iCompj).getIntProperty( Component.MEMORY );
                    }
                }

                oCompi.getPropertyMap().put( Component.GROUP_MEMORY, totalMemRequired[iCompi] );
            }

            //every component now has a list of incompatible components.
            // among groups, these can differ.
            //now we have to combine the lists of the group members for easier checks.
		
            for (Component currentComp : groupMembers) {
                //System.out.println();
                //System.out.println("Group member: "+currentComp.i_index+" hates ");
                List<Component> buddies = currentComp.getMustColocateWith();

                //System.out.println();
                //System.out.print("; his buddies also hate: ");
                for (Component buddy : buddies) {
                    List<Component> buddysEnemies = buddy.getMustNotColocateWith();
                    for (Component mutualEnemy : buddysEnemies) {
                        //means it doesn't appear in the enemies' list yet
                        if (currentComp.canColocateWith(mutualEnemy)) {
                            currentComp.addMustNotCoLocate(mutualEnemy);
                            // System.out.print(mutualEnemy.i_index+", ");
                        }
                    }
                    // System.out.flush();
                }
            }

            //fill list of possible ECUs for the component
            for (int iHost = 0; iHost < i_ECUs; iHost++) {
                Host oECU = lst_ECUs.get(iHost);
                int memAvailable = oECU.getIntProperty( Host.MEMORY );

                List<Integer> lstLoc = (List<Integer>) oECU.getIntListProperty(Host.LOC_LIST);
                for (int iCompIndex = 0; iCompIndex < i_Comps; iCompIndex++) {
                    if (lstLoc.get(iCompIndex).intValue() == 1) {
                        Component comp = lst_Comps.get(iCompIndex);
                        //if (memAvailable >= totalMemRequired[iCompIndex] )
                        comp.addCanUseHost(oECU);
                        /*	else {
                        if (log.isErrorEnabled())
                            log.error("Misstated problem: Claims Host " + iHost + " can host component " + iCompIndex +
									(comp.getMustColocateWith().isEmpty() ? "" : " (and its " + comp.getMustColocateWith().size() + " other group member(s))") +						" but total memory required is " + totalMemRequired[iCompIndex] + " > " + memAvailable + " available");
                        }*/
                    }
                }
            }
            //sort only after finding suitable ECUs
            Collections.sort(lst_compConstrainedAssignments, componentOptionComparator);
            lst_compRefinedConstrainedAssignments.addAll(lst_compConstrainedAssignments);
            //Collections.sort(lst_compRefinedConstrainedAssignments, componentOptionAndSizeComparator);
    
            //From this point on no one should be able to modify these lists
            lst_compConstrainedAssignments = Collections.unmodifiableList(lst_compConstrainedAssignments);
            lst_compRefinedConstrainedAssignments = Collections.unmodifiableList(lst_compRefinedConstrainedAssignments);
	}

	public List<Component> getCompsSortedByPossibleAssignments() {
            return lst_compConstrainedAssignments;
	}
	
	public List<Component> getCompsSortedByPossibleAssignmentsAndSize() {
            return lst_compRefinedConstrainedAssignments;
	}

	public Service getService(int iServiceID) {
            //throws ParameterException{
            if (iServiceID >= lst_Services.size()) {
                System.err.println("Service not Found :" + iServiceID);
            }
            //throw new ParameterException("Service not Found :"+ iServiceID);
            return lst_Services.get(iServiceID);
	}

	//EJM notes that: If this calls System.exit() then WE CAN'T USE EXHAUSTIVE SEARCH AS THE PROBLEM HAS NOT BEEN SAVED TO DISK!
	//It may be appropriate when trying to solve the problem, but it makes it impossible to get problem generator to work
	/** do we have an appropriate search space with sufficient possible solutions? */
	private void realityCheck() {
            long searchSpaceSize = 1;  // multiplication
            Component comp = null;
            Component buddy = null;
            BitSet possibleECUs = new BitSet(i_ECUs);
            BitSet buddysECUs = new BitSet(i_ECUs);

            HashMap skipList = new HashMap();
            for (int i = 0; i < lst_Comps.size(); i++) {
                comp = lst_Comps.get(i);
                if (skipList.containsKey(comp))//the ones from the colocation lists
                {
                    continue;
                }

                possibleECUs.clear();

                for (int j = 0; j < comp.getCanUseHostListLength(); j++) {
                    if (comp.getIntProperty(Component.MEMORY) > comp.getHostOption(j).getIntProperty(Host.MEMORY)) {
                        System.err.println("PROBLEM CONF ERROR: Location list contains Comp[" + i
                                            + "] for which this [" + comp.getHostOption(j).getName() + "] has not enough memory!"
                                            + " ( " + comp.getIntProperty(Component.MEMORY) + ">" + comp.getHostOption(j).getIntProperty(Host.MEMORY) + ")");
//                        System.exit(0);
                    }

                    possibleECUs.set(comp.getHostOption(j).index);
                }

                if (comp.mustColocate()) {
                    List<Component> complist = comp.getMustColocateWith();
                    for (int j = 0; j < complist.size(); j++) {
                        buddy = complist.get(j);
                        skipList.put(buddy, null);
                    }
                    buddysECUs.clear();
                    for (int k = 0; k < buddy.getCanUseHostListLength(); k++) {
                        buddysECUs.set(buddy.getHostOption(k).index);
                    }
                    int options = possibleECUs.cardinality();
                    possibleECUs.and(buddysECUs);
                    if (options > possibleECUs.cardinality()) //they don't all share the same options
                    {
                        System.err.println("PROBLEM CONF ERROR: Colocated comps do not share same location options!");
                    }
                }
                int temp = possibleECUs.cardinality();
                if (temp == 0)
                    System.err.println("No ECU options for comp: "+comp.index);
                if (searchSpaceSize < Integer.MAX_VALUE)
                    searchSpaceSize *= temp;
                // System.out.println("Comp "+i+ " has "+ lst_Comps.get(i).getCanUseHostListLength()+" options.");
            }
            if (searchSpaceSize == 0) {
                    System.err.println("PROBLEM CONF ERROR: There are no possible solutions!");
//                System.exit(0);
            } else if (searchSpaceSize >= 0 && searchSpaceSize < 1000) {
                    System.err.println("SEARCH SPACE TOO SMALL: Only " + searchSpaceSize + " possible solutions - use exhaustive search!");
//                System.exit(0);
            }

	}
	
	public void gaugeProblemInstance(){
            int avgMemReq = 0;
            int avgMemAvail = 0;
            int avgHostListLength =0;
            int minHostListLength = Integer.MAX_VALUE;

            for (int i = 0; i < lst_Comps.size(); i++)
            {
                Component component = lst_Comps.get(i);
                avgMemReq += component.getIntProperty(Component.MEMORY);
                if (component.getCanUseHostListLength() < minHostListLength)
                    minHostListLength = component.getCanUseHostListLength();
                avgHostListLength += component.getCanUseHostListLength();
            }
            for (int i = 0; i < lst_ECUs.size(); i++){
                Host ecu = lst_ECUs.get(i);
                avgMemAvail += ecu.getIntProperty(Host.MEMORY);
            }
            avgMemReq /= lst_Comps.size();
            avgMemAvail /= lst_ECUs.size();
            avgHostListLength /= lst_Comps.size();

            System.out.println("Average mem requirement "+avgMemReq+ " average memory available "+avgMemAvail);
            System.out.println("According to mem req, a host accommodates an average "+(avgMemAvail/avgMemReq)+" components.");
            System.out.println("Average number of hosts for a component acc to location list "+avgHostListLength);
            System.out.println("Smallest number of hosts for a component acc to location list "+minHostListLength);
            System.out.flush();
	}
}
