package archeopterix.problem;


import java.util.List;

import archeopterix.util.PropertyMap;
import archeopterix.util.statistics.ProbabilityDistribution;
import java.io.Serializable;

public abstract class ObjectWithProperties implements Serializable{
    public static final String OBJECT_ID = "ID";
	protected String name;
	transient protected PropertyMap propertyMap;

	protected ObjectWithProperties(String name, PropertyMap propertyMap) {
		this.name = name;
		this.propertyMap = propertyMap;
                this.propertyMap.put(OBJECT_ID, name);
	}
	
	public String getName() { return name; }
	
	public PropertyMap getPropertyMap() { return propertyMap; }
	
	/** Convenience method to retrieve {@code double}-valued property. */ 
	public double getDoubleProperty(String propertyName) { return propertyMap.getDouble(propertyName); }

        //PDF Related methods
        public ProbabilityDistribution getPDFProperty(String propertyName) {return propertyMap.getPDF(propertyName);}
        public List<ProbabilityDistribution> getPDFListProperty(String propertyName) {return propertyMap.getPDFList(propertyName);}
        public ProbabilityDistribution getPDFFromListProperty(String propertyName, int iIndex) {return propertyMap.getPDFFromList(propertyName, iIndex);}

	/** Convenience method to retrieve {@code int}-valued property. */ 
	public int getIntProperty(String propertyName) { return propertyMap.getInt(propertyName); }

	/** Convenience method to retrieve a list of doubles property. */
	public List<Double> getDoubleListProperty(String propertyName) { return propertyMap.getDoubleList(propertyName); }

	/** Convenience method to retrieve value from a list of doubles property. */
	public double getDoubleFromListProperty(String propertyName, int index) { return propertyMap.getDoubleFromList(propertyName, index); }
	
	/** Convenience method to retrieve a list of ints property. */
	public List<Integer> getIntListProperty(String propertyName) { return propertyMap.getIntList(propertyName); }

	/** Convenience method to retrieve value from a list of ints property. */
	public int getIntFromListProperty(String propertyName, int index) { return propertyMap.getIntFromList(propertyName, index); }

	public String toString() { return "(" + name + ", properties " + propertyMap + ")"; }

}