package archeopterix.problem.componentselection.algorithm.aco;
import java.util.*;

import archeopterix.algorithm.Stats;
import archeopterix.algorithm.aco.ACO;
import archeopterix.algorithm.aco.Ant;
import archeopterix.conf.Conf;
import archeopterix.problem.Component;
import archeopterix.problem.componentselection.solution.ComponentSelectionSolution;

//import common.Logger;

public class ComponentSelectionAnt extends Ant {
	
	public  	LinkedList<Integer> m_CompsToAllocate;
	protected 	Component	m_StartPoint;

	public ComponentSelectionAnt(ACO oAlgo, int iIndex){
		m_Algo = oAlgo;
		m_Memory = new ComponentSelectionSolution(m_Algo.getProblem());
		m_CompsToAllocate = new LinkedList<Integer>();

		int iArrSize = m_Algo.getPheromoneMapsCount() >1? m_Algo.getPheromoneMapsCount():2;
		ad_Weights= new double [iArrSize];
		//ad_Weights[0] = ((double)iIndex/(double)iMax);
		//ad_Weights[1] = (1- ad_Weights[0]);
		
		double dMid = (double)1/(double)m_Algo.getPheromoneMapsCount();
		int iSteps = (int)((double)m_Algo.getBatchSize()/(double)m_Algo.getPheromoneMapsCount());
		int iPriority = (int)((double)iIndex / ((double)m_Algo.getBatchSize()/(double)m_Algo.getPheromoneMapsCount()));
		int iTick = (iIndex % iSteps);
		//System.out.print("A["+ iAnt + "] Priority=" + iPriority + "Tick="+ iTick );
		//double[] aiARR = new double[iObjectives];
		for(int i=0; i< m_Algo.getPheromoneMapsCount(); i++){
			if (i== iPriority){
				ad_Weights[i] = dMid + (iTick * (dMid/(double)iSteps));
			}
			else
				ad_Weights[i] = dMid - (iTick * (dMid/(double)iSteps));
			//System.out.print(aiARR[i] + ",");
		}
		
		//Random rnd = Conf.instance().random;
		/*for (int i=0; i< Defs.NO_OF_ATTRIBUTES; i++){
			ad_Weights[i] = rnd.nextDouble();
		}*/
	}
	
	public ComponentSelectionAnt(ComponentSelectionSolution oCA, ACO oAlgo, int iIndex, int iMax){
		m_Algo = oAlgo;
		m_Memory = oCA;
		m_CompsToAllocate = new LinkedList<Integer>();
		ad_Weights= new double [Conf.instance().getNEvaluators()];
		ad_Weights[0] = ((double)iIndex/(double)iMax);
		ad_Weights[1] = (1- ad_Weights[0]);
	}
	
	
	public boolean Travel(){
		m_Memory.reset();
		Random rnd = Conf.instance().random;
		for (int iCompIndex=0; iCompIndex < m_Algo.getProblem().getComponentCount(); iCompIndex++){
			int iOption = decideOption(iCompIndex);
            ComponentSelectionSolution mySelections = (ComponentSelectionSolution)m_Memory;
			mySelections.changeSelection(iCompIndex, iOption);

			if (! m_Algo.getProblem().validate(m_Memory)){
				iOption = rnd.nextInt(mySelections.getNOptions(iCompIndex));
				boolean bSuccess =false;
				for (int iTry=0; iTry<mySelections.getNOptions(iCompIndex) ; iTry++){
					mySelections.changeSelection(iCompIndex, iOption);
					if ( m_Algo.getProblem().validate(m_Memory) ){
						bSuccess = true;
						break;
					}
					iOption = (++iOption == mySelections.getNOptions(iCompIndex))? 0: iOption;
					Stats.instance().l_CntRndD++;
				}
				if ( !bSuccess){
					//Logger.Logln("Can't proceed Dep :" + m_Memory.getDepString());
					return false;
				}
				//TODO:: Do BackTaracking here
			}
					
			//Local Pheromone Update
			for( int iAtt=0; iAtt<m_Algo.getPheromoneMapsCount(); iAtt++){
				double dPM = m_Algo.getPheromoneValue(iAtt,iCompIndex, iOption);
				dPM = m_Algo.evaporatePheromone(dPM);
                                m_Algo.setPheromeneValue(iAtt, iCompIndex, iOption, dPM);
			}
		}
		return true;
		
		/*for (int i=0; i< m_CompsToAllocate.size(); i++){
			m_CompsToAllocate.remove(i);
			int iHostIndex = DecideHostIndex(i);
			m_Memory.Set(i, iHostIndex);
			double dPM = m_Algo.arr_PhMap[0].GetPheromone(i, iHostIndex);
			m_Algo.arr_PhMap[0].UpdatePheromone(i, iHostIndex, dPM * (1- m_Algo.d_PhDecay));//Local Pheromone Update
		}*/
	}
	
	private int decideOption(int iCompIndex){
		LinkedList<Double> lstCum = new LinkedList<Double>();
		//Calculating Cumulative Values for probability
		double dTot=0.0;
		double dMax=0.0;
		int iMaxIndex=0;
        ComponentSelectionSolution mySelections = (ComponentSelectionSolution)m_Memory;
		//Index starts from 0, to MaxRALevels
		for (int iOption=0; iOption<mySelections.getNOptions(iCompIndex) ; iOption++){
			double dF = 0.0;
			//TODO::Uncomment Following
			 for(int iAttr=0; iAttr < m_Algo.getPheromoneMapsCount(); iAttr++){
				dF+= m_Algo.getPheromoneValue(iAttr,iCompIndex, iOption)*ad_Weights[iAttr];
			}
			//dF = m_Algo.arr_PhMap[0].GetPheromone(iCompIndex, iHostIndex);
			dTot+=dF;		
			if(dF>dMax){
				dMax	= dF;
				iMaxIndex = iOption;
			}
			lstCum.add(dTot);
		}
		Random rnd = Conf.instance().random;
		if ( dTot < 1E-4 ){
			Stats.instance().l_CntRndD++;
			return rnd.nextInt(m_Algo.getProblem().getECUCount());
		}
		double dQ = rnd.nextDouble();
		if(dQ <= m_Algo.getThPheormone()){
			Stats.instance().l_CntMPU++;
			return iMaxIndex;
		}
		else{
			double dTh = rnd.nextDouble()* dTot;
			int iOptionIndex=0;
			//Selecting the host based on probability
			while( lstCum.get(iOptionIndex)< dTh){
				iOptionIndex++;
			}
			//System.out.println("Decide " + lstCum.toString() + "Rnd="+ dTh+ " =" + iHostIndex);
			Stats.instance().l_CntPPU++;
			return iOptionIndex;
		}
	}
	
	@Override
	public void UpdateTrack(int iAtt, int iQuantity){
//		LinkedList<Attribute> lstAttributes = m_Memory.GetAttributeList();
//		Attribute oAtt = lstAttributes.get(iAtt);
		for (int i=0; i< m_Algo.getProblem().getComponentCount(); i++){
			LinkedList<Double> lstRow = m_Algo.getPheromoneMap(iAtt).GetRow(i);
			int j = ((ComponentSelectionSolution)m_Memory).getCurrentSelection(i);
			double dPmOld = lstRow.get(j);
			double dPm =  m_Algo.calculateNewPheromone(dPmOld, iQuantity);
			//double dPm = dPmOld + oAtt.f_Value * iQuantity;
			m_Algo.setPheromeneValue(iAtt, i, j, dPm);
			//System.out.println("Update Att"+ iAtt+ " Old="+ dPmOld + " New="+ dPm); 
		}
	}


	
	/*public void UpdateTrack(int i_Quantity){
		for ( int iAtt=0; iAtt< Defs.NO_OF_ATTRIBUTES; iAtt++){
			for (int i=0; i< m_Algo.i_Comps; i++){
				LinkedList<Double> lstRow = m_Algo.arr_PhMap[iAtt].GetRow(i);
				int j = m_Memory.GetHostIndex(i);
				double dPmOld = lstRow.get(j);
				double dPm = (1-m_Algo.d_PhDecay) * dPmOld + m_Algo.d_PhDecay * i_Quantity;
				m_Algo.arr_PhMap[iAtt].UpdatePheromone(i, j, dPm);
				System.out.println("Update Att"+ iAtt+ " Old="+ dPmOld + " New="+ dPm); 
			}
		}
	}*/
	

    @Override
	public String toString(){
		return  m_Memory.toString();
	}

}


