/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package archeopterix.problem.componentselection.solution;

import archeopterix.conf.Conf;
import archeopterix.problem.Component;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.solution.Architecture;
import archeopterix.solution.OutOfOptionsException;
import archeopterix.solution.Repairable;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 *
 * @author indika
 */
public class ComponentSelectionSolution extends Architecture implements Repairable{

    private int i_Components;
    private ArrayList<Integer> lst_Selections;
    private ArrayList<Integer> lst_NOptions;

    public ComponentSelectionSolution(EmbeddedSystemProblem problem) {
        i_Components = problem.getComponentCount();
        lst_Selections = new ArrayList<Integer>();
        lst_NOptions = new ArrayList<Integer>();
        for (int i = 0; i < i_Components; i++) {
            lst_Selections.add(0);//TODO::DoubleCheck.Initially RA level is 0 for all.
            lst_NOptions.add(problem.getComponent(i).getNumberOptions());
        }
        randomlyAllocate();
    }

    public ComponentSelectionSolution (ComponentSelectionSolution old){
        i_Components = old.i_Components;
        lst_Selections = new ArrayList<Integer>();
        lst_Selections.addAll(old.lst_Selections);
        lst_NOptions = new ArrayList<Integer>();
        lst_NOptions.addAll(old.lst_NOptions);
    }

    public final void randomlyAllocate(){
		Random rnd = Conf.instance().random;
		for(int i=0; i <i_Components; i++){
            int iOption = rnd.nextInt(lst_NOptions.get(i));
			lst_Selections.set(i, iOption);
		}
	}
    
    @Override
    public Architecture clone() {
        return new ComponentSelectionSolution(this);
    }

    @Override
    public boolean isSameAs(Architecture other) {
        if (! (other instanceof ComponentSelectionSolution))
                return false;
        ComponentSelectionSolution oCS = (ComponentSelectionSolution)other;

        for (int i = 0; i < lst_Selections.size(); i++)
		{
			if (lst_Selections.get(i) != oCS.lst_Selections.get(i))
				return false;
		}
		return true;
    }

    @Override
    public void reset() {
        for (int i = 0; i < lst_Selections.size(); i++)
            lst_Selections.set(i,0);
    }

    /**
     * Format : nComponents, Comma Seperated Selections
     * @return
     */
    @Override
    public String getSerializedString() {
        String sRet = new String();
        sRet = i_Components + ",";
        /*for (int i=0; i< lst_NOptions.size(); i++){
                sRet = sRet + lst_NOptions.get(i).toString()+ ",";
        }*/
        for (int i=0; i< lst_Selections.size(); i++){
            sRet = sRet + lst_Selections.get(i).toString()+ ",";
        }
        return sRet;
    }

    @Override
    public void initializeFromString(String sStr, EmbeddedSystemProblem oProblem) {
		String[] aSt = sStr.split(",");
		i_Components = Integer.parseInt(aSt[0]);
		if ( aSt.length != (i_Components + 1 )){
			System.out.println("String Sizes not mach! " + aSt.length + "," + (i_Components +1) );
		}
		lst_Attributes = null;
        lst_Selections.clear();
		for ( int i= 1 ; i< (i_Components +1)  ; i++){
			lst_Selections.add(Integer.parseInt(aSt[i]));
		}
        lst_NOptions.clear();
        List<Component> comps = oProblem.getComponents();
		for ( int i= 0; i< i_Components; i++){
            int iOptions = comps.get(i).getNumberOptions();
			lst_NOptions.add(iOptions);
		}


    }

    @Override
    public void attemptRepair(EmbeddedSystemProblem problem) throws OutOfOptionsException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void changeSelection(int iCompIndex, int iOption){
        if (iOption >= lst_NOptions.get(iCompIndex)){
            System.err.println("Invalid Option :" + iOption + " NOptions ="+ lst_NOptions.get(iCompIndex));
            return;
        }
        lst_Selections.set(iCompIndex, iOption);
    }

    public int getNOptions(int iCompIndex){
        if (iCompIndex >= i_Components){
            System.err.println("Invalid Component Index to Set :" + iCompIndex + " NComps ="+ i_Components);
            return 0;
        }
        return lst_NOptions.get(iCompIndex);
    }

    public int getCurrentSelection( int iCompIndex){
        return lst_Selections.get(iCompIndex);
    }

}
