package archeopterix.problem.deployment.algorithm.aco;

import java.util.*;

import archeopterix.algorithm.Stats;
import archeopterix.algorithm.aco.ACO;
import archeopterix.algorithm.aco.Ant;
import archeopterix.conf.Conf;
import archeopterix.problem.Component;
import archeopterix.problem.deployment.solution.DeploymentSolution;
import archeopterix.problem.deployment.solution.OriginalDeploymentSolution;
import archeopterix.solution.Architecture;

//import common.Logger;

public class DeploymentAnt extends Ant {
	
	public  LinkedList<Integer> m_CompsToAllocate;
	protected Component 		m_StartPoint;
        protected int i_ECUs;
        protected int i_Comps;


	public DeploymentAnt(ACO oAlgo, int iIndex, int iMax){
		m_Algo = oAlgo;
                i_ECUs = m_Algo.getProblem().getECUCount();
                i_Comps = m_Algo.getProblem().getComponentCount();
		m_Memory = new OriginalDeploymentSolution(i_ECUs , i_Comps);
		m_CompsToAllocate = new LinkedList<Integer>();

		int iArrSize = Conf.instance().getNEvaluators()>1? Conf.instance().getNEvaluators():2;
		ad_Weights= new double [iArrSize];
		//ad_Weights[0] = ((double)iIndex/(double)iMax);
		//ad_Weights[1] = (1- ad_Weights[0]);
		
		// The following section assigns homgenous sweep of parameters for the batch of ands for any no of attributs
		/*
		double dMid = (double)1/(double)Conf.Ins().i_Attributes;
		int iSteps = (int)((double)m_Algo.i_BatchSize/(double)Conf.Ins().i_Attributes);
		int iPriority = (int)((double)iIndex / ((double)m_Algo.i_BatchSize/(double)Conf.Ins().i_Attributes));
		int iTick = (iIndex% iSteps);
		//System.out.print("A["+ iAnt + "] Priority=" + iPriority + "Tick="+ iTick );
		//double[] aiARR = new double[iObjectives];
		for(int i=0; i< Conf.Ins().i_Attributes; i++){
			if (i== iPriority){
				ad_Weights[i] = dMid + (iTick * (dMid/(double)iSteps));
			}
			else
				ad_Weights[i] = dMid - (iTick * (dMid/(double)iSteps));
			//System.out.print(aiARR[i] + ",");
		}
		*/
		
		//Random allocation of weights, for any of the number of objectives.
		Random rnd = Conf.instance().random;
		double dRemain = 1.0;
		int i=0;
		do {
			if (i == ( Conf.instance().getNEvaluators()-1) )//Last one
			{
				ad_Weights[i] = dRemain;
				break;
			}
			else
			{
				ad_Weights[i] = rnd.nextDouble()* dRemain;
				dRemain -= ad_Weights[i];
				i++;
			}
		}while(i>=0); //Safety
	}
	
	public DeploymentAnt(Architecture oDep, ACO oAlgo, int iIndex, int iMax){
		m_Algo = oAlgo;
		m_Memory = oDep;
		m_CompsToAllocate = new LinkedList<Integer>();
		ad_Weights= new double [Conf.instance().getNEvaluators()];
		ad_Weights[0] = ((double)iIndex/(double)iMax);
		ad_Weights[1] = (1- ad_Weights[0]);
	}
	
	
	public boolean Travel(){
		((DeploymentSolution)m_Memory).reset();
		Random rnd = Conf.instance().random;
		for (int iCompIndex=0; iCompIndex < i_Comps; iCompIndex++){
			int iHostIndex = DecideHostIndex(iCompIndex);
			((OriginalDeploymentSolution)m_Memory).set(iCompIndex, iHostIndex);
			if (! m_Algo.getProblem().validate(m_Memory) ){
				iHostIndex = rnd.nextInt(i_ECUs);
				boolean bSuccess =false;
				for (int iTry=0; iTry<i_ECUs ; iTry++){
					((OriginalDeploymentSolution)m_Memory).set(iCompIndex, iHostIndex);
					if ( m_Algo.getProblem().validate(m_Memory) ){
						bSuccess = true;
						break;
					}
					iHostIndex = (++iHostIndex == i_ECUs)? 0: iHostIndex; 
					Stats.instance().l_CntRndD++;
				}
				if ( !bSuccess){
					//Logger.Logln("Can't proceed Dep :" + m_Memory.getDepString());
					return false;
				}
				//TODO:: Do BackTaracking here
			}
					
			//Local Pheromone Update
			DoLocalPheromoneUpdate(iCompIndex, iHostIndex);
		}
		return true;
		
		/*for (int i=0; i< m_CompsToAllocate.size(); i++){
			m_CompsToAllocate.remove(i);
			int iHostIndex = DecideHostIndex(i);
			m_Memory.set(i, iHostIndex);
			double dPM = m_Algo.arr_PhMap[0].GetPheromone(i, iHostIndex);
			m_Algo.arr_PhMap[0].UpdatePheromone(i, iHostIndex, dPM * (1- m_Algo.d_PhDecay));//Local Pheromone Update
		}*/
	}
	
	protected void DoLocalPheromoneUpdate(int iCompIndex, int iHostIndex){
		for( int iAtt=0; iAtt < m_Algo.getPheromoneMapsCount(); iAtt++){
			double dPM = m_Algo.getPheromoneValue(iAtt,iCompIndex, iHostIndex);
			dPM = m_Algo.evaporatePheromone(dPM);
			m_Algo.setPheromeneValue(iAtt,iCompIndex, iHostIndex, dPM );
		}
	}
	
	protected int DecideHostIndex(int iCompIndex){
		LinkedList<Double> lstCum = new LinkedList<Double>();
		//Calculating Cumulative Values for probability
		double dTot=0.0;
		double dMax=0.0;
		int iMaxIndex=0;
		for (int iHostIndex=0; iHostIndex < i_ECUs; iHostIndex++){
			double dF = 0.0;
			//TODO::Uncomment Following
			 for(int iAttr=0; iAttr < m_Algo.getPheromoneMapsCount() ; iAttr++){
				dF+= m_Algo.getPheromoneValue(iAttr, iCompIndex, iHostIndex)*ad_Weights[iAttr];
			}
			//dF = m_Algo.arr_PhMap[0].GetPheromone(iCompIndex, iHostIndex);
			dTot+=dF;		
			if(dF>dMax){
				dMax	= dF;
				iMaxIndex = iHostIndex;
			}
			lstCum.add(dTot);
		}
		Random rnd = Conf.instance().random;
		if ( dTot < 1E-4 ){
			Stats.instance().l_CntRndD++;
			return rnd.nextInt(i_ECUs);
		}
		double dQ = rnd.nextDouble();
		if(dQ <= m_Algo.getThPheormone()){
			Stats.instance().l_CntMPU++;
			return iMaxIndex;
		}
		else{
			double dTh = rnd.nextDouble()* dTot;
			int iHostIndex=0;
			//Selecting the host based on probability
			while( lstCum.get(iHostIndex)< dTh){
				iHostIndex++;
			}
			//System.out.println("Decide " + lstCum.toString() + "Rnd="+ dTh+ " =" + iHostIndex);
			Stats.instance().l_CntPPU++;
			return iHostIndex;
		}
	}
	

	
	/*public void UpdateTrack(int i_Quantity){
		for ( int iAtt=0; iAtt< Defs.NO_OF_ATTRIBUTES; iAtt++){
			for (int i=0; i< m_Algo.i_Comps; i++){
				LinkedList<Double> lstRow = m_Algo.arr_PhMap[iAtt].GetRow(i);
				int j = m_Memory.GetHostIndex(i);
				double dPmOld = lstRow.get(j);
				double dPm = (1-m_Algo.d_PhDecay) * dPmOld + m_Algo.d_PhDecay * i_Quantity;
				m_Algo.arr_PhMap[iAtt].UpdatePheromone(i, j, dPm);
				System.out.println("Update Att"+ iAtt+ " Old="+ dPmOld + " New="+ dPm); 
			}
		}
	}*/
	

        @Override
	public String toString(){
		
		return  m_Memory.toString();
	}

	@Override
	public void UpdateTrack(int iAtt, int iQuantity){
//		LinkedList<Attribute> lstAttributes = m_Memory.GetAttributeList();
//		Attribute oAtt = lstAttributes.get(iAtt);
		for (int i=0; i< i_Comps; i++){
                    LinkedList<Double> lstRow = m_Algo.getPheromoneMap(iAtt).GetRow(i);
                    int j = ((DeploymentSolution)m_Memory).getHostIndex(i);
                    double dPmOld = lstRow.get(j);
                    double dPm =  m_Algo.calculateNewPheromone(dPmOld, iQuantity);
                    //double dPm = dPmOld + oAtt.f_Value * iQuantity;
                    m_Algo.setPheromeneValue(iAtt, i, j, dPm);
                    //System.out.println("Update Att"+ iAtt+ " Old="+ dPmOld + " New="+ dPm);
                    }
		}
	}
