package archeopterix.problem.deployment.algorithm.exact;

import archeopterix.algorithm.Algorithm;
import java.util.Collection;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import archeopterix.algorithm.PopulationGenerator;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.deployment.solution.OriginalDeploymentSolution;
import archeopterix.solution.Architecture;
import archeopterix.util.PropertyMap;

public class ExactAlgoDep extends Algorithm{
	private static final Log log = LogFactory.getLog(ExactAlgoDep.class);

	OriginalDeploymentSolution m_Dep;
	private int i_Feasible;
        EmbeddedSystemProblem problem;
        PopulationGenerator populationGenerator;
        PropertyMap props;
        Collection<Architecture> initialPop;
        
        @Override
	public void initialise(EmbeddedSystemProblem prob, PropertyMap params, Collection<Architecture> partialInitPop, PopulationGenerator popGenerator) {
            problem = prob;
            props = params;
            populationGenerator = popGenerator;
	initialPop = partialInitPop;
		m_Dep = new OriginalDeploymentSolution(i_ECUs, i_Comps);
	}

        @Override
	public void oneIteration(){
		i_Feasible = 0;
		if (log.isInfoEnabled())
			log.info(m_Watch.GetElapsed()+": H" +i_ECUs + "_C"+ i_Comps);
		for (int i=0;i<i_Comps; i++)
			m_Dep.set(i, 0);
		/*for ( int iC=0; iC<i_Comps; iC++ ){
                //for ( int iC=(i_Comps-1); iC>=0; iC-- ){
			TryAllCompFrom(iC);
			PrintCurrentSolutions("Feasible Count="+i_Feasible+" at Component :"+ iC);
		}*/
		Try(0);
		if (log.isInfoEnabled())
			log.info("Search Complete.Search Space="+ Math.pow(i_Comps, i_ECUs) + " Feasible="+ i_Feasible);
		printCurrentNondominatedSolutions("Final");
	}

	private void TryAllCompFrom(int iCompIndex){
		for (int iC=iCompIndex ; iC>=0 ; iC--){
			m_Dep.set(iC, -1);
		}
		if (log.isDebugEnabled())
			log.debug("Check:"+ m_Dep.getSerializedString());
		if ( !problem.validate(m_Dep)){
			//Logger.Logln("Ignore branch:"+ m_Dep.getDepString());
			return;
		}

		for (int iH=0; iH<i_ECUs; iH++){
			m_Dep.set(iCompIndex, iH);
			if ( iCompIndex == 0 ){ //Go here only get VALI+FULL deployment
				if ( problem.validate(m_Dep)){
					i_Feasible++;
					if (log.isDebugEnabled())
						log.debug("FEASEIBLE:"+ m_Dep.getSerializedString());
					OriginalDeploymentSolution oDep = m_Dep.clone();
					problem.evaluateAttributes(oDep);
					checkDominance(oDep);
				}
			}
			//                        else if (!m_Context.validate(m_Dep)){
			//                            Logger.Logln("--Not Valid:"+ m_Dep.getDepString());
			//                            continue;
			//                        }
			else{
				//validate and Evaluate Here
				for (int iC=(iCompIndex -1) ; iC>=0 ; iC--){
					TryAllCompFrom(iC);
				}
			}
		}
	}

	private void Try(int iCompIndex) {
		if (iCompIndex == i_Comps) { //Go here only get VALI+FULL deployment
			if (problem.validate(m_Dep)) {
				i_Feasible++;
				if (log.isDebugEnabled())
					log.debug("FEASEIBLE:" + m_Dep.getSerializedString());
				OriginalDeploymentSolution oDep = m_Dep.clone();
				problem.evaluateAttributes(oDep);
				checkDominance(oDep);
			}
		} else {
			for (int iH = 0; iH < i_ECUs; iH++) {
				m_Dep.set(iCompIndex, iH);
				Try(iCompIndex+1);
			}
			//
		}
	}

	
}