/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package archeopterix.problem.deployment.algorithm.heuristic;

import archeopterix.problem.deployment.solution.DeploymentSolution;
import archeopterix.problem.deployment.solution.OriginalDeploymentSolution;
import archeopterix.solution.Architecture;
import archeopterix.solution.OutOfOptionsException;
import java.util.LinkedList;
import java.util.List;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * BHCDO extension to compute conditional probabilities throughout the process
 * @author indika
 */
public class BHCDO2 extends BHCDO{
    private static final Log log = LogFactory.getLog(BHCDO2.class);

    private List<int[]> localEliteSet;
    private List<int[]> localTotalSet;


	@Override
   protected void initStructures(){
	    localEliteSet = new LinkedList<int[]>();
        localTotalSet = new LinkedList<int[]>();
		currentAllocation = new OriginalDeploymentSolution(i_ECUs, i_Comps);
   }
    /**
     * Create local structures to represent the population and elite set
     */
	@Override
   protected void populateBayesianStructures(){
       localEliteSet.clear();
       for(Architecture arch:nondominatedSolutions){
           int[] arrArc = new int[i_Comps];
           DeploymentSolution dep = (DeploymentSolution)arch;
           for (int iC=0; iC<i_Comps; iC++){
               arrArc[iC] = dep.getHostIndex(iC);
           }
           localEliteSet.add(arrArc);
       }

       localTotalSet.clear();
       for(Architecture arch:population){
           int[] arrArc = new int[i_Comps];
           DeploymentSolution dep = (DeploymentSolution)arch;
           for (int iC=0; iC<i_Comps; iC++){
               arrArc[iC] = dep.getHostIndex(iC);
           }
           localTotalSet.add(arrArc);
       }

   }

   /**
    * Filtereing the local structures for current allocation
    * @param iC
    * @param iH
    */
   private List<int[]> filterAllocations(List<int[]> sourceList, int iC, int iH){
	   List<int[]> tempList = new LinkedList<int[]>();
       for(int[] arr:sourceList){
           if (arr[iC] == iH)
               tempList.add(arr);
       }
	   return tempList;
   }

   /**
    * Count number of H-C combinations in filtered elite set
    * @param lstDep list of allocation arrays
    * @param iC component index
    * @return array of counts for host indexs
    */
   private int[] getHostCounts(List<int[]> lstDep, int iC){
       int[] arrCounts = new int[i_ECUs];
	   for (int iH=0; iH<i_ECUs; iH++)
		   arrCounts[iH] = 0;
       for(int[] arrDep : lstDep){
           arrCounts[arrDep[iC]] += 1;
       }
       return arrCounts;
   }

   @Override
   protected DeploymentSolution createSolution()throws OutOfOptionsException{
	   List<int[]> currentEliteSet = new LinkedList<int[]>();
	   currentEliteSet.addAll(localEliteSet);
	   List<int[]> currentTotalSet = new LinkedList<int[]>();
	   currentTotalSet.addAll(localTotalSet);
	   List<Integer> lstCompsToAllocate = new LinkedList<Integer>();
       for (int i=0; i< i_Comps; i++){
            lstCompsToAllocate.add(i);
            currentAllocation.set(i,-1); // Intialising
       }
       while ( lstCompsToAllocate.size() > 0){
            int iCIdx = rnd.nextInt(lstCompsToAllocate.size());
            int iC = lstCompsToAllocate.remove(iCIdx);
			int iAttempts = 0;
			currentAllocation.set(iC,selectHostBayesian(iC, currentTotalSet, currentEliteSet));
			while(! problem.validate(currentAllocation)){
				currentAllocation.set(iC,selectHostBayesian(iC, currentTotalSet, currentEliteSet));
				if (iAttempts > 100){
					throw new OutOfOptionsException();
				}
			}
            if (lstCompsToAllocate.size() > 0){
				currentTotalSet = filterAllocations(currentTotalSet, iC, currentAllocation.getHostIndex(iC));
                currentEliteSet = filterAllocations(currentEliteSet, iC, currentAllocation.getHostIndex(iC));
			}
        }
        return new OriginalDeploymentSolution(currentAllocation);
    }

   private int selectHostBayesian( int iC, List<int[]> totalSet, List<int[]> eliteSet){
        double[] adWeighs = computeConditionalProbabilities(iC, totalSet, eliteSet );
        return doRoulette(adWeighs);
    }

   private double[] computeConditionalProbabilities(int iC, List<int[]> totalSet, List<int[]> eliteSet ){
       double dSumCP = 0.0;
       double[] adCP = new double[i_ECUs];
       int[] arrE = getHostCounts(eliteSet, iC);
       int[] arrT = getHostCounts(totalSet, iC);
        for (int iH = 0; iH < i_ECUs; iH++) {
            double dE = arrE[iH];
            double dT = arrT[iH];
            double dCP = 0.0;
            if (arrT[iH] != 0) {
                dCP = dE / dT;
            }
            dSumCP += dCP;
            adCP[iH] = dCP;
        }
        //Normalising
        double dRowSum = dSumCP *(1 + exploraitionRate);
        double dCompensation = dSumCP * exploraitionRate / (double)i_ECUs;
        for (int iH = 0; iH < i_ECUs; iH++) {
            double dCP = adCP[iH];
            dCP = (dCP + dCompensation) / dRowSum;
            adCP[iH] = dCP;
        }
        return adCP;
   }

}
