/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package archeopterix.problem.deployment.algorithm.nsga;


import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

import archeopterix.algorithm.ea.EAOperators;
import archeopterix.algorithm.ea.MutationOperator;
import archeopterix.conf.Conf;
import archeopterix.problem.Component;
import archeopterix.problem.Host;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.deployment.solution.ECUBasedDeployment;
import archeopterix.solution.Architecture;
import archeopterix.solution.OutOfOptionsException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *s
 * @author imeedeniya
 */
public class ECUBasedDepEAOperators extends EAOperators {
	
	public boolean canOperateOn(Class<?> solRepClass) {
		return solRepClass.equals( ECUBasedDeployment.class );
	}

	@Override
	public Architecture singlePointMutation(Architecture oArch, EmbeddedSystemProblem problem) throws OutOfOptionsException {
		Random random = Conf.instance().random;
		ECUBasedDeployment parent = (ECUBasedDeployment)oArch;
		ECUBasedDeployment dep = new ECUBasedDeployment(parent);

		int nECUs = problem.getECUCount();

		boolean foundOption = false;
		Component comp = null;
		List<Component> source= null;
		List<Component> target=null;
		int counter = 0;
		for (int i = 0; i < nECUs; i++)
		{
			counter = 0;
			while(!foundOption && counter < nECUs)
			{
				counter++;
				source = dep.getComponentListForHost( random.nextInt(nECUs), problem);
				for(int j=0; j<source.size(); j++){
					comp = source.get(random.nextInt(source.size()));

					if(comp.getMustColocateWith().size() ==0){
						foundOption = true;
						break;
					}
				}
			}
			if(!foundOption)
				throw new OutOfOptionsException("Not enough options for pointMutate.");
			foundOption = false;
			counter = 0;
			List<Host> hosts = problem.getECUs();
			while(!foundOption && counter < nECUs){
				counter++;
				int targetHost = random.nextInt(nECUs);
				target = dep.getComponentListForHost(targetHost, problem);
				if(!comp.isCompatibleHost(hosts.get(targetHost)))
					continue;
				if(comp.conflicts(target))
					continue;
				else
					foundOption = true;
			}
			//foundOption = true;
			if(foundOption)
			{
				target.add(comp);
				source.remove(comp);
                                 
				return dep;
			}
		}
		throw new OutOfOptionsException("Not enough options for pointMutate.");

	}

	@Override
	public Architecture swapMutation(Architecture oArch, EmbeddedSystemProblem context) throws OutOfOptionsException {
		ECUBasedDeployment parent = (ECUBasedDeployment) oArch;
		Random random = Conf.instance().random;
		int nECUs = context.getECUCount();

		List<Host> hosts = context.getECUs();
		boolean allocationImpossible;
		for (int i = 0; i < nECUs*2; i++)
		{
			allocationImpossible = false;
			int iMutationID1 = random.nextInt(nECUs);
			int iMutationID2;
			do {
				iMutationID2 = random.nextInt(nECUs);
			} while (iMutationID1 == iMutationID2);
			ECUBasedDeployment oDep = new ECUBasedDeployment(parent);
			Iterator<Component> compsIterator1 = oDep.getComponentListForHost(iMutationID1, context).iterator();

			while(compsIterator1.hasNext())
			{
				if(!compsIterator1.next().isCompatibleHost(hosts.get(iMutationID2)))
				{
					allocationImpossible = true;
					break;
				}
			}
			if (allocationImpossible)
				break;

			Iterator<Component> compsIterator2 = oDep.getComponentListForHost(iMutationID2, context).iterator();

			while(compsIterator2.hasNext())
			{
				if(!compsIterator2.next().isCompatibleHost(hosts.get(iMutationID1)))
				{
					allocationImpossible = true;
					break;
				}
			}
			if (allocationImpossible)
				continue;
			//dep is a copy and only two lists have to be adjusted
			oDep.getComponentListForHost(iMutationID1, context).clear();
			oDep.getComponentListForHost(iMutationID2, context).clear();
			// System.out.println("After clearing, before adding: "+dep.getDepString());
			oDep.getComponentListForHost(iMutationID1, context).addAll(parent.getComponentListForHost(iMutationID2, context));
			// System.out.println("After adding first: "+dep.getDepString()+" clearing index "+iMutationID1);
			oDep.getComponentListForHost(iMutationID2, context).addAll(parent.getComponentListForHost(iMutationID1, context));
			// System.out.println("After adding second: "+dep.getDepString()+" clearing index "+iMutationID2);
			return oDep;
		}
		throw new OutOfOptionsException("Could not find suitable swap using "+2*nECUs+" attempts.");

	}

	public Architecture swapMutatation(Architecture oArch, EmbeddedSystemProblem context, int index) throws OutOfOptionsException {
		ECUBasedDeployment parent = (ECUBasedDeployment) oArch;
		Random random = Conf.instance().random;
		int nECUs = context.getECUCount();

		List<Host> hosts = context.getECUs();
		boolean allocationImpossible;
		for (int i = 0; i < nECUs*2; i++)
		{
			allocationImpossible = false;
			int iMutationID2;
			do {
				iMutationID2 = random.nextInt(nECUs);
			} while (index == iMutationID2);
			ECUBasedDeployment oDep = new ECUBasedDeployment(parent);
			Iterator<Component> compsIterator1 = oDep.getComponentListForHost(index, context).iterator();

			while(compsIterator1.hasNext())
			{
				if(!compsIterator1.next().isCompatibleHost(hosts.get(iMutationID2)))
				{
					allocationImpossible = true;
					break;
				}
			}
			if (allocationImpossible)
				break;

			Iterator<Component> compsIterator2 = oDep.getComponentListForHost(iMutationID2, context).iterator();

			while(compsIterator2.hasNext())
			{
				if(!compsIterator2.next().isCompatibleHost(hosts.get(index)))
				{
					allocationImpossible = true;
					break;
				}
			}
			if (allocationImpossible)
				continue;
			//dep is a copy and only two lists have to be adjusted
			oDep.getComponentListForHost(index, context).clear();
			oDep.getComponentListForHost(iMutationID2, context).clear();
			// System.out.println("After clearing, before adding: "+dep.getDepString());
			oDep.getComponentListForHost(index, context).addAll(parent.getComponentListForHost(iMutationID2, context));
			// System.out.println("After adding first: "+dep.getDepString()+" clearing index "+iMutationID1);
			oDep.getComponentListForHost(iMutationID2, context).addAll(parent.getComponentListForHost(index, context));
			// System.out.println("After adding second: "+dep.getDepString()+" clearing index "+iMutationID2);
			return oDep;
		}
		throw new OutOfOptionsException("Could not find suitable swap using "+2*nECUs+" attempts.");

	}

	@Override
	public Architecture[] singlePointCrossover(Architecture oParent1, Architecture oParent2, EmbeddedSystemProblem context) throws OutOfOptionsException {
		Random random = Conf.instance().random;
		int nComponents = context.getComponentCount();
		int nECUs = context.getECUCount();


		ECUBasedDeployment parent1 = (ECUBasedDeployment) oParent1;
		ECUBasedDeployment parent2 = (ECUBasedDeployment) oParent2;
		ECUBasedDeployment[] children = new ECUBasedDeployment[2];
		children[0] = new ECUBasedDeployment(nECUs, nComponents);
		children[1] = new ECUBasedDeployment(nECUs, nComponents);
		ArrayList<LinkedList<Component>> table = new ArrayList<LinkedList<Component>>();
		//  System.out.println("New singlePointCrossover");
		for ( int i=0; i< nECUs ; i++)
		{
			LinkedList<Component> lst = new LinkedList<Component>();
			table.add(lst);
		}
		boolean [] child1Has;
		//counting whether child 1 has all its components
		boolean success = true;
		// needs timeout
		for (int x = 0; x < 2*nComponents; x++)
		{
			child1Has = new boolean[nComponents];
			int counter = 0;
			for (int i = 0; i < nECUs; i++)
			{
				children[0].getComponentListForHost(i, context).clear();
				children[1].getComponentListForHost(i, context).clear();
				table.get(i).clear();
				table.get(i).addAll(parent1.getComponentListForHost(i, context));
				table.get(i).addAll(parent2.getComponentListForHost(i, context));
			}
			int timeout = 0;

			while (counter < nComponents)
			{
				int chosenHost;
				do{
					chosenHost = random.nextInt(nECUs);
				}while (table.get(chosenHost).size() == 0);

				int chosenComp = random.nextInt(table.get(chosenHost).size());

				Component currentComp = table.get(chosenHost).get(chosenComp);

				//have to ensure that no deployment has twice the same component
				if (child1Has[currentComp.index])
				{
					if (timeout > nComponents)
					{
						success = false;
						break;
					}

					timeout++;
					continue;
				}

				if(currentComp.conflicts(children[0].getComponentListForHost(chosenHost, context)) )
				{
					if (timeout > nComponents)
					{
						success = false;
						break;
					}
					timeout++;
					continue;
				}
				//add since there are no conflicts
				children[0].getComponentListForHost(chosenHost, context).add(currentComp);
				child1Has[currentComp.index] = true;
				table.get(chosenHost).remove(currentComp);
				counter++;
				//add its buddies
				List<Component> buddies = currentComp.getMustColocateWith();
				for (Component comp : buddies)
				{
					children[0].getComponentListForHost(chosenHost, context).add(comp);
					table.get(chosenHost).remove(comp);
					child1Has[comp.index] = true;
					counter++;
				}
			}

			for (int i = 0; i < table.size(); i++)
			{
				List<Component> lst = table.get(i);
				for (int j = 0; j < lst.size(); j++)
				{
					Component comp = lst.get(j);
					if(comp.conflicts(children[1].getComponentListForHost(i, context)) )
					{
						success = false;
						break;
					}
					children[1].getComponentListForHost(i, context).add(comp);
				}
			}
			if (!success)
				continue;

			return children;
		}
		throw new OutOfOptionsException("Could not find possible crossover within "+(4*nComponents)+" attempts.");

	}

	@Override
	public Architecture uniformMutation(Architecture oArch, EmbeddedSystemProblem context, double prob) {
		int nECUs = context.getECUCount();
		Random random = Conf.instance().random;
		for (int i=0; i<nECUs;i++){

			if(random.nextDouble()<prob){
				try {
					oArch = swapMutatation(oArch, context, i);
				} catch (OutOfOptionsException ex) {
					Logger.getLogger(ECUBasedDepEAOperators.class.getName()).log(Level.SEVERE, null, ex);
				}
			}
		}
		return oArch;
	}

	@Override
	public Architecture[] uniformCrossover(Architecture oParent1, Architecture oParent2, EmbeddedSystemProblem context, double prob) {
		
		int nECUs = context.getECUCount();
		Architecture[] children = new Architecture[2];
		children[0] = oParent1;
		children[1] = oParent2;
		Random random = Conf.instance().random;
		for (int i=0; i<nECUs;i++){
                    if(random.nextDouble()<prob){
                        try {
                                children= singlePointCrossover(children[0],children[1], context);
                        } catch (OutOfOptionsException ex) {
                                Logger.getLogger(ECUBasedDepEAOperators.class.getName()).log(Level.SEVERE, null, ex);
                        }
                    }
		}
		return children;
	}

   
}
