/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package archeopterix.problem.deployment.algorithm.nsga;

import java.util.List;
import java.util.Random;

import archeopterix.algorithm.ea.EAOperators;
import archeopterix.conf.Conf;
import archeopterix.problem.Component;
import archeopterix.problem.Host;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.deployment.solution.OriginalDeploymentSolution;
import archeopterix.solution.Architecture;
import archeopterix.solution.OutOfOptionsException;

/**
 *
 * @author imeedeniya
 */
public class OriginalDepEAOperators extends EAOperators {

	public boolean canOperateOn(Class<?> solRepClass) {
		return solRepClass.equals(OriginalDeploymentSolution.class);
	}


	@Override
	public Architecture singlePointMutation(Architecture oArch, EmbeddedSystemProblem problem) throws OutOfOptionsException {
            OriginalDeploymentSolution parent = (OriginalDeploymentSolution) oArch;

            OriginalDeploymentSolution dep = new OriginalDeploymentSolution(parent);

            boolean foundOption = false;
            Component comp = null;

            Random rand = Conf.instance().random;
            int nComponents = problem.getComponentCount();

            List<Component> comps = problem.getComponents();
            List<Host> ecus = problem.getECUs();

            int iMutationID = 0;
            int targetECUIndex = -1;
            int counter = 0;

            while(!foundOption && counter < nComponents*2)
            {
                counter++;
                iMutationID = rand.nextInt(nComponents);
                comp = comps.get(iMutationID);
                int nECUs = comp.getCanUseHostListLength();
                int count = 0;

                again:
                    while(!foundOption && count < nECUs*2)
                    {
                        count++;
                        targetECUIndex = comp.getHostOption(rand.nextInt(nECUs)).index ;

                        if(!comp.isCompatibleHost(ecus.get(targetECUIndex)))
                                continue again;
                        int i;
                        for(i = 0; i < nComponents; i++)
                        {
                            if(parent.getHostIndex(i)==targetECUIndex && !comps.get(iMutationID).canColocateWith(comps.get(i)))
                                continue again;
                        }
                        if(i == nComponents)
                            foundOption = true;
                    }
             }
            if(foundOption)
            {
                dep.set(iMutationID, targetECUIndex);
                List<Component> buddies = comp.getMustColocateWith();
                for (Component buddy : buddies)
                {
                    dep.set(buddy.index, targetECUIndex);
                }
               
                return dep;
            }
            else
                throw new OutOfOptionsException("Not enough options for pointMutate.");

	}

	@Override
	public Architecture swapMutation(Architecture oArch, EmbeddedSystemProblem context) throws OutOfOptionsException {
            OriginalDeploymentSolution parent = (OriginalDeploymentSolution) oArch;
            OriginalDeploymentSolution oDep = new OriginalDeploymentSolution(parent);
            boolean coLocViolated;
            boolean swapImpossible;

            Random rand = Conf.instance().random;
            int nComponents = context.getComponentCount();
            List<Component> comps = context.getComponents();

            // if we don't find a possible swap after twice the number of components (list length)
            // we assume something is wrong and moves are impossible
            for (int i = 0; i < nComponents * 2; i++) {
                swapImpossible = false;
                coLocViolated = false;
                int iMutationID1 = rand.nextInt(nComponents);
                int iMutationID2;
                // if (comps.get(iMutationID1).mustColocate())
                //     continue;
                int ecuIndex1 = parent.getHostIndex(iMutationID1);
                int ecuIndex2 = 0;
                int timeout = 0;
                do {
                        iMutationID2 = rand.nextInt(nComponents);
                        //  if (comps.get(iMutationID2).mustColocate())
                        //     continue;
                        ecuIndex2 = parent.getHostIndex(iMutationID2);
                        if (timeout > comps.get(iMutationID1).getCanUseHostListLength()) {
                                swapImpossible = true;
                                break;
                        }
                        timeout++;
                } while (ecuIndex1 == ecuIndex2);

                if (swapImpossible) {
                        continue;
                }

                //localisation constraint

                if (!comps.get(iMutationID1).isCompatibleHost(context.getECUs().get(ecuIndex2))) {
                        continue;
                }

                if (!comps.get(iMutationID2).isCompatibleHost(context.getECUs().get(ecuIndex1))) {
                        continue;
                }


                //check all other allocations to same Host - are there incompatibilities?

                for (int j = 0; j < nComponents; j++) {
                        //assumption is that components are not prohibited from colocating with themselves
                        if (j != iMutationID1 && parent.getHostIndex(j) == ecuIndex1) {
                                if (!comps.get(iMutationID2).canColocateWith(comps.get(j))) {
                                        coLocViolated = true;
                                        break;
                                }
                        } else if (j != iMutationID2 && parent.getHostIndex(j) == ecuIndex2) {
                                if (!comps.get(iMutationID1).canColocateWith(comps.get(j))) {
                                        coLocViolated = true;
                                        break;
                                }
                        }
                }
                if (coLocViolated) {
                        continue;
                }
                //assuming here that colocation restrictions and poss Host assignments are common to the group members.
                if (comps.get(iMutationID1).mustColocate()) {
                        List<Component> sourceGroupMembers = comps.get(iMutationID1).getMustColocateWith();
                        for (Component comp : sourceGroupMembers) {
                                oDep.set(comp.index, ecuIndex2);
                        }
                }

                if (comps.get(iMutationID2).mustColocate()) {
                        List<Component> targetGroupMembers = comps.get(iMutationID2).getMustColocateWith();
                        for (Component comp : targetGroupMembers) {
                                oDep.set(comp.index, ecuIndex1);
                        }

                }
                oDep.set(iMutationID1, ecuIndex2);
                oDep.set(iMutationID2, ecuIndex1);
                //System.out.println("Swapped "+iMutationID1+", host "+ecuIndex1+ " with "+iMutationID2+", host "+ecuIndex2);
                return oDep;

            }
            throw new OutOfOptionsException("Could not find suitable swap using " + 2 * nComponents + " attempts.");

	}

	@Override
	public Architecture[] singlePointCrossover(Architecture oParent1, Architecture oParent2, EmbeddedSystemProblem context) throws OutOfOptionsException {
		int iIndex ;
		OriginalDeploymentSolution[] parents = {(OriginalDeploymentSolution)oParent1, (OriginalDeploymentSolution) oParent2};
		OriginalDeploymentSolution[] oOffspring = { new OriginalDeploymentSolution( parents[0] ), new OriginalDeploymentSolution( parents[1] ) };
		int[] parentHosts = new int[2];

		boolean choiceIsFixed = false;
		boolean outOfOptions = false;
		boolean swap ;

		Random rand = Conf.instance().random;
		int nComponents = context.getComponentCount();
		List<Component> comps = context.getComponents();


		//FIXME Returns a solution during first iteration; should it do something different?
		for(int timeout = 0; timeout < nComponents*2; timeout++)
		{
			do
			{
				iIndex = rand.nextInt(nComponents);
			}while (iIndex == 0 || iIndex == (nComponents-1));
			for (int i=iIndex; i<nComponents;i++)
			{
				swap = true;
				Component currentComp = comps.get(i);
				parentHosts[0] = (parents[0].getHostIndex(i));
				parentHosts[1] = (parents[1].getHostIndex(i));
				if (currentComp.mustColocate())
				{
					List<Component> colocationGroup = currentComp.getMustColocateWith(); // see if any of the buddies is already assigned
					for (Component component : colocationGroup)
					{
						if (component.index < i)//we have already assigned it
						{
							choiceIsFixed = true;

							if (oOffspring[0].getHostIndex(component.index) == parentHosts[0])
								swap = false;
						}
					}
				}
				List<Component> enemies = currentComp.getMustNotColocateWith();
				for (Component component : enemies)
				{
					if (component.index < i)//we have already assigned it
					{
						if (swap)
						{
							if (oOffspring[0].getHostIndex(component.index) == parentHosts[1]
								|| oOffspring[1].getHostIndex(component.index) == parentHosts[0])
							{
								if (choiceIsFixed)
									outOfOptions = true;
								else
								{
									swap = !swap;
									choiceIsFixed = true;
								}
							}
						}
						if(!swap)
						{
							if (oOffspring[0].getHostIndex(component.index) == parentHosts[0]
								|| oOffspring[1].getHostIndex(component.index) == parentHosts[1])
							{
								if (choiceIsFixed)
									outOfOptions = true;
								else
									swap = !swap;
							}
						}
					}
				}
				if (outOfOptions)
				{
					outOfOptions = false;
					timeout++;
					continue;
				}
				if (swap)
				{
					oOffspring[0].set(i, parentHosts[1]);
					oOffspring[1].set(i, parentHosts[0]);
				}
				else
				{
					oOffspring[0].set(i, parentHosts[0]);
					oOffspring[1].set(i, parentHosts[1]);
				}
			}
			return oOffspring;
		}
		throw new OutOfOptionsException("Could not find suitable crossover using " + context.getComponentCount() + " attempts.");

	}

	@Override
	public Architecture uniformMutation(Architecture oArch, EmbeddedSystemProblem context, double prob) {
		throw new UnsupportedOperationException("Not supported yet.");
	}

	@Override
	public Architecture[] uniformCrossover(Architecture oParent1, Architecture oParent2, EmbeddedSystemProblem context, double prob) {
		throw new UnsupportedOperationException("Not supported yet.");
	}
}
