/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package archeopterix.problem.deployment.algorithm.sa;

import archeopterix.algorithm.sa.*;
import archeopterix.conf.Conf;
import archeopterix.problem.Component;
import archeopterix.problem.Host;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.deployment.solution.ECUBasedDeployment;
import archeopterix.solution.Architecture;
import archeopterix.solution.OutOfOptionsException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 *
 * @author imeedeniya
 */
public class ECUBasedDepSAOperators extends SAOperators {

	// Random variable
	private static final Log log = LogFactory.getLog(ECUBasedDepSAOperators.class);
	private Random rnd;
	int noComps;
	int noECUs;
	List<Component> componentsList = null;
	List<Host> ecusList = null;
	/**
	 * For each component the total memory size required by it and any other
	 * components with which it must be colocated.
	 */
	private int[] compWithGroupSizes;

	public ECUBasedDepSAOperators() {
		rnd = Conf.instance().random;
	}

	@Override
	public Architecture mutationMove(Architecture oArch, EmbeddedSystemProblem oProblem) throws OutOfOptionsException {
		//return swapMutate(oArch, oProblem);
		return newSAMutate(oArch, oProblem);
	}

	public boolean canOperateOn(Class<?> solRepClass) {
		return solRepClass.equals(ECUBasedDeployment.class);
	}

	private void init(EmbeddedSystemProblem oProblem) {
		noECUs = oProblem.getECUCount();
		noComps = oProblem.getComponentCount();
		componentsList = oProblem.getComponents();
		ecusList = oProblem.getECUs();

		//Preprocess component sizes to create fast lookup of total size of component and its buddies
		if (compWithGroupSizes == null || compWithGroupSizes.length != noComps) {
			compWithGroupSizes = new int[noComps];
		}
		Arrays.fill(compWithGroupSizes, -1); //just in case there exists a component with size zero
		for (int i = 0; i < compWithGroupSizes.length; i++) {
			if (compWithGroupSizes[i] == -1) {
				int totalSize = componentsList.get(i).getIntProperty(Component.MEMORY);
				for (Component buddy : componentsList.get(i).getMustColocateWith()) {
					totalSize += buddy.getIntProperty(Component.MEMORY);
				}
				compWithGroupSizes[i] = totalSize;
				for (Component buddy : componentsList.get(i).getMustColocateWith()) {
					compWithGroupSizes[buddy.index] = totalSize;
				}
			}
		}
	}

	//----------------------- Copied from NSGA Operators
	public Architecture swapMutate(Architecture oArch, EmbeddedSystemProblem context) throws OutOfOptionsException {
		ECUBasedDeployment parent = (ECUBasedDeployment) oArch;
		Random random = Conf.instance().random;
		int nECUs = context.getECUCount();

		List<Host> hosts = context.getECUs();
		boolean allocationImpossible;
		for (int i = 0; i < nECUs * 2; i++) {
			allocationImpossible = false;
			int iMutationID1 = random.nextInt(nECUs);
			int iMutationID2;
			do {
				iMutationID2 = random.nextInt(nECUs);
			} while (iMutationID1 == iMutationID2);
			ECUBasedDeployment oDep = new ECUBasedDeployment(parent);
			Iterator<Component> compsIterator1 = oDep.getComponentListForHost(iMutationID1, context).iterator();

			while (compsIterator1.hasNext()) {
				if (!compsIterator1.next().isCompatibleHost(hosts.get(iMutationID2))) {
					allocationImpossible = true;
					break;
				}
			}
			if (allocationImpossible) {
				break;
			}

			Iterator<Component> compsIterator2 = oDep.getComponentListForHost(iMutationID2, context).iterator();

			while (compsIterator2.hasNext()) {
				if (!compsIterator2.next().isCompatibleHost(hosts.get(iMutationID1))) {
					allocationImpossible = true;
					break;
				}
			}
			if (allocationImpossible) {
				continue;
			}
			//dep is a copy and only two lists have to be adjusted
			oDep.getComponentListForHost(iMutationID1, context).clear();
			oDep.getComponentListForHost(iMutationID2, context).clear();
			// System.out.println("After clearing, before adding: "+dep.getDepString());
			oDep.getComponentListForHost(iMutationID1, context).addAll(parent.getComponentListForHost(iMutationID2, context));
			// System.out.println("After adding first: "+dep.getDepString()+" clearing index "+iMutationID1);
			oDep.getComponentListForHost(iMutationID2, context).addAll(parent.getComponentListForHost(iMutationID1, context));
			// System.out.println("After adding second: "+dep.getDepString()+" clearing index "+iMutationID2);
			return oDep;
		}
		throw new OutOfOptionsException("Could not find suitable swap using " + 2 * nECUs + " attempts.");
	}

	public Architecture newSAMutate(Architecture oArch, EmbeddedSystemProblem oProblem) throws OutOfOptionsException {
		ECUBasedDeployment parent = (ECUBasedDeployment) oArch;
		ECUBasedDeployment oChild = new ECUBasedDeployment(parent);
		if (componentsList == null) //If not intialized, do it now.
		{
			init(oProblem);
		}
		// Select a component randomly for relocation
		Component comp = componentsList.get(rnd.nextInt(noComps));

		// get the list of possible Host's as well as the
		// base Host of this component (using the solution)
		// and a randomly selected target Host for relocation
		List<Host> possibleEcus = new LinkedList<Host>(comp.getCanUseHosts());
		Host baseEcu = ecusList.get(oChild.getHostIndex(comp.index));
		possibleEcus.remove(baseEcu);
		Host targetEcu = possibleEcus.get(rnd.nextInt(possibleEcus.size()));

		// Check if component group fits on target Host
		int compGroupSize = compWithGroupSizes[comp.index];
		if (compGroupSize > targetEcu.getIntProperty(Host.MEMORY)) {
			throw new OutOfOptionsException("Not enough options for Mutation Move[GRP].");
		}

		// create lists for the components on the target Host with
		// components that must be relocated to the base due to colocation
		// conflict and components that not necessarily have to
		int sizeCompsOnTarget = 0;
		int sizeCompsMustMove = 0;
		List<Component> compsOnTarget = oChild.getComponentListForHost(targetEcu.index, oProblem);
		List<Component> compsMustMove = new LinkedList<Component>();
		for (Component c : compsOnTarget) {
			if (!comp.canColocateWith(c)) {
				compsMustMove.add(c);
				sizeCompsMustMove += c.getIntProperty(Component.MEMORY);
			} else {
				sizeCompsOnTarget += c.getIntProperty(Component.MEMORY);
			}
		}


		// get list and memory size of components on base Host (without the
		// component to be relocated)
		List<Component> compsOnBase = oChild.getComponentListForHost(baseEcu.index, oProblem);
		compsOnBase.remove(comp);
		if (comp.mustColocate()) {
			for (Component c : comp.getMustColocateWith()) {
				compsOnBase.remove(c);
			}
		}

		int sizeCompsOnBase = 0;
		for (Component c : compsOnBase) {
			sizeCompsOnBase += c.getIntProperty(Component.MEMORY);
		}

		// try to move all colocation conflicts to the base Host
		if (sizeCompsMustMove + sizeCompsOnBase > baseEcu.getIntProperty(Host.MEMORY)) {
			throw new OutOfOptionsException("Not enough options for Mutation Move [MEM].");
		}
		if (!compsMustMove.isEmpty()) {
			for (Component m : compsMustMove) {
				for (Component b : compsOnBase) {
					if (!m.canColocateWith(b)) {
						throw new OutOfOptionsException("Not enough options for Mutation Move[COL].");
					}
				}
			}
			compsOnBase.addAll(compsMustMove);
			compsOnTarget.removeAll(compsMustMove);
			sizeCompsOnBase += sizeCompsMustMove;
		}

		// try to move the target component (and its group) to the target Host
		// if this is not possible due to memory constraints try to move components
		// from the target Host to the base Host
		boolean compMoved = false;
		while (!compMoved) {
			if (sizeCompsOnTarget > targetEcu.getIntProperty(Host.MEMORY)) {
				if (compsOnTarget.isEmpty())//this means the initial comp by itself is too big for the target
				{
					if (log.isDebugEnabled()) {
						log.debug("Comp to shift is too big for target");
					}
					throw new OutOfOptionsException("Not enough options for Mutation Move[TB].");
				} else {
					Component compMoveToBase = compsOnTarget.get(rnd.nextInt(compsOnTarget.size()));
					if (compMoveToBase.getIntProperty(Component.MEMORY) + sizeCompsOnBase <= baseEcu.getIntProperty(Host.MEMORY)) {
						for (Component b : compsOnBase) {
							if (!compMoveToBase.canColocateWith(b)) {
								throw new OutOfOptionsException("Not enough options for Mutation Move[TD]."); //too difficult. giving up
							}
						}
						compsOnBase.add(compMoveToBase);
						compsOnTarget.remove(compMoveToBase);
						sizeCompsOnBase += compMoveToBase.getIntProperty(Component.MEMORY);
						sizeCompsOnTarget -= compMoveToBase.getIntProperty(Component.MEMORY);
					} else {
						throw new OutOfOptionsException("Not enough options for Mutation Move[NR]."); //too difficult; base has no room for comps moved out of the way
					}
				}

			} else// if(!compsOnTarget.isEmpty())//how could it be empty if the size of the resident components is bigger than the memory size?
			{
				compMoved = true;
				//do this last
				compsOnTarget.add(comp);
			}
		}

		if (!compMoved) {
			throw new OutOfOptionsException("Not enough options for Mutation Move[FN].");
		}

		return oChild;
	}
}
