/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package archeopterix.problem.deployment.constraints;

import archeopterix.constraints.ConstraintHandler;
import archeopterix.problem.Component;
import archeopterix.problem.InteractionProperties;
import archeopterix.problem.NetworkProperties;
import archeopterix.problem.deployment.Deployment;
import archeopterix.problem.deployment.solution.DeploymentSolution;
import archeopterix.solution.Architecture;
import java.util.List;

/**
 *
 * @author indika
 */
public class DeploymentBandwidthConstraint extends ConstraintHandler{

    public DeploymentBandwidthConstraint(Deployment oProblem){
        super(oProblem);
    }
    @Override
    protected boolean Validate(Architecture oArch) {
        DeploymentSolution oDep = (DeploymentSolution) oArch;
        int iECUs = m_Problem.getECUCount();
        for(int iHi=0; iHi<iECUs; iHi++){
            for(int iHj=0; iHj<iECUs; iHj++){
                if(iHi!=iHj){
                    if( ! checkBandwidthCapacityBetweenTwoHosts(oDep, iHi, iHj))
                        return false;
                }
            }
        }
        return true;
    }

    private boolean checkBandwidthCapacityBetweenTwoHosts(DeploymentSolution oDep, int iHi, int iHj){
        Deployment oDepProblem = (Deployment)m_Problem;
        List<Component> lstCompsi = oDep.getComponentListForHost(iHi, oDepProblem) ;
        List<Component> lstCompsj = oDep.getComponentListForHost(iHj, oDepProblem) ;
        int iDRij 	= oDepProblem.getNetworkProperties(iHi, iHj).getInt(NetworkProperties.DATA_RATE_KBPS );
        for(int obj=0; obj<3;obj++){
            for(int i=0; i< lstCompsi.size(); i++){
                int iCi = lstCompsi.get(i).index;
                for (int j=0; j<lstCompsj.size(); j++){
                    int iCj = lstCompsj.get(j).index;
                    if ( oDepProblem.areInteracting(iCi, iCj)){
                        Double iCLij = oDepProblem.getInteractionProperties(iCi, iCj).getDoubleFromList(InteractionProperties.COMM_LOAD_KB, 0 ); //SERVICE 0
                        double dFRij = oDepProblem.getInteractionProperties(iCi, iCj).getDouble( InteractionProperties.FREQUENCY  );
                        iDRij -= (iCLij* dFRij);
                    }
                    if (iDRij <= 0){
                        return false;
                    }
                }
            }
        }
        return true;
    }

    private int getBandwidthViolationBetweenTwoHosts(DeploymentSolution oDep, int iHi, int iHj){
        int violation=0;
        Deployment oDepProblem = (Deployment)m_Problem;
        List<Component> lstCompsi = oDep.getComponentListForHost(iHi, oDepProblem) ;
        List<Component> lstCompsj = oDep.getComponentListForHost(iHj, oDepProblem) ;
        if(oDepProblem.getNetworkProperties(iHi, iHj)!=null){
            int iDRij = oDepProblem.getNetworkProperties(iHi, iHj).getInt(NetworkProperties.DATA_RATE_KBPS );
            for(int obj=0; obj<3;obj++){
                for(int i=0; i< lstCompsi.size(); i++){
                    int iCi = lstCompsi.get(i).index;
                    for (int j=0; j<lstCompsj.size(); j++){
                        int iCj = lstCompsj.get(j).index;
                        if ( oDepProblem.areInteracting(iCi, iCj)){
                            double iCLij = oDepProblem.getInteractionProperties(iCi, iCj).getDoubleFromList(InteractionProperties.COMM_LOAD_KB, 0); //SERVICE 0
                            double dFRij = oDepProblem.getInteractionProperties(iCi, iCj).getDouble( InteractionProperties.FREQUENCY  );
                            iDRij -= (iCLij* dFRij);
                        }
                        if (iDRij <= 0){
                            violation+=-iDRij;
                        }
                    }
                }
            }
        }
        return violation;
    }
    @Override
    public int quantifySatisfaction(Architecture oArch) {
        int violations = 0;
        DeploymentSolution oDep = (DeploymentSolution) oArch;
        int iECUs = m_Problem.getECUCount();
        for(int iHi=0; iHi<iECUs; iHi++){
            for(int iHj=0; iHj<iECUs; iHj++){
                violations=violations+getBandwidthViolationBetweenTwoHosts(oDep, iHi, iHj);
            }
        }
        return violations;
     }
}
