/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package archeopterix.problem.generators;

import archeopterix.problem.Component;
import archeopterix.problem.Host;
import archeopterix.problem.InteractionProperties;
import archeopterix.problem.NetworkProperties;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.util.statistics.NormalDistribution;
import archeopterix.util.statistics.ProbabilityDistribution;
import java.util.Arrays;
import java.util.List;
import java.util.Random;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import archeopterix.util.PropertyMap;
import archeopterix.util.matrix.Matrix;
import archeopterix.util.probability.DiscreteDistribution;
import java.util.LinkedList;


public class EmbeddedSystem {
     private static final Log log = LogFactory.getLog(EmbeddedSystem.class);

    private static EmbeddedSystem m_Instance = null;

    private Random random;
    private List<Component>lst_Comps;
    private List<Host> lst_ECUs;
    private List<List<PropertyMap>> mat_Network;
    private List<List<PropertyMap>> mat_Interactions;
    //Temprary variables
    Integer[][] mat_CoLocation;
    Integer[][] mat_Localization;
    Matrix[]    amat_Pij;
    Matrix[]    amat_Q0;

    private int i_ECUs;
    private int i_Comps;
    private int i_Buses;
    private int i_Services;

    public synchronized static EmbeddedSystem instance() {
        if (m_Instance == null)
            m_Instance = new EmbeddedSystem();
        return m_Instance;
    }
  public EmbeddedSystemProblem generateInstance(int iECUs, int iComps, int iBuses, int iServices){
    random = new Random(System.currentTimeMillis());
    i_ECUs = iECUs;
    i_Comps = iComps;
    i_Buses = iBuses;
    i_Services = iServices;
    lst_Comps   = new LinkedList<Component>();
    lst_ECUs         =  new LinkedList<Host>();
    mat_Network      = new LinkedList<List<PropertyMap>>();
    mat_Interactions = new LinkedList<List<PropertyMap>>();

    List<PropertyMap> lstBuses = new LinkedList<PropertyMap>();
    //Create Tempprary matrices for transition probabilities and init probs
    createTransistionMatrices();
    for (int i=0; i<i_ECUs; i++){
        List<PropertyMap> oNetworkList = new LinkedList<PropertyMap>();
        for (int j=0; j<i_ECUs; j++){
            oNetworkList.add(null);
        }
        mat_Network.add(oNetworkList);
    }
    for (int i=0; i<i_Comps; i++){
        List<PropertyMap> oInterconList = new LinkedList<PropertyMap>();
        for (int j=0; j<i_Comps; j++){
            oInterconList.add(null);
        }
        mat_Interactions.add(oInterconList);
    }
    //Creating ECUs
    for ( int i=0; i<i_ECUs; i++){
        PropertyMap oPropMap = new PropertyMap();
        int iMIPS = (random.nextInt(6)+1)* 20 ; //20 *(1~6)
        int iLambdaClass = random.nextInt(3);
        double dLamda 	= ( random.nextInt(5) + 1 ) * Math.pow( 10 , iLambdaClass );
                // (1~5) * 1E(0~2)
        int iMem = random.nextInt(2) + 9 ;  //2^(9~10)
        oPropMap.put(Host.MEMORY, (int)Math.pow(2, iMem) );
        oPropMap.put(Host.SPEED_MIPS, iMIPS );
        double dFR = dLamda * 1E-5;
        double dRR = dLamda * 1E-2;
        oPropMap.put(Host.FAILURE_RATE, dFR  );
        oPropMap.put(Host.REPAIR_RATE, dRR );
        oPropMap.put(Host.SOJOURN_TIME, random.nextInt(5) );
        oPropMap.put(Host.FAILURE_RATE_DT, new DiscreteDistribution( new double[]{ dFR * 0.1, 0.2, dFR, 0.4, dFR*5, 0.2, dFR*10, 0.2} ) );
        oPropMap.put(Host.COST, (double)((iMem+iMIPS- iLambdaClass) * 5) );
        List<Integer> oLocList = new LinkedList<Integer>();
        //Create empt localizations
        for (int j=0; j<iComps; j++){
            oLocList.add(0);
        }
        oPropMap.put( Host.LOC_LIST, oLocList );
        Host oECU = new Host(Host.LABEL + i, i, oPropMap);
        lst_ECUs.add(oECU);
    }
    //Creating components
    for ( int i=0; i<i_Comps; i++){
        PropertyMap oPropMap = new PropertyMap();
        int iMem = random.nextInt(3) + 6 ; // Range between 2^(6-8)
        List<Double> lstMIs = new LinkedList<Double>();
        List<Double> lstSTs = new LinkedList<Double>();
        List<Double> lstInitProb = new LinkedList<Double>();
        List<Double> lstEnergyPerVisit =new LinkedList<Double>();
        List<Double> lstIdleEnergy =new LinkedList<Double>();
        List<ProbabilityDistribution> lstInitProbDT = new LinkedList<ProbabilityDistribution>();
        for (int iSrv=0; iSrv<i_Services; iSrv++){
            double dVal = amat_Q0[iSrv].get(i, 0);
            double dMI = random.nextInt(7) * 0.4;
            lstMIs.add(dMI);
            lstInitProb.add(dVal);
            lstInitProbDT.add( new NormalDistribution(dVal, 0.001) );
            lstEnergyPerVisit.add( (random.nextInt(5)+1)*10.0); //10~50 mJ
            lstIdleEnergy.add( (random.nextInt(10)+1)*0.5); //1~5 mW
            lstSTs.add (dMI*5.0); //FIXME:: Sojurn Time does not belong to components
        }
        if (i_Services > 0) {
            oPropMap.put( Component.COMPUTATIONS_MI , lstMIs );
            oPropMap.put( Component.ENERGY_PER_VISIT , lstEnergyPerVisit );
            oPropMap.put( Component.IDLE_ENERGY , lstIdleEnergy );
            oPropMap.put( Component.INITILIZE_PROB , lstInitProb );
            oPropMap.put( Component.INITILIZE_PROB_DT ,  lstInitProbDT );
            oPropMap.put( Component.SOJOURN_TIME , lstSTs ); //FIXME:: Sojurn Time does not belong to components
        }
        else
            oPropMap.put( Component.COMPUTATIONS_MI , (random.nextInt(6)+1) * 0.4 );
            int iLambdaClass = random.nextInt(3);
            double dLamda 	= ( random.nextInt(5) + 1 ) * Math.pow( 10 , iLambdaClass );
            double dFR = dLamda *10 ; //* 1E-5;
            oPropMap.put(Component.FAILURE_RATE, dFR  );
            oPropMap.put(Component.COST, (double)((iMem - iLambdaClass) * 5) );
            oPropMap.put( Component.MEMORY , (int)Math.pow(2, iMem) );
            
            
            //Colocalization list
            //oPropMap.put(Component.COLOC_LIST, Arrays.asList( coLocMatrix[i] ) );

            //this will be needed if we go for service reliability
            /*List<Integer> lstWorkLoad = new LinkedList<Integer>();
            for (int j=0; j<Conf.Ins().i_Services; j++){
                    int iWorkLoad = random.nextInt(11)+10;
                    lstWorkLoad.add( (int)Math.pow(2, iWorkLoad)); // 2^10~20 MIs
            }
            oPropMap.put( Defs.STR_COMP +"::" +  Defs.COMPUTATIONS_MI , lstWorkLoad);
             */
            Component oComp = new Component(Component.LABEL + i, i, oPropMap);
            lst_Comps.add(oComp);
        }

        //Busses
        for (int i=0; i< i_Buses; i++){
            PropertyMap oBus = new PropertyMap();
            double dLamda = (random.nextInt(8)+3) * Math.pow( 10 , random.nextInt(2) ); // 2~10 E^(-3~-4)
            double dFR = dLamda * 1E-4 ;
            oBus.put(NetworkProperties.FAILURE_RATE, dFR);
            oBus.put(NetworkProperties.FAILURE_RATE_DT, new DiscreteDistribution( new double[]{dFR, 1.0} ) );
            int iKBPS = random.nextInt(6)+ 5; //2^5~10
            oBus.put(NetworkProperties.DATA_RATE_KBPS ,(int)Math.pow(2, iKBPS) );
            double dFailureRate = random.nextInt(1000) * 1E-5; //0.01 ~ 0.00001
            oBus.put( NetworkProperties.RELIABILITY , (1 - dFailureRate));
            oBus.put( NetworkProperties.NW_DELAY ,random.nextDouble()* 0.2 ); //1~100 ms
            lstBuses.add(oBus);
        }


        //Create temporary arrays
        List<Integer>[] alstECUsOfBus = (List<Integer>[]) new LinkedList<?>[i_Buses];
        List<Integer>[] alstCompsOfBus = (List<Integer>[]) new LinkedList<?>[i_Buses];
        mat_CoLocation = new Integer[i_Comps][i_Comps];
        for (int i = 0; i < i_Comps; i++) {
                Arrays.fill(mat_CoLocation[i], 0);
        }
        mat_Localization = new Integer[i_ECUs][i_Comps];
        for (int i = 0; i < i_ECUs; i++) {
                Arrays.fill(mat_Localization[i], 0);
        }
        for (int i=0;i<i_Buses; i++){
            alstECUsOfBus[i] = new LinkedList<Integer>();
            alstCompsOfBus[i] = new LinkedList<Integer>();
        }
        
        //Assign ECUs for Buses
        for(int i=0; i<i_ECUs; i++){
            int iBus = random.nextInt(i_Buses);
            alstECUsOfBus[iBus].add(i);
        }
        //Similarly, Assign Comps for Buses
        for(int i=0; i<i_Comps; i++){
            int iBus = random.nextInt(i_Buses);
            alstCompsOfBus[iBus].add(i);
        }

        //Problem without GW concept. Removed the newly implemented GW concept, in oreder to comply with the Old AOCmd Code.
        // We may enable it later.
        for(int iB=0; iB<i_Buses; iB++){
            if (log.isDebugEnabled())
            log.debug("Bus["+iB+"]:ECUS:"+alstECUsOfBus[iB].toString()+
                    " Comps:"+ alstCompsOfBus[iB].toString());
            localizeComponents( alstECUsOfBus[iB], alstCompsOfBus[iB]);
            colocateComponents( alstCompsOfBus[iB] );
            //--mat_Localization[iGWECU][iGWComp] = 1;

            establishNetworks(alstECUsOfBus[iB], lstBuses.get(iB));
            establishInteractions(alstCompsOfBus[iB]);
        }
/*
 * To enable the GW concept, comment the above, and uncomment the following block.
 * At the same time, needs to fix the localizeComponenst method as well.
 * 
        if (log.isDebugEnabled())
    		log.debug("Bus["+ 0+"]:ECUS:"+alstECUsOfBus[0].toString()+ " Comps:"+ alstCompsOfBus[0].toString());
        //Select Gateway ECUs, GWComps for Each Bus
        List<Integer> lstGWECUs = new LinkedList<Integer>();
        List<Integer> lstGWComps = new LinkedList<Integer>();
        for(int iB=1; iB<i_Buses; iB++){
            int iGWECU = 0;
            int iGWComp = 0;
            if (alstECUsOfBus[iB].size()>0)
                iGWECU = alstECUsOfBus[iB].get(0);
            else if (alstECUsOfBus[0].size()>0)
                iGWECU =  alstECUsOfBus[0].get(0);
            else
            	if (log.isDebugEnabled())
            		log.debug("ERR:Could not find a GW Host for Bus:"+iB);

            if (alstCompsOfBus[iB].size()>0)
                iGWComp = alstCompsOfBus[iB].get(0);
            else if (alstCompsOfBus[0].size()>0)
                iGWComp = alstCompsOfBus[iB].get(0);
            else
            	if (log.isDebugEnabled())
            		log.debug("ERR:Could not find a GW Comp for Bus:"+iB);

            if (log.isDebugEnabled())
        		log.debug("Bus["+iB+"]:ECUS:"+alstECUsOfBus[iB].toString()+
        				" Comps:"+ alstCompsOfBus[iB].toString()+" GWECU:"+ iGWECU+" GWComp:"+ iGWComp);
            lstGWECUs.add(iGWECU);
            lstGWComps.add(iGWComp);

            localizeComponents( alstECUsOfBus[iB], alstCompsOfBus[iB], iGWComp);
            colocateComponents( alstCompsOfBus[iB] );
            mat_Localization[iGWECU][iGWComp] = 1;
            
            establishNetworks(alstECUsOfBus[iB], lstBuses.get(iB));
            establishInterconnections(alstCompsOfBus[iB]);
        }
        localizeComponents( alstECUsOfBus[0], alstCompsOfBus[0], -1); // -1 to skip
        colocateComponents( alstCompsOfBus[0] );
        
        List<Integer> lstECUsForBus0 = new LinkedList<Integer>();
        lstECUsForBus0.addAll(alstECUsOfBus[0]);
        lstECUsForBus0.addAll(lstGWECUs);//TODO::there is a posibility that some from Bus0 will be also there
        establishNetworks(lstECUsForBus0, lstBuses.get(0)); //Bus0 is assumed to be the gw bus

        List<Integer> lstCompsForBus0 = new LinkedList<Integer>();
        lstCompsForBus0.addAll(alstCompsOfBus[0]);
        lstCompsForBus0.addAll(lstGWComps);//TODO::there is a posibility that some from Bus0 will be also there
        //Establish communication //Bus0 is assumed to be the gw bus
        establishInterconnections(lstCompsForBus0);
*/
        //Update the componet's colocation list
        for(int i=0;i<i_Comps; i++){
            PropertyMap oPropMap = lst_Comps.get(i).getPropertyMap();
            List<Integer> lstcoloc = new LinkedList<Integer>();
            for (int j=0; j<i_Comps; j++)
                lstcoloc.add(mat_CoLocation[i][j]);
            oPropMap.put(Component.COLOC_LIST, lstcoloc);
        }

        //Update the ECUs localiztion list
        for(int i=0;i<i_ECUs; i++){
            PropertyMap oPropMap = lst_ECUs.get(i).getPropertyMap();
            List<Integer> lstloc = new LinkedList<Integer>();
            for (int j=0; j<i_Comps; j++)
                lstloc.add(mat_Localization[i][j]);
            oPropMap.put(Host.LOC_LIST, lstloc );
        }
        return new EmbeddedSystemProblem(lst_ECUs,lst_Comps, mat_Network,mat_Interactions);
    }


    
    private void establishInteractions(List<Integer> lstCompsOfBus){
        for(int i=0; i<lstCompsOfBus.size(); i++){
            int iCi = lstCompsOfBus.get(i);
            for (int j=0; j<lstCompsOfBus.size(); j++){
                int iCj = lstCompsOfBus.get(j);
                if ( (iCi!=iCj) && (random.nextDouble()<1.00)){  // % Interacts network
                    PropertyMap oInteraction = new PropertyMap();
                    List<Double> lstCommLoad = new LinkedList<Double>();
                    List<Double> lstTransProb = new LinkedList<Double>();
                    List<Double> lstEnergyPerTr = new LinkedList<Double>();
                    List<ProbabilityDistribution> lstTransProbDT = new LinkedList<ProbabilityDistribution>();
                    for (int iSrv=0; iSrv<i_Services; iSrv++){
                        double dVal = amat_Pij[iSrv].get(iCi, iCj);
                        int iCommLoad = random.nextInt(3);
                        lstCommLoad.add( (double)iCommLoad); // 0~4
                        lstTransProb.add(dVal);
                        lstTransProbDT.add ( new NormalDistribution(dVal, 0.01) );
                        lstEnergyPerTr.add( (random.nextInt(9)+4)*5.0 ); //20~60 in 5 intervals
                    }
                    if (i_Services > 0) {
                    	oInteraction.put(InteractionProperties.COMM_LOAD_KB, lstCommLoad);
                    	oInteraction.put(InteractionProperties.ENERGY_PER_TRANSITION, lstEnergyPerTr);
                    	oInteraction.put(InteractionProperties.TRANSITION_PROB , lstTransProb);
                    	oInteraction.put( InteractionProperties.TRANSITION_PROB_DT ,  lstTransProbDT );
                    }
                    oInteraction.put(InteractionProperties.FREQUENCY , random.nextDouble());
                    int iEvtSize = random.nextInt(3)+3;
                    oInteraction.put(InteractionProperties.EVENT_SIZE , Math.pow(2, iEvtSize)); //2^(3~5)
                    oInteraction.put(InteractionProperties.MEAN_EXEC_TIME , random.nextDouble());
                    setInteraction(iCi, iCj, oInteraction);
                }
            }
        }
    }
    private void establishNetworks(List<Integer> lstECUsofBus,  PropertyMap oBus){
        for(int i=0; i<lstECUsofBus.size(); i++){
            int iHi = lstECUsofBus.get(i);
            for (int j=i+1; j<lstECUsofBus.size(); j++){
                int iHj = lstECUsofBus.get(j);
                setNetwork(iHi, iHj, oBus);
            }
        }
    }
    
    private void setNetwork(int iHi, int iHj, PropertyMap oBus){
        //TODO Implement
        List<PropertyMap> oNetworkList = mat_Network.get(iHi);
        oNetworkList.set(iHj, oBus);
        oNetworkList = mat_Network.get(iHj);
        oNetworkList.set(iHi, oBus);
        // Should update iHi->iHj and iHj-iHi both
    }

    private void setInteraction(int iCi, int iCj, PropertyMap oIntercon){
        mat_Interactions.get(iCi).set(iCj,oIntercon);
    }
    private int selectGWECUforBus(Integer[] aIECUsBus, int iBusID){
        for(int i=0;i<aIECUsBus.length; i++){
            if (aIECUsBus[i]==iBusID)
                return i;
        }
        System.out.println("ERR: Could not find a GW for BUS:"+ iBusID);
        return random.nextInt(i_ECUs);
    }

    private void localizeComponents(List<Integer> lstECUsofBus, List<Integer> lstCompsofBus){//, int iGWComp){
        for(int i=0;i<lstECUsofBus.size(); i++){
            int iECU = lstECUsofBus.get(i);
            for(int j=0; j<lstCompsofBus.size(); j++){
              int iComp = lstCompsofBus.get(j);
              //if (iComp != iGWComp) //GW components' loclalization is set somewhere else
                mat_Localization[iECU][iComp] = 1;
            }
        }
    }

    private void colocateComponents(List<Integer> lstCompsofBus){
        
     /*   for(int iC=0; iC<lstCompsofBus.size(); ){ //iC++ will be cloed in sub loops
            //Create group
            List<Integer> lstGrp= new LinkedList<Integer>();
            lstGrp.clear();
            int rand = random.nextInt(2)+2;
            for(int iLoc=0; iLoc<rand && iC<lstCompsofBus.size() ; iLoc++,iC++) {// Loop may run for 2 or 3 times
                lstGrp.add( lstCompsofBus.get(iC));
            }
            collocateToAGroup(lstGrp);

            //Let some non-related components
            iC+= 4;

            //Make two from the rest not to stay together
            /*if ( (iC+2) < lstCompsofBus.size()){
                int iC1 = lstCompsofBus.get(iC);
                int iC2 = lstCompsofBus.get(iC+1);
                mat_CoLocation[iC1][iC2] = mat_CoLocation[iC2][iC1] = -1;
                if (log.isDebugEnabled())
    				log.debug("Inverse coloclize :"+ iC1+ ", " + iC2 );
                iC+=2;
            }*/
       // }

        //Add exclusions
        for (int i = 0; i < lstCompsofBus.size(); i++) {
            int iCi = lstCompsofBus.get(i);
            for (int j=i+1; j<lstCompsofBus.size(); j++) {
                int iCj = lstCompsofBus.get(j);
                if ( (mat_CoLocation[iCi][iCj] == 0) && (iCi != iCj) && (random.nextDouble() < 0.1) ) {
                    mat_CoLocation[iCi][iCj] = mat_CoLocation[iCj][iCi] = -1;
                    //if (log.isDebugEnabled())
                      //  log.debug("Inverse coloclize :"+ iCi+ ", " + iCj );
                }
            }
        }

    }

    private void collocateToAGroup(List<Integer> lstComps){
    	if (log.isDebugEnabled())
    		log.debug("Creating a group of components:"+ lstComps.toString());
        for(int i=0; i<lstComps.size(); i++){
            int iCi = lstComps.get(i);
            //mat_CoLocation[iCi][iCi] = 0;
            for(int j=0; j<lstComps.size(); j++){
                int iCj = lstComps.get(j);
                if (iCi== iCj)
                    mat_CoLocation[iCi][iCj] = 0;//Required as per the algo
                else
                    mat_CoLocation[iCi][iCj] = mat_CoLocation[iCj][iCi] = 1;
            }
        }
    }

    private void createTransistionMatrices(){
        amat_Pij = new Matrix[i_Services];
        amat_Q0 = new Matrix[i_Services];
        for (int iSrv=0; iSrv<i_Services; iSrv++){
            amat_Pij[iSrv] = Matrix.random(i_Comps, i_Comps);
            amat_Q0[iSrv] = Matrix.random(i_Comps, 1);
            int iAC =  random.nextInt(i_Comps);
            //Make the matrix absoarbing
            for(int j=0;j<i_Comps; j++){
                amat_Pij[iSrv].set(iAC, j, 0.0);
                amat_Q0[iSrv].set(iAC,0, 0.0);
                amat_Pij[iSrv].set(j, j, 0.0); //Remove transitions to the same
            }
            System.out.println("Component " + iAC + " is set to Absoarbing for Service: "+ iSrv);

            //Normalize
            for (int i=0; i<i_Comps; i++){
                double dT= 0.0;
                double dQT = 0.0;
                for (int j=0; j<i_Comps; j++){
                    dT +=  amat_Pij[iSrv].get(i,j);
                    dQT += amat_Q0[iSrv].get(j, 0);
                }
                if (dT > 0){
                    for (int j=0; j<i_Comps; j++)
                     amat_Pij[iSrv].set( i,j, ( amat_Pij[iSrv].get(i,j)/dT));
                }
                if (dQT > 0){
                    for (int j=0; j<i_Comps; j++)
                    amat_Q0[iSrv].set( j,0, (amat_Q0[iSrv].get(j,0)/dQT));
                }
            }
        }
    }
}
