package archeopterix.problem.redundancy.algorithm.aco;

import java.util.*;

import archeopterix.algorithm.Stats;
import archeopterix.algorithm.aco.ACO;
import archeopterix.algorithm.aco.Ant;
import archeopterix.conf.Conf;
import archeopterix.problem.Component;
import archeopterix.problem.redundancy.solution.RedundancyAlloc;

//import common.Logger;

public class RedundancyAnt extends Ant {
	
	public  	LinkedList<Integer> m_CompsToAllocate;
	protected 	Component	m_StartPoint;
	private         int i_MaxRALevel;

	public RedundancyAnt(ACO oAlgo, int iIndex, int iMax){
		m_Algo = oAlgo;
		i_MaxRALevel = Conf.instance().getMaxRedundancyLevel();
		m_Memory = new RedundancyAlloc(m_Algo.getProblem().getComponentCount(), i_MaxRALevel );
		m_CompsToAllocate = new LinkedList<Integer>();

		int iArrSize = m_Algo.getPheromoneMapsCount() >1? m_Algo.getPheromoneMapsCount():2;
		ad_Weights= new double [iArrSize];
		//ad_Weights[0] = ((double)iIndex/(double)iMax);
		//ad_Weights[1] = (1- ad_Weights[0]);
		
		double dMid = (double)1/(double)m_Algo.getPheromoneMapsCount();
		int iSteps = (int)((double)m_Algo.getBatchSize()/(double)m_Algo.getPheromoneMapsCount());
		int iPriority = (int)((double)iIndex / ((double)m_Algo.getBatchSize()/(double)m_Algo.getPheromoneMapsCount()));
		int iTick = (iIndex% iSteps);
		//System.out.print("A["+ iAnt + "] Priority=" + iPriority + "Tick="+ iTick );
		//double[] aiARR = new double[iObjectives];
		for(int i=0; i< m_Algo.getPheromoneMapsCount(); i++){
			if (i== iPriority){
				ad_Weights[i] = dMid + (iTick * (dMid/(double)iSteps));
			}
			else
				ad_Weights[i] = dMid - (iTick * (dMid/(double)iSteps));
			//System.out.print(aiARR[i] + ",");
		}
		
		//Random rnd = Conf.instance().random;
		/*for (int i=0; i< Defs.NO_OF_ATTRIBUTES; i++){
			ad_Weights[i] = rnd.nextDouble();
		}*/
	}
	
	public RedundancyAnt(RedundancyAlloc oRA, ACO oAlgo, int iIndex, int iMax){
		m_Algo = oAlgo;
		m_Memory = oRA;
		m_CompsToAllocate = new LinkedList<Integer>();

		ad_Weights= new double [Conf.instance().getNEvaluators()];
		ad_Weights[0] = ((double)iIndex/(double)iMax);
		ad_Weights[1] = (1- ad_Weights[0]);
	}
	
	
	public boolean Travel(){
		m_Memory.reset();
		Random rnd = Conf.instance().random;
		for (int iCompIndex=0; iCompIndex < m_Algo.getProblem().getComponentCount(); iCompIndex++){
			int iRALevelIndex = DecideRAlevelIndex(iCompIndex);
			((RedundancyAlloc)m_Memory).setRALevel(iCompIndex, iRALevelIndex);

			if (! m_Algo.getProblem().validate(m_Memory)){
				iRALevelIndex = rnd.nextInt(i_MaxRALevel);
				boolean bSuccess =false;
				for (int iTry=0; iTry<i_MaxRALevel ; iTry++){
					((RedundancyAlloc)m_Memory).setRALevel(iCompIndex, iRALevelIndex);
					if ( m_Algo.getProblem().validate(m_Memory) ){
						bSuccess = true;
						break;
					}
					iRALevelIndex = (++iRALevelIndex == i_MaxRALevel)? 0: iRALevelIndex; 
					Stats.instance().l_CntRndD++;
				}
				if ( !bSuccess){
					//Logger.Logln("Can't proceed Dep :" + m_Memory.getDepString());
					return false;
				}
				//TODO:: Do BackTaracking here
			}
					
			//Local Pheromone Update
			for( int iAtt=0; iAtt<m_Algo.getPheromoneMapsCount(); iAtt++){
				double dPM = m_Algo.getPheromoneValue(iAtt,iCompIndex, iRALevelIndex);
				dPM = m_Algo.evaporatePheromone(dPM);
                                m_Algo.setPheromeneValue(iAtt, iCompIndex, iRALevelIndex, dPM);
			}
		}
		return true;
		
		/*for (int i=0; i< m_CompsToAllocate.size(); i++){
			m_CompsToAllocate.remove(i);
			int iHostIndex = DecideHostIndex(i);
			m_Memory.Set(i, iHostIndex);
			double dPM = m_Algo.arr_PhMap[0].GetPheromone(i, iHostIndex);
			m_Algo.arr_PhMap[0].UpdatePheromone(i, iHostIndex, dPM * (1- m_Algo.d_PhDecay));//Local Pheromone Update
		}*/
	}
	
	private int DecideRAlevelIndex(int iCompIndex){
		LinkedList<Double> lstCum = new LinkedList<Double>();
		//Calculating Cumulative Values for probability
		double dTot=0.0;
		double dMax=0.0;
		int iMaxIndex=0;
		//Index starts from 0, to MaxRALevels
		for (int iRALevelIndex=0; iRALevelIndex<i_MaxRALevel ; iRALevelIndex++){
			double dF = 0.0;
			//TODO::Uncomment Following
			 for(int iAttr=0; iAttr < m_Algo.getPheromoneMapsCount(); iAttr++){
				dF+= m_Algo.getPheromoneValue(iAttr,iCompIndex, iRALevelIndex)*ad_Weights[iAttr];
			}
			//dF = m_Algo.arr_PhMap[0].GetPheromone(iCompIndex, iHostIndex);
			dTot+=dF;		
			if(dF>dMax){
				dMax	= dF;
				iMaxIndex = iRALevelIndex;
			}
			lstCum.add(dTot);
		}
		Random rnd = Conf.instance().random;
		if ( dTot < 1E-4 ){
			Stats.instance().l_CntRndD++;
			return rnd.nextInt(m_Algo.getProblem().getECUCount());
		}
		double dQ = rnd.nextDouble();
		if(dQ <= m_Algo.getThPheormone()){
			Stats.instance().l_CntMPU++;
			return iMaxIndex;
		}
		else{
			double dTh = rnd.nextDouble()* dTot;
			int iRALevelIndex=0;
			//Selecting the host based on probability
			while( lstCum.get(iRALevelIndex)< dTh){
				iRALevelIndex++;
			}
			//System.out.println("Decide " + lstCum.toString() + "Rnd="+ dTh+ " =" + iHostIndex);
			Stats.instance().l_CntPPU++;
			return iRALevelIndex;
		}
	}
	
	@Override
	public void UpdateTrack(int iAtt, int iQuantity){
//		LinkedList<Attribute> lstAttributes = m_Memory.GetAttributeList();
//		Attribute oAtt = lstAttributes.get(iAtt);
		for (int i=0; i< m_Algo.getProblem().getComponentCount(); i++){
			LinkedList<Double> lstRow = m_Algo.getPheromoneMap(iAtt).GetRow(i);
			int j = ((RedundancyAlloc)m_Memory).getRALevel(i);
			double dPmOld = lstRow.get(j);
			double dPm =  m_Algo.calculateNewPheromone(dPmOld, iQuantity);
			//double dPm = dPmOld + oAtt.f_Value * iQuantity;
			m_Algo.setPheromeneValue(iAtt, i, j, dPm);
			//System.out.println("Update Att"+ iAtt+ " Old="+ dPmOld + " New="+ dPm); 
		}
	}


	
	/*public void UpdateTrack(int i_Quantity){
		for ( int iAtt=0; iAtt< Defs.NO_OF_ATTRIBUTES; iAtt++){
			for (int i=0; i< m_Algo.i_Comps; i++){
				LinkedList<Double> lstRow = m_Algo.arr_PhMap[iAtt].GetRow(i);
				int j = m_Memory.GetHostIndex(i);
				double dPmOld = lstRow.get(j);
				double dPm = (1-m_Algo.d_PhDecay) * dPmOld + m_Algo.d_PhDecay * i_Quantity;
				m_Algo.arr_PhMap[iAtt].UpdatePheromone(i, j, dPm);
				System.out.println("Update Att"+ iAtt+ " Old="+ dPmOld + " New="+ dPm); 
			}
		}
	}*/
	

	public String toString(){
		
		return  m_Memory.toString();
	}

}


//OLD CODE for TRAVEL with random ordering
/*
 		m_CompsToAllocate.clear();
		for ( int i=0; i< m_Algo.i_Comps; i++){
			m_CompsToAllocate.add(i);
			
		}
		
		while (m_CompsToAllocate.size() > 0 ){
			int iArrIndex = rnd.nextInt(m_CompsToAllocate.size());
			int iCompIndex = m_CompsToAllocate.get(iArrIndex).intValue();
			int iHostIndex = DecideHostIndex(iCompIndex);
			m_Memory.Set(iCompIndex, iHostIndex);
			if (!m_Algo.m_Context.validate(m_Memory)){
				iHostIndex = rnd.nextInt(m_Algo.i_Hosts);
				boolean bSuccess =false;
				for (int iTry=0; iTry<m_Algo.i_Hosts ; iTry++){
					m_Memory.Set(iCompIndex, iHostIndex);
					if ( m_Algo.m_Context.validate(m_Memory) ){
						bSuccess = true;
						break;
					}
					iHostIndex = (++iHostIndex == m_Algo.i_Hosts)? 0: iHostIndex; 
					Stats.Ins().l_CntRndD++;
				}
				if ( !bSuccess){
					Logger.Logln("Can't proceed Dep :" + m_Memory.getDepString());
					return false;
				}
				//TODO:: Do BackTaracking here
			}
			m_CompsToAllocate.remove(iArrIndex);
			//Local Pheromone Update
			for( int iAtt=0; iAtt<Conf.Ins().i_Attributes; iAtt++){
				double dPM = m_Algo.arr_PhMap[iAtt].GetPheromone(iCompIndex, iHostIndex);
				m_Algo.arr_PhMap[iAtt].UpdatePheromone(iCompIndex, iHostIndex, ( dPM * m_Algo.d_PhDecay));
			}
		}
 */

