/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package archeopterix.problem.redundancy.algorithm.nsga;

import archeopterix.algorithm.ea.EAOperators;
import archeopterix.conf.Conf;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.redundancy.solution.RedundancyAlloc;
import archeopterix.solution.Architecture;
import archeopterix.solution.OutOfOptionsException;
import java.util.Random;

/**
 *
 * @author indika
 */
public class RAEAOperators extends EAOperators {

    @Override
    public Architecture singlePointMutation(Architecture oArch, EmbeddedSystemProblem context) throws OutOfOptionsException {
        Architecture oNew = new RedundancyAlloc((RedundancyAlloc)oArch);
        Random random = Conf.instance().random;
        int indx1 = random.nextInt(context.getComponentCount());
        ((RedundancyAlloc) oNew).changeRAValue(indx1);
        return oNew;
    }

    @Override
    public Architecture swapMutation(Architecture oArch, EmbeddedSystemProblem context) throws OutOfOptionsException {
        Architecture oNew = new RedundancyAlloc((RedundancyAlloc)oArch);
        Random random = Conf.instance().random;
        int indx1 = random.nextInt(context.getComponentCount());
        int indx2;
        do {
            indx2 = random.nextInt(context.getComponentCount());
        } while (indx1 == indx2);
        ((RedundancyAlloc) oNew).swapRAValues(indx1, indx2);
        return oNew;
    }

    @Override
    public Architecture[] singlePointCrossover(Architecture oParent1, Architecture oParent2, EmbeddedSystemProblem context) throws OutOfOptionsException {
        RedundancyAlloc[] parents = {(RedundancyAlloc) oParent1, (RedundancyAlloc) oParent2};
        RedundancyAlloc[] oOffspring = {new RedundancyAlloc(parents[0]), new RedundancyAlloc(parents[1])};
        Random rand = Conf.instance().random;
        int iSwapAt = rand.nextInt(context.getComponentCount());
        for (int iTry = 0; iTry < context.getComponentCount(); iTry++) {
            for (int i = 0; i < context.getComponentCount(); i++) {
                if (i < iSwapAt) {
                    oOffspring[0].setRALevel(i, parents[0].getRALevel(i));
                    oOffspring[1].setRALevel(i, parents[1].getRALevel(i));
                } else {
                    oOffspring[0].setRALevel(i, parents[1].getRALevel(i));
                    oOffspring[1].setRALevel(i, parents[0].getRALevel(i));
                }
            }
            if (context.validate(oOffspring[0])) {
                return oOffspring;
            }
            iSwapAt = (++iSwapAt == context.getComponentCount()) ? 0 : iSwapAt;
        }
        throw new OutOfOptionsException("Could not find suitable crossover using " + context.getComponentCount() + " attempts.");
    }

    public boolean canOperateOn(Class<?> solRepClass) {
        return solRepClass.equals(RedundancyAlloc.class);
    }
	@Override
	public Architecture uniformMutation(Architecture oArch, EmbeddedSystemProblem context, double prob) {
		throw new UnsupportedOperationException("Not supported yet.");
	}

	@Override
	public Architecture[] uniformCrossover(Architecture oParent1, Architecture oParent2, EmbeddedSystemProblem context, double prob) {
		throw new UnsupportedOperationException("Not supported yet.");
	}
}
