package archeopterix.problem.redundancy.algorithm.nsga;

import archeopterix.algorithm.ea.EAOperators;
import archeopterix.conf.Conf;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.royalroad.solution.RoyalRoadSolution;
import archeopterix.solution.Architecture;
import archeopterix.solution.OutOfOptionsException;
import java.util.Random;



public class RoyalRoadOperators extends EAOperators {

    @Override
    public Architecture singlePointMutation(Architecture oArch, EmbeddedSystemProblem context) throws OutOfOptionsException {
        Architecture oNew = new RoyalRoadSolution((RoyalRoadSolution)oArch);
        Random random = Conf.instance().random;
        int indx1 = random.nextInt(context.getComponentCount());
        ((RoyalRoadSolution) oNew).changeRAValue(indx1);
        return oNew;
    }

    @Override
    public Architecture swapMutation(Architecture oArch, EmbeddedSystemProblem context) throws OutOfOptionsException {
        Architecture oNew = new RoyalRoadSolution((RoyalRoadSolution)oArch);
        Random random = Conf.instance().random;
        int indx1 = random.nextInt(context.getComponentCount());
        int indx2;
        do {
            indx2 = random.nextInt(context.getComponentCount());
        } while (indx1 == indx2);
        ((RoyalRoadSolution) oNew).swapRAValues(indx1, indx2);
        return oNew;
    }

    @Override
    public Architecture[] singlePointCrossover(Architecture oParent1, Architecture oParent2, EmbeddedSystemProblem context) throws OutOfOptionsException {
        RoyalRoadSolution[] parents = {(RoyalRoadSolution) oParent1, (RoyalRoadSolution) oParent2};
        RoyalRoadSolution[] oOffspring = {new RoyalRoadSolution(parents[0]), new RoyalRoadSolution(parents[1])};
        Random rand = Conf.instance().random;
        int iSwapAt = rand.nextInt(context.getComponentCount());
        for (int i = 0; i < context.getComponentCount(); i++) {
           if (i < iSwapAt) {
              oOffspring[0].setRALevel(i, parents[0].getRALevel(i));
              oOffspring[1].setRALevel(i, parents[1].getRALevel(i));
           } else {
              oOffspring[0].setRALevel(i, parents[1].getRALevel(i));
              oOffspring[1].setRALevel(i, parents[0].getRALevel(i));
           }
        }

     return oOffspring;
   }


	public Architecture[] uniformCrossover(Architecture oParent1, Architecture oParent2, EmbeddedSystemProblem context, double prob)
	{
		RoyalRoadSolution[] parents = {(RoyalRoadSolution) oParent1, (RoyalRoadSolution) oParent2};
		RoyalRoadSolution[] offsprings = {new RoyalRoadSolution(context), new RoyalRoadSolution(context)};
		Random rand = Conf.instance().random;
		for (int i = 0; i < context.getComponentCount(); i++)
		{
			if (rand.nextDouble() < prob)
			{
				offsprings[0].setRALevel(i, parents[0].getRALevel(i));
				offsprings[1].setRALevel(i, parents[1].getRALevel(i));
			}
			else
			{
				offsprings[0].setRALevel(i, parents[1].getRALevel(i));
				offsprings[1].setRALevel(i, parents[0].getRALevel(i));
			}
		}

		return offsprings;

	}

	public boolean canOperateOn(Class<?> solRepClass) {
		return solRepClass.equals( RoyalRoadSolution.class );
	}

	@Override
	public Architecture uniformMutation(Architecture oArch, EmbeddedSystemProblem context, double prob) {
		Architecture oNew = new RoyalRoadSolution((RoyalRoadSolution)oArch);
        Random random = Conf.instance().random;

        for(int indx =0; indx<context.getComponentCount();indx++){
			if(random.nextDouble()<prob){
				((RoyalRoadSolution) oNew).changeRAValue(indx);
			}
		}
        return oNew;
	}



}
