package archeopterix.problem.redundancy.solution;

import archeopterix.problem.EmbeddedSystemProblem;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Random;

import archeopterix.conf.Conf;
import archeopterix.evaluation.Attribute;
import archeopterix.solution.Architecture;
import archeopterix.util.CommonFormats;

public class RedundancyAlloc extends Architecture {

    protected int i_Components;
    protected int i_MaxRALevel;
    protected LinkedList<Integer> lst_Alloc;

    public RedundancyAlloc(EmbeddedSystemProblem problem) {
        i_Components = problem.getComponentCount();
        i_MaxRALevel = Conf.instance().getMaxRedundancyLevel();
        lst_Alloc = new LinkedList<Integer>();
        for (int i = 0; i < i_Components; i++) {
            lst_Alloc.add(0);//TODO::DoubleCheck.Initially RA level is 0 for all.
        }
        randomlyAllocate();
    }

    public RedundancyAlloc(int iComps, int iMaxRAlevel) {
        i_Components = iComps;
        i_MaxRALevel = iMaxRAlevel;
        lst_Alloc = new LinkedList<Integer>();
        for (int i = 0; i < iComps; i++) {
            lst_Alloc.add(0);//TODO::DoubleCheck.Initially RA level is 0 for all.
        }
        randomlyAllocate();
    }

    public RedundancyAlloc(RedundancyAlloc oOld) {
        super(oOld);
        lst_Alloc = new LinkedList<Integer>();
        i_Components = oOld.i_Components;
        i_MaxRALevel = oOld.i_MaxRALevel;
        lst_Alloc.addAll(oOld.lst_Alloc);
        lst_Attributes = null;
    }

    public int getNComponents() {
        return i_Components;
    }

    @Override
    public String toString() {
        //FIXME Since all Architectures (solutions) have attributes, this code does not belong here and could be simplified
        String sRet = ";" + lst_Alloc.toString() + ";";
        ListIterator<Attribute> itrAttList = getAttributeList().listIterator();
        while (itrAttList.hasNext()) {
            Attribute oAtt = itrAttList.next();
            sRet += oAtt.getName() + "=;" + CommonFormats.dFormatter.format(oAtt.getValue()) + ";"; // TODO::Remove the Cast
            //sRet +=  oAtt.getName() + "=;" + oAtt.GetFaceValue() + ";"; // TODO::Remove the Cast
        }
        return sRet;
    }

    @Override
    public Architecture clone() {
        return new RedundancyAlloc(this);
    }

    public void randomlyAllocate() {
        Random random = Conf.instance().random;
        for (int i = 0; i < i_Components; i++) {
            setRALevel(i, random.nextInt(i_MaxRALevel));
        }
    }

    public void setRALevel(int iCompIndex, int iRALevel) {
        lst_Alloc.set(iCompIndex, iRALevel);
    }

    public int getRALevel(int iCompIndex) {
        return lst_Alloc.get(iCompIndex).intValue();
    }

    /*public Architecture crossOver(Architecture oArch, int iIndex) {
    if (! (oArch instanceof RedundancyAlloc))
    return null;
    RedundancyAlloc oRA = (RedundancyAlloc)oArch;
    RedundancyAlloc oOffspring = new RedundancyAlloc(i_Components,i_MaxRALevel);
    for (int i=0; i < iIndex; i++){
    oOffspring.setRALevel(i, oRA.getRALevel(i) );
    }
    for (int i=iIndex; i<i_Components;i++){
    oOffspring.setRALevel(i, getRALevel(i));
    }
    return oOffspring;
    }*/
    public void swapRAValues(int iComp1, int iComp2) {
        Integer oInd2 = lst_Alloc.get(iComp2);
        lst_Alloc.set(iComp2, lst_Alloc.get(iComp1));
        lst_Alloc.set(iComp1, oInd2);
    }

	public void changeRAValue(int iComp) {
		 Random random = Conf.instance().random;
		 int index = random.nextInt(i_MaxRALevel);
        lst_Alloc.set(iComp, index);
    }

    @Override
    public boolean isSameAs(Architecture other) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void reset() {
        for (int i = 0; i < i_Components; i++) {
            lst_Alloc.add(0);//TODO::DoubleCheck.Initially RA level is 0 for all.
        }
    }

    @Override
    public String getSerializedString() {
        return lst_Alloc.toString();
    }

    @Override
    public void initializeFromString(String sStr, EmbeddedSystemProblem oProblem) {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    /*public void Reset() {
    for (int i = 0; i < i_Components; i++) {
    SetRALevel(i, 0);
    }
    }*/
}
