package archeopterix.problem.royalroad.solution;

import archeopterix.conf.Conf;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.problem.redundancy.solution.RedundancyAlloc;
import java.util.LinkedList;


public class RoyalRoadSolution extends RedundancyAlloc
{
    public static String PARAM_BLOCK_SIZE = "blockSize";
    public static String PARAM_GAP_SIZE = "gapSize";
    public static String PARAM_LEVELS = "levels";
    public static String PARAM_PART_FITNESS= "partFitness";
    public static String PARAM_PART_VALUE= "partValue";
    public static String PARAM_BONUS_START_VALUE= "bonusStartValue";
    public static String PARAM_BONUS_VALUE= "bonusValue";


    public int blockSize;
    public int gapSize;
    public int regionSize;
    public int blockCount;
    public int levels;
    public int stringSize;
    public int partFitness;
    public double partValue;
    public double bonusStartValue;
    public double bonusValue;
	
	
    public RoyalRoadSolution(EmbeddedSystemProblem problem) {
        super(problem);
        setFieldsFromProperties(Conf.instance());
		i_MaxRALevel = 3;
        
    }
	public RoyalRoadSolution(int iComps, int iMaxRAlevel) {
        super(iComps,2);
    }
public RoyalRoadSolution(RoyalRoadSolution otherSolution) {
		super(otherSolution);
        lst_Alloc = new LinkedList<Integer>();
        i_Components = otherSolution.i_Components;
        i_MaxRALevel = otherSolution.i_MaxRALevel;
        lst_Alloc.addAll(otherSolution.lst_Alloc);
        lst_Attributes = null;
        blockSize = otherSolution.blockSize;
        gapSize = otherSolution.gapSize;
        regionSize = otherSolution.regionSize;
        blockCount = otherSolution.blockCount;
        levels = otherSolution.levels;
        stringSize = otherSolution.stringSize;
        partFitness = otherSolution.partFitness;
        partValue = otherSolution.partValue;
        bonusStartValue=otherSolution.bonusStartValue;
        bonusValue=otherSolution.bonusValue;

    }
    
    public LinkedList<Integer> getLstAlloc(){
        return this.lst_Alloc;
    }
    
    private void setFieldsFromProperties(Conf conf)
	{
            Class<?> algClass = this.getClass();
            blockSize = conf.getApplicableParameters(algClass).getInt(PARAM_BLOCK_SIZE);
            gapSize = conf.getApplicableParameters(algClass).getInt(PARAM_GAP_SIZE);
		
            regionSize = blockSize + gapSize;
		
            levels = conf.getApplicableParameters(algClass).getInt(PARAM_LEVELS);
		
            blockCount = (int) Math.pow(2, levels);
            stringSize = (blockSize+gapSize)* blockCount;
           
		
            partFitness = conf.getApplicableParameters(algClass).getInt(PARAM_PART_FITNESS);
            partValue = conf.getApplicableParameters(algClass).getDouble(PARAM_PART_VALUE);
            bonusStartValue = conf.getApplicableParameters(algClass).getDouble(PARAM_BONUS_START_VALUE);
            bonusValue = conf.getApplicableParameters(algClass).getDouble(PARAM_BONUS_VALUE);

	}
}
