/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package archeopterix.reporting;

import archeopterix.conf.Conf;
import archeopterix.evaluation.Attribute;
import archeopterix.solution.Architecture;
import archeopterix.solution.comparator.SolutionComparator;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Collections;
import java.util.List;

/**
 *
 * @author IMoser
 */
public class HypervolumeDevelopment
{

    private double[] hypervolumeList;
    private double[] hypervolumeDevelopmentList;
    private int[] denominators;
    private int numberOfDevelopmentEntries;
    private PrintWriter printWriter;
    private int runRepeats;
    private static HypervolumeDevelopment hvDevelopment = null;
    private int checkinCounter = 0;
    private int bucketsize;
    private HypervolumeDevelopment(int iterations, int runRepeats, int bucketsize){
        if (bucketsize > 0)
            numberOfDevelopmentEntries = iterations/bucketsize + 10;
        else
            numberOfDevelopmentEntries = iterations;
         
         hypervolumeList = new double[runRepeats];
         hypervolumeDevelopmentList = new double[numberOfDevelopmentEntries];
         denominators = new int [numberOfDevelopmentEntries];
         this.runRepeats = runRepeats;
         this.bucketsize = bucketsize;
         String fileName = Conf.instance().getProblemFileRootName() ;
         //String resultsPrefix = Conf.instance().getResultsFileNamePrefix().replaceFirst("results", "");
         String resultsPrefix = Conf.instance().getAlgorithmClass().getSimpleName() + "_results";
         fileName += resultsPrefix + ".log";
         try{
            FileWriter outFile = new FileWriter(fileName);
            printWriter = new PrintWriter(outFile);
         } catch (IOException e) {
            System.out.println("LOG:Error occurred creating file : " + fileName);
         }
    }

    public static HypervolumeDevelopment getInstance (){
        return hvDevelopment;
    }

    public static void createInstance(int iterations, int runRepeats, int bucketsize){
        hvDevelopment = new HypervolumeDevelopment(iterations, runRepeats, bucketsize);
    }


    public void addHypervolumeForIteration(double hv, int it){
        //writeToResultFile("AT;"+it+";HV;" + hv);
        int place = it;
        if (bucketsize > 0)
            place = it / bucketsize;
		if (place >= hypervolumeDevelopmentList.length)
			return;
        this.hypervolumeDevelopmentList[place] += hv;
        this.denominators[place]++;
    }

    public void addHypervolumeForRun(double hv){
        this.hypervolumeList[checkinCounter++] = hv;
    }

    private double averagedHypervolumeForIteration( int it){
        if (bucketsize == 0)
            return this.hypervolumeDevelopmentList[it]/runRepeats;       
        return this.hypervolumeDevelopmentList[it]/denominators[it];      
    }

    public String hypervolumeDevelopmentsToString(){
        String result = System.getProperty("line.separator")+"Averaged hypervolumes for iterations:"+System.getProperty("line.separator");
        for (int i = 0; i < hypervolumeDevelopmentList.length; i++) {
            int iteration = i;
            if (bucketsize > 0)
                iteration = i * bucketsize;
            if(hypervolumeDevelopmentList[i] > 0)
                result += iteration+" "+ averagedHypervolumeForIteration(i)+System.getProperty("line.separator");

        }
        return result;
    }

    public String hypervolumesForRunsToString(){
        String result = System.getProperty("line.separator")+"Final hypervolumes for trials:"+System.getProperty("line.separator");
        for (int i = 0; i < hypervolumeList.length; i++) {
            result += (i+1)+" "+ hypervolumeList[i] +System.getProperty("line.separator");
        }
        return result;
    }

    public void logHypervolumes(){
        printWriter.println(this.hypervolumeDevelopmentsToString());
        printWriter.println(this.hypervolumesForRunsToString());
        printWriter.flush();
    }

    public void writeToResultFile(String printable){
        printWriter.println(printable);
        printWriter.flush();
    }

    public double getHypervolume(List<Architecture> nondominatedSolutions){
        Collections.sort(nondominatedSolutions, new SolutionComparator());
        double fHV = 0;
        for( int i=0; i< nondominatedSolutions.size(); i++){
            Architecture dep = nondominatedSolutions.get(i);
            List<Attribute> lstAttributes =  dep.getAttributeList();
            if(lstAttributes.size()<2){
                return lstAttributes.get(0).getNormalisedOptimiserValue();
            }
            double[] afAtts = new double[lstAttributes.size()];
            double[] afPrvAtts = new double[lstAttributes.size()];
            for (int j=0; j<lstAttributes.size(); j++){
                Attribute att = lstAttributes.get(j);
                if (att.isMinimizing())
                    afAtts[j] = ( att.getMaxValue() - att.getValue());
                else
                    afAtts[j] = att.getValue();
            }
            //afAtts[0] = dep.GetAttributeValue(0);
            //afAtts[1] = dep.GetAttributeValue(1);
            if (i == 0){
                fHV += afAtts[0]*afAtts[1];
            }
            else{
                //fHV += Math.abs( 0.5 * (afAtts[0]-afPrvAtts[0])*(afAtts[1]+afPrvAtts[1])) ;//.5 (y1+y2)(x2-x1)
                fHV += Math.abs( (afAtts[0]-afPrvAtts[0])*afAtts[1]) ;//(x2-x1)*y2
            }
            afPrvAtts[0] = afAtts[0];
            afPrvAtts[1] = afAtts[1];
        }
        return fHV;
    }

}
