package archeopterix.util;

import archeopterix.conf.Conf;
import java.util.List;
/**
 *
 * @author ${user}
 */
public class CommonFunctions{

	/** execute a rouletted with the given weights and return an index
     *
     * @param arrValues
     * @return
     */
    public static int doRoulette(double[] arrWeights){

        double dSum = 0.0;
        for ( int i=0; i< arrWeights.length; i++)
            dSum += Math.abs(arrWeights[i]);
        double dTh = Conf.instance().random.nextDouble()* dSum;
        int iIndex=0;
        double dCum = Math.abs(arrWeights[iIndex]);
        //Selecting the host based on probability
        while( dCum < dTh){
            iIndex++;
            dCum += Math.abs(arrWeights[iIndex]);
        }
        return iIndex;
    }

	/**
	 * Roulette Wheel selection with certain degree of exploration
	 * @param arrWeights Weights of indexes
	 * @param dExplorationRate (0,1) : Allow unwieghted selection some chance to get selected
	 * @return
	 */
    public static int doRouletteWithExploration(double[] arrWeights, double dExplorationRate){
		if ((dExplorationRate > 1 ) || (dExplorationRate <0)){
			System.err.println("doRouletteWithExploration : rate should be within (0,1)");
			return 0;
		}
        double dSum = 0.0;
        for ( int i=0; i< arrWeights.length; i++)
            dSum += Math.abs(arrWeights[i]);
		double dExplore = (dExplorationRate* dSum ) / (double)arrWeights.length;
		dSum = (1+ dExplorationRate)* dSum;
        double dTh = Conf.instance().random.nextDouble()* dSum;
        int iIndex=0;
        double dCum = (Math.abs(arrWeights[iIndex]) + dExplore);
        //Selecting the host based on probability
        while( dCum < dTh){
            iIndex++;
            dCum += (Math.abs(arrWeights[iIndex]) + dExplore);
        }
        return iIndex;
    }




	//==========================================================================
	//                    Utility Functions
	//==========================================================================

	/**
	 * Compute the average of given list of double values
	 * @param lstVals
	 * @return
	 */
	public static double getAverage(List<Double> lstVals){
		int iCnt = lstVals.size();
		if ( iCnt== 0 )
			return 0;
		double dSum = 0.0;
		for(Double d : lstVals)
			dSum += d.doubleValue();
		return dSum/ (double)iCnt;
	}


		public static double getAverage(double[] adVals){
		int iCnt = adVals.length;
		if ( iCnt== 0 )
			return 0;
		double dSum = 0.0;
		for (int i=0; i< iCnt; i++)
			dSum += adVals[i];
		return dSum/ (double)iCnt;
	}

	/**
	 * Compute number of elements that are greater that a given Threshold
	 * @param lstVals
	 * @param dTh
	 * @return
	 */
	public static int getOutperformerCount(List<Double>lstVals, double dTh){
		int iCnt = 0;
		for(Double d : lstVals){
			if (d >= dTh)
				iCnt++;
		}
		return iCnt;
	}
        public static double[] arrayFromList(List<Double> source){
            int size = source.size();
            double[] newArray = new double[size];
            for(int i=0; i<size;i++){
                newArray[i]=source.get(i);
            }
            return newArray;
        }

	/**
	 * Returns the index set as a double array. This is useful applications like
	 * linear regression
	 * @param adVals
	 * @return
	 */
	public static double[] getIndexSet(double[] adVals){
		double[] adIdx = new double[adVals.length];
		for (int i=0; i< adIdx.length; i++)
			adIdx[i] = i;
		return adIdx;
	}

	public static int[] getIntIndexSet(List<Double> adVals){
		int[] adIdx = new int[adVals.size()];
		for (int i=0; i< adIdx.length; i++)
			adIdx[i] = i+1;
		return adIdx;
	}
        public static int[] getIntIndexSet(double[] adVals){
		int[] adIdx = new int[adVals.length];
		for (int i=0; i< adIdx.length; i++)
			adIdx[i] = i+1;
		return adIdx;
	}
	public static double getMin(double[] adVals){
		double dMin = Double.MAX_VALUE;
		for (int i=0; i< adVals.length; i++){
			if (dMin> adVals[i] )
				dMin = adVals[i];
		}
		return dMin;
	}

	public static double getMax(double[] adVals){
		double dMax = Double.MIN_VALUE;
		for (int i=0; i< adVals.length; i++){
			if (dMax < adVals[i] )
				dMax = adVals[i];
		}
		return dMax;
	}

}
