/**
 * 
 */
package archeopterix.util;

import archeopterix.util.statistics.ProbabilityDistribution;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author JMontgomery
 *
 */
public class PropertyMap extends HashMap<String, Object> {
	private static final long serialVersionUID = 1L;

	//FIXME Decide if performance hit from using generic function worth it given ease of data conversion elsewhere
	@SuppressWarnings("unchecked") //No worse than the type casts in the original version
	public <E> E get(String propertyName, E defaultIfNull) {
		Object result = get(propertyName);
		return (E) (result == null ? defaultIfNull : result);
	}
	
	public double getDouble(String propertyName) { return get(propertyName, 0.0); }

	public int getInt(String propertyName) { return get(propertyName, 0); }

        public ProbabilityDistribution getPDF(String propertyName){
            return get(propertyName, (ProbabilityDistribution) null);
        }

        public List<ProbabilityDistribution> getPDFList(String propertyName){
            return get(propertyName, (List<ProbabilityDistribution>) null);
        }

        public ProbabilityDistribution getPDFFromList(String propertyName, int index) {
		List<ProbabilityDistribution> lstItems = getPDFList(propertyName);
		if (lstItems != null)
			return lstItems.get(index);
		return null;
	}

	public List<Double> getDoubleList(String propertyName) {
		return get(propertyName, (List<Double>) null);
	}
	
	public double getDoubleFromList(String propertyName, int index) {
		List<Double> lstItems = getDoubleList(propertyName);
		if (lstItems != null)
			return lstItems.get(index);
		return 0;
	}
	
	public List<Integer> getIntList(String propertyName) {
		return get(propertyName, (List<Integer>) null);
	}

	public int getIntFromList(String propertyName, int index) {
		List<Integer> lstItems = getIntList(propertyName);
		if (lstItems != null)
			return lstItems.get(index);
		return 0;
	}
	
	public boolean getBoolean(String propertyName) {
		return (Boolean) get(propertyName); }
	
	public Class<?> getClass(String propertyName) { return (Class<?>) get(propertyName); }
	
	public float getFloat(String propertyName) { return (Float) get(propertyName); }
	
	public String getString(String propertyName) { return (String) get(propertyName, (String) null); }
	
	public List<Class<?>> getClassList(String propertyName) { return get(propertyName, (List<Class<?>>) null); }

	/**
	 * Convenience method for creating a union of {@code PropertyMap}s.
	 */
	public void addAll(PropertyMap properties) {
		for (Map.Entry<String, Object> otherParam : properties.entrySet())
			put(otherParam.getKey(), otherParam.getValue());
	}
	
	public String toFormattedString(String linePrefix) {
		final String NL = System.getProperty("line.separator");
		StringBuilder builder = new StringBuilder();
		for (Map.Entry<String,Object> entry : entrySet())
			builder.append(linePrefix).append(entry.getKey()).append(" = ").append(entry.getValue()).append(NL);
		return builder.toString();
	}

}
