/**
 * 
 */
package archeopterix.util.io;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import archeopterix.problem.Component;
import archeopterix.problem.Host;
import archeopterix.problem.InteractionProperties;
import archeopterix.problem.NetworkProperties;
import archeopterix.problem.EmbeddedSystemProblem;
import archeopterix.util.PropertyMap;

/**
 * Text and AADL output methods formerly located in Context/Problem.
 */
public abstract class ProblemToTextConverter {
	private final static Log log = LogFactory.getLog(ProblemToTextConverter.class);
	public static boolean saveToAADL(EmbeddedSystemProblem problem, String fileName) {
		boolean success = true;
		PrintWriter out = null;
		try {
			out = new PrintWriter( new FileWriter(fileName) );
			out.print( toAADL(problem) );
			System.out.println("AADL file created ! :" + fileName);
		} catch (IOException e) {
			if (log.isErrorEnabled())
				log.error("Error occured opening file for writing AADL", e);
			success = false;
		} finally {
			if (out != null) out.close();
		}
		return success;
	}
	
	/**
	 * Copied with minimal changes from original version in Context (EmbeddedSystemProblem).
	 */
	public static String toAADL(EmbeddedSystemProblem problem) {
		StringWriter stringOut = new StringWriter();
		PrintWriter out = new PrintWriter( stringOut );
		out.println("system Main" );
		out.println("end Main;" );
		out.println("" );
		out.println("property set "+ NetworkProperties.LABEL + " is" );
		out.println("  "+NetworkProperties.FAILURE_RATE+": aadlreal  applies to (connections);" );
		out.println("  "+NetworkProperties.DATA_RATE_KBPS+": aadlinteger  applies to (connections);" );
		out.println("  "+NetworkProperties.RELIABILITY+": aadlreal  applies to (connections);" );
		out.println("  "+NetworkProperties.NW_DELAY+": aadlreal  applies to (connections);" );
		out.println("  "+NetworkProperties.BANDWIDTH+": aadlreal  applies to (connections);" );
		out.println("end " + NetworkProperties.LABEL + ";" );
		out.println("" );
		out.println("property set " + InteractionProperties.LABEL + " is" );
		out.println("  "+InteractionProperties.INTERACTION_PROB+": aadlreal  applies to (connections);" ); 
		out.println("  "+InteractionProperties.TRANSITION_PROB+": aadlreal  applies to (connections);" );
		out.println("  "+InteractionProperties.FREQUENCY+": aadlreal  applies to (connections);" );
		out.println("  "+InteractionProperties.EVENT_SIZE+": aadlreal  applies to (connections);" );
		out.println("  "+InteractionProperties.OBJECTIVE+": aadlreal  applies to (connections);" );
		out.println("  "+InteractionProperties.MEAN_EXEC_TIME+": aadlreal  applies to (connections);" );
		out.println("  "+InteractionProperties.COMM_LOAD_KB +": list of aadlinteger  applies to (connections);" );
		out.println("end " + InteractionProperties.LABEL + ";" );
		out.println("" );
		out.println("property set " + Host.LABEL + " is" );
		out.println("  "+Host.MEMORY+": aadlinteger  applies to (processor);" );
		out.println("  "+Host.LOC_LIST+": list of aadlinteger  applies to (processor);" );
		out.println("  "+Host.FAILURE_RATE+": aadlreal  applies to (processor);" );
		out.println("  "+Host.SPEED_MIPS +": aadlinteger  applies to (processor);" );
		out.println("end "+ Host.LABEL+ ";" );
		out.println("" );
		out.println("property set "+ Component.LABEL +" is" );
		out.println("  "+Component.MEMORY+": aadlinteger  applies to (process);" );
		out.println("  "+Component.COLOC_LIST+": list of aadlinteger  applies to (process);" );
		out.println("  "+Component.COMPUTATIONS_MI+": list of aadlinteger  applies to (process);" );
		out.println("  "+Component.INITILIZE_PROB+": list of aadlreal  applies to (process);" );
		out.println("end "+ Component.LABEL+";" );
		out.println("" );
		out.println("property set Main is" );
		out.println("  MainSystemImplementation: constant aadlstring => \"Main.impl\";" );
		out.println("end Main;" );

		out.println("");
		out.println("system implementation Main.impl" );
		out.println("  subcomponents" );

		for (Host oHost : problem.getECUs()){
			out.println("	"+ Host.LABEL +oHost.index+": processor "+ Host.LABEL +oHost.index+"{" );
			PropertyMap oPropMap = oHost.getPropertyMap();
			Iterator<String> it = oPropMap.keySet().iterator(); 
			while(it.hasNext()) { 
				Object key = it.next(); 
				Object val = oPropMap.get(key); 
				out.println("		" + key.toString() + " => " + replaceBraces(val.toString()) + ";" );
			} 
			out.println("	};"); 
		}

		for (Component oComp : problem.getComponents()){
			out.println("	"+ Component.LABEL +oComp.index+": process "+ Component.LABEL+oComp.index+"{" );
			PropertyMap oPropMap = oComp.getPropertyMap();
			Iterator<String> it = oPropMap.keySet().iterator(); 
			while(it.hasNext()) { 
				Object key = it.next(); 
				Object val = oPropMap.get(key); 
				out.println("		" + key.toString() + " => " + replaceBraces(val.toString()) + ";" );
			} 
			out.println("	};"); 
		}
		out.println("  connections" );

		//NETWORK CONNECTIONS
		List<List<PropertyMap>> networkMatrix = problem.getNetworkMatrix();
		for (int i=0, c = problem.getECUCount(); i < c; i++){
			List<PropertyMap> lstRow = networkMatrix.get(i);
			for ( int j=0; j<c; j++ ){
				PropertyMap mapParams = lstRow.get(j);
				if ( mapParams != null){
					out.println("	event port "+ Host.LABEL +i+ ".output -> "+ Host.LABEL + j+ ".input {");
					Iterator<String> it = mapParams.keySet().iterator(); 
					while(it.hasNext()) { 
						Object key = it.next(); 
						Object val = mapParams.get(key); 
						out.println("		" + key.toString() + " => " + val.toString() + ";" );
					} 
					out.println("	};"); 
				}
			}

		}
		out.println(""); 
		//INTERACTIONS
		List<List<PropertyMap>> interactionsMatrix = problem.getInteractionsMatrix();
                for ( int i=0, c = problem.getComponentCount(); i < c; i++){
                    List<PropertyMap> lstRow =  interactionsMatrix.get(i);
                    for ( int j=0; j<c; j++ ){
                        PropertyMap mapParams = lstRow.get(j);
                        if ( mapParams != null){
                            out.println("	event port "+ Component.LABEL +i+ ".output -> "+ Component.LABEL + j+ ".input {");
                            Iterator<String> it = mapParams.keySet().iterator();
                            while(it.hasNext()) {
                                Object key = it.next();
                                Object val = mapParams.get(key);
                                out.println("		" + key.toString() + " => " + replaceBraces(val.toString()) + ";" );
                            }
                            out.println("	};");
                        }
                    }
                }
		
		out.println("  end Main.impl;");
		out.println("");
		//
		for ( int i=0, c = problem.getECUCount(); i < c; i++){
			out.println("processor "+ Host.LABEL +i );
			out.println("	features" );
			out.println("		input: in event port;" );
			out.println("		output: out event port;" );
			out.println("end "+ Host.LABEL +i + ";" );
		}

		for ( int i=0, c = problem.getComponentCount(); i < c; i++){
			out.println("process "+ Component.LABEL + i );
			out.println("	features" );
			out.println("		input: in event port;" );
			out.println("		output: out event port;" );
			out.println("end "+ Component.LABEL + i + ";" );
		}
		out.println("</System>");
		out.close();
		
		return stringOut.toString();
	}

	public static boolean saveToText(EmbeddedSystemProblem problem, String fileName) {
		boolean success = true;
		PrintWriter out = null;
		try {
			out = new PrintWriter( new FileWriter(fileName) );
			out.print( toText(problem) );
			System.out.println("TEXT file created ! :" + fileName);
		} catch (IOException e) {
			if (log.isErrorEnabled())
				log.error("Error occured opening file for writing AADL", e);
			success = false;
		} finally {
			if (out != null) out.close();
		}
		return success;
	}
	
	/**
	 * No functional changes from String generation in DumpToText in Context
	 * (now EmbeddedSystemProblem).
	 * FIXME Could still do with considerable clean up and streamlining.
	 */
	public static String toText(EmbeddedSystemProblem problem) {
		StringWriter stringOut = new StringWriter();
		PrintWriter out = new PrintWriter(stringOut);

		final int nComponents = problem.getComponentCount();
		final int nECUs = problem.getECUCount();
		
		List<Component> comps = problem.getComponents();
		List<Host> ecus = problem.getECUs();
		List<List<PropertyMap>> networkMatrix = problem.getNetworkMatrix();
		List<List<PropertyMap>> interactionsMatrix = problem.getInteractionsMatrix();

		out.println("==== Components ==== ");
		for(Component oComp : comps)
			out.println(oComp);

		out.println("==== ECUs ==== ");
		for(Host oECU : ecus)
			out.println(oECU);


		out.println("==== Interconnections ==== "+ System.getProperty("line.separator"));
		
                for(int i=0; i<nComponents; i++){
                    for(int j=0; j<nComponents; j++){ //NOTE:: Ends at j. lower half of the matrix
                            PropertyMap mapInteractionProperties = interactionsMatrix.get(i).get(j);
                            out.print("(" +i + "->" + j +")");
                            if ( mapInteractionProperties != null )
                                    out.print(mapInteractionProperties.toString() );
                            else
                                    out.print("null");
                    }
                }
		

		out.println("==== Networks ==== "+ System.getProperty("line.separator"));
		for(int i=0; i<nECUs; i++){
			//Network
			for(int j=0; j<nECUs; j++){
				PropertyMap mapNWProperties = networkMatrix.get(i).get(j);
				out.print("(" +i + "->" + j +")");
				if ( mapNWProperties != null )
					out.print(mapNWProperties.toString() );
				else
					out.print("null");
			}
		}

		out.println("==== Colocalization Matrix ( Comp x Comp )==== ");
		for(int i=0; i<nComponents; i++){
			out.print("[  ");
			for(int j=0; j<nComponents; j++){ //NOTE:: Ends at j. lower half of the matrix
				int iColoc = comps.get(i).getIntFromListProperty(Component.COLOC_LIST, j);
				if (iColoc < 0)
					out.print(iColoc +"  ");
				else
					out.print(iColoc +"   ");
			}
			out.print("]");
		}

		out.println("==== Localization Matrix ( ECU x Comp )==== ");
		for(int i=0; i<nECUs; i++){
			out.print("[  ");
			for(int j=0; j<nComponents; j++){
				int iLoc = ecus.get(i).getIntFromListProperty(Host.LOC_LIST, j);
				if (iLoc < 0)
					out.print(iLoc +"  ");
				else
					out.print(iLoc +"   ");
			}
			out.print("]");
		}


		out.close();
		
		return stringOut.toString();
	}
	
	private static String replaceBraces(String sInput){
		String sOut = new String(sInput);
		//System.out.println("In = "+ sInput);
		sOut = sOut.replace('[', '(');
		sOut = sOut.replace(']' , ')');
		//System.out.println("Out = "+ sOut);
		return sOut;
	}


}
