/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package archeopterix.util.probability;
import archeopterix.util.statistics.ProbabilityDistribution;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;

/**
 *
 * @author indika
 */
public class DiscreteDistribution extends ProbabilityDistribution implements Serializable{

	private Map<Double,Double> map_Values;
	private Map<Double,Double> map_CDF;
	private double d_Min = Double.MAX_VALUE;
	private double d_Max = Double.MIN_VALUE;

	//2010-09-06: Changed by EJM. ALL other distributions take doubles, so this one should as well.
	//Furthermore, the identically-named DiscreteDistribution that comes in the JSci package takes doubles.
	//String parsing moved to XMLParser.
	//Some reformatting for clarity and brevity.
	public DiscreteDistribution(double[] params) {
            s_Name = PD_DISCRETE;
		if (params.length % 2 != 0)
			throw new IllegalArgumentException(getClass().getSimpleName() + " requires even number of parameters");

		map_Values = new TreeMap<Double,Double>();
		for (int i = 0, maxI = params.length - 1; i < maxI; i += 2)
			map_Values.put( params[i], params[i+1] );

		double dPrvKey = 0.0;
		double dCum = 0.0;
		map_CDF = new TreeMap<Double, Double>();
		for (Map.Entry<Double,Double> pair : map_Values.entrySet()) {
			double dKey = pair.getKey();
			double dVal = pair.getValue();
			if (dVal < d_Min)
				d_Min = dKey;
			if (dVal > d_Max)
				d_Max = dKey;
			dCum += dVal;
			map_CDF.put(dKey, dCum);
			//System.out.println("PDF" + pairs.getKey() + " = " + pairs.getValue());
			//System.out.println("CDF" + dKey + " = " + dCum);
		}
	}

    @Override
        public Map<Double,Double> getPDF(double dMin, double dMax, int iSteps){
            return map_Values;
        }

    @Override
	public Map<Double,Double> getCDF(double dMin, double dMax, int iSteps) {
            return map_CDF;
        }

	@Override
	public double probability(double dX) {
		Iterator it = map_Values.entrySet().iterator();
		double dPrvKey = 0.0;
		double dPrvVal = 0.0;
		while (it.hasNext()) {
			Map.Entry pairs = (Map.Entry)it.next();
			double dKey = ((Double)pairs.getKey()).doubleValue();
			double dVal = ((Double)pairs.getValue()).doubleValue();
			if (dX<=dKey) //TODO::Fix later
                        //if ( Math.abs(dX-dKey) <= Double.MIN_VALUE)
                            return dPrvVal;
			dPrvVal = dVal;
			//System.out.println(pairs.getKey() + " = " + pairs.getValue());
		}
		return 0.0;
	}

	@Override
	public double cumulative(double dX) {
		Iterator it = map_CDF.entrySet().iterator();
		double dPrvKey = 0.0;
		double dPrvVal = 0.0;
		while (it.hasNext()) {
			Map.Entry pairs = (Map.Entry)it.next();
			double dKey = ((Double)pairs.getKey()).doubleValue();
			double dVal = ((Double)pairs.getValue()).doubleValue();
			if (dX<=dKey) //TODO::Fix later
                return dPrvVal;
			dPrvVal = dVal;
			//System.out.println(pairs.getKey() + " = " + pairs.getValue());
		}
		return 0.0;
	}

	@Override
	public double inverse(double probability) {
		checkRange(probability);
		if(probability==0.0)
			return d_Min;
		if(probability==1.0)
			return d_Max;
		//return findRoot(probability, 0.5*(d_Max+d_Min), d_Min, d_Max);
        Iterator it = map_CDF.entrySet().iterator();
		double dPrvKey = d_Min;
		double dPrvVal = 0.0;
		while (it.hasNext()) {
			Map.Entry pairs = (Map.Entry)it.next();
			double dKey = ((Double)pairs.getKey()).doubleValue();
			double dVal = ((Double)pairs.getValue()).doubleValue();
			if (probability<=dVal) //TODO::Fix later
                return dPrvKey;
			dPrvKey = dKey;
			//System.out.println(pairs.getKey() + " = " + pairs.getValue());
		}
		return d_Max;
	}

	@Override
	public String toString(){
		String sStr = ProbabilityDistribution.PD_DISCRETE;
		Iterator it = map_Values.entrySet().iterator();
		double dPrvVal = 0.0;
		while (it.hasNext()) {
			Map.Entry pairs = (Map.Entry)it.next();
			double dKey = ((Double)pairs.getKey()).doubleValue();
			double dVal = ((Double)pairs.getValue()).doubleValue();
			sStr += (","+ dKey +","+ dVal);

		}
		return sStr;
	}


        public double getMean(){
            	Iterator it = map_Values.entrySet().iterator();
		double dSum = 0.0;
		while (it.hasNext()) {
			Map.Entry pairs = (Map.Entry)it.next();
			double dKey = ((Double)pairs.getKey()).doubleValue();
			double dVal = ((Double)pairs.getValue()).doubleValue();
                        dSum += (dKey*dVal);
		}
                return dSum; //Since Sum(dVal) =1.0
        }

}
