/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package archeopterix.util.probability;

import archeopterix.util.statistics.BetaDistribution;
import archeopterix.util.statistics.ExponentialDistribution;
import archeopterix.util.statistics.NormalDistribution;
import archeopterix.util.statistics.ProbabilityDistribution;
import archeopterix.gui.ChartViewer;
import java.util.Arrays;
import java.util.Map;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 *
 * @author imeedeniya
 */
public class Distribution {


    private static final Log log = LogFactory.getLog(Distribution.class);

    public static double getNormalEstimate(double dMean, double dVar, double dConfidance) {
        NormalDistribution oNorm = new NormalDistribution(dMean, dVar);
        return oNorm.inverse(1 - dConfidance);
    }

    public static double getTriangularEstimate(double dMean, double dVar, double dConfidance) {
        ProbabilityDistribution oDist = getBestfitTriangleDist(dMean, dVar);
        return oDist.inverse(1 - dConfidance);
    }

    public static TriangularDistribution getBestfitTriangleDist(double dMean, double dVar) {
        double dMin, dMid, dMax;
        dMid = dMean;
        dMin = dMid - 5 * dMean * dVar;
        dMax = dMid + 5 * dMean * dVar;
        if (dMin < 0.0) {
            dMin = 0.0;
        }
        if (dMax > 1.0) {
            dMax = 1.0;
        }
        return new TriangularDistribution(dMin, dMid, dMax);
    }

    public static BetaDistribution getBestfitBetaDist(double dMean, double dVar) {
        double dAlpha = (dMean * dMean * (1 - dMean) / dVar) - dMean;
        double dBeta = (dAlpha / dMean) - dAlpha;
        if ((dAlpha < 0) || (dBeta < 0)) {
            if (log.isDebugEnabled()) {
                log.debug("Mean=" + dMean + " Var=" + dVar + " Alpha=" + dAlpha + " Beta =" + dBeta);
            }
            dAlpha = Math.abs(dAlpha); //HACK. ABS is a hack
            dBeta = Math.abs(dBeta);
        }
        return new BetaDistribution(dAlpha, dBeta);

    }

    public static double getBetaEstimate(double dMean, double dVar, double dConfidance) {
        ProbabilityDistribution oDist = getBestfitBetaDist(dMean, dVar);
        return oDist.inverse(1 - dConfidance);
    }

    public static double getNonParametricEstimate(double[] adRawData, double dConfidance) {
        Arrays.sort(adRawData); //Ascending order
        int iPos = (int) ((1 - dConfidance) * adRawData.length);
        return adRawData[iPos];
    }

    /**
     * Plot the probability distribution function. If Plot name is not given (null),
     * the graph will be viewed in a window instead of saving.
     * @param oDist
     * @param sName
     * @param dMin
     * @param dMax
     * @param iSteps
     * @param sPlotName
     */
    public static void plotPDF(ProbabilityDistribution oDist, String sName, double dMin, double dMax, int iSteps, String sPlotName) {
        ChartViewer oChart = null;
        if ( oDist instanceof DiscreteDistribution){
            oChart = new ChartViewer("PDF", ChartViewer.CHART_TYPE_XY_SCATTER, true, false);
        } else {
            oChart = new ChartViewer("PDF", ChartViewer.CHART_TYPE_XY_LINE, true, false);
        }
        oChart.SetData(sName, oDist.getPDF(dMin, dMax, iSteps));
        if (sPlotName != null )
            oChart.saveChart(".", sPlotName);
        else
            oChart.View();
    }

    /**
     * Plot the cumulative distibution function
     * @param oDist
     * @param sName
     * @param dMin
     * @param dMax
     * @param iSteps
     * @param sPlotName
     */
    public static void plotCDF(ProbabilityDistribution oDist, String sName, double dMin, double dMax, int iSteps, String sPlotName) {
        ChartViewer oChart = null;
        if ( oDist instanceof DiscreteDistribution){
            oChart = new ChartViewer("CDF", ChartViewer.CHART_TYPE_XY_SCATTER, true, false);
        } else {
            oChart = new ChartViewer("CDF", ChartViewer.CHART_TYPE_XY_LINE, true, false);
        }
        oChart.SetData(sName, oDist.getCDF(dMin, dMax, iSteps));
        if (sPlotName != null )
            oChart.saveChart(".", sPlotName);
        else
            oChart.View();
    }

    public static double getAnalyticalMean(ProbabilityDistribution oDist) {
        if (oDist instanceof NormalDistribution) {
            return ((NormalDistribution) oDist).getMean();
        } else if (oDist instanceof ExponentialDistribution) {
            return ((ExponentialDistribution) oDist).getMean();
        } else if (oDist instanceof DiscreteDistribution) {
            return ((DiscreteDistribution) oDist).getMean();
        } else if (oDist instanceof TriangularDistribution) {
            return ((TriangularDistribution) oDist).getMean();
        } else if (oDist instanceof BetaDistribution) {
            BetaDistribution oBeta = (BetaDistribution) oDist;
            return oBeta.getDegreesOfFreedomP() / (oBeta.getDegreesOfFreedomP() + oBeta.getDegreesOfFreedomQ());
            //P/(P+Q);
        } else {
            if (log.isErrorEnabled()) {
                log.error("Analitical mean for Distribution" + oDist.toString() + "  not implemented!");
            }
            return 0.0;
        }
    }

    /*
     *         public static HashMap<Double, Double> getFreqDistribution(int iIntervals) {
        Arrays.sort(adRawData); //Ascending order
        double dTick = ((adRawData[adRawData.length - 1] - adRawData[0]) / (double) iIntervals);
        double dHalfTick = 0.5 * dTick;
        double dCurrent = adRawData[0] + dTick;
        int iCount = 0;
        HashMap<Double, Double> mapFD = new HashMap<Double, Double>();
        int iMaxIndex = adRawData.length - 1;
        for (int i = 0; i <= iMaxIndex; i++) {
            iCount++;
            if ((adRawData[i] > dCurrent) || (i == iMaxIndex)) {
                double dRelFreq = ((double) iCount) / (double) adRawData.length;
                mapFD.put(dCurrent - dHalfTick, dRelFreq);
                iCount = 0;
                dCurrent += dTick;
                if (dCurrent > adRawData[iMaxIndex]) {
                    dCurrent = adRawData[iMaxIndex];
                }
            }
        }
        return mapFD;
    }
    */
}
