/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 *  * @author indika
 */

package archeopterix.util.probability;
import archeopterix.util.statistics.*;
import archeopterix.experiments.SlidingWindow;

import java.util.Random;
public class MonteCarlo {
    private Random m_Rnd;
    private UncertaintyHandler uh = new UncertaintyHandler();
    private int i_CurrentRun = 0;
    private SlidingWindow m_EstimatesWindow;
    public MonteCarlo(){
        m_Rnd = new Random();
        i_CurrentRun = 0;
        m_EstimatesWindow = new SlidingWindow(uh.getMCEpochSize());
    }

    public void ResetRuns(){
        i_CurrentRun = 0;
    }

    public static MonteCarlo instance(){return new MonteCarlo();}

    public boolean hasMoreRuns(MyStatistics oStat){
        i_CurrentRun++;
        if (uh.getMCStopCriteria() == MCStopCriteria.FIXED_RUNS){
            return (i_CurrentRun < uh.getMCRuns());
        } else {
            if ( i_CurrentRun >= uh.getMCEpochSize()){
                //double dVal =  oStat.getEstimate();
                //m_EstimatesWindow.add(dVal);
                //ChartViewer.Ins().UpdateXY("ESTIMATE",null, i_CurrentRun, dVal);
                if ( i_CurrentRun >= (2* uh.getMCEpochSize()) ){
                double dRE = m_EstimatesWindow.getRelativeError();
                return (dRE > uh.getMCErrorTh());
                //ChartViewer.Ins().UpdateXY("ERROR",null, i_CurrentRun, dRE);
                //System.out.println( i_CurrentRun +"," + dVal + "," + dRE );
                }
            }
            return true;
            //return (i_CurrentRun < conf.getMCRuns());
        }
    }
    
    public double Sample(ProbabilityDistribution oDist){
        return oDist!=null?oDist.inverse(m_Rnd.nextDouble()):0.0;
    }
    
    public double SampleTriangleDistribution(double dMin, double dMid, double dMax){
        ProbabilityDistribution oDist = new TriangularDistribution(dMin, dMid, dMax);
        //Utils.PlotDistribution(oDist, dMin, dMax, 100);
        return oDist.inverse(m_Rnd.nextDouble());
    }

    public double SampleNormal(double dMean, double dVar){
        NormalDistribution oDist = new NormalDistribution(dMean, dVar);
        return oDist.inverse(m_Rnd.nextDouble());
    }

    public double SampleBeta(double dMean){
        if ((dMean > 0 ) && ( dMean <1.0)){
            double dAlpha = dMean;
            double dBeta = (1-dMean);
            BetaDistribution oBeta = new BetaDistribution(dAlpha, dBeta);
            return oBeta.inverse(m_Rnd.nextDouble());
        }
        else
            return dMean;
        
    }

    public double SampleGamma(double dMean){
        GammaDistribution oGamma = new GammaDistribution(dMean);
        return oGamma.inverse(m_Rnd.nextDouble());
    }

    public double[] SampleDirchelet(double[] adMean){
        double dSum = 0.0;
        int iLen = adMean.length;
        double[] adSamples = new double[iLen];
        for(int i=0; i<iLen; i++){
            //adSamples[i] = SampleBeta(adMean[i]);
            if (adMean[i] > 0.0 )
                adSamples[i] = SampleGamma(adMean[i]);
            dSum += adSamples[i];
        }
        if (dSum >0 ){
            for(int i=0; i<iLen; i++)
                adSamples[i] = ( adSamples[i]/dSum);
        }
        return adSamples;
    }

    /**
     *
     * @param dVal Point estimate of the value
     * @return Sample from specified probability distribution
     */
    public double Sample(double dVal, int iID){
        double dMin,dMid, dMax;
        dMid = dVal;
        if (iID%2==0 ){
            dMin = dMid *(1-0.2);
            dMax = dMid *(1+0.6);
        }
        else{
            dMin = dMid *(1-0.6);
            dMax = dMid *(1+0.2);
        }
        return SampleTriangleDistribution(dMin,dMid,dMax);
    }
}
