/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package archeopterix.util.probability;

import archeopterix.problem.RobustnessMeasure;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;

/**
 *
 * @author indika
 */
public class MyStatistics {

    private LinkedList<Double> lst_SortedValues;
    private int n = 0;
    private double sum = 0.0;
    private double sumSqr = 0.0;
    private double min = Double.POSITIVE_INFINITY;
    private double max = Double.NEGATIVE_INFINITY;
    private double d_Mode = 0.0; //Temporary

    public MyStatistics() {
        lst_SortedValues = new LinkedList<Double>();
    }

    public void update(MyStatistics oMoreStats){
        for( Double dVal: oMoreStats.lst_SortedValues)
            update(dVal, true);
    }
     public void update(double dVal) {
        if (UncertaintyHandler.instance().getRobustnessMeasure() == RobustnessMeasure.NON_PARAMETRIC) {
            int iNewIndex = Collections.binarySearch(lst_SortedValues, dVal);
            if (iNewIndex < 0) {
                lst_SortedValues.add(-iNewIndex - 1, dVal);
            } else {
                lst_SortedValues.add(iNewIndex, dVal);
            }
        }
        n++;
        sum += dVal;
        sumSqr += dVal * dVal;
        min = Math.min(dVal, min);
        max = Math.max(dVal, max);
    }

     public double getEstimate() {

        switch (UncertaintyHandler.instance().getRobustnessMeasure()) {
            case NON_PARAMETRIC:
                return getNonParametricEstimate(UncertaintyHandler.instance().getRobustnessConfidance());
            case NORMAL_ESTIMATE:
                return Distribution.getNormalEstimate(getMean(), getVariance(), UncertaintyHandler.instance().getRobustnessConfidance());
            case RMS_VALUE:
                return Math.sqrt(sumSqr / n);
        }
        return 0.0;
    }
    public void update(double dVal, boolean bSorted) {
        if (bSorted) {
            int iNewIndex = Collections.binarySearch(lst_SortedValues, dVal);
            if (iNewIndex < 0) {
                lst_SortedValues.add(-iNewIndex - 1, dVal);
            } else {
                lst_SortedValues.add(iNewIndex, dVal);
            }
        }
        n++;
        sum += dVal;
        sumSqr += dVal * dVal;
        min = Math.min(dVal, min);
        max = Math.max(dVal, max);
    }

    public double getRMS(){
        return Math.sqrt(sumSqr / n);
    }
    

    public double getNonParametricEstimate(double dConfidance) {
        int iPos = (int) ((1 - dConfidance) * lst_SortedValues.size());
        return lst_SortedValues.get(iPos);
    }

    public int getCount() {
        return n;
    }

    public double getMean() {
        return sum /(double) n;
    }

    public double getVariance() {
        return (sumSqr - sum * sum / n) / (n - 1);
    }

    public double getMinimum() {
        return min;
    }

    public double getMaximum() {
        return max;
    }

    /**
     * returns the frequency distribution of the statistics
     * @param iIntervals
     * @return
     */
    public HashMap<Double, Double> getFreqDistribution(int iIntervals) {
        HashMap<Double, Double> mapFD = new HashMap<Double, Double>();
        if (n < 1)
            return mapFD;
        double dTick = ((lst_SortedValues.get(n - 1) - lst_SortedValues.get(0))/ (double)iIntervals);
        double dHalfTick = 0.5 * dTick;
        double dCurrent = lst_SortedValues.get(0) + dTick;
        int iCount = 0;
        
        int iMaxIndex = n - 1;
        double dModeFreq = 0.0; //To track the mode
        for (int i = 0; i <= iMaxIndex; i++) {
            iCount++;
            if ((lst_SortedValues.get(i) > dCurrent) || (i == iMaxIndex)) {
                double dRelFreq = ((double) iCount) / (double) n;
                mapFD.put(dCurrent - dHalfTick, dRelFreq);
                if ( dRelFreq > dModeFreq){
                    d_Mode = (dCurrent - dHalfTick);
                    dModeFreq = dRelFreq;
                }
                iCount = 0;
                dCurrent += dTick;
                if (dCurrent > lst_SortedValues.get(iMaxIndex)) {
                    dCurrent = lst_SortedValues.get(iMaxIndex);
                }
            }
        }
        return mapFD;
    }

    /**
     * Returns the cumulative distribution statistics
     * @param iIntervals
     * @return
     */
    public HashMap<Double, Double> getCumulativeDistribution(int iIntervals) {
        HashMap<Double, Double> mapFD = new HashMap<Double, Double>();
        if (n < 1)
            return mapFD;
        double dTick = ((lst_SortedValues.get(n - 1) - lst_SortedValues.get(0))/ (double)iIntervals);
        double dHalfTick = 0.5 * dTick;
        double dCurrent = lst_SortedValues.get(0) + dTick;
        int iCount = 0;

        int iMaxIndex = n - 1;
        for (int i = 0; i <= iMaxIndex; i++) {
            iCount++;
            if ((lst_SortedValues.get(i) > dCurrent) || (i == iMaxIndex)) {
                double dRelFreq = ((double) iCount) / (double) n;
                //mapFD.put(dCurrent - dHalfTick, dRelFreq);
                mapFD.put(dCurrent, dRelFreq);
                //iCount = 0;
                dCurrent += dTick;
                if (dCurrent > lst_SortedValues.get(iMaxIndex)) {
                    dCurrent = lst_SortedValues.get(iMaxIndex);
                }
            }
        }
        return mapFD;
    }

    /**
     * This requires descritization. 
     * @return the mode
     */
    public double getMode(int iIntervals){
        getFreqDistribution(iIntervals);
        return d_Mode;
    }

    /**
     *
     * @return Median
     */
    public double getMedian(){
        return getNonParametricEstimate(0.5);
    }

    public Double[] getSortedValues(){
        return lst_SortedValues.toArray(new Double[]{});
    }

    @Override
    public String toString(){
        String str= "";
        for( Double dVal: lst_SortedValues)
            str += (dVal + "\n");
        return str;
    }
}
