/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package archeopterix.util.probability;

import archeopterix.util.statistics.BetaDistribution;
import archeopterix.util.statistics.ProbabilityDistribution;

/**
 *
 * @author indika
 */
public class ShiftedBetaDistribution extends ProbabilityDistribution{

    private double d_Min;
    private double d_Span;
    private double d_Max;
    private BetaDistribution m_Beta;
    public ShiftedBetaDistribution(double dMin, double dMax, double dAlpha, double dBeta){
        s_Name = PD_BETA_SHD;
        if (dMax <= dMin){
            System.err.println("Invalid arguments for Shifted Beta distribution. Max["+dMax+"]<=Min["+dMin+"]");
            return;
        }
        d_Min= dMin;
        d_Max = dMax;
        d_Span = (dMax-dMin);
        m_Beta = new BetaDistribution(dAlpha, dBeta);
    }

    private double getBetaX(double dVal){
        return ((dVal<d_Min)||(dVal>d_Max))? 0 : (dVal-d_Min)/d_Span;
    }
    @Override
    public double probability(double dVal) {
        return m_Beta.probability( getBetaX(dVal) );
    }

    @Override
    public double cumulative(double dVal) {
        return m_Beta.cumulative( getBetaX(dVal) );
    }

    @Override
    public double inverse(double dProb) {
        double dBetaX = m_Beta.inverse(dProb);
        return (dBetaX*d_Span+ d_Min);
    }

    @Override
    public String getBufString(){
        return PD_BETA_SHD + "," + d_Min + "," + d_Max + "," + m_Beta.getDegreesOfFreedomP() + "," + m_Beta.getDegreesOfFreedomQ();
      }

    @Override
    public String toString(){
        return getName() + "(" + df.format(d_Min) + ","+ df.format(d_Max) + ","+ df.format(m_Beta.getDegreesOfFreedomP())+ ","+ df.format(m_Beta.getDegreesOfFreedomQ()) +")";
    }
}
