/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package archeopterix.util.probability;

import archeopterix.experiments.SlidingWindow;
import archeopterix.problem.RobustnessMeasure;
import archeopterix.util.PropertyMap;

/**
 * Takes care of evaluation under uncertainty. Keep the parameters related to
 * Monte-Carlo Simulations and Dynamic Stopping Criterion
 * @author indika
 */
public class UncertaintyHandler {

    public static String FIXED_MC_RUNS = "FixedMCRuns";
    public static String MC_EPOCH_SIZE = "MCEpochSize";
    public static String ROBUSTNESS_CONFIDENCE = "RobustnessConfidence";
    public static String DYNAMIC_MC_ERROR_TH = "DynamicMCErrorTh";
    public static String ROBUSTNESS_MEASEURE = "RobustnessMeasure";
    public static String MC_STOPPING_CRITERION = "MCStoppingCriterion";
    

    private PropertyMap parameterMap;

    private int i_MCEpochSize = 10;
    private int i_MCRuns = 100;
    private double d_RobustnessConfidance = 0.8;
    private double d_MCErrorTh = 0.05;

    private RobustnessMeasure robustnessMeasure = RobustnessMeasure.NON_PARAMETRIC;
    private MCStopCriteria monteCarloStopCriteria = MCStopCriteria.DYNAMIC;

    private int i_CurrentRun = 0;
    private SlidingWindow m_EstimatesWindow;


    public static UncertaintyHandler instance(){
        return new UncertaintyHandler();
    }
    public UncertaintyHandler(PropertyMap parameters){
        //parameterMap = Conf.instance().getApplicableParameters(this.getClass());
        i_MCEpochSize = parameters.getInt(MC_EPOCH_SIZE);
        i_MCRuns = parameters.getInt(FIXED_MC_RUNS);
        d_RobustnessConfidance = parameters.getDouble(ROBUSTNESS_CONFIDENCE);
        d_MCErrorTh = parameters.getDouble(DYNAMIC_MC_ERROR_TH);
        
        robustnessMeasure = (RobustnessMeasure)parameters.get(ROBUSTNESS_MEASEURE);
        monteCarloStopCriteria = (MCStopCriteria)parameters.get(MC_STOPPING_CRITERION);

        i_CurrentRun = 0;
        
        m_EstimatesWindow = new SlidingWindow(i_MCEpochSize);
    }
    
    //For testing purposes
    public UncertaintyHandler(){
        //parameterMap = Conf.instance().getApplicableParameters(this.getClass());
    }


    /*public synchronized static UncertaintyHandler instance() {
		if (instance == null)
			instance = new UncertaintyHandler();
		return instance;
	}*/

    public void ResetRuns(){
        i_CurrentRun = 0;
    }
    
    public boolean hasMoreRuns(MyStatistics oStat){
        i_CurrentRun++;
        if ( monteCarloStopCriteria == MCStopCriteria.FIXED_RUNS){
            return (i_CurrentRun <= i_MCRuns);
        } else {
            if ( i_CurrentRun >= i_MCEpochSize){
                double dVal = getEstimate(oStat);
                m_EstimatesWindow.add(dVal);
                //ChartViewer.Ins().UpdateXY("ESTIMATE",null, i_CurrentRun, dVal);
                if ( i_CurrentRun >= (2* i_MCEpochSize) ){
                double dRE = m_EstimatesWindow.getRelativeError();
                if (dRE < d_MCErrorTh){
                    //System.out.println("Stop at MC Runs " + i_CurrentRun);
                    return false;
                } else
                    return true;
                //ChartViewer.Ins().UpdateXY("ERROR",null, i_CurrentRun, dRE);
                //System.out.println( i_CurrentRun +"," + dVal + "," + dRE + " NEED MORE:"+ (dRE > uh.getMCErrorTh()));
                //return (dRE > uh.getMCErrorTh());
                //return (i_CurrentRun <1000);
                }
            }
            return true;
            //return (i_CurrentRun < conf.getMCRuns());
        }
    }

        public double getEstimate(MyStatistics oStat) {

        switch (robustnessMeasure) {
            case NON_PARAMETRIC:
                return  oStat.getNonParametricEstimate(d_RobustnessConfidance);
            case NORMAL_ESTIMATE:
                return Distribution.getNormalEstimate(oStat.getMean(), oStat.getVariance(), d_RobustnessConfidance);
            case RMS_VALUE:
                return oStat.getRMS();
        }
        return 0.0;
    }

    public void setMCRuns(int iRuns){i_MCRuns = iRuns;}
    public void setMCEpochSize (int iSize) {
        i_MCEpochSize = iSize;
        m_EstimatesWindow = new SlidingWindow(i_MCEpochSize);
    }
    public void setMCStopCriteria(MCStopCriteria sc)  { monteCarloStopCriteria = sc;}
    public void setRobustnessConfidance(double dConf){d_RobustnessConfidance=dConf;}
    public void setRobustnessMeasure(RobustnessMeasure measure){robustnessMeasure = measure;}
    public void setMCErrorTh(double dErrTh){ d_MCErrorTh = dErrTh;}

    public int getMCRuns(){return i_MCRuns;}
    public int getMCEpochSize(){return i_MCEpochSize;}
    public double getRobustnessConfidance(){return d_RobustnessConfidance;}
    public RobustnessMeasure    getRobustnessMeasure(){return robustnessMeasure;}
    public MCStopCriteria getMCStopCriteria(){return monteCarloStopCriteria;}
    public double getMCErrorTh(){ return d_MCErrorTh;}


}
