
package archeopterix.util.statistics;

import archeopterix.util.CommonFunctions;

/**
 * Exponential Weighted Moving Average Model
 * http://en.wikipedia.org/wiki/Moving_average
 * 
 * @author imeedeniya
 */
public class EWMA {

	private double[] ad_St; //Moving Average Data
	private double d_Lambda;
	private double d_LastYt;
	private int i_DataLength;
	public EWMA(double[] adYt, double dLambda){
		d_Lambda = dLambda;
		i_DataLength = adYt.length;
		ad_St = new double[i_DataLength];
		ad_St[0] = CommonFunctions.getAverage(adYt);
		for (int t=0; t<i_DataLength; t++ ){
			if (t>0)
				ad_St[t] = getNextSt(ad_St[t-1], adYt[t-1]);
			d_LastYt = adYt[t];
		}
	}


	private double getNextSt(double dPrvSt, double dPrvYt){
		return ( d_Lambda*dPrvYt + (1- d_Lambda)* dPrvSt );
	}

	/**
	 *  Using the data given in the constructor, simply predicts one next time
	 *  series value
	 * @return
	 */
	public double getNextPrediction(){
		return getNextSt(ad_St[i_DataLength -1], d_LastYt);
	}
	/**
	 * Forecast the future
	 * @param iLen : Length of the future to forecast
	 * @return
	 */
	public double[] getForecast(int iLen){
		double[] adVals = new double[iLen];
		double dYt = d_LastYt;
		double dSt = ad_St[i_DataLength -1];
		for ( int t=0; t< iLen; t++){
			adVals[t] = getNextSt(dSt, dYt);
			dYt = dSt;
			dSt = adVals[t];
		}
		return adVals;
	}

	/**
	 * Returns the old and future forecastes of the time series
	 * @param iLen
	 * @return
	 */
	public double[] getTimeSeries(int iLen){
		double[] adVals = new double[iLen];
		for (int t=0; ((t< i_DataLength) &&  (t< iLen)); t++)
			adVals[t] = ad_St[t];
		int iNFuture = iLen - i_DataLength;
		if (iNFuture > 0){
			double[] adFuture = getForecast(iNFuture);
			for (int t=i_DataLength; t<iLen; t++){
				adVals[t] = adFuture[t- i_DataLength];
			}
		}
		return adVals;
	}
}
