package  archeopterix.util.statistics.timeseries.ARIMA;


import archeopterix.util.statistics.timeseries.ARIMA.Cholesky.NotSPDException;
import archeopterix.util.statistics.timeseries.ARIMA.ZeroPolynomial.DidNotConvergeException;
import java.io.Serializable;

// Referenced classes of package 
//            ARMAEstimateMissing, ARSeasonalFit, ARMAOutlierIdentification, Cdfau/edu swin.archeopterix.util.statistics.timeseries    ARMA, ARAutoUnivariate, ARMAMaxLikelihood

public class AutoARIMA
    implements Serializable, Cloneable
{

    private static final long serialVersionUID = 0xc3a097925d2bd8f2L;
    public static final int INNOVATIONAL = 0;
    public static final int ADDITIVE = 1;
    public static final int LEVEL_SHIFT = 2;
    public static final int TEMPORARY_CHANGE = 3;
    public static final int UNABLE_TO_IDENTIFY = 4;
    public static final int AIC = 0;
    public static final int AICC = 1;
    public static final int BIC = 2;
    private boolean isComputed;
    private boolean isForecasted;
    private int n_obs;
    private double res_sigma[];
    private double res_series[];
    private double out_free_series[];
    private int outlier_stat[][];
    private double omega[];
    private int num_outliers[];
    private double l_delta;
    private double l_critical;
    private double l_epsilon;
    private double l_relativeError;
    private double l_confidence;
    private double l_aic[];
    private double l_aicc[];
    private double l_bic[];
    private double l_parameters[];
    private double l_outfree_forecast[];
    private double l_outlier_forecast[];
    private double l_psi_weights[];
    private double l_dev[];
    private int nObs;
    private int nObsActual;
    private int l_maxARLag;
    private int l_infoCriterion;
    private int l_periods[][];
    private int l_orders[][];
    private int optimumModel[];
    private double l_gapFreeSeries[];
    private int l_completeTimes[];
    private double l_x[];
    private int l_times[];
    private static final int EMPTY_INTEGER_ARRAY[][] = new int[0][0];
    private static final double EMPTY_DOUBLE_VECTOR[] = new double[0];

    public static class NoAcceptableModelFoundException extends Exception
    {

        public NoAcceptableModelFoundException(String s)
        {
            super(s);
        }

        public NoAcceptableModelFoundException(String s, Object aobj[])
        {
            super(s+ aobj);
        }
    }


    public AutoARIMA(int ai[], double ad[])
    {
		nObs = ai.length;
        if(ad.length != nObs)
        {
            System.out.println("Error: x "+ new Integer(ad.length)+ " times "+ new Integer(ai.length));


        }
        nObsActual = (ai[nObs - 1] - ai[0]) + 1;
        if(nObs < 2)
        {
            System.out.println("Error: x.length"+ new Integer(ad.length)+ "1");
        }
        for(int i = 1; i < nObs; i++)
            if(ai[i] <= ai[i - 1])
            {
                 System.out.println(new Integer(i)+ new Integer(ai[i])+ new Integer(i + 1)+ new Integer(ai[i + 1]));
                };
		l_x = ad;
        l_times = ai;
        optimumModel = new int[4];
        l_orders = new int[1][1];
        l_periods = new int[1][1];
        num_outliers = new int[1];
        out_free_series = new double[nObsActual];
        res_series = new double[nObsActual];
        l_gapFreeSeries = new double[nObsActual];
        l_completeTimes = new int[nObsActual];
        res_sigma = new double[1];
        l_aic = new double[1];
        l_aicc = new double[1];
        l_bic = new double[1];
        optimumModel[0] = 0;
        optimumModel[1] = 0;
        optimumModel[2] = 1;
        optimumModel[3] = 0;
        num_outliers[0] = 0;
        l_orders[0][0] = 0;
        l_periods[0][0] = 1;
        l_aic[0] = (1.0D / 0.0D);
        l_aicc[0] = (1.0D / 0.0D);
        l_bic[0] = (1.0D / 0.0D);
		
        isComputed = false;
        isForecasted = false;
        l_delta = 0.69999999999999996D;
        l_critical = 3D;
        l_epsilon = 0.001D;
        l_relativeError = 2.2204460492503131E-012D;
        l_confidence = 95D;
        l_maxARLag = 10;
        l_infoCriterion = 0;
        
			}

        

    public final void compute(int i)
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, ARAutoUnivariate.TriangularMatrixSingularException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.InitialMAException, SingularMatrixException, NoAcceptableModelFoundException, NotSPDException, DidNotConvergeException
    {
        int j1 = 1;
        int ai1[] = new int[0];
        int ai2[] = new int[0];
        if(i < 1 || i >= nObsActual)
        {
            Object aobj[] = {
                "maxlag", new Integer(i), "1", new Integer(nObsActual - 1)
            };
            System.out.println("GreaterEqual"+ aobj);
        }
        int j = i;
        int k = 0;
        double ad[] = new double[1 + j + k];
        int ai[][] = new int[nObsActual][2];
        double ad1[] = new double[nObsActual];
        num_outliers[0] = 0;
        l_auto_arima_driver(nObsActual, l_x, l_times, j1, i, optimumModel, l_delta, l_critical, l_epsilon, res_sigma, num_outliers, ai,     ad,    l_aic,  l_aicc, l_bic, res_series, ad1, out_free_series, l_gapFreeSeries, l_completeTimes, ai1, ai2, l_periods, l_orders, l_infoCriterion, l_relativeError);
       // l_auto_arima_driver( i,       ad[], ai[],    j, k,  ai1[],           d,      d1,            d2,     ad1[],     ai2[],    ai3[][], ad2[],  ad3[], ad4[],  ad5[],  ad6[],  ad7[],   ad8[],            ad9[],             ai4[],       ai5[],ai6[], ai7[][], ai8[][], l,                d3)

		int l = optimumModel[0];
        int i1 = optimumModel[1];
        l_parameters = new double[1 + l + i1];
        if(num_outliers[0] > 0)
        {
            omega = new double[num_outliers[0]];
            outlier_stat = new int[num_outliers[0]][2];
        } else
        {
            omega = EMPTY_DOUBLE_VECTOR;
            outlier_stat = EMPTY_INTEGER_ARRAY;
        }
        System.arraycopy(ad1, 0, omega, 0, num_outliers[0]);
        for(int k1 = 0; k1 < num_outliers[0]; k1++)
            System.arraycopy(ai[k1], 0, outlier_stat[k1], 0, 2);

        System.arraycopy(ad, 0, l_parameters, 0, 1 + l + i1);
        isComputed = true;
    }

    public final void compute(int ai[], int ai1[])
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, ARAutoUnivariate.TriangularMatrixSingularException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.InitialMAException, SingularMatrixException, NoAcceptableModelFoundException, NotSPDException, DidNotConvergeException
    {
        byte byte0 = 2;
        int i = ai.length;
        if(i == 0)
        {
            Object aobj[] = {
                "arOrders.length", new Integer(0)
            };
            System.out.println("NotPositive"+ aobj);
        }
        int j = ai1.length;
        if(j == 0)
        {
            Object aobj1[] = {
                "maOrders.length", new Integer(0)
            };
            System.out.println("NotPositive"+ aobj1);
        }
        for(int l1 = 0; l1 < i; l1++)
            if(ai[l1] < 0)
            {
                Object aobj2[] = {
                    (new StringBuilder()).append("arOrders[").append(l1).append("]").toString(), new Integer(ai[l1])
                };
                System.out.println("Negative"+ aobj2);
            }

        for(int i2 = 0; i2 < j; i2++)
            if(ai1[i2] < 0)
            {
                Object aobj3[] = {
                    (new StringBuilder()).append("maOrders[").append(i2).append("]").toString(), new Integer(ai1[i2])
                };
                System.out.println("Negative"+ aobj3);
            }

        int i1 = BLAS.iimax(i, ai, 0, 1) - 1;
        int k = ai[i1];
        i1 = BLAS.iimax(j, ai1, 0, 1) - 1;
        int l = ai1[i1];
        double ad[] = new double[1 + k + l];
        int ai2[][] = new int[nObsActual][2];
        double ad1[] = new double[nObsActual];
        num_outliers[0] = 0;
        l_auto_arima_driver(nObsActual, l_x, l_times, byte0, l_maxARLag, optimumModel, l_delta, l_critical, l_epsilon, res_sigma, num_outliers, ai2, ad, l_aic, l_aicc, l_bic, res_series, ad1, out_free_series, l_gapFreeSeries, l_completeTimes, ai, ai1, l_periods, l_orders, l_infoCriterion, l_relativeError);
        int j1 = optimumModel[0];
        int k1 = optimumModel[1];
        l_parameters = new double[1 + j1 + k1];
        if(num_outliers[0] > 0)
        {
            omega = new double[num_outliers[0]];
            outlier_stat = new int[num_outliers[0]][2];
        } else
        {
            omega = EMPTY_DOUBLE_VECTOR;
            outlier_stat = EMPTY_INTEGER_ARRAY;
        }
        System.arraycopy(ad1, 0, omega, 0, num_outliers[0]);
        for(int j2 = 0; j2 < num_outliers[0]; j2++)
            System.arraycopy(ai2[j2], 0, outlier_stat[j2], 0, 2);

        System.arraycopy(ad, 0, l_parameters, 0, 1 + j1 + k1);
        isComputed = true;
    }

    public final void compute(int i, int j, int k, int l)
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, ARAutoUnivariate.TriangularMatrixSingularException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.InitialMAException, SingularMatrixException, NoAcceptableModelFoundException, NotSPDException, DidNotConvergeException
    {
        byte byte0 = 3;
        int ai1[] = new int[0];
        int ai2[] = new int[0];
        if(i < 0)
        {
            Object aobj[] = {
                "p", new Integer(i)
            };
            System.out.println("Negative"+ aobj);
        }
        if(j < 0)
        {
            Object aobj1[] = {
                "q", new Integer(j)
            };
            System.out.println("Negative"+ aobj1);
        }
        if(k < 1)
        {
            Object aobj2[] = {
                "s", new Integer(k)
            };
            System.out.println("NotPositive"+ aobj2);
        }
        if(l < 0)
        {
            Object aobj3[] = {
                "d", new Integer(l)
            };
            System.out.println("Negative"+ aobj3);
        }
        double ad[] = new double[1 + i + j];
        int ai[][] = new int[nObsActual][2];
        double ad1[] = new double[nObsActual];
        num_outliers[0] = 0;
        optimumModel[0] = i;
        optimumModel[1] = j;
        optimumModel[2] = k;
        optimumModel[3] = l;
        l_auto_arima_driver(nObsActual, l_x, l_times, byte0, l_maxARLag, optimumModel, l_delta, l_critical, l_epsilon, res_sigma, num_outliers, ai, ad, l_aic, l_aicc, l_bic, res_series, ad1, out_free_series, l_gapFreeSeries, l_completeTimes, ai1, ai2, l_periods, l_orders, l_infoCriterion, l_relativeError);
        l_parameters = new double[1 + i + j];
        if(num_outliers[0] > 0)
        {
            omega = new double[num_outliers[0]];
            outlier_stat = new int[num_outliers[0]][2];
        } else
        {
            omega = EMPTY_DOUBLE_VECTOR;
            outlier_stat = EMPTY_INTEGER_ARRAY;
        }
        System.arraycopy(ad1, 0, omega, 0, num_outliers[0]);
        for(int i1 = 0; i1 < num_outliers[0]; i1++)
            System.arraycopy(ai[i1], 0, outlier_stat[i1], 0, 2);

        System.arraycopy(ad, 0, l_parameters, 0, 1 + i + j);
        isComputed = true;
    }

    public void setPeriods(int ai[])
    {
        int i = ai.length;
        if(i == 0)
        {
            l_periods = new int[1][1];
            l_periods[0][0] = 1;
        } else
        {
            l_periods = new int[i][1];
            for(int j = 0; j < i; j++)
                if(ai[j] <= 0)
                {
                    Object aobj[] = {
                        (new StringBuilder()).append("periods[").append(j).append("]").toString(), new Integer(ai[j])
                    };
                    System.out.println("NotPositive"+ aobj);
                } else
                {
                    l_periods[j][0] = ai[j];
                }

        }
    }

    public void setDifferenceOrders(int ai[])
    {
        int i = ai.length;
        if(i == 0)
        {
            l_orders = new int[1][1];
            l_orders[0][0] = 0;
        } else
        {
            l_orders = new int[i][1];
            for(int j = 0; j < i; j++)
                if(ai[j] < 0)
                {
                    Object aobj[] = {
                        (new StringBuilder()).append("orders[").append(j).append("]").toString(), new Integer(ai[j])
                    };
                    System.out.println("Negative"+ aobj);
                } else
                {
                    l_orders[j][0] = ai[j];
                }

        }
    }

    public void setMaximumARLag(int i)
    {
        if(i < 1 || i >= nObsActual)
        {
            Object aobj[] = {
                "maxARLag", new Integer(i), "1", new Integer(nObsActual - 1)
            };
            System.out.println("GreaterEqual"+ aobj);
        }
        l_maxARLag = i;
    }

    public int[] getOptimumModelOrder()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getOptimumModelOrder"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        int ai[] = new int[4];
        System.arraycopy(optimumModel, 0, ai, 0, 4);
        return ai;
    }

    public void setDelta(double d)
    {
        if(d <= 0.0D || d >= 1.0D)
        {
            Object aobj[] = {
                "delta", new Double(d), new Double(0.0D), new Double(1.0D)
            };
            System.out.println("GreaterLess"+ aobj);
        }
        l_delta = d;
    }

    public void setCriticalValue(double d)
    {
        if(d <= 0.0D)
        {
            Object aobj[] = {
                "critical", new Double(d)
            };
            System.out.println("NotPositive"+ aobj);
        }
        l_critical = d;
    }

    public void setAccuracyTolerance(double d)
    {
        if(d <= 0.0D)
        {
            Object aobj[] = {
                "epsilon", new Double(d)
            };
            System.out.println("NotPositive"+ aobj);
        }
        l_epsilon = d;
    }

    public void setRelativeError(double d)
    {
        if(d <= 0.0D)
        {
            Object aobj[] = {
                "relativeError", new Double(d)
            };
            System.out.println("NotPositive"+ aobj);
        }
        l_relativeError = d;
    }

    public double getResidualStandardError()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getResidualStandardError"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        return res_sigma[0];
    }

    public double[] getResiduals()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getResiduals"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        double ad[] = new double[nObsActual];
        System.arraycopy(res_series, 0, ad, 0, nObsActual);
        return ad;
    }

    public int getNumberOfOutliers()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getNumberOfOutliers"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        return num_outliers[0];
    }

    public int[][] getOutlierStatistics()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getOutlierStatistics"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        if(num_outliers[0] > 0)
        {
            int ai[][] = new int[num_outliers[0]][2];
            for(int i = 0; i < num_outliers[0]; i++)
                System.arraycopy(outlier_stat[i], 0, ai[i], 0, 2);

            return ai;
        } else
        {
            return EMPTY_INTEGER_ARRAY;
        }
    }

    public double getAIC()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getAIC"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        return l_aic[0];
    }

    public double getAICC()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getAICC"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        return l_aicc[0];
    }

    public double getBIC()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getBIC"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        return l_bic[0];
    }

    public void setModelSelectionCriterion(int i)
    {
        if(i < 0 || i > 2)
        {
            Object aobj[] = {
                "infoCriterion", new Integer(i), "0", "2"
            };
            System.out.println("GreaterEqual"+ aobj);
        }
        l_infoCriterion = i;
    }

    public double[] getMA()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getMA"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        int i = optimumModel[0];
        int j = optimumModel[1];
        if(j > 0)
        {
            double ad[] = new double[j];
            System.arraycopy(l_parameters, 1 + i, ad, 0, j);
            return ad;
        } else
        {
            return EMPTY_DOUBLE_VECTOR;
        }
    }

    public double[] getAR()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getAR"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        int i = optimumModel[0];
        if(i > 0)
        {
            double ad[] = new double[i];
            System.arraycopy(l_parameters, 1, ad, 0, i);
            return ad;
        } else
        {
            return EMPTY_DOUBLE_VECTOR;
        }
    }

    public double getConstant()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getConstant"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        return l_parameters[0];
    }

    public double[] getCompleteTimeSeries()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getCompleteTimeSeries"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        double ad[] = new double[nObsActual];
        System.arraycopy(l_gapFreeSeries, 0, ad, 0, nObsActual);
        return ad;
    }

    public int[] getCompleteTimes()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getCompleteTimes"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        int ai[] = new int[nObsActual];
        System.arraycopy(l_completeTimes, 0, ai, 0, nObsActual);
        return ai;
    }

    public double[] getOutlierFreeSeries()
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "getOutlierFreeSeries"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        double ad[] = new double[nObsActual];
        System.arraycopy(out_free_series, 0, ad, 0, nObsActual);
        return ad;
    }

    public final void forecast(int i)
    {
        if(!isComputed)
        {
            Object aobj[] = {
                "forecast"
            };
            System.out.println("ARMAOutlierIdentification.ComputeMethodRequired"+ aobj);
        }
        if(i <= 0)
        {
            Object aobj1[] = {
                "nForecast", new Integer(i), new Integer(0)
            };
            System.out.println("NotLargeEnough"+ aobj1);
        }
        double ad[] = new double[i];
        double ad1[] = new double[i];
        double ad2[][] = new double[i][3];
        double ad3[][] = new double[i][3];
        l_outfree_forecast = new double[i];
        l_outlier_forecast = new double[i];
        l_psi_weights = new double[i];
        l_dev = new double[i];
        l_ts_outlier_forecast_driver(nObsActual, out_free_series, optimumModel, outlier_stat, omega, res_series, ad, ad1, i, num_outliers[0], l_delta, ad3, ad2, l_confidence, l_parameters);
        for(int j = 0; j < i; j++)
        {
            l_outfree_forecast[j] = ad2[j][0];
            l_outlier_forecast[j] = ad3[j][0];
            l_dev[j] = ad2[j][1];
            l_psi_weights[j] = ad2[j][2];
        }

        isForecasted = true;
    }

    public void setConfidence(double d)
    {
        if(d <= 0.0D || d >= 1.0D)
        {
            Object aobj[] = {
                "confidence", new Double(d), new Double(0.0D), new Double(1.0D)
            };
            System.out.println("GreaterLess"+ aobj);
        }
        d *= 100D;
        l_confidence = d;
    }

    public double[] getDeviations()
    {
        if(!isForecasted)
        {
            Object aobj[] = {
                "forecast", "getDeviations"
            };
            System.out.println("CallMethodFirst"+ aobj);
        }
        int i = l_dev.length;
        double ad[] = new double[i];
        System.arraycopy(l_dev, 0, ad, 0, i);
        return ad;
    }

    public double[] getForecast()
    {
        if(!isForecasted)
        {
            Object aobj[] = {
                "forecast", "getForecast"
            };
            System.out.println("CallMethodFirst"+ aobj);
        }
        int i = l_outlier_forecast.length;
        double ad[] = new double[i];
        System.arraycopy(l_outlier_forecast, 0, ad, 0, i);
        return ad;
    }

    public double[] getOutlierFreeForecast()
    {
        if(!isForecasted)
        {
            Object aobj[] = {
                "forecast", "getOutlierFreeForecast"
            };
            System.out.println("CallMethodFirst"+ aobj);
        }
        int i = l_outfree_forecast.length;
        double ad[] = new double[i];
        System.arraycopy(l_outfree_forecast, 0, ad, 0, i);
        return ad;
    }

    public double[] getPsiWeights()
    {
        if(!isForecasted)
        {
            Object aobj[] = {
                "forecast", "getPsiWeights"
            };
            System.out.println("CallMethodFirst"+ aobj);
        }
        int i = l_psi_weights.length;
        double ad[] = new double[i];
        System.arraycopy(l_psi_weights, 0, ad, 0, i);
        return ad;
    }

    private void l_auto_arima_driver(int i, double ad[], int ai[], int j, int k, int ai1[], double d, double d1, double d2, double ad1[], int ai2[], 
            int ai3[][], double ad2[], double ad3[], double ad4[], double ad5[], double ad6[], double ad7[], 
            double ad8[], double ad9[], int ai4[], int ai5[], int ai6[], int ai7[][], int ai8[][], 
            int l, double d3)
        throws ARMA.MatrixSingularException, ARMA.TooManyCallsException, ARMA.IncreaseErrRelException, ARMA.NewInitialGuessException, ARMA.IllConditionedException, ARMA.TooManyITNException, ARMA.TooManyFcnEvalException, ARMA.TooManyJacobianEvalException, ARMA.NoProgressException, ARAutoUnivariate.TriangularMatrixSingularException, ARMAMaxLikelihood.NonInvertibleException, ARMAMaxLikelihood.NonStationaryException, ARMAMaxLikelihood.InitialMAException, SingularMatrixException, NoAcceptableModelFoundException, NotSPDException, DidNotConvergeException
    {
        double d4 = (1.0D / 0.0D);
        double d5 = (1.0D / 0.0D);
        double d6 = (1.0D / 0.0D);
        double ad10[] = null;
        double ad12[] = null;
        int ai13[][] = (int[][])null;
        double ad23[] = null;
        boolean flag = false;
        boolean flag1 = false;
        ARMAEstimateMissing armaestimatemissing = new ARMAEstimateMissing(ai, ad);
        armaestimatemissing.setMaxlag(k);
        armaestimatemissing.setMissingValueMethod(3);
        double ad25[] = armaestimatemissing.getCompleteTimeSeries();
        ai4[0] = ai[0];
        ad9[0] = ad25[0];
        for(int i1 = 1; i1 < i; i1++)
        {
            ad9[i1] = ad25[i1];
            ai4[i1] = ai4[i1 - 1] + 1;
        }

        if(j == 1)
        {
            ARSeasonalFit arseasonalfit = new ARSeasonalFit(k, ai7, ad9);
            arseasonalfit.setDInitial(ai8);
            arseasonalfit.setExclude(false);
            arseasonalfit.setCenter(2);
            arseasonalfit.setModelSelectionCriterion(l_infoCriterion);
            int l2 = arseasonalfit.getNLost();
            arseasonalfit.compute();
            int ai11[] = arseasonalfit.getOptimumD();
            int ai9[] = arseasonalfit.getOptimumS();
            int j3 = arseasonalfit.getAROrder();
            int l3 = j3;
            int k4 = 0;
            int j5 = ai9[0];
            int i6 = ai11[0];
            ai1[0] = l3;
            ai1[1] = k4;
            ai1[2] = j5;
            ai1[3] = i6;
            ARMAOutlierIdentification armaoutlieridentification2 = new ARMAOutlierIdentification(ad9);
            armaoutlieridentification2.setDelta(d);
            armaoutlieridentification2.setCriticalValue(d1);
            armaoutlieridentification2.setAccuracyTolerance(d2);
            armaoutlieridentification2.setRelativeError(d3);
            armaoutlieridentification2.compute(ai1);
            ad10 = armaoutlieridentification2.getResidual();
            ai2[0] = armaoutlieridentification2.getNumberOfOutliers();
            ad1[0] = armaoutlieridentification2.getResidualStandardError();
            ai13 = armaoutlieridentification2.getOutlierStatistics();
            ad12 = armaoutlieridentification2.getOmegaWeights();
            ad3[0] = armaoutlieridentification2.getAIC();
            ad4[0] = armaoutlieridentification2.getAICC();
            ad5[0] = armaoutlieridentification2.getBIC();
            ad23 = armaoutlieridentification2.getOutlierFreeSeries();
            System.arraycopy(ad10, 0, ad6, 0, i);
            System.arraycopy(ad23, 0, ad8, 0, i);
            if(ai2[0] > 0)
                System.arraycopy(ad12, 0, ad7, 0, ai2[0]);
            for(int j1 = 0; j1 < ai2[0]; j1++)
                System.arraycopy(ai13[j1], 0, ai3[j1], 0, 2);

            ad2[0] = armaoutlieridentification2.getConstant();
            if(l3 > 0)
            {
                double ad14[] = armaoutlieridentification2.getAR();
                System.arraycopy(ad14, 0, ad2, 1, l3);
            }
        }
        if(j == 2)
            if(ai7.length == 1 && ai8.length == 1)
            {
                int k5 = ai7[0][0];
                int j6 = ai8[0][0];
                ai1[2] = k5;
                ai1[3] = j6;
            } else
            {
                ARSeasonalFit arseasonalfit1 = new ARSeasonalFit(k, ai7, ad9);
                arseasonalfit1.setDInitial(ai8);
                arseasonalfit1.setExclude(false);
                arseasonalfit1.setCenter(2);
                arseasonalfit1.setModelSelectionCriterion(l_infoCriterion);
                arseasonalfit1.compute();
                int i3 = arseasonalfit1.getNLost();
                int ai12[] = arseasonalfit1.getOptimumD();
                int ai10[] = arseasonalfit1.getOptimumS();
                int k3 = arseasonalfit1.getAROrder();
                int l5 = ai10[0];
                int k6 = ai12[0];
                ai1[2] = l5;
                ai1[3] = k6;
            }
        if(j == 2)
        {
            int l6 = -1;
            int i7 = -1;
            ARMAOutlierIdentification armaoutlieridentification = new ARMAOutlierIdentification(ad9);
            armaoutlieridentification.setDelta(d);
            armaoutlieridentification.setCriticalValue(d1);
            armaoutlieridentification.setAccuracyTolerance(d2);
            armaoutlieridentification.setRelativeError(d3);
            int j7 = ai5.length;
            int k7 = ai6.length;
            for(int k1 = 0; k1 < j7; k1++)
            {
                for(int k2 = 0; k2 < k7; k2++)
                {
                    ai1[0] = ai5[k1];
                    ai1[1] = ai6[k2];
                    int i4 = ai1[0];
                    int l4 = ai1[1];
                    try
                    {
                        armaoutlieridentification.compute(ai1);
                        ad3[0] = armaoutlieridentification.getAIC();
                        ad4[0] = armaoutlieridentification.getAICC();
                        ad5[0] = armaoutlieridentification.getBIC();
                        if(l == 0 && ad3[0] < d4)
                        {
                            ad10 = armaoutlieridentification.getResidual();
                            ai2[0] = armaoutlieridentification.getNumberOfOutliers();
                            ad1[0] = armaoutlieridentification.getResidualStandardError();
                            ai13 = armaoutlieridentification.getOutlierStatistics();
                            ad12 = armaoutlieridentification.getOmegaWeights();
                            ad23 = armaoutlieridentification.getOutlierFreeSeries();
                            ad2[0] = armaoutlieridentification.getConstant();
                            if(ai1[0] > 0)
                            {
                                double ad15[] = armaoutlieridentification.getAR();
                                System.arraycopy(ad15, 0, ad2, 1, i4);
                            }
                            if(ai1[1] > 0)
                            {
                                double ad19[] = armaoutlieridentification.getMA();
                                System.arraycopy(ad19, 0, ad2, 1 + i4, l4);
                            }
                            d4 = ad3[0];
                            d5 = ad4[0];
                            d6 = ad5[0];
                            l6 = ai1[0];
                            i7 = ai1[1];
                        }
                        if(l == 1 && ad4[0] < d5)
                        {
                            ad10 = armaoutlieridentification.getResidual();
                            ai2[0] = armaoutlieridentification.getNumberOfOutliers();
                            ad1[0] = armaoutlieridentification.getResidualStandardError();
                            ai13 = armaoutlieridentification.getOutlierStatistics();
                            ad12 = armaoutlieridentification.getOmegaWeights();
                            ad23 = armaoutlieridentification.getOutlierFreeSeries();
                            ad2[0] = armaoutlieridentification.getConstant();
                            if(ai1[0] > 0)
                            {
                                double ad16[] = armaoutlieridentification.getAR();
                                System.arraycopy(ad16, 0, ad2, 1, i4);
                            }
                            if(ai1[1] > 0)
                            {
                                double ad20[] = armaoutlieridentification.getMA();
                                System.arraycopy(ad20, 0, ad2, 1 + i4, l4);
                            }
                            d4 = ad3[0];
                            d5 = ad4[0];
                            d6 = ad5[0];
                            l6 = ai1[0];
                            i7 = ai1[1];
                        }
                        if(l != 2 || ad5[0] >= d6)
                            continue;
                        ad10 = armaoutlieridentification.getResidual();
                        ai2[0] = armaoutlieridentification.getNumberOfOutliers();
                        ad1[0] = armaoutlieridentification.getResidualStandardError();
                        ai13 = armaoutlieridentification.getOutlierStatistics();
                        ad12 = armaoutlieridentification.getOmegaWeights();
                        ad23 = armaoutlieridentification.getOutlierFreeSeries();
                        ad2[0] = armaoutlieridentification.getConstant();
                        if(ai1[0] > 0)
                        {
                            double ad17[] = armaoutlieridentification.getAR();
                            System.arraycopy(ad17, 0, ad2, 1, i4);
                        }
                        if(ai1[1] > 0)
                        {
                            double ad21[] = armaoutlieridentification.getMA();
                            System.arraycopy(ad21, 0, ad2, 1 + i4, l4);
                        }
                        d4 = ad3[0];
                        d5 = ad4[0];
                        d6 = ad5[0];
                        l6 = ai1[0];
                        i7 = ai1[1];
                    }
                    catch(Exception ex) {System.out.println(ex.toString()); }
                }

            }

            ai1[0] = l6;
            ai1[1] = i7;
            if(ai1[0] < 0 || ai1[1] < 0)
                throw new NoAcceptableModelFoundException("AutoARIMA.NoAcceptableModel", null);
            System.arraycopy(ad10, 0, ad6, 0, i);
            System.arraycopy(ad23, 0, ad8, 0, i);
            if(ai2[0] > 0)
                System.arraycopy(ad12, 0, ad7, 0, ai2[0]);
            for(int l1 = 0; l1 < ai2[0]; l1++)
                System.arraycopy(ai13[l1], 0, ai3[l1], 0, 2);

            ad3[0] = d4;
            ad4[0] = d5;
            ad5[0] = d6;
        }
        if(j == 3)
        {
            int j4 = ai1[0];
            int i5 = ai1[1];
            ARMAOutlierIdentification armaoutlieridentification1 = new ARMAOutlierIdentification(ad9);
            armaoutlieridentification1.setDelta(d);
            armaoutlieridentification1.setCriticalValue(d1);
            armaoutlieridentification1.setAccuracyTolerance(d2);
            armaoutlieridentification1.setRelativeError(d3);
            armaoutlieridentification1.compute(ai1);
            double ad11[] = armaoutlieridentification1.getResidual();
            ai2[0] = armaoutlieridentification1.getNumberOfOutliers();
            ad1[0] = armaoutlieridentification1.getResidualStandardError();
            int ai14[][] = armaoutlieridentification1.getOutlierStatistics();
            double ad13[] = armaoutlieridentification1.getOmegaWeights();
            ad3[0] = armaoutlieridentification1.getAIC();
            ad4[0] = armaoutlieridentification1.getAICC();
            ad5[0] = armaoutlieridentification1.getBIC();
            double ad24[] = armaoutlieridentification1.getOutlierFreeSeries();
            System.arraycopy(ad11, 0, ad6, 0, i);
            System.arraycopy(ad24, 0, ad8, 0, i);
            if(ai2[0] > 0)
                System.arraycopy(ad13, 0, ad7, 0, ai2[0]);
            for(int i2 = 0; i2 < ai2[0]; i2++)
                System.arraycopy(ai14[i2], 0, ai3[i2], 0, 2);

            ad2[0] = armaoutlieridentification1.getConstant();
            if(j4 > 0)
            {
                double ad18[] = armaoutlieridentification1.getAR();
                System.arraycopy(ad18, 0, ad2, 1, j4);
            }
            if(i5 > 0)
            {
                double ad22[] = armaoutlieridentification1.getMA();
                System.arraycopy(ad22, 0, ad2, 1 + j4, i5);
            }
        }
        for(int j2 = 0; j2 < ai2[0]; j2++)
            ai3[j2][0] += ai[0] - 1;

    }

    private void l_ts_outlier_forecast_driver(int i, double ad[], int ai[], int ai1[][], double ad1[], double ad2[], double ad3[], 
            double ad4[], int j, int k, double d, double ad5[][], double ad6[][], 
            double d1, double ad7[])
    {
        int l = ai[0];
        int i1 = ai[1];
        int j1 = ai[2];
        int k1 = ai[3];
        double ad8[] = new double[l + j1 * k1 + 1];
        double ad9[] = new double[i + j + 1];
        double d6 = 0.0D;
        for(int l1 = Math.max(l + j1 * k1, i1); l1 < i; l1++)
            d6 += ad2[l1] * ad2[l1];

        if(i - Math.max(l + j1 * k1, i1) > 0)
            d6 /= i - Math.max(l + j1 * k1, i1);
        double d2 = (100D - d1) * 0.01D;
        double d4 = Cdf.inverseNormal(1.0D - d2 / 2D);
        double d3 = 0.0D;
        l_compute_psi(ad7, ad8, ad9, ai, i + j);
        l_compute_forecasts(i, j, d3, ai, ad3, ad, ad2, ad7, ad8);
        BLAS.copy(j, ad3, ad4);
        for(int k3 = 0; k3 < k; k3++)
        {
            int l3 = ai1[k3][1];
            int i4 = ai1[k3][0];
            if(l3 == 0 || l3 == 4)
            {
                for(int i2 = 0; i2 < j; i2++)
                    ad4[i2] += ad1[k3] * ad9[(i - i4) + i2 + 1];

            }
            if(l3 == 2)
            {
                for(int j2 = 0; j2 < j; j2++)
                    ad4[j2] += ad1[k3];

            }
            if(l3 != 3)
                continue;
            for(int k2 = 0; k2 < j; k2++)
                ad4[k2] += ad1[k3] * Math.pow(d, (i - i4) + k2 + 1);

        }

        for(int l2 = 0; l2 < j; l2++)
        {
            ad5[l2][2] = ad9[l2 + 1];
            ad6[l2][2] = ad9[l2 + 1];
        }

        double d5 = 0.0D;
        ad5[0][1] = d4 * Math.sqrt(d6);
        ad6[0][1] = d4 * Math.sqrt(d6);
        for(int i3 = 2; i3 <= j; i3++)
        {
            d5 += Math.pow(ad5[i3 - 2][2], 2D);
            ad5[i3 - 1][1] = d4 * Math.sqrt((1.0D + d5) * d6);
            ad6[i3 - 1][1] = d4 * Math.sqrt((1.0D + d5) * d6);
        }

        for(int j3 = 0; j3 < j; j3++)
        {
            ad5[j3][0] = ad4[j3];
            ad6[j3][0] = ad3[j3];
        }

    }

    private void l_compute_forecasts(int i, int j, double d, int ai[], double ad[], double ad1[], 
            double ad2[], double ad3[], double ad4[])
    {
        int k = ai[0];
        int l = ai[1];
        int i1 = ai[2];
        int j1 = ai[3];
        double d1 = ad3[0];
        for(int l1 = 1; l1 <= j; l1++)
        {
            int k1 = l1 - 1;
            ad[k1] = d1;
            for(int i2 = 1; i2 <= k + i1 * j1; i2++)
                if(l1 - i2 <= 0)
                    ad[k1] += ad4[i2] * ad1[((i - 1) + l1) - i2];
                else
                    ad[k1] += ad4[i2] * ad[l1 - i2 - 1];

            for(int j2 = l1; j2 <= l; j2++)
                ad[k1] -= ad3[k + j2] * ad2[((i - 1) + l1) - j2];

        }

    }

    private void l_compute_psi(double ad[], double ad1[], double ad2[], int ai[], int i)
    {
        int j = ai[0];
        int k = ai[1];
        int l = ai[2];
        int i1 = ai[3];
        ad1[0] = 1.0D;
        for(int j1 = 1; j1 <= j + l * i1; j1++)
        {
            if(j1 <= j)
                ad1[j1] = ad[j1];
            else
                ad1[j1] = 0.0D;
            double d = 1.0D;
            for(int l1 = 1; l1 <= i1; l1++)
            {
                if(j1 - l * l1 == 0)
                    ad1[j1] += l_binom(i1, l1) * d;
                if(j1 - l * l1 > 0 && j1 - l * l1 <= j)
                    ad1[j1] += l_binom(i1, l1) * d * -ad[j1 - l * l1];
                d *= -1D;
            }

        }

        ad2[0] = 1.0D;
        for(int k1 = 1; k1 <= i; k1++)
        {
            if(k1 <= k)
                ad2[k1] = -ad[j + k1];
            else
                ad2[k1] = 0.0D;
            for(int i2 = 1; i2 <= k1; i2++)
                if(i2 <= j + l * i1)
                    ad2[k1] += ad2[k1 - i2] * ad1[i2];

        }

    }

    private double l_binom(int i, int j)
    {
        boolean flag1 = false;
        double d8 = Math.log(1.7976931348623157E+308D) - 0.0001D;
        double d = (0.0D / 0.0D);
        int l = Math.min(j, i - j);
        d = 1.0D;
        if(l == 0 || i == 1)
            return d;
        double d5 = i;
        double d7 = d5 + 1.0D;
        double d3 = l;
        double d4 = d3 + 1.0D;
        boolean flag;
        if(l <= 20)
        {
            flag = false;
            if(d3 * Math.log(d5) > d8)
                flag = true;
        } else
        {
            flag = true;
        }
        if(!flag)
        {
            double d2 = 0.0D;
            for(int k = 1; k <= l; k++)
            {
                d2++;
                d = (d * (d7 - d2)) / d2;
            }

        } else
        {
            if(l < 9)
            {
                System.out.println("ANOVAFactorial.BinomialOverflow");
                d = (0.0D / 0.0D);
                return d;
            }
            double d6 = d7 - d3;
            double d1 = Sfun.r9lgmc(d7) - Sfun.r9lgmc(d4) - Sfun.r9lgmc(d6);
            d = (((d4 * Math.log(d6 / d4) - d7 * dlnrel(-d3 / d7) - 0.5D * Math.log((d7 * d6) / d4)) + 1.0D) - 0.91893853320467278D) + d1;
            if(d > d8)
            {
                System.out.println("ANOVAFactorial.BinomialOverflow");
                d = (0.0D / 0.0D);
                return d;
            }
            d = Math.exp(d);
        }
        if(d < 8106479329266893D)
            d = (int)d;
        return d;
    }

    private double csevl(double d, double ad[])
    {
        double d2 = 0.0D;
        double d1 = 0.0D;
        double d3 = 0.0D;
        double d4 = 2D * d;
        for(int i = ad.length - 1; i >= 0; i--)
        {
            d3 = d2;
            d2 = d1;
            d1 = (d4 * d2 - d3) + ad[i];
        }

        return 0.5D * (d1 - d3);
    }

    private double dlnrel(double d)
    {
        double ad[] = {
            1.037869356274377D, -0.13364301504908918D, 0.019408249135520562D, -0.0030107551127535777D, 0.00048694614797154852D, -8.1054881893175362E-005D, 1.3778847799559525E-005D, -2.380221089435897E-006D, 4.1640416213865184E-007D, -7.3595828378075992E-008D, 
            1.3117611876241675E-008D, -2.3546709317742423E-009D, 4.2522773276035E-010D, -7.7190894134840799E-011D, 1.407574648135907E-011D, -2.5769072058024682E-012D, 4.7342406666294419E-013D, -8.7249012674742641E-014D, 1.6124614902740551E-014D, -2.9875652015665774E-015D, 
            5.5480701209082887E-016D, -1.0324619158271569E-016D, 1.9250239203049852E-017D, -3.5955073465265147E-018D, 6.726454253787686E-019D, -1.260262416873522E-019D
        };
        double d1;
        if(d <= -1D)
            d1 = (0.0D / 0.0D);
        else
        if(Math.abs(d) <= 0.375D)
            d1 = d * (1.0D - d * csevl(d / 0.375D, ad));
        else
            d1 = Math.log(1.0D + d);
        return d1;
    }

   

}