

package archeopterix.util.statistics.timeseries.ARIMA;

import java.io.Serializable;

public class AutoCorrelation
    implements Serializable, Cloneable
{
    private double data[];
    private double xmean;
    private int maxlag;
    private int nobs;
    public static final int BARTLETTS_FORMULA = 1;
    public static final int MORANS_FORMULA = 2;

    public static class NonPosVariancesException extends STATException
    {

        private static final long serialVersionUID = 0xbe9f9a7227d5047dL;

        public NonPosVariancesException(String s)
        {
            super(s);
        }

        public NonPosVariancesException(String s, Object aobj[])
        {
            super(s+aobj);
        }
    }


    public AutoCorrelation(double ad[], int i)
    {
        data = new double[ad.length];
        System.arraycopy(ad, 0, data, 0, ad.length);
        maxlag = i;
        nobs = data.length;
        if(nobs < 2)
        {
            Object aobj[] = {
                "x", new Integer(data.length), new Integer(2)
            };
            System.out.println("ArrayLength"+aobj);
        }
        if(maxlag < 1 || maxlag >= nobs)
        {
            Object aobj1[] = {
                "maximum_lag", new Integer(maxlag), new Integer(1), new Integer(nobs)
            };
            System.out.println("OutOfRange"+aobj1);
        }
        for(int j = 0; j < nobs; j++)
            if(Double.isNaN(data[j]))
            {
                Object aobj2[] = {
                    "x", new Integer(j)
                };
                System.out.println("ContainsNaN"+aobj2);
            }

        if(nobs > 1 && ad[BLAS.ismin(nobs, ad, 0, 1) - 1] == ad[BLAS.ismax(nobs, ad, 0, 1) - 1])
        {
            Object aobj3[] = {
                "x", new Double(ad[BLAS.ismin(nobs, ad, 0, 1) - 1])
            };
            System.out.println("ConstantSeries"+aobj3);
        }
        xmean = BLAS.sum(nobs, ad, 0, 1) / (double)nobs;
    }

    public void setMean(double d)
    {
        xmean = d;
    }

    public double getMean()
    {
        return xmean;
    }

    public double[] getStandardErrors(int i)
    {
        if(i < 1 && i > 2)
        {
            Object aobj[] = {
                "stderrMethod", new Integer(i), new Integer(0), new Integer(3)
            };
            System.out.println("OutOfRange"+aobj);
        }
        double ad1[] = new double[maxlag + 1];
        ad1 = getAutoCorrelations();
        double ad[] = new double[maxlag];
        if(i == 1)
        {
            BLAS.set(maxlag, 0.0D, ad, 0);
            for(int j = 1; j <= maxlag; j++)
            {
                for(int l = -maxlag; l <= maxlag; l++)
                    ad[j - 1] += ad1[Math.abs(l)] * ad1[Math.abs(l)] * (1.0D + 2D * ad1[j] * ad1[j]);

                for(int i1 = -(maxlag - j); i1 <= maxlag - j; i1++)
                    ad[j - 1] += ad1[Math.abs(i1 + j)] * ad1[Math.abs(i1 - j)];

                for(int j1 = -(maxlag - j); j1 <= maxlag; j1++)
                    ad[j - 1] += -4D * ad1[j] * ad1[Math.abs(j1)] * ad1[Math.abs(j1 - j)];

                ad[j - 1] = Math.sqrt((1.0D / (double)nobs) * ad[j - 1]);
            }

        } else
        if(i == 2)
        {
            for(int k = 1; k <= maxlag; k++)
                ad[k - 1] = Math.sqrt((double)(nobs - k) / (double)(nobs * (nobs + 2)));

        }
        return ad;
    }

    public double[] getAutoCorrelations()
    {
        double ad[] = new double[maxlag + 1];
        double ad1[] = new double[maxlag + 1];
        try
        {
            ad = getAutoCovariances();
        }
        catch(Exception exception) { }
        int i = maxlag + 1;
        double d = 1.0D / ad[0];
        int j = 1;
        int k = 1;
        BLAS.dvcal(i, d, ad, 0, j, ad1, 0, k);
        return ad1;
    }

    public double[] getAutoCovariances()
        throws NonPosVariancesException
    {
        double ad[] = new double[maxlag + 1];
        BLAS.set(maxlag + 1, 0.0D, ad, 0);
        for(int j = 0; j <= maxlag; j++)
        {
            for(int i = 1; i <= nobs - j; i++)
                ad[j] += ((data[i - 1] - xmean) * (data[(i + j) - 1] - xmean)) / (double)nobs;

        }

        if(ad[0] < 4.9406564584124654E-324D)
        {
            Object aobj[] = {
                "Variance", new Double(ad[0])
            };
            throw new NonPosVariancesException("NotPositive"+aobj);
        } else
        {
            return ad;
        }
    }

    public double getVariance()
    {
        double ad[] = new double[maxlag + 1];
        try
        {
            ad = getAutoCovariances();
        }
        catch(Exception exception) { }
        return ad[0];
    }

    public double[] getPartialAutoCorrelations()
    {
        double ad[] = new double[maxlag];
        double ad1[] = new double[2 * maxlag];
        double ad2[] = new double[maxlag + 1];
        try
        {
            ad2 = getAutoCorrelations();
        }
        catch(Exception exception) { }
        ad1[0] = ad2[1];
        ad[0] = ad2[1];
        for(int k = 2; k <= maxlag; k++)
        {
            int j1 = k - 1;
            ad1[j1] = (ad2[k] - BLAS.dot(k - 1, ad1, 0, 1, ad2, 1, -1)) / (1.0D - BLAS.dot(k - 1, ad1, 0, 1, ad2, 1, 1));
            ad[j1] = ad1[j1];
            for(int i = 1; i <= k - 1; i++)
            {
                int l = i - 1;
                ad1[maxlag + l] = ad1[l] - ad[j1] * ad1[j1 - i];
            }

            for(int j = 1; j <= k - 1; j++)
            {
                int i1 = j - 1;
                ad1[i1] = ad1[maxlag + i1];
            }

        }

        return ad;
    }

  

}