
package archeopterix.util.statistics.timeseries.ARIMA;

import java.io.Serializable;

// Referenced classes of package
//            Complex, JMath

public class ZeroPolynomial
    implements Serializable, Cloneable
{
	    private static final long serialVersionUID = 0x966087631a82fe24L;
    private Complex root[];
    private double radius[];
    private boolean err[];
    private int maxIterations;
    public static final double xsmall = JMath.log(4.9406564584124654E-324D);
    public static final double xbig = JMath.log(1.7976931348623157E+308D);
    public static final double EPSILON_SMALL = 1.1102230246251999E-016D;
    private class State
    {

        public double radius;
        public Complex corr;
        public boolean again;
        public final ZeroPolynomial this$0;

        private State()
        {
			super();
            this$0 = ZeroPolynomial.this;
            
        }

    }

    public static class DidNotConvergeException extends STATException
    {

        private static final long serialVersionUID = 0x2418ff52fe8d540eL;

        public DidNotConvergeException(String s)
        {
            super(s);
        }

        public DidNotConvergeException(String s, Object aobj[])
        {
            super( s+ aobj);
        }
    }


    public ZeroPolynomial()
    {
        maxIterations = 30;
        radius = null;
        err = null;
    }

    public synchronized void setMaxIterations(int i)
    {
        if(i <= 0)
        {
            Object aobj[] = {
                "MaxIterations", new Integer(i)
            };
            System.out.println("NotPositive"+ aobj);
        }
        maxIterations = i;
    }

    public Complex getRoot(int i)
    {
        return root[i];
    }

    public Complex[] getRoots()
    {
        return (Complex[])(Complex[])root.clone();
    }

    public double getRadius(int i)
    {
        return radius[i];
    }

    public boolean getStatus(int i)
    {
        return err[i];
    }

    public synchronized Complex[] computeRoots(double ad[])
        throws DidNotConvergeException
    {
        int i = ad.length;
        Complex acomplex[] = new Complex[i];
        for(int j = 0; j < i; j++)
            acomplex[j] = new Complex(ad[j]);

        return computeRoots(acomplex);
    }

    public synchronized Complex[] computeRoots(Complex acomplex[])
        throws DidNotConvergeException
    {
        int i = acomplex.length - 1;
        Complex complex = new Complex();
        double ad[] = new double[i + 1];
        double ad1[] = new double[i + 1];
        if(Complex.abs(acomplex[i]) == 0.0D)
            System.out.println("ZeroPolynomial.Coef0");
        if(Complex.abs(acomplex[0]) == 0.0D)
            System.out.println("ZeroPolynomial.CoefN");
        root = new Complex[i];
        radius = new double[i];
        err = new boolean[i + 1];
        double d = 0.0D;
        for(int k = 0; k < i + 1; k++)
        {
            ad[k] = Complex.abs(acomplex[k]);
            d = Math.max(d, ad[k]);
            ad1[k] = ad[k];
        }

        if(d >= 1.7976931348623157E+308D / (double)(i + 1))
            System.out.println(this+ "ZeroPolynomial.LargeCoef");
        for(int l = 0; l < i; l++)
        {
            radius[l] = 0.0D;
            err[l] = true;
        }

        int j = start(i, ad1, root, err);
        for(int i1 = 0; i1 < i + 1; i1++)
        {
            ad1[i - i1] = 1.1102230246251999E-016D * ad[i1] * (3.7999999999999998D * (double)(i - i1) + 1.0D);
            ad[i1] = 1.1102230246251999E-016D * ad[i1] * (3.7999999999999998D * (double)i1 + 1.0D);
        }

        if(ad[0] == 0.0D || ad[i] == 0.0D)
            System.out.println(this+ "ZeroPolynomial.InclusionRadius");
        for(int j1 = 0; j1 < i; j1++)
            err[j1] = !Double.isNaN(radius[j1]);

        State state = new State();
        for(int k1 = 0; k1 < maxIterations; k1++)
        {
            for(int l1 = 0; l1 < i; l1++)
            {
                if(!err[l1])
                    continue;
                newton(i, acomplex, ad, ad1, root[l1], state);
                radius[l1] = state.radius;
                err[l1] = state.again;
                if(err[l1])
                {
                    Complex complex1 = aberth(i, l1, root);
                    root[l1] = Complex.subtract(root[l1], Complex.divide(state.corr, Complex.subtract(1.0D, Complex.multiply(state.corr, complex1))));
                    continue;
                }
                if(++j == i)
                    return (Complex[])(Complex[])root.clone();
            }

        }

        throw new DidNotConvergeException("ZeroPolynomial.DidNotConverge", null);
    }

    private void newton(int i, Complex acomplex[], double ad[], double ad1[], Complex complex, State state)
    {
        double d5 = Complex.abs(complex);
        if(d5 <= 1.0D)
        {
            Complex complex1 = acomplex[i];
            double d = ad[i];
            Complex complex3 = complex1;
            for(int j = i; j >= 2; j--)
            {
                complex1 = Complex.add(Complex.multiply(complex1, complex), acomplex[j - 1]);
                complex3 = Complex.add(Complex.multiply(complex3, complex), complex1);
                d = d * d5 + ad[j - 1];
            }

            complex1 = Complex.add(Complex.multiply(complex1, complex), acomplex[0]);
            d = d * d5 + ad[0];
            state.corr = Complex.divide(complex1, complex3);
            double d3 = Complex.abs(complex1);
            d = d;
            state.again = d3 > 4.9406564584124654E-324D + d;
            if(!state.again)
                state.radius = ((double)i * (d3 + d)) / Complex.abs(complex3);
        } else
        {
            Complex complex5 = Complex.divide(1.0D, complex);
            double d2 = 1.0D / d5;
            Complex complex2 = acomplex[0];
            Complex complex4 = complex2;
            double d1 = ad1[i];
            for(int k = i; k >= 2; k--)
            {
                complex2 = Complex.add(Complex.multiply(complex2, complex5), acomplex[(i - k) + 1]);
                complex4 = Complex.add(Complex.multiply(complex4, complex5), complex2);
                d1 = d1 * d2 + ad1[k - 1];
            }

            complex2 = Complex.add(Complex.multiply(complex2, complex5), acomplex[i]);
            d1 = d1 * d2 + ad1[0];
            double d4 = Complex.abs(complex2);
            state.again = d4 > 4.9406564584124654E-324D + d1;
            Complex complex7 = Complex.divide(Complex.multiply(complex2, complex), complex4);
            Complex complex6 = Complex.subtract(Complex.multiply(i, complex7), 1.0D);
            state.corr = Complex.multiply(complex, Complex.divide(complex7, complex6));
            if(state.again)
                return;
            state.radius = Complex.abs(complex7) + (d1 * d5) / Complex.abs(complex4);
            state.radius = ((double)i * state.radius) / Complex.abs(complex6);
            state.radius *= d5;
        }
    }

    private Complex aberth(int i, int j, Complex acomplex[])
    {
        Complex complex = new Complex();
        Complex complex1 = acomplex[j];
        for(int k = 0; k < i; k++)
            if(k != j)
                complex = Complex.add(complex, Complex.divide(1.0D, Complex.subtract(complex1, acomplex[k])));

        return complex;
    }

    private int start(int i, double ad[], Complex acomplex[], boolean aflag[])
    {
        double d1 = 0.0D;
        double d2 = 0.69999999999999996D;
        int j = 0;
        for(int k = 0; k < i + 1; k++)
            ad[k] = ad[k] != 0.0D ? JMath.log(ad[k]) : -1E+030D;

        computeConvexHull(i + 1, ad, aflag);
        int l = 1;
        double d3 = 6.2831853071795862D / (double)i;
        for(int i1 = 2; i1 <= i + 1; i1++)
        {
            if(!aflag[i1 - 1])
                continue;
            int j1 = i1 - l;
            double d4 = (ad[l - 1] - ad[i1 - 1]) / (double)j1;
            if(d4 < -xbig && d4 >= xsmall)
            {
                System.out.println(this+ "ZeroPolynomial.SetSmall");
                j += j1;
                d1 = 5.5626846462680035E-309D;
            }
            if(d4 < xsmall)
            {
                j += j1;
                System.out.println(this+ "ZeroPolynomial.SetZero");
            }
            if(d4 > xbig)
            {
                d1 = 1.7976931348623157E+308D;
                j += j1;
                System.out.println(this+ "ZeroPolynomial.TooBig");
            }
            if(d4 <= xbig && d4 > Math.max(-xbig, xsmall))
                d1 = JMath.exp(d4);
            double d = 6.2831853071795862D / (double)j1;
            for(int k1 = l; k1 <= i1 - 1; k1++)
            {
                int l1 = (k1 - l) + 1;
                if(d1 <= 5.5626846462680035E-309D || d1 == 1.7976931348623157E+308D)
                    radius[k1 - 1] = (0.0D / 0.0D);
                double d5 = d * (double)l1 + d3 * (double)i1 + d2;
                acomplex[k1 - 1] = new Complex(d1 * JMath.cos(d5), d1 * JMath.sin(d5));
            }

            l = i1;
        }

        return j;
    }

    private void computeConvexHull(int i, double ad[], boolean aflag[])
    {
        for(int j = 0; j < i; j++)
            aflag[j] = true;

        int k = (int)(JMath.log((double)i - 2D) / JMath.log(2D));
        if((int)JMath.pow(2D, k + 1) <= i - 2)
            k++;
        int l = 1;
        for(int i1 = 0; i1 <= k; i1++)
        {
            int j1 = Math.max(0, (i - 2 - l) / (l + l));
            for(int k1 = 0; k1 <= j1; k1++)
            {
                int l1 = (k1 + k1 + 1) * l + 1;
                cmerge(i, ad, l1, l, aflag);
            }

            l += l;
        }

    }

    private int left(int i, boolean aflag[], int j)
    {
        for(int k = j - 1; k >= 0; k--)
            if(aflag[k - 1])
                return k;

        return 0;
    }

    private int right(int i, boolean aflag[], int j)
    {
        for(int k = j + 1; k <= i; k++)
            if(aflag[k - 1])
                return k;

        return i;
    }

    private void cmerge(int i, double ad[], int j, int k, boolean aflag[])
    {
        int l = 0;
        int i1 = 0;
        int j1 = left(i, aflag, j);
        int k1 = right(i, aflag, j);
        if(ctest(i, ad, j1, j, k1))
            return;
        aflag[j - 1] = false;
        do
        {
            boolean flag1;
            do
            {
                boolean flag;
                if(j1 == j - k)
                {
                    flag = true;
                } else
                {
                    l = left(i, aflag, j1);
                    flag = ctest(i, ad, l, j1, k1);
                }
                if(k1 == Math.min(i, j + k))
                {
                    flag1 = true;
                } else
                {
                    i1 = right(i, aflag, k1);
                    flag1 = ctest(i, ad, j1, k1, i1);
                }
                aflag[j1 - 1] = flag;
                aflag[k1 - 1] = flag1;
                if(flag && flag1)
                    return;
                if(!flag)
                    j1 = l;
            } while(flag1);
            k1 = i1;
        } while(true);
    }

    private boolean ctest(int i, double ad[], int j, int k, int l)
    {
        double d = ad[k - 1] - ad[j - 1];
        double d1 = ad[l - 1] - ad[k - 1];
        double d2 = 0.40000000000000002D;
        d *= l - k;
        d1 *= k - j;
        return d > d1 + d2;
    }



}
