/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package archeopterix.algorithm.learning.benchmarks;

import archeopterix.conf.Conf;
import java.util.Random;

public class ProbMatching extends OperatorSelection {

	double p_min;
	double alpha;
	double[] quality;
	double[] prob;
	int[] opTimes;
	double best_quality;

	public ProbMatching(int N_operators, double P_min, double Alpha) {

		super(N_operators, true);
		p_min = P_min;
		alpha = Alpha;
		quality = new double[n_operators];
		prob = new double[n_operators];
		opTimes = new int[n_operators];
	}

	public void Reset() {
		for (int i = 0; i < n_operators; i++) {
			prob[i] = 1.0 / ((double) n_operators);
			quality[i] = 1.0;
			opTimes[i] = 0;
		}
		best_quality = 0;
	}

	@Override
	public double applyReward(int op) {
            if (best_op == op) {
                exploit++;
            } else {
                explore++;
            }
            double reward = getReward(op);
            updateQualityVector(op, reward);
            updateProbabilityVector();
            updateBestOp(op);
            opTimes[op]++;
            return reward;
	}

	public void updateBestOp(int op) {
		double exploit_op = quality[op];
		//if the one that was the best has a lower \hat p, check between all; or if the previous best changed
		if ((best_op == op && exploit_op < best_quality) || (best_op >= 0 && quality[best_op] != best_quality)) {
			best_quality = -1;
			best_op = 0;
			double exploit_aux = -1;
			for (int i = 0; i < n_operators; i++) {
				exploit_aux = quality[i];
				if (exploit_aux > best_quality) {
					best_quality = exploit_aux;
					best_op = i;
				}
			}
		} else if (exploit_op >= best_quality) { //else if it is best than the previous best
			best_op = op;
			best_quality = exploit_op;
		}
	}

	public void updateQualityVector(int op, double reward) {
		if (credit.get(op).getType() < 3) {
			quality[op] = quality[op] + (alpha * (reward - quality[op]));
		} else {
			quality[op] = reward;
		}
	}

	public void updateProbabilityVector() {
		double sum = 0.0;
		for (int i = 0; i < n_operators; i++) {
			sum += quality[i];
		}
		for (int i = 0; i < n_operators; i++) {
			prob[i] = p_min + ((1.0 - ((double) n_operators) * p_min) * (quality[i] / sum));
		}
	}

	void printSpecificResults() {
	}

    @Override
    public int selectOperator(int times_total) {    
        Random rand= Conf.instance().random;
        int i = (int) (rand.nextDouble() * n_operators);
        for (int j = 0; j < n_operators; j++) {
            int k = (i + j) % n_operators;
            if (opTimes[k] == 0) {
                return k;
            }
        }
        double sorted = rand.nextDouble();
        double sum = 0;
        int op;
        for (op = 0; op < n_operators - 1; op++) {
            sum += prob[op];
            if (sum > sorted) {
                return op;
            }
        }
        return op;
        
    }
}
